
// **************************************
// Header Files
// **************************************

#include "device.h"

// **************************
// Class Declaration
// **************************

Device::Device(void)
{  // constructor
}

Device::~Device(void)
{  // destructor
}

Device::Device(const Device &dev)
{  // copy constructor

  identifier=dev.GetId();
  driver=dev.GetDriver();
  busID=dev.GetBus();
  screen=dev.GetScreen();
  videoRam=dev.GetVideoRam();
  comment=dev.GetComment();

  identifierComment=dev.GetIdComment();
  driverComment=dev.GetDriverComment();
  busIDComment=dev.GetBusComment();
  screenComment=dev.GetScreenComment();
  videoRamComment=dev.GetVideoRamComment();

  for (int i=0; i<dev.GetOptionSize(); i++)
  {
    AddOption(dev.GetOption(i));
  }

}

Device& Device::operator=(const Device &dev)
{  // copy constructor

  Clear();

  identifier=dev.GetId();
  driver=dev.GetDriver();
  busID=dev.GetBus();
  screen=dev.GetScreen();
  videoRam=dev.GetVideoRam();
  comment=dev.GetComment();

  identifierComment=dev.GetIdComment();
  driverComment=dev.GetDriverComment();
  busIDComment=dev.GetBusComment();
  screenComment=dev.GetScreenComment();
  videoRamComment=dev.GetVideoRamComment();

  for (int i=0; i<dev.GetOptionSize(); i++)
  {
    AddOption(dev.GetOption(i));
  }

  return(*this);
}

void Device::AddComment(const wxString& com)
{  // set comment

  comment.Add(com);
}

void Device::SetId(const wxString& id, const wxString& com)
{  // set identifier

  identifier=id;
  identifierComment=com;
}

void Device::SetDriver(const wxString& dr, const wxString& com)
{  // set driver

  driver=dr;
  driverComment=com;
}

void Device::SetBus(const wxString& bus, const wxString& com)
{  // set bus type

  busID=bus;
  busIDComment=com;
}

void Device::SetScreen(const wxString& scr, const wxString& com)
{  // set screen to use

  screen=scr;
  screenComment=com;
}

void Device::SetVideoRam(const wxString& ram, const wxString& com)
{  // set video ram size

  videoRam=ram;
  videoRamComment=com;
}

void Device::RemoveOption(const int index)
{  // remove option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Device::RemoveOption"), wxICON_ERROR);
    return;
  }

  options.Remove(index);
}

void Device::AddOption(const Option &opt)
{  // add option

  options.Add(opt);
}

void Device::SetOption(const int index, const Option& opt)
{  // set option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Device::SetOption"), wxICON_ERROR);
    return;
  }

  options[index]=opt;
}

void Device::Clear(void)
{    // clear everything

  identifier=wxEmptyString;
  driver=wxEmptyString;
  busID=wxEmptyString;
  screen=wxEmptyString;
  videoRam=wxEmptyString;
  identifierComment=wxEmptyString;
  driverComment=wxEmptyString;
  busIDComment=wxEmptyString;
  screenComment=wxEmptyString;
  videoRamComment=wxEmptyString;
  comment.Clear();
  options.Clear();
}

const wxArrayString& Device::GetComment(void) const
{  // return comment

  return(comment);
}

const wxString& Device::GetId(void) const
{  // return identifier

  return(identifier);
}

const wxString& Device::GetDriver(void) const
{  // retun driver name

  return(driver);
}

const wxString& Device::GetBus(void) const
{  // return bus id

  return(busID);
}

const wxString& Device::GetScreen(void) const
{  // return used screen

  return(screen);
}

const wxString& Device::GetVideoRam(void) const
{  // return video ram size

  return(videoRam);
}

const wxString& Device::GetIdComment(void) const
{  // return identifier comment

  return(identifierComment);
}

const wxString& Device::GetDriverComment(void) const
{  // retun driver name comment

  return(driverComment);
}

const wxString& Device::GetBusComment(void) const
{  // return bus id comment

  return(busIDComment);
}

const wxString& Device::GetScreenComment(void) const
{  // return used screen comment

  return(screenComment);
}

const wxString& Device::GetVideoRamComment(void) const
{  // return video ram size comment

  return(videoRamComment);
}

const Option& Device::GetOption(const int index) const
{  // get single option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Device::GetOption"), wxICON_ERROR);
  }

  return(options[index]);
}

Option& Device::GetOption(const int index)
{  // get single option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Device::GetOption"), wxICON_ERROR);
  }

  return(options[index]);
}

const int Device::GetOptionSize(void) const
{  // return size of options

  return(options.GetSize());
}

