
// **************************************
// Header Files
// **************************************

#include "devicelist.h"

// **************************
// Class Declaration
// **************************

DeviceList::DeviceList(void)
{  // constructor
  size=0;
  dList=NULL;
}

DeviceList::~DeviceList(void)
{  // destructor
  if (dList) delete [] dList;
  dList=NULL;
}

DeviceList::DeviceList(const DeviceList &dis)
{  // copy constructor

  size=dis.GetSize();
  dList=new Device[size];

  for (int i=0; i<size;i++)
  {
    dList[i]=dis[i];
  }
}

DeviceList& DeviceList::operator=(const DeviceList &dis)
{  // copy constructor

  Clear();

  size=dis.GetSize();
  dList=new Device[size];

  for (int i=0; i<size;i++)
  {
    dList[i]=dis[i];
  }

  return(*this);
}

void DeviceList::Remove(const int index)
{  // remove a Device

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("DeviceList::Remove"), wxICON_ERROR);
    return;
  }

  int i;  // loop counter

  if (size>1)
  {
    Device *dis=new Device[size-1];

    for (i=0; i<index; i++) dis[i]=dList[i];        // copy old entries
    for (i=index+1; i<size; i++) dis[i-1]=dList[i]; // and ignore index

    if (dList) delete [] dList;   // delete old list
    dList=dis;         // set pointer
    dis=NULL;
  }
  else
  {
    if (dList) delete [] dList;   // delete old list
    dList=NULL;         // set pointer
  }

  size--;              // set size
}

void DeviceList::Add(const Device& dis)
{  // add an option and value

  Device *dL=new Device[size+1];

  for (int i=0; i<size; i++) dL[i]=dList[i]; // copy old entries

  dL[size]=dis;   // set las entry

  if (dList) delete [] dList;   // delete old list
  dList=dL;         // set pointer
  dL=NULL;

  size++;
}

void DeviceList::Clear(void)
{    // clear everything

  if (dList) delete [] dList;
  dList=NULL;

  size=0;
}

const Device& DeviceList::operator[](const int index) const
{  // get single Device

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("DeviceList::Get"), wxICON_ERROR);
  }

  return(dList[index]);
}

Device& DeviceList::operator[](const int index)
{  // get single Device

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("DeviceList::operator[]"), wxICON_ERROR);
    return(dList[index]);  // return empty Device
  }

  return(dList[index]);
}

const int DeviceList::GetSize(void) const
{  // return size

  return(size);

}
