
// **************************************
// Header Files
// **************************************

#include "display.h"

// **************************
// Class Declaration
// **************************

Display::Display(void)
{  // constructor
}

Display::~Display(void)
{  // destructor
}

Display::Display(const Display &dis)
{  // copy constructor

  depth=dis.GetDepth();
  viewport=dis.GetViewPort();
  virtualE=dis.GetVirtual();
  comment=dis.GetComment();
  depthComment=dis.GetDepthComment();
  viewportComment=dis.GetViewPortComment();
  virtualEComment=dis.GetVirtualComment();
  modesComment=dis.GetModeComment();

  for (int i=0; i<dis.GetOptionSize(); i++)
  {
    AddOption(dis.GetOption(i));
  }

  for (int j=0; j<dis.GetModeSize(); j++)
  {
    AddMode(dis.GetMode(j));
  }

}

Display& Display::operator=(const Display &dis)
{  // copy constructor

  Clear();

  depth=dis.GetDepth();
  viewport=dis.GetViewPort();
  virtualE=dis.GetVirtual();
  comment=dis.GetComment();
  depthComment=dis.GetDepthComment();
  viewportComment=dis.GetViewPortComment();
  virtualEComment=dis.GetVirtualComment();
  modesComment=dis.GetModeComment();

  for (int i=0; i<dis.GetOptionSize(); i++)
  {
    AddOption(dis.GetOption(i));
  }

  for (int j=0; j<dis.GetModeSize(); j++)
  {
    AddMode(dis.GetMode(j));
  }

  return(*this);
}

void Display::AddComment(const wxString& com)
{  // set comment

  comment.Add(com);
}


void Display::SetDepth(const wxString& dd, const wxString& com)
{  // set depth

  depth=dd;
  depthComment=com;
}

void Display::SetViewPort(const wxString& vw, const wxString& com)
{  // set viewport

  viewport=vw;
  viewportComment=com;
}

void Display::SetVirtual(const wxString& vi, const wxString& com)
{  // set virtual

  virtualE=vi;
  virtualEComment=com;
}

void Display::RemoveOption(const int index)
{  // remove Option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Display::RemoveOption"), wxICON_ERROR);
    return;
  }

  options.Remove(index);
}

void Display::AddOption(const Option &opt)
{  // add Option

  options.Add(opt);
}

void Display::SetOption(const int index, const Option& opt)
{  // set Option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Display::SetOption"), wxICON_ERROR);
    return;
  }

  options[index]=opt;
}

void Display::RemoveMode(const int index)
{  // remove modeline

  if (index>=(int)modes.GetCount())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Display::RemoveMode"), wxICON_ERROR);
    return;
  }

  modes.RemoveAt(index);
}

void Display::AddMode(const wxString& mode)
{  // add modeline

  modes.Add(mode);
}

void Display::SetModeComment(const wxString& com)
{  // set mode comment
  modesComment=com;
}

void Display::SetMode(const int index, const wxString& mode)
{  // set mode

  if (index>=(int)modes.GetCount())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Display::SetMode"), wxICON_ERROR);
    return;
  }

  modes[index]=mode;
}

void Display::Clear(void)
{    // clear everything

  depth=wxEmptyString;
  virtualE=wxEmptyString;
  viewport=wxEmptyString;
  depthComment=wxEmptyString;
  virtualEComment=wxEmptyString;
  viewportComment=wxEmptyString;
  comment.Clear();
  options.Clear();
  modes.Clear();
  modesComment=wxEmptyString;
}

const wxString& Display::GetDepth(void) const
{  // return depth

  return(depth);
}

const wxString& Display::GetViewPort(void) const
{  // return viewport

  return(viewport);
}

const wxString& Display::GetVirtual(void) const
{  // return virtual

  return(virtualE);
}

const wxArrayString& Display::GetComment(void) const
{  // return comment

  return(comment);
}

const wxString& Display::GetDepthComment(void) const
{  // return depth Comment

  return(depthComment);
}

const wxString& Display::GetViewPortComment(void) const
{  // return viewport Comment

  return(viewportComment);
}

const wxString& Display::GetVirtualComment(void) const
{  // return virtual Comment

  return(virtualEComment);
}

const Option& Display::GetOption(const int index) const
{  // get single Option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Display::GetOption"), wxICON_ERROR);
  }

  return(options[index]);
}

Option& Display::GetOption(const int index)
{  // get single Option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Display::GetOption"), wxICON_ERROR);
  }

  return(options[index]);
}

const int Display::GetOptionSize(void) const
{  // return size of options

  return(options.GetSize());
}

const wxString& Display::GetMode(const int index) const
{  // get modeline name

  if (index>=(int)modes.GetCount())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Display::GetMode"), wxICON_ERROR);
  }

  return(modes.Item(index));
}

const wxString& Display::GetModeComment(void) const
{  // get modeline comment

  return(modesComment);
}

const int Display::GetModeSize(void) const
{  // return number of modes

  return(modes.GetCount());

}




