
// **************************************
// Header Files
// **************************************

#include "displaylist.h"

// **************************
// Class Declaration
// **************************

DisplayList::DisplayList(void)
{  // constructor
  size=0;
  dList=NULL;
}

DisplayList::~DisplayList(void)
{  // destructor
  if (dList) delete [] dList;
  dList=NULL;
}

DisplayList::DisplayList(const DisplayList &dis)
{  // copy constructor

  size=dis.GetSize();
  dList=new Display[size];

  for (int i=0; i<size;i++)
  {
    dList[i]=dis[i];
  }
}

DisplayList& DisplayList::operator=(const DisplayList &dis)
{  // copy constructor

  Clear();

  size=dis.GetSize();
  dList=new Display[size];

  for (int i=0; i<size;i++)
  {
    dList[i]=dis[i];
  }

  return(*this);
}

void DisplayList::Remove(const int index)
{  // remove a Display

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("DisplayList::Remove"), wxICON_ERROR);
    return;
  }

  int i;  // loop counter

  if (size>1)
  {
    Display *dis=new Display[size-1];

    for (i=0; i<index; i++) dis[i]=dList[i];        // copy old entries
    for (i=index+1; i<size; i++) dis[i-1]=dList[i]; // and ignore index

    if (dList) delete [] dList;   // delete old list
    dList=dis;         // set pointer
    dis=NULL;
  }
  else
  {
    if (dList) delete [] dList;   // delete old list
    dList=NULL;         // set pointer
  }

  size--;              // set size
}

void DisplayList::Add(const Display& dis)
{  // add an option and value

  Display *dL=new Display[size+1];

  for (int i=0; i<size; i++) dL[i]=dList[i]; // copy old entries

  dL[size]=dis;   // set las entry

  if (dList) delete [] dList;   // delete old list
  dList=dL;         // set pointer
  dL=NULL;

  size++;
}

void DisplayList::Clear(void)
{    // clear everything

  if (dList) delete [] dList;
  dList=NULL;

  size=0;
}

const Display& DisplayList::operator[](const int index) const
{  // get single Display

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("DisplayList::Get"), wxICON_ERROR);
  }

  return(dList[index]);
}

Display& DisplayList::operator[](const int index)
{  // get single device

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("DisplayList::operator[]"), wxICON_ERROR);
    return(dList[index]);  // return empty device
  }

  return(dList[index]);
}


const int DisplayList::GetSize(void) const
{  // return size

  return(size);

}
