#include "driveroption.h"

// **************************
// Class Declaration
// **************************

DriverOption::DriverOption(void) : Option()
{   // constructor
  strict=false;
  value=_T("string");
}

DriverOption::~DriverOption(void)
{
}

DriverOption::DriverOption(const DriverOption &dev) : Option(dev)
{ // copy constructor

  defArg=dev.GetDefault();
  arguments=dev.GetArguments();
  strict=dev.IsStrict();
}

DriverOption& DriverOption::operator=(const DriverOption &dev)
{ // copy constructor

  Option::operator=(dev);

  defArg=dev.GetDefault();
  arguments=dev.GetArguments();
  strict=dev.IsStrict();
  return(*this);
}

const wxArrayString& DriverOption::GetArguments(void) const
{  // return argument list
  return(arguments);
}

const bool DriverOption::IsStrict(void) const
{                // return if strict
  return(strict);
}

const wxString& DriverOption::GetDefault(void) const
{        // return default argument
  return(defArg);
}

void DriverOption::SetDefault(const wxString& def)
{    // set default argument

  defArg=def;
}

void DriverOption::AddArgument(const wxString& arg)
{  // add argument

  if (!arg.IsEmpty() && arguments.Index(arg)==wxNOT_FOUND)
  {  // only add if not in list already
    arguments.Add(arg);
  }
}

void DriverOption::SetStrict(const bool str)
{          // set if strict
  strict=str;
}

void DriverOption::Clear(void)
{   // clear all Options and values

  Option::Clear();
  strict=false;
  defArg=wxEmptyString;
  value=_T("string");
  arguments.Clear();
}

