#ifndef DRIVEROPTIONLIST_H
#define DRIVEROPTIONLIST_H

// **************************************
// Header Files
// **************************************

#include "global.h"
#include "driveroption.h"
#include <wx/textfile.h>

// **************************
// Constants
// **************************

enum optionErrorID
{ // IDs for errors while reading option list

  UNKNOWNL=400,
  NOOPTION,
  TWOOPTIONS,
  TWODEFAULT,
  NOOPTIONEND,
  NODEFAULTEND,
  NOARGUMENTEND,
  NAMEERROR,
  NAMEMISSING,
  VALUEERROR,
  VALUEVALUE,
  COMMENTERROR,
  STRICTERROR,
  STRICTVALUE,
  CLOSEOPTION,
  EMPTYARGUMENT2,
  EMPTYDEFAULT,

};


// **************************
// Class Definition
// **************************

class DriverOptionList
{
public:
  DriverOptionList(void);   // constructor
  DriverOptionList(const DriverOptionList &dis); // copy constructor
  ~DriverOptionList(void);   // destructor

  DriverOptionList& operator=(const DriverOptionList &dis); // copy constructor

  void Remove(const int index);  // remove Option
  void Add(const DriverOption& dis);           // add Option

  DriverOption& operator[](const int index);               // get device entry
  const DriverOption& operator[](const int index) const;   // get Option entry
  const int GetSize(void) const;                     // return size

  void Clear(void);          // clear everything

// special functions
  const bool Load(const wxString& filename); // load driver option lists

  const int Find(const wxString& name) const; // search option

private:
  const wxString GetNextLine(wxTextFile& file);
  void StripSpaces(wxString& line) const;         // convert tabs to spaces, delete spaces from left and right
  void StripRight(wxString& line, const int num) const;    // delete n right characters

  const bool ShowError(const optionErrorID error, const wxString& filename, const wxString& line);
    // show error message when reading file, return true if the user wants to stop loading

  DriverOption *dList;       // Option list
  int size;         // size

  int lineNr;       // line number

};

#endif // DRIVEROPTIONLIST_H
