
// **************************************
// Header Files
// **************************************

#include "filelist.h"

// **************************
// Class Declaration
// **************************

FileList::FileList(void)
{  // constructor
}

FileList::~FileList(void)
{  // destructor
}

FileList::FileList(const FileList &list)
{  // copy constructor

  int i;

  comment=list.GetComment();

  for (i=0; i<list.GetFontPathSize(); i++)
  {
    AddFontPath(list.GetFontPath(i), list.FontPathIsActive(i), list.GetFontPathComment(i));
  }
  for (i=0; i<list.GetModulePathSize(); i++)
  {
    AddModulePath(list.GetModulePath(i), list.ModulePathIsActive(i), list.GetModulePathComment(i));
  }
  for (i=0; i<list.GetRgbPathSize(); i++)
  {
    AddRgbPath(list.GetRgbPath(i), list.RgbPathIsActive(i), list.GetRgbPathComment(i));
  }
}

FileList& FileList::operator=(const FileList &list)
{  // copy constructor

  // clear old entries
  Clear();

  int i;


  comment=list.GetComment();

  for (i=0; i<list.GetFontPathSize(); i++)
  {
    AddFontPath(list.GetFontPath(i), list.FontPathIsActive(i), list.GetFontPathComment(i));
  }
  for (i=0; i<list.GetModulePathSize(); i++)
  {
    AddModulePath(list.GetModulePath(i), list.ModulePathIsActive(i), list.GetModulePathComment(i));
  }
  for (i=0; i<list.GetRgbPathSize(); i++)
  {
    AddRgbPath(list.GetRgbPath(i), list.RgbPathIsActive(i), list.GetRgbPathComment(i));
  }

  return(*this);
}

void FileList::Clear(void)
{  // clear all

  comment.Clear();
  fontPaths.Clear();
  modulePaths.Clear();
  rgbPaths.Clear();
}

void FileList::RemoveFontPath(const int index)
{  // remove a font path

  if (index>=fontPaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::RemoveFontPath"), wxICON_ERROR);
    return;
  }

  fontPaths.Remove(index);
}

void FileList::AddFontPath(const wxString& path, const bool act, const wxString& com)
{  // add a font path

  fontPaths.Add(path, wxEmptyString, com);
  fontPaths[fontPaths.GetSize()-1].SetActive(act);
}

void FileList::SetFontPath(const int index, const wxString& path, const wxString& com)
{ // set font path

  if (index>=fontPaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::SetFontPath"), wxICON_ERROR);
    return;
  }

  fontPaths[index].SetName(path);
  fontPaths[index].SetComment(com);
}

void FileList::SetFontPathActive(const int index, const bool act)
{ // set font path active

  if (index>=fontPaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::SetFontPathActive"), wxICON_ERROR);
  }

  fontPaths[index].SetActive(act);
}

const wxString& FileList::GetFontPath(const int index) const
{  // get single font path

  if (index>=fontPaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::GetFontPath"), wxICON_ERROR);
  }

  return(fontPaths[index].GetName());
}

const wxString& FileList::GetFontPathComment(const int index) const
{  // get single font path

  if (index>=fontPaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::GetFontPathComment"), wxICON_ERROR);
  }

  return(fontPaths[index].GetComment());
}

const int FileList::GetFontPathSize(void) const
{  // return size

  return(fontPaths.GetSize());
}

const bool FileList::FontPathIsActive(const int index) const
{    // return status

  if (index>=fontPaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::FontPathIsActive"), wxICON_ERROR);
  }

  return(fontPaths[index].IsActive());
}

void FileList::RemoveModulePath(const int index)
{  // remove a module path

  if (index>=modulePaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::RemoveModulePath"), wxICON_ERROR);
    return;
  }

  modulePaths.Remove(index);
}

void FileList::AddModulePath(const wxString& path, const bool act, const wxString& com)
{  // add a module path

  modulePaths.Add(path, wxEmptyString, com);
  modulePaths[modulePaths.GetSize()-1].SetActive(act);
}

void FileList::SetModulePath(const int index, const wxString& path, const wxString& com)
{ // set module path

  if (index>=modulePaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::SetModulePath"), wxICON_ERROR);
    return;
  }

  modulePaths[index].SetName(path);
  modulePaths[index].SetComment(com);
}

void FileList::SetModulePathActive(const int index, const bool act)
{ // set module path active

  if (index>=modulePaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::SetModulePathActive"), wxICON_ERROR);
  }

  modulePaths[index].SetActive(act);
}

const wxString& FileList::GetModulePath(const int index) const
{  // get single module path

  if (index>=modulePaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::GetModulePath"), wxICON_ERROR);
  }

  return(modulePaths[index].GetName());
}

const wxString& FileList::GetModulePathComment(const int index) const
{  // get single module path comment

  if (index>=modulePaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::GetModulePathComment"), wxICON_ERROR);
  }

  return(modulePaths[index].GetComment());
}

const int FileList::GetModulePathSize(void) const
{  // return size

  return(modulePaths.GetSize());
}

const bool FileList::ModulePathIsActive(const int index) const
{    // return status

  if (index>=modulePaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::ModulePathIsActive"), wxICON_ERROR);
  }

  return(modulePaths[index].IsActive());
}

void FileList::RemoveRgbPath(const int index)
{  // remove a rgb path

  if (index>=rgbPaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::RemoveRgbPath"), wxICON_ERROR);
    return;
  }

  rgbPaths.Remove(index);
}

void FileList::AddRgbPath(const wxString& path, const bool act, const wxString& com)
{  // add a rgb path

  rgbPaths.Add(path, wxEmptyString, com);
  rgbPaths[rgbPaths.GetSize()-1].SetActive(act);
}

void FileList::SetRgbPath(const int index, const wxString& path, const wxString& com)
{ // set rgb path

  if (index>=rgbPaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::SetRgbPath"), wxICON_ERROR);
    return;
  }

  rgbPaths[index].SetName(path);
  rgbPaths[index].SetComment(com);
}

void FileList::SetRgbPathActive(const int index, const bool act)
{ // set rgb path active

  if (index>=rgbPaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::SetRgbPathActive"), wxICON_ERROR);
  }

  rgbPaths[index].SetActive(act);
}

const wxString& FileList::GetRgbPath(const int index) const
{  // get single rgb path

  if (index>=rgbPaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::GetRgbPath"), wxICON_ERROR);
  }

  return(rgbPaths[index].GetName());
}

const wxString& FileList::GetRgbPathComment(const int index) const
{  // get single rgb path comment

  if (index>=rgbPaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::GetRgbPathComment"), wxICON_ERROR);
  }

  return(rgbPaths[index].GetComment());
}

const int FileList::GetRgbPathSize(void) const
{  // return size

  return(rgbPaths.GetSize());
}

const bool FileList::RgbPathIsActive(const int index) const
{    // return status

  if (index>=rgbPaths.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("FileList::RgbPathIsActive"), wxICON_ERROR);
  }

  return(rgbPaths[index].IsActive());
}

void FileList::AddComment(wxString& com)
{     // set comment
  comment.Add(com);
}

const wxArrayString& FileList::GetComment(void) const
{    // get comment
  return(comment);
}



