#ifndef FILELIST_H
#define FILELIST_H

// **************************************
// Header Files
// **************************************

#include "global.h"
#include "optionlist.h"

// **************************
// Class Definition
// **************************

class FileList
{
public:
  FileList(void);   // constructor
  ~FileList(void);   // destructor
  FileList(const FileList &list); // copy constructor

  FileList& operator=(const FileList &list); // copy constructor

  void RemoveFontPath(const int index);  // remove a font path
  void AddFontPath(const wxString& path, const bool act, const wxString& com=wxEmptyString);  // add a font path
  void SetFontPath(const int index, const wxString& path, const wxString& com=wxEmptyString); // set font path
  void SetFontPathActive(const int index, const bool act); // set font path active
  const wxString& GetFontPath(const int index) const;    // get single font path
  const wxString& GetFontPathComment(const int index) const;    // get single font path comment
  const bool FontPathIsActive(const int index) const;    // return status
  const int GetFontPathSize(void) const;                // return size

  void RemoveModulePath(const int index);  // remove a module path
  void AddModulePath(const wxString& path, const bool act, const wxString& com=wxEmptyString);  // add a module path
  void SetModulePath(const int index, const wxString& path, const wxString& com=wxEmptyString); // set module path
  void SetModulePathActive(const int index, const bool act); // set module path active
  const wxString& GetModulePath(const int index) const;    // get single module path
  const wxString& GetModulePathComment(const int index) const;    // get single module path comment
  const bool ModulePathIsActive(const int index) const;    // return status
  const int GetModulePathSize(void) const;                // return size

  void RemoveRgbPath(const int index);  // remove a Rgb path
  void AddRgbPath(const wxString& path, const bool act, const wxString& com=wxEmptyString);  // add a Rgb path
  void SetRgbPath(const int index, const wxString& path, const wxString& com=wxEmptyString); // set Rgb path
  void SetRgbPathActive(const int index, const bool act); // set Rgb path active
  const wxString& GetRgbPath(const int index) const;    // get single Rgb path
  const wxString& GetRgbPathComment(const int index) const;    // get single Rgb path comment
  const bool RgbPathIsActive(const int index) const;    // return status
  const int GetRgbPathSize(void) const;                // return size

  void AddComment(wxString& com);     // set comment
  const wxArrayString& GetComment(void) const;    // get comment

  void Clear(void);   // clear everything

private:
  wxArrayString comment;        // comment
  OptionList fontPaths;       // font paths
  OptionList modulePaths;       // module paths
  OptionList rgbPaths;       // rgb paths
};

#endif // FILELIST_H
