#ifndef GUIFRAME_H
#define GUIFRAME_H

// **************************************
// Header Files
// **************************************

#include "global.h"
#include "xorgfile.h"
#include "settings.h"
#include "filehandler.h"
#include "driveroptionlist.h"
#include <wx/notebook.h>

// **************************
// Constants
// **************************

// IDs for the controls and the menu commands in guiFrame
enum myFrameID
{
  // menu items
  NEWXORG=0,
  LOADXORG,
  SAVEXORG,
  SAVEAS,
  RESTORE,
  CLOSEXORG,
  QUITPROG,
  SHOWXORG,
  TESTXORG,
  ABOUTPROG,
  LANGGER,
  LANGENG,
  SETOPTDIR,
  SAVESET,
  SAVESETEXIT,
  LOADLAST,
  SHOWSPLASH,
  SHOWBLANK,
  REMTEST,

  // device items
  DEVID,
  DEVDRI,
  DEVBUS,
  DEVSCR,
  DEVRAM,
  DEVOPT,
  DEVOVAL,

  // input device items
  IDEVID,
  IDEVDRI,
  IDEVOPT,
  IDEVOVAL,

  // monitor items
  MONID,
  MONHSY,
  MONVREF,
  MONDISP,
  MONGAM,
  MONOPT,
  MONOVAL,
  MONMODL,
  MONMODLOPT,
  MONFREQ,
  MONDISPSIZE,
  MONMODLGEN,

  // screen items
  SCRID,
  SCRDEP,
  SCRDEV,
  SCRMON,
  SCROPT,
  SCROVAL,
  SCRDISP,
  SCRDISPVP,
  SCRDISPVIR,
  SCRDISPMOD,
  SCRDISPOPT,
  SCRDISPOVAL,

  // server layout items
  SERLID,
  SERLSCR,
  SERLSCRNUM,
  SERLSCRPOS,
  SERLSCR1,
  SERLSCR2,
  SERLIDEV,
  SERLIDEV1,
  SERLIDEVOPT,
  SERLOPT,
  SERLOVAL,

  // server flags items
  SERFOPT,
  SERFOVAL,

  // extension items
  EXTOPT,
  EXTOVAL,

  // files items
  FONTPATH,
  MODPATH,
  RGBPATH,

  // modules items
  MODID,
  MODOPT,
  MODOVAL,

  // dri items
  DRIMOD,

  // global items
  CHANGETAB
};

// **************************
// Class Definition
// **************************

class XorgApp;

class GuiFrame: public wxFrame
{
public:
  // GuiFrame(const wxString& title, wxLocale *lang);
  GuiFrame(const wxString& title);
  virtual ~GuiFrame();

  void OnNew(wxCommandEvent& event);      // create new empty xorg.conf
  void OnLoad(wxCommandEvent& event);     // load xorg.conf
  void OnSave(wxCommandEvent& event);     // save xorg.conf
  void OnSaveAs(wxCommandEvent& event);   // save config file
  void OnRestore(wxCommandEvent& event);   // restoreconfig file
  void OnClose(wxCommandEvent& event);   // close config file
  void OnQuit(wxCommandEvent& event);     // quit program
  void OnShow(wxCommandEvent& event);     // show xorg file
  void OnTest(wxCommandEvent& event);     // test xorg file
  void OnAbout(wxCommandEvent& event);    // show about dialog
  void OnSaveSettings(wxCommandEvent& event);    // save settings

  void SetLanguage(wxCommandEvent& event);    // set language
  void SetSaveSettings(wxCommandEvent& event);     // set if settings should be saved
  void SetLoadLoastFile(wxCommandEvent& event);    // set if last file should be loaded
  void SetShowSplash(wxCommandEvent& event);       // set if warning should be shown
  void SetShowBlankTest(wxCommandEvent& event);       // set if blank message at test should be shown
  void SetRememberTest(wxCommandEvent& event);       // set if test should be remembered
  void SetOptionDirectory(wxCommandEvent& event);  // set option directory

  void OnTabChange(wxNotebookEvent& event);     // some tab has changed

  // select wxComboBox elements
  void SelectDevice(wxCommandEvent& event);
  void SelectDeviceOption(wxCommandEvent& event);
  void SelectInputDevice(wxCommandEvent& event);
  void SelectInputDeviceOption(wxCommandEvent& event);
  void SelectMonitor(wxCommandEvent& event);
  void SelectMonitorOption(wxCommandEvent& event);
  void SelectMonitorModeline(wxCommandEvent& event);
  void SelectScreen(wxCommandEvent& event);
  void SelectScreenOption(wxCommandEvent& event);
  void SelectScreenDiplay(wxCommandEvent& event);
  void SelectScreenDiplayMode(wxCommandEvent& event);
  void SelectScreenDiplayOption(wxCommandEvent& event);
  void SelectServerLayout(wxCommandEvent& event);
  void SelectServerLayoutScreen(wxCommandEvent& event);
  void SelectServerLayoutInputDevice(wxCommandEvent& event);
  void SelectServerLayoutOption(wxCommandEvent& event);
  void SelectServerLayoutInputDeviceOption(wxCommandEvent& event);
  void SelectServerFlagOption(wxCommandEvent& event);
  void SelectExtension(wxCommandEvent& event);
  void SelectFontPath(wxCommandEvent& event);
  void SelectModulePath(wxCommandEvent& event);
  void SelectRgbPath(wxCommandEvent& event);
  void SelectModule(wxCommandEvent& event);
  void SelectModuleOption(wxCommandEvent& event);

  // select wxChoice elements
  void SelectScreenDevice(wxCommandEvent& event);
  void SelectScreenMonitor(wxCommandEvent& event);
  void SelectServerLayoutScreen1(wxCommandEvent& event);
  void SelectServerLayoutScreen2(wxCommandEvent& event);
  void SelectServerLayoutInputDevice1(wxCommandEvent& event);

  // enter text in wxComboBox elements
  void EnterDevice(wxCommandEvent& event);
  void EnterDeviceOption(wxCommandEvent& event);
  void EnterInputDevice(wxCommandEvent& event);
  void EnterInputDeviceOption(wxCommandEvent& event);
  void EnterMonitor(wxCommandEvent& event);
  void EnterMonitorOption(wxCommandEvent& event);
  void EnterMonitorModeline(wxCommandEvent& event);
  void EnterScreen(wxCommandEvent& event);
  void EnterScreenOption(wxCommandEvent& event);
  void EnterScreenDiplay(wxCommandEvent& event);
  void EnterScreenDiplayMode(wxCommandEvent& event);
  void EnterScreenDiplayOption(wxCommandEvent& event);
  void EnterServerLayout(wxCommandEvent& event);
  void EnterServerLayoutOption(wxCommandEvent& event);
  void EnterServerLayoutInputDeviceOption(wxCommandEvent& event);
  void EnterServerFlagOption(wxCommandEvent& event);
  void EnterExtension(wxCommandEvent& event);
  void EnterFontPath(wxCommandEvent& event);
  void EnterModulePath(wxCommandEvent& event);
  void EnterRgbPath(wxCommandEvent& event);
  void EnterModule(wxCommandEvent& event);
  void EnterModuleOption(wxCommandEvent& event);

  // enter text in text fields
  void EnterDeviceDriver(wxCommandEvent& event);
  void EnterDeviceBusID(wxCommandEvent& event);
  void EnterDeviceScreen(wxCommandEvent& event);
  void EnterDevicVideoRam(wxCommandEvent& event);
  void EnterDeviceOptValue(wxCommandEvent& event);
  void EnterMonitorHorizSync(wxCommandEvent& event);
  void EnterMonitorVertRefresh(wxCommandEvent& event);
  void EnterMonitorDisplay(wxCommandEvent& event);
  void EnterMonitorGamma(wxCommandEvent& event);
  void EnterMonitorOptValue(wxCommandEvent& event);
  void EnterMonitorModelineOption(wxCommandEvent& event);
  void EnterInputDeviceDriver(wxCommandEvent& event);
  void EnterInputDeviceOptValue(wxCommandEvent& event);
  void EnterScreenDepth(wxCommandEvent& event);
  void EnterScreenOptValue(wxCommandEvent& event);
  void EnterScreenDisplayViewport(wxCommandEvent& event);
  void EnterScreenDisplayVirtual(wxCommandEvent& event);
  void EnterScreenDisplayOptValue(wxCommandEvent& event);
  void EnterServerLayoutScreenNumber(wxCommandEvent& event);
  void EnterServerLayoutScreenPosition(wxCommandEvent& event);
  void EnterServerLayoutOptValue(wxCommandEvent& event);
  void EnterServerFlagsOptValue(wxCommandEvent& event);
  void EnterExtensionOptValue(wxCommandEvent& event);
  void EnterModuleOptValue(wxCommandEvent& event);
  void EnterDriMode(wxCommandEvent& event);

  // buttons
  void GetRefreshRate(wxCommandEvent& event);
  void GetDisplaySize(wxCommandEvent& event);
  void GenerateModeline(wxCommandEvent& event);

  void ShowWarning(void) const; // show warning message

private:
  void StripSpaces(wxString& line) const;         // convert tabs to spaces, delete spaces from left and right
  void StripRight(wxString& line, const int num) const;    // delete n right characters

  bool TestXorg(void);       // test xorg.conf file
  void ClearElements(void);  // clear all elements
  void SetFrame(void);       // set all elements on screen
  void SetSaved(bool save);  // set saved status and set title

  void LoadDrivers(void);    // load driver lists

  void CreateGUIControls(void);  // create all gui controls, like text fields, menus, buttons

  void EnableMenuItems(const bool enable);

  // check integrity functions
  bool CheckDeviceTab(void);
  bool CheckDevice(void);
  bool CheckDeviceOption(void);
  bool CheckMonitorTab(void) const;
  bool CheckMonitor(void) const;
  bool CheckMonitorOption(void) const;
  bool CheckMonitorModeline(void) const;
  bool CheckInputDeviceTab(void);
  bool CheckInputDevice(void);
  bool CheckInputDeviceOption(void);
  bool CheckScreenTab(void) const;
  bool CheckScreen(void) const;
  bool CheckScreenOption(void) const;
  bool CheckScreenDisplay(void) const;
  bool CheckScreenDisplayMode(void) const;
  bool CheckScreenDisplayOption(void) const;
  bool CheckServerTab(void) const;
  bool CheckServerLayout(void) const;
  bool CheckServerLayoutOption(void) const;
  bool CheckServerLayoutScreen(void) const;
  bool CheckServerLayoutInputDevice(void) const;
  bool CheckServerLayoutInputDeviceOption(void) const;
  bool CheckServerFlagOption(void) const;
  bool CheckOtherSectionTab(void) const;
  bool CheckExtension(void) const;
  bool CheckModule(void) const;
  bool CheckModuleOption(void) const;
  bool CheckFiles(void) const;
  bool CheckFontPath(void) const;
  bool CheckModulePath(void) const;
  bool CheckRgbPath(void) const;
  bool CheckDRI(void) const;

  // accept functions
  void AcceptDevice(void);
  void AcceptInputDevice(void);
  void AcceptMonitor(void);
  void AcceptScreen(void);

  // Update screen functions
  void UpdateDevice(void);
  void UpdateDeviceOption(void);
  void UpdateInputDevice(void);
  void UpdateInputDeviceOption(void);
  void UpdateMonitor(void);
  void UpdateMonitorOption(void);
  void UpdateMonitorModeline(void);
  void UpdateScreen(void);
  void UpdateScreenOption(void);
  void UpdateScreenDisplay(void);
  void UpdateScreenDisplayMode(void);
  void UpdateScreenDisplayOption(void);
  void UpdateServerLayout(void);
  void UpdateServerLayoutOption(void);
  void UpdateServerLayoutScreen(void);
  void UpdateServerLayoutInputDevice(void);
  void UpdateServerLayoutInputDeviceOption(void);
  void UpdateServerFlagOption(void);
  void UpdateExtension(void);
  void UpdateFontPath(void);
  void UpdateModulePath(void);
  void UpdateRgbPath(void);
  void UpdateModule(void);
  void UpdateModuleOption(void);
  void UpdateDRI(void);

  // delete, add and activate functions
  void AddDevice(void);
  void DelDevice(void);
  void CopyDevice(void);
  void AddDeviceOption(void);
  void DelDeviceOption(void);
  void DeActDeviceOption(void);
  void AddInputDevice(void);
  void DelInputDevice(void);
  void CopyInputDevice(void);
  void AddInputDeviceOption(void);
  void DelInputDeviceOption(void);
  void DeActInputDeviceOption(void);
  void AddMonitor(void);
  void DelMonitor(void);
  void CopyMonitor(void);
  void AddMonitorOption(void);
  void DelMonitorOption(void);
  void DeActMonitorOption(void);
  void AddMonitorModeline(void);
  void DelMonitorModeline(void);
  void DeActMonitorModeline(void);
  void AddScreen(void);
  void DelScreen(void);
  void CopyScreen(void);
  void AddScreenOption(void);
  void DelScreenOption(void);
  void DeActScreenOption(void);
  void AddScreenDisplay(void);
  void DelScreenDisplay(void);
  void AddScreenDisplayMode(void);
  void DelScreenDisplayMode(void);
  void AddScreenDisplayOption(void);
  void DelScreenDisplayOption(void);
  void DeActScreenDisplayOption(void);
  void AddServerLayout(void);
  void DelServerLayout(void);
  void CopyServerLayout(void);
  void AddServerLayoutScreen(void);
  void DelServerLayoutScreen(void);
  void AddServerLayoutInputDevice(void);
  void DelServerLayoutInputDevice(void);
  void AddServerLayoutOption(void);
  void DelServerLayoutOption(void);
  void DeActServerLayoutOption(void);
  void AddServerLayoutInputDeviceOption(void);
  void DelServerLayoutInputDeviceOption(void);
  void AddServerFlagOption(void);
  void DelServerFlagOption(void);
  void DeActServerFlagOption(void);
  void AddExtension(void);
  void DelExtension(void);
  void DeActExtension(void);
  void AddFontPath(void);
  void DelFontPath(void);
  void DeActFontPath(void);
  void AddModulePath(void);
  void DelModulePath(void);
  void DeActModulePath(void);
  void AddRgbPath(void);
  void DelRgbPath(void);
  void DeActRgbPath(void);
  void AddModule(void);
  void DelModule(void);
  void DeActModule(void);
  void AddModuleOption(void);
  void DelModuleOption(void);
  void DeActModuleOption(void);

  // global GUI elements

  wxMenu* fileMenu;                    // file menu
  wxMenu* extrasMenu;                  // extras menu

  wxNotebook* layout;                   // layout for tabs

  wxPanel* deviceTab;                   // device tab
  wxPanel* monitorTab;                  // monitor tab
  wxPanel* inputDeviceTab;              // inputdevice tab
  wxPanel* screenTab;                   // screen tab
  wxPanel* serverTab;                   // server layout/server flags tab
  wxPanel* otherSectionTab;             // files/modules/extensions/dri tab
  wxStaticText* warningText;

  // "local" GUI elements

  // section DEVICE elements
  wxStaticBox *deviceBox;
  wxComboBox *deviceIdentifier;
  wxComboBox *deviceDriver;
  wxTextCtrl *deviceBusID;
  wxTextCtrl *deviceScreen;
  wxTextCtrl *deviceVideoRam;
  wxComboBox *deviceOption;
  wxComboBox *deviceOptValue;
  wxStaticText *deviceDriverText;
  wxStaticText *deviceBusIDText;
  wxStaticText *deviceVideoRamText;
  wxStaticText *deviceScreenNumberText;
  wxStaticText *deviceOptionText;

  // section MONITOR elements
  wxStaticBox *monitorBox;
  wxComboBox *monitorIdentifier;
  wxTextCtrl *monitorHorizSync;
  wxTextCtrl *monitorVertRefresh;
  wxTextCtrl *monitorDisplay;
  wxTextCtrl *monitorGamma;
  wxComboBox *monitorOption;
  wxTextCtrl *monitorOptValue;
  wxComboBox *monitorModelineName;
  wxTextCtrl *monitorModelineOption;
  wxStaticText *monitorHorizSyncText;
  wxStaticText *monitorVertRefreshText;
  wxStaticText *monitorDisplayText;
  wxStaticText *monitorGammaText;
  wxStaticText *monitorOptionText;
  wxStaticText *monitorModelineText;
  wxButton* monitorFreqButton;
  wxButton* monitorDisplaySizeButton;
  wxStaticBox *monitorModelineGeneratorBox;
  wxTextCtrl *monitorModelineGeneratorXRes;
  wxTextCtrl *monitorModelineGeneratorYRes;
  wxTextCtrl *monitorModelineGeneratorFreq;
  wxTextCtrl *monitorModelineGeneratorOutput1;
  wxTextCtrl *monitorModelineGeneratorOutput2;
  wxStaticText *monitorModelineGeneratorXResText;
  wxStaticText *monitorModelineGeneratorYResText;
  wxStaticText *monitorModelineGeneratorFreqText;
  wxButton* monitorModelineGeneratorButton;

  // section INPUTDEVICE elements
  wxStaticBox *inputDeviceBox;
  wxComboBox *inputDeviceIdentifier;
  wxComboBox *inputDeviceDriver;
  wxComboBox *inputDeviceOption;
  wxComboBox *inputDeviceOptValue;
  wxStaticText *inputDeviceDriverText;
  wxStaticText *inputDeviceOptionText;

  // section SCREEN elements
  wxStaticBox *screenBox;
  wxComboBox *screenIdentifier;
  wxChoice *screenDevice;
  wxChoice *screenMonitor;
  wxTextCtrl *screenDepth;
  wxComboBox *screenOption;
  wxTextCtrl *screenOptValue;
  wxComboBox *screenDisplayDepth;
  wxTextCtrl *screenDisplayViewport;
  wxTextCtrl *screenDisplayVirtual;
  wxComboBox *screenDisplayModes;
  wxComboBox *screenDisplayOption;
  wxTextCtrl *screenDisplayOptValue;
  wxStaticText *screenDeviceText;
  wxStaticText *screenMonitorText;
  wxStaticText *screenDepthText;
  wxStaticText *screenOptionText;
  wxStaticText *screenDisplayDepthText;
  wxStaticText *screenDisplayViewportText;
  wxStaticText *screenDisplayVirtualText;
  wxStaticText *screenDisplayModesText;
  wxStaticText *screenDisplayOptionText;

  // section SERVERLAYOUT elements
  wxStaticBox *serverLayoutBox;
  wxComboBox *serverLayoutIdentifier;
  wxComboBox *serverLayoutScreen;
  wxChoice *serverLayoutScreen1;
  wxChoice *serverLayoutScreen2;
  wxTextCtrl *serverLayoutScreenNumber;
  wxTextCtrl *serverLayoutScreenPosition;
  wxComboBox *serverLayoutInputDevice;
  wxChoice *serverLayoutInputDevice1;
  wxComboBox *serverLayoutInputDeviceOption;
  // wxTextCtrl *serverLayoutInputDeviceOptValue;
  wxComboBox *serverLayoutOption;
  wxTextCtrl *serverLayoutOptValue;
  wxStaticText *serverLayoutScreenNumberText;
  wxStaticText *serverLayoutScreenPositionText;
  wxStaticText *serverLayoutOptionText;
  wxStaticText *serverLayoutInputDeviceOptionText;

  // section SERVERFLAGS elements
  wxStaticBox *serverFlagBox;
  wxComboBox *serverFlagsOption;
  wxTextCtrl *serverFlagsOptValue;

  // section EXTENSION elements
  wxStaticBox *extensionBox;
  wxComboBox *extensionOption;
  wxTextCtrl *extensionOptValue;

  // section MODULE elements
  wxStaticBox *moduleBox;
  wxComboBox *moduleIdentifier;
  wxComboBox *moduleOption;
  wxTextCtrl *moduleOptValue;
  wxStaticText *moduleOptionText;

  // section FILES elements
  wxStaticBox *filesBox;
  wxComboBox *fontPath;
  wxStaticText *fontPathText;
  wxComboBox *modulePath;
  wxStaticText *modulePathText;
  wxComboBox *rgbPath;
  wxStaticText *rgbPathText;

  // section DRI elements
  wxStaticBox *driBox;
  wxTextCtrl *driMode;

  // non GUI elements
  int fontPathCurrent;
  int modulePathCurrent;
  int rgbPathCurrent;

  int moduleCurrent;
  int moduleOptionCurrent;

  int serverLayoutCurrent;
  int serverLayoutScreenCurrent;
  int serverLayoutOptionCurrent;
  int serverLayoutIDevCurrent;
  int serverLayoutIDevOptionCurrent;

  int inputDeviceCurrent;
  int inputDeviceOptionCurrent;

  int deviceCurrent;
  int deviceOptionCurrent;

  int monitorCurrent;
  int monitorOptionCurrent;
  int monitorModelineCurrent;

  int screenCurrent;
  int screenOptionCurrent;
  int screenDisplayCurrent;
  int screenDisplayModeCurrent;
  int screenDisplayOptionCurrent;

  int serverFlagCurrent;

  int extensionCurrent;

  // backup elements
  wxString deviceIdentifierText;
  wxString monitorIdentifierText;
  wxString inputDeviceIdentifierText;
  wxString screenIdentifierText;

  // other elements
  XorgFile xorg;
  Settings settings;
  FileHandler fileHandler;
  DriverOptionList deviceDriversOption;
  DriverOptionList inputDeviceDriversOption;

  wxArrayString deviceDriverList;
  wxArrayString inputDeviceDriverList;

  int optListSize;
  int devListSize;

  wxString version;   // current version
  bool isSaved;       // flag if file is saved or there are some changes

  // wxLocale *appLocale; // language

  DECLARE_EVENT_TABLE();
};

#endif // GUIFRAME_H

