
// **************************************
// Header Files
// **************************************

#include "inputdevice.h"

// **************************
// Class Declaration
// **************************

InputDevice::InputDevice(void)
{  // constructor
}

InputDevice::~InputDevice(void)
{  // destructor
}

InputDevice::InputDevice(const InputDevice &dev)
{  // copy constructor

  identifier=dev.GetId();
  driver=dev.GetDriver();
  identifierComment=dev.GetIdComment();
  driverComment=dev.GetDriverComment();
  comment=dev.GetComment();

  for (int i=0; i<dev.GetOptionSize(); i++)
  {
    AddOption(dev.GetOption(i));
  }

}

InputDevice& InputDevice::operator=(const InputDevice &dev)
{  // copy constructor

  Clear(); // clear all

  identifier=dev.GetId();
  driver=dev.GetDriver();
  identifierComment=dev.GetIdComment();
  driverComment=dev.GetDriverComment();
  comment=dev.GetComment();

  for (int i=0; i<dev.GetOptionSize(); i++)
  {
    AddOption(dev.GetOption(i));
  }

  return(*this);
}

void InputDevice::AddComment(const wxString& com)
{  // set comment

  comment.Add(com);
}

void InputDevice::SetId(const wxString& id, const wxString& com)
{  // set identifier

  identifier=id;
  identifierComment=com;
}

void InputDevice::SetDriver(const wxString& dr, const wxString& com)
{  // set driver

  driver=dr;
  driverComment=com;
}

const wxArrayString& InputDevice::GetComment(void) const
{  // return comment

  return(comment);
}

const wxString& InputDevice::GetId(void) const
{  // return identifier

  return(identifier);
}

const wxString& InputDevice::GetDriver(void) const
{  // retun driver name

  return(driver);
}

const wxString& InputDevice::GetIdComment(void) const
{  // return identifier comment

  return(identifierComment);
}

const wxString& InputDevice::GetDriverComment(void) const
{  // retun driver name comment

  return(driverComment);
}

void InputDevice::RemoveOption(const int index)
{  // remove option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("InputDevice::RemoveOption"), wxICON_ERROR);
    return;
  }

  options.Remove(index);
}

void InputDevice::AddOption(const Option &opt)
{  // add option

  options.Add(opt);
}

void InputDevice::SetOption(const int index, const Option& opt)
{  // set option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("InputDevice::SetOption"), wxICON_ERROR);
    return;
  }

  options[index]=opt;
}

void InputDevice::Clear(void)
{    // clear everything

  comment.Clear();
  identifier=wxEmptyString;
  driver=wxEmptyString;
  identifierComment=wxEmptyString;
  driverComment=wxEmptyString;
  options.Clear();
}

const Option& InputDevice::GetOption(const int index) const
{  // get option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("InputDevice::GetOption"), wxICON_ERROR);
  }

  return(options[index]);
}

Option& InputDevice::GetOption(const int index)
{  // get option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("InputDevice::GetOptionE"), wxICON_ERROR);
  }

  return(options[index]);
}

const int InputDevice::GetOptionSize(void) const
{  // return size of options

  return(options.GetSize());
}

