#ifndef INPUTDEVICE_H
#define INPUTDEVICE_H

// **************************************
// Header Files
// **************************************

#include "global.h"
#include "optionlist.h"

// **************************
// Class Definition
// **************************

class InputDevice
{
public:
  InputDevice(void);   // constructor
  ~InputDevice(void);   // destructor
  InputDevice(const InputDevice &dev); // copy constructor

  InputDevice& operator=(const InputDevice &dev); // copy constructor

  void AddComment(const wxString& com);      // set comment
  void SetId(const wxString& id, const wxString& com=wxEmptyString);      // set identifier
  void SetDriver(const wxString& dr, const wxString& com=wxEmptyString);  // set driver

  const wxArrayString& GetComment(void) const;           // return comment
  const wxString& GetId(void) const;           // return identifier
  const wxString& GetDriver(void) const;       // retun driver name
  const wxString& GetIdComment(void) const;           // return identifier comment
  const wxString& GetDriverComment(void) const;       // retun driver name comment

  void RemoveOption(const int index);    // remove option
  void AddOption(const Option &opt);              // add option
  void SetOption(const int index, const Option &opt); // set option

  Option& GetOption(const int index);                // get option
  const Option& GetOption(const int index) const;     // get option
  const int GetOptionSize(void) const;                      // return size of options

  void Clear(void);          // clear everything

private:
  wxString identifier;      // identifier
  wxString driver;          // driver name
  wxString identifierComment;      // identifier comment
  wxString driverComment;          // driver name comment
  wxArrayString comment;
  OptionList options;       // options

};

#endif // INPUTDEVICE_H
