
// **************************************
// Header Files
// **************************************

#include "inputdevicelist.h"

// **************************
// Class Declaration
// **************************

InputDeviceList::InputDeviceList(void)
{  // constructor
  size=0;
  dList=NULL;
}

InputDeviceList::~InputDeviceList(void)
{  // destructor
  if (dList) delete [] dList;
  dList=NULL;
}

InputDeviceList::InputDeviceList(const InputDeviceList &dis)
{  // copy constructor

  size=dis.GetSize();
  dList=new InputDevice[size];

  for (int i=0; i<size;i++)
  {
    dList[i]=dis[i];
  }
}

InputDeviceList& InputDeviceList::operator=(const InputDeviceList &dis)
{  // copy constructor

  Clear();

  size=dis.GetSize();
  dList=new InputDevice[size];

  for (int i=0; i<size;i++)
  {
    dList[i]=dis[i];
  }

  return(*this);
}

void InputDeviceList::Remove(const int index)
{  // remove a InputDevice

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("InputDeviceList::Remove"), wxICON_ERROR);
    return;
  }

  int i;  // loop counter

  if (size>1)
  {
    InputDevice *dis=new InputDevice[size-1];

    for (i=0; i<index; i++) dis[i]=dList[i];        // copy old entries
    for (i=index+1; i<size; i++) dis[i-1]=dList[i]; // and ignore index

    if (dList) delete [] dList;   // delete old list
    dList=dis;         // set pointer
    dis=NULL;
  }
  else
  {
    if (dList) delete [] dList;   // delete old list
    dList=NULL;         // set pointer
  }

  size--;              // set size
}

void InputDeviceList::Add(const InputDevice& dis)
{  // add an option and value

  InputDevice *dL=new InputDevice[size+1];

  for (int i=0; i<size; i++) dL[i]=dList[i]; // copy old entries

  dL[size]=dis;   // set las entry

  if (dList) delete [] dList;   // delete old list
  dList=dL;         // set pointer
  dL=NULL;

  size++;
}

void InputDeviceList::Clear(void)
{    // clear everything

  if (dList) delete [] dList;
  dList=NULL;

  size=0;
}

const InputDevice& InputDeviceList::operator[](const int index) const
{  // get single InputDevice

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("InputDeviceList::Get"), wxICON_ERROR);
  }

  return(dList[index]);
}

InputDevice& InputDeviceList::operator[](const int index)
{  // get single device

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("InputDeviceList::operator[]"), wxICON_ERROR);
    return(dList[index]);  // return empty device
  }

  return(dList[index]);
}


const int InputDeviceList::GetSize(void) const
{  // return size

  return(size);

}
