
// **************************************
// Header Files
// **************************************

#include "modulelist.h"

// **************************
// Class Declaration
// **************************

ModuleList::ModuleList(void)
{  // constructor
  options=NULL;
}

ModuleList::~ModuleList(void)
{  // destructor

  if (options) delete [] options;
  options=NULL;
}

ModuleList::ModuleList(const ModuleList &list)
{  // copy constructor

  options=NULL;
  comment=list.GetComment();

  for (int i=0; i<list.GetSize(); i++)
  {
    Add(list.Get(i), list.IsActive(i), list.GetModuleComment(i));

    for (int j=0; j<list.GetOptionSize(i); j++)
    {
      AddOption(i, list.GetOption(i,j));
    }
  }
}

ModuleList& ModuleList::operator=(const ModuleList &list)
{  // copy constructor

  // clear old entries
  Clear();

  comment=list.GetComment();

  for (int i=0; i<list.GetSize(); i++)
  {
    Add(list.Get(i), list.IsActive(i), list.GetModuleComment(i));

    for (int j=0; j<list.GetOptionSize(i); j++)
    {
      AddOption(i, list.GetOption(i,j));
    }
  }

  return(*this);
}

void ModuleList::Remove(const int index)
{  // remove a Module

  if (index>=Modules.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ModuleList::Remove"), wxICON_ERROR);
    return;
  }

  Modules.Remove(index);  // remove Module

  int i;  // loop counter
  int size=Modules.GetSize();

  if (size>1)
  {
    OptionList *opt=new OptionList[size-1];

    for (i=0; i<index; i++) opt[i]=options[i];        // copy old entries
    for (i=index+1; i<size; i++) opt[i-1]=options[i]; // and ignore index

    if (options) delete [] options;   // delete old options
    options=opt;         // set pointer
    opt=NULL;
  }
  else
  {
    if (options) delete [] options;   // delete old list
    options=NULL;         // set pointer
  }
}

void ModuleList::Add(const wxString& mod, const bool act, const wxString& com)
{  // add a Module

  Modules.Add(mod, wxEmptyString, com);
  Modules[Modules.GetSize()-1].SetActive(act);

  OptionList *opt=new OptionList[Modules.GetSize()];

  for (int i=0; i<Modules.GetSize()-1; i++) opt[i]=options[i]; // copy old entries

  // Note: Last new option in opt keeps empty.

  if (options) delete [] options;   // delete old options
  options=opt;         // set pointer
  opt=NULL;
}

void ModuleList::Set(const int index, const wxString& mod, const wxString& com)   // set Module
{
  if (index>=Modules.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ModuleList::Set"), wxICON_ERROR);
    return;
  }

  Modules[index].SetName(mod);
  Modules[index].SetComment(com);

}

void ModuleList::Clear(void)
{  // clear all

  Modules.Clear();
  comment.Clear();

  if (options) delete [] options;
  options=NULL;
}

const wxString& ModuleList::Get(const int index) const
{  // get Module name

  if (index>=Modules.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ModuleList::Get"), wxICON_ERROR);
  }

  return(Modules[index].GetName());
}

const wxString& ModuleList::GetModuleComment(const int index) const
{  // get Module name

  if (index>=Modules.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ModuleList::GetModuleComment"), wxICON_ERROR);
  }

  return(Modules[index].GetComment());
}

void ModuleList::RemoveOption(const int index1, const int index2)
{  // remove an option Module

  if (index1>=Modules.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ModuleList::RemoveOption"), wxICON_ERROR);
    return;
  }

  options[index1].Remove(index2);

}

void ModuleList::AddOption(const int index, const Option& opt)
{  // add an option to a Module

  if (index>=Modules.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ModuleList::AddOption"), wxICON_ERROR);
    return;
  }

  options[index].Add(opt);

}

void ModuleList::SetOption(const int index1, const int index2, const Option& opt)
{  // set an option and value

  if (index1>=Modules.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ModuleList::SetOption"), wxICON_ERROR);
    return;
  }

  options[index1][index2]=opt;
}

const Option& ModuleList::GetOption(const int index1, const int index2) const
{  // get option of a Module

  if (index1>=Modules.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ModuleList::GetOption"), wxICON_ERROR);
  }

  return(options[index1][index2]);
}

Option& ModuleList::GetOption(const int index1, const int index2)
{  // get option of a Module

  if (index1>=Modules.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ModuleList::GetOptionE"), wxICON_ERROR);
  }

  return(options[index1][index2]);
}

const int ModuleList::GetSize(void) const
{  // return size

  return(Modules.GetSize());
}

const int ModuleList::GetOptionSize(const int index) const
{  // return size

  if (index>=Modules.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ModuleList::GetOptionSize"), wxICON_ERROR);
    return(0);
  }

  return(options[index].GetSize());
}

void ModuleList::SetActive(const int index, const bool act)
{  // set active status

  if (index>=Modules.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ModuleList::SetActive"), wxICON_ERROR);
    return;
  }

  Modules[index].SetActive(act);
}

const bool ModuleList::IsActive(const int index) const
{  // return active status

  if (index>=Modules.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ModuleList::IsActive"), wxICON_ERROR);
    return(false);
  }

  return(Modules[index].IsActive());
}

const wxArrayString& ModuleList::GetComment(void) const
{    // get comment
  return(comment);
}

void ModuleList::AddComment(const wxString& com)
{   // set comment
  comment.Add(com);
}


