#ifndef MODULELIST_H
#define MODULELIST_H

// **************************************
// Header Files
// **************************************

#include "global.h"
#include "optionlist.h"

// **************************
// Class Definition
// **************************

class ModuleList
{
public:
  ModuleList(void);    // constructor
  ~ModuleList(void);   // destructor
  ModuleList(const ModuleList &list); // copy constructor

  ModuleList& operator=(const ModuleList &list); // copy constructor

  void Remove(const int index);  // remove a Module
  void Add(const wxString& mod, const bool act, const wxString& com=wxEmptyString);           // add a Module
  const wxString& Get(const int index) const;    // get Module name
  const wxString& GetModuleComment(const int index) const;    // get comment
  void Set(const int index, const wxString& mod, const wxString& com=wxEmptyString);   // set Module
  void SetActive(const int index, const bool act);  // set active status
  const bool IsActive(const int index) const;  // return active status

  void RemoveOption(const int index1, const int index2);    // remove option
  void AddOption(const int index, const Option &opt);              // add option
  void SetOption(const int index1, const int index2, const Option &opt); // set option

  Option& GetOption(const int index1, const int index2);                // get option
  const Option& GetOption(const int index1, const int index2) const;     // get option
  const int GetOptionSize(const int index) const;                      // return size of options

  const wxArrayString& GetComment(void) const;    // get comment
  void AddComment(const wxString& com);   // set comment

  const int GetSize(void) const;                // return size
  void Clear(void);   // clear everything

private:
  wxArrayString comment;         // comment
  OptionList Modules;            // Modules
  OptionList *options;           // options for each Module
};

#endif // FILELIST_H
