
// **************************************
// Header Files
// **************************************

#include "monitor.h"

// **************************
// Class Declaration
// **************************

Monitor::Monitor(void)
{  // constructor
}

Monitor::~Monitor(void)
{  // destructor
}

Monitor::Monitor(const Monitor &mon)
{  // copy constructor

  identifier=mon.GetId();
  horSync=mon.GetHorSync();
  vertRefresh=mon.GetVertRef();
  displaySize=mon.GetDisplaySize();
  gamma=mon.GetGamma();
  identifierComment=mon.GetIdComment();
  horSyncComment=mon.GetHorSyncComment();
  vertRefreshComment=mon.GetVertRefComment();
  displaySizeComment=mon.GetDisplaySizeComment();
  gammaComment=mon.GetGammaComment();
  comment=mon.GetComment();

  for (int i=0; i<mon.GetOptionSize(); i++)
  {
    AddOption(mon.GetOption(i));
  }

  for (int j=0; j<mon.GetModelineSize(); j++)
  {
    AddModeline(mon.GetModeline(j));
  }

}

Monitor& Monitor::operator=(const Monitor &mon)
{  // copy constructor

  Clear();

  identifier=mon.GetId();
  horSync=mon.GetHorSync();
  vertRefresh=mon.GetVertRef();
  displaySize=mon.GetDisplaySize();
  gamma=mon.GetGamma();
  identifierComment=mon.GetIdComment();
  horSyncComment=mon.GetHorSyncComment();
  vertRefreshComment=mon.GetVertRefComment();
  displaySizeComment=mon.GetDisplaySizeComment();
  gammaComment=mon.GetGammaComment();
  comment=mon.GetComment();

  for (int i=0; i<mon.GetOptionSize(); i++)
  {
    AddOption(mon.GetOption(i));
  }

  for (int j=0; j<mon.GetModelineSize(); j++)
  {
    AddModeline(mon.GetModeline(j));
  }

  return(*this);
}

void Monitor::AddComment(const wxString& com)
{  // set comment

  comment.Add(com);
}

void Monitor::SetId(const wxString& id, const wxString& com)
{  // set identifier

  identifier=id;
  identifierComment=com;
}

void Monitor::SetHorSync(const wxString& hor, const wxString& com)
{  // set horizontal sync

  horSync=hor;
  horSyncComment=com;
}

void Monitor::SetVertRef(const wxString& ver, const wxString& com)
{  // set vertical refresh

  vertRefresh=ver;
  vertRefreshComment=com;
}

void Monitor::SetDisplaySize(const wxString& size, const wxString& com)
{  // set display size

  displaySize=size;
  displaySizeComment=com;
}

void Monitor::SetGamma(const wxString& gam, const wxString& com)
{  // set gamma

  gamma=gam;
  gammaComment=com;
}

void Monitor::RemoveOption(const int index)
{  // remove option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Monitor::RemoveOption"), wxICON_ERROR);
    return;
  }

  options.Remove(index);
}

void Monitor::AddOption(const Option &opt)
{  // add option

  options.Add(opt);
}

void Monitor::SetOption(const int index, const Option& opt)
{  // set option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Monitor::SetOption"), wxICON_ERROR);
    return;
  }

  options[index]=opt;
}

void Monitor::RemoveModeline(const int index)
{  // remove modeline

  if (index>=modelines.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Monitor::RemoveModeline"), wxICON_ERROR);
    return;
  }

  modelines.Remove(index);
}

void Monitor::AddModeline(const Option &opt)
{  // add modeline

  modelines.Add(opt);
}

void Monitor::SetModeline(const int index, const Option& opt)
{  // set modeline

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Monitor::SetModeline"), wxICON_ERROR);
    return;
  }

  modelines[index]=opt;
}

void Monitor::Clear(void)
{    // clear everything

  identifier=wxEmptyString;
  horSync=wxEmptyString;
  vertRefresh=wxEmptyString;
  displaySize=wxEmptyString;
  gamma=wxEmptyString;
  identifierComment=wxEmptyString;
  horSyncComment=wxEmptyString;
  vertRefreshComment=wxEmptyString;
  displaySizeComment=wxEmptyString;
  gammaComment=wxEmptyString;
  comment.Clear();
  options.Clear();
  modelines.Clear();
}

const wxArrayString& Monitor::GetComment(void) const
{  // return comment

  return(comment);
}

const wxString& Monitor::GetId(void) const
{  // return identifier

  return(identifier);
}

const wxString& Monitor::GetHorSync(void) const
{  // return horizontal sync

  return(horSync);
}

const wxString& Monitor::GetVertRef(void) const
{  // return vertical refresh

  return(vertRefresh);
}

const wxString& Monitor::GetDisplaySize(void) const
{  // return display size

  return(displaySize);
}

const wxString& Monitor::GetGamma(void) const
{  // return gamma

  return(gamma);
}

const wxString& Monitor::GetIdComment(void) const
{  // return identifier Comment

  return(identifierComment);
}

const wxString& Monitor::GetHorSyncComment(void) const
{  // return horizontal sync Comment

  return(horSyncComment);
}

const wxString& Monitor::GetVertRefComment(void) const
{  // return vertical refresh Comment

  return(vertRefreshComment);
}

const wxString& Monitor::GetDisplaySizeComment(void) const
{  // return display size Comment

  return(displaySizeComment);
}

const wxString& Monitor::GetGammaComment(void) const
{  // return gamma Comment

  return(gammaComment);
}

const Option& Monitor::GetOption(const int index) const
{  // get option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Monitor::GetOption"), wxICON_ERROR);
  }

  return(options[index]);
}

Option& Monitor::GetOption(const int index)
{  // get option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Monitor::GetOptionE"), wxICON_ERROR);
  }

  return(options[index]);
}

const int Monitor::GetOptionSize(void) const
{  // return size of options

  return(options.GetSize());
}

const Option& Monitor::GetModeline(const int index) const
{  // get modeline

  if (index>=modelines.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Monitor::GetModeline"), wxICON_ERROR);
  }

  return(modelines[index]);
}

Option& Monitor::GetModeline(const int index)
{  // get modeline

  if (index>=modelines.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Monitor::GetModelineE"), wxICON_ERROR);
  }

  return(modelines[index]);
}

const int Monitor::GetModelineSize(void) const
{  // return number of modelines

  return(modelines.GetSize());
}


