
// **************************************
// Header Files
// **************************************

#include "optionlist.h"

// **************************
// Class Declaration
// **************************

OptionList::OptionList(void)
{  // constructor
  size=0;
  dList=NULL;
}

OptionList::~OptionList(void)
{  // destructor
  if (dList) delete [] dList;
  dList=NULL;
}

OptionList::OptionList(const OptionList &dis)
{  // copy constructor

  size=dis.GetSize();
  dList=new Option[size];

  for (int i=0; i<size;i++)
  {
    dList[i]=dis[i];
  }
}

OptionList& OptionList::operator=(const OptionList &dis)
{  // copy constructor

  Clear();

  size=dis.GetSize();
  dList=new Option[size];

  for (int i=0; i<size;i++)
  {
    dList[i]=dis[i];
  }

  return(*this);
}

void OptionList::Remove(const int index)
{  // remove a Option

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("OptionList::Remove"), wxICON_ERROR);
    return;
  }

  int i;  // loop counter

  if (size>1)
  {
    Option *dis=new Option[size-1];

    for (i=0; i<index; i++) dis[i]=dList[i];        // copy old entries
    for (i=index+1; i<size; i++) dis[i-1]=dList[i]; // and ignore index

    if (dList) delete [] dList;   // delete old list
    dList=dis;         // set pointer
    dis=NULL;
  }
  else
  {
    if (dList) delete [] dList;   // delete old list
    dList=NULL;         // set pointer
  }

  size--;              // set size
}

void OptionList::Add(const Option& dis)
{  // add an Option and value

  Option *dL=new Option[size+1];

  for (int i=0; i<size; i++) dL[i]=dList[i]; // copy old entries

  dL[size]=dis;   // set las entry

  if (dList) delete [] dList;   // delete old list
  dList=dL;         // set pointer
  dL=NULL;

  size++;
}

void OptionList::Add(const wxString& opt, const wxString& val, const wxString& com)
{  // add Option

  Option tempOption(opt, val, com);

  Option *dL=new Option[size+1];

  for (int i=0; i<size; i++) dL[i]=dList[i]; // copy old entries

  dL[size]=tempOption;   // set las entry

  if (dList) delete [] dList;   // delete old list
  dList=dL;         // set pointer
  dL=NULL;

  size++;
}
void OptionList::Clear(void)
{    // clear everything

  if (dList) delete [] dList;
  dList=NULL;

  size=0;
}

const Option& OptionList::operator[](const int index) const
{  // get single Option

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("OptionList::Get"), wxICON_ERROR);
  }

  return(dList[index]);
}

Option& OptionList::operator[](const int index)
{  // get single device

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("OptionList::operator[]"), wxICON_ERROR);
    return(dList[index]);  // return empty device
  }

  return(dList[index]);
}

const int OptionList::GetSize(void) const
{  // return size

  return(size);

}
