
// **************************************
// Header Files
// **************************************

#include "screen.h"

// **************************
// Class Declaration
// **************************

Screen::Screen(void)
{  // constructor
}

Screen::~Screen(void)
{  // destructor
}

Screen::Screen(const Screen &scr)
{  // copy constructor

  identifier=scr.GetId();
  monitorEntry=scr.GetMonitor();
  deviceEntry=scr.GetDevice();
  defaultDepth=scr.GetDefaultDepth();
  identifierComment=scr.GetIdComment();
  monitorEntryComment=scr.GetMonitorComment();
  deviceEntryComment=scr.GetDeviceComment();
  defaultDepthComment=scr.GetDefaultDepthComment();
  comment=scr.GetComment();

  for (int i=0; i<scr.GetOptionSize(); i++)
  {
    AddOption(scr.GetOption(i));
  }

  for (int j=0; j<scr.GetDisplaySize(); j++)
  {
    AddDisplay(scr.GetDisplay(j));
  }

}

Screen& Screen::operator=(const Screen &scr)
{  // copy constructor

  Clear();

  identifier=scr.GetId();
  monitorEntry=scr.GetMonitor();
  deviceEntry=scr.GetDevice();
  defaultDepth=scr.GetDefaultDepth();
  identifierComment=scr.GetIdComment();
  monitorEntryComment=scr.GetMonitorComment();
  deviceEntryComment=scr.GetDeviceComment();
  defaultDepthComment=scr.GetDefaultDepthComment();
  comment=scr.GetComment();

  for (int i=0; i<scr.GetOptionSize(); i++)
  {
    AddOption(scr.GetOption(i));
  }

  for (int j=0; j<scr.GetDisplaySize(); j++)
  {
    AddDisplay(scr.GetDisplay(j));
  }

  return(*this);
}

void Screen::AddComment(const wxString& com)
{  // set comment
  comment.Add(com);
}

void Screen::SetId(const wxString& id, const wxString& com)
{  // set identifier

  identifier=id;
  identifierComment=com;
}

void Screen::SetMonitor(const wxString& mon, const wxString& com)
{  // set monitor

  monitorEntry=mon;
  monitorEntryComment=com;
}

void Screen::SetDevice(const wxString& dev, const wxString& com)
{  // set device

  deviceEntry=dev;
  deviceEntryComment=com;
}

void Screen::SetDefaultDepth(const wxString& dd, const wxString& com)
{  // set default depth

  defaultDepth=dd;
  defaultDepthComment=com;
}

void Screen::RemoveOption(const int index)
{  // remove option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Screen::RemoveOption"), wxICON_ERROR);
    return;
  }

  options.Remove(index);
}

void Screen::AddOption(const Option &opt)
{  // add option

  options.Add(opt);
}

void Screen::SetOption(const int index, const Option& opt)
{  // set option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Screen::SetOption"), wxICON_ERROR);
    return;
  }

  options[index]=opt;
}

void Screen::RemoveDisplay(const int index)
{  // remove Display

  if (index>=dList.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Screen::RemoveDisplay"), wxICON_ERROR);
    return;
  }

  dList.Remove(index);
}

void Screen::AddDisplay(const Display& dis)
{  // add Display

  dList.Add(dis);
}

void Screen::Clear(void)
{    // clear everything

  identifier=wxEmptyString;
  monitorEntry=wxEmptyString;
  deviceEntry=wxEmptyString;
  defaultDepth=wxEmptyString;
  identifierComment=wxEmptyString;
  monitorEntryComment=wxEmptyString;
  deviceEntryComment=wxEmptyString;
  defaultDepthComment=wxEmptyString;
  comment.Clear();
  options.Clear();
  dList.Clear();

}

const wxArrayString& Screen::GetComment(void) const
{  // return comment

  return(comment);
}

const wxString& Screen::GetId(void) const
{  // return identifier

  return(identifier);
}

const wxString& Screen::GetMonitor(void) const
{  // return monitor

  return(monitorEntry);
}

const wxString& Screen::GetDevice(void) const
{  // return device

  return(deviceEntry);
}

const wxString& Screen::GetDefaultDepth(void) const
{  // return default depth

  return(defaultDepth);
}

const wxString& Screen::GetIdComment(void) const
{  // return identifier Comment

  return(identifierComment);
}

const wxString& Screen::GetMonitorComment(void) const
{  // return monitor Comment

  return(monitorEntryComment);
}

const wxString& Screen::GetDeviceComment(void) const
{  // return device Comment

  return(deviceEntryComment);
}

const wxString& Screen::GetDefaultDepthComment(void) const
{  // return default depth Comment

  return(defaultDepthComment);
}

const Option& Screen::GetOption(const int index) const
{  // get option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Screen::GetOption"), wxICON_ERROR);
  }

  return(options[index]);
}

Option& Screen::GetOption(const int index)
{  // get option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Screen::GetOptionE"), wxICON_ERROR);
  }

  return(options[index]);
}

const int Screen::GetOptionSize(void) const
{  // return size of options

  return(options.GetSize());
}


const Display& Screen::GetDisplay(const int index) const
{  // get Display

  if (index>=dList.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Screen::GetDisplay"), wxICON_ERROR);
  }

  return(dList[index]);
}

Display& Screen::GetDisplay(const int index)
{  // get Display

  if (index>=dList.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("Screen::GetDisplay"), wxICON_ERROR);
    return(dList[index]);
  }
  return(dList[index]);
}

const int Screen::GetDisplaySize(void) const
{  // return number of Display sections

  return(dList.GetSize());

}

