#ifndef SCREEN_H
#define SCREEN_H

// **************************************
// Header Files
// **************************************

#include "global.h"
#include "optionlist.h"
#include "displaylist.h"

// **************************
// Class Definition
// **************************

class Screen
{
public:
  Screen(void);   // constructor
  ~Screen(void);   // destructor
  Screen(const Screen &scr); // copy constructor

  Screen& operator=(const Screen &scr); // copy constructor

  void AddComment(const wxString& com);      // set comment
  void SetId(const wxString& id, const wxString& com=wxEmptyString);      // set identifier
  void SetMonitor(const wxString& mon, const wxString& com=wxEmptyString);  // set monitor
  void SetDevice(const wxString& dev, const wxString& com=wxEmptyString);    // set device
  void SetDefaultDepth(const wxString& dd, const wxString& com=wxEmptyString);  // set default depth

  const wxArrayString& GetComment(void) const;           // return comment
  const wxString& GetId(void) const;           // return identifier
  const wxString& GetMonitor(void) const;       // return monitor
  const wxString& GetDevice(void) const;         // return device
  const wxString& GetDefaultDepth(void) const;    // return default depth
  const wxString& GetIdComment(void) const;           // return identifier comment
  const wxString& GetMonitorComment(void) const;       // return monitor comment
  const wxString& GetDeviceComment(void) const;         // return device comment
  const wxString& GetDefaultDepthComment(void) const;    // return default depth comment

  void RemoveDisplay(const int index);  // remove display
  void AddDisplay(const Display& dis);        // add display

  const Display& GetDisplay(const int index) const;     // get single display
  Display& GetDisplay(const int index);     // get single display
  const int GetDisplaySize(void) const;                 // return number of displays

  void RemoveOption(const int index);    // remove option
  void AddOption(const Option &opt);              // add option
  void SetOption(const int index, const Option &opt); // set option

  Option& GetOption(const int index);                // get option
  const Option& GetOption(const int index) const;     // get option
  const int GetOptionSize(void) const;                      // return size of options

  void Clear(void);          // clear everything

private:
  wxString identifier;      // Screen identifier
  wxString monitorEntry;     // monitor
  wxString deviceEntry;       // device entry
  wxString defaultDepth;    // default depth

  // Note: Any additional settings will not be recognized like:
  // VideoAdaptor, DefaultFbBpp.

  wxArrayString comment;
  wxString identifierComment;
  wxString monitorEntryComment;
  wxString deviceEntryComment;
  wxString defaultDepthComment;
  DisplayList dList;     // all display subsections
  OptionList options;    // all options
};

#endif // SCREEN_H
