
// **************************************
// Header Files
// **************************************

#include "screenlist.h"

// **************************
// Class Declaration
// **************************

ScreenList::ScreenList(void)
{  // constructor
  size=0;
  dList=NULL;
}

ScreenList::~ScreenList(void)
{  // destructor
  if (dList) delete [] dList;
  dList=NULL;
}

ScreenList::ScreenList(const ScreenList &dis)
{  // copy constructor

  size=dis.GetSize();
  dList=new Screen[size];

  for (int i=0; i<size;i++)
  {
    dList[i]=dis[i];
  }
}

ScreenList& ScreenList::operator=(const ScreenList &dis)
{  // copy constructor

  Clear();

  size=dis.GetSize();
  dList=new Screen[size];

  for (int i=0; i<size;i++)
  {
    dList[i]=dis[i];
  }

  return(*this);
}

void ScreenList::Remove(const int index)
{  // remove a Screen

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ScreenList::Remove"), wxICON_ERROR);
    return;
  }

  int i;  // loop counter

  if (size>1)
  {
    Screen *dis=new Screen[size-1];

    for (i=0; i<index; i++) dis[i]=dList[i];        // copy old entries
    for (i=index+1; i<size; i++) dis[i-1]=dList[i]; // and ignore index

    if (dList) delete [] dList;   // delete old list
    dList=dis;         // set pointer
    dis=NULL;
  }
  else
  {
    if (dList) delete [] dList;   // delete old list
    dList=NULL;         // set pointer
  }

  size--;              // set size
}

void ScreenList::Add(const Screen& dis)
{  // add an option and value

  Screen *dL=new Screen[size+1];

  for (int i=0; i<size; i++) dL[i]=dList[i]; // copy old entries

  dL[size]=dis;   // set las entry

  if (dList) delete [] dList;   // delete old list
  dList=dL;         // set pointer
  dL=NULL;

  size++;
}

void ScreenList::Clear(void)
{    // clear everything

  if (dList) delete [] dList;
  dList=NULL;

  size=0;
}

const Screen& ScreenList::operator[](const int index) const
{  // get single Screen

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ScreenList::Get"), wxICON_ERROR);
  }

  return(dList[index]);
}

Screen& ScreenList::operator[](const int index)
{  // get single device

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ScreenList::operator[]"), wxICON_ERROR);
    return(dList[index]);  // return empty device
  }

  return(dList[index]);
}


const int ScreenList::GetSize(void) const
{  // return size

  return(size);

}
