
// **************************************
// Header Files
// **************************************

#include "serverlayout.h"

// **************************
// Class Declaration
// **************************

ServerLayout::ServerLayout(void)
{  // constructor
  screenSize=0;
  iDeviceSize=0;
  iDeviceOptions=NULL;
}

ServerLayout::~ServerLayout(void)
{  // destructor

  if (iDeviceOptions) delete [] iDeviceOptions;
  iDeviceOptions=NULL;
}

ServerLayout::ServerLayout(const ServerLayout &ser)
{  // copy constructor

  screenSize=0;
  iDeviceSize=0;
  iDeviceOptions=NULL;

  identifier=ser.GetId();
  identifierComment=ser.GetIdComment();
  comment=ser.GetComment();

  for (int i=0; i<ser.GetScreenSize(); i++)
  {
    AddScreen(ser.GetScreen(i), ser.GetScreenNumber(i), ser.GetScreenPosition(i), ser.GetSecondScreen(i), ser.GetScreenComment(i));
  }

  for (int j=0; j<ser.GetInputDeviceSize(); j++)
  {
    AddInputDevice(ser.GetInputDevice(j), ser.GetInputDeviceComment(j));

    for (int l=0; l<ser.GetInputDeviceOptionSize(j); l++)
    {
      AddInputDeviceOption(j, ser.GetInputDeviceOption(j,l));
    }
  }

  for (int k=0; k<ser.GetOptionSize(); k++)
  {
    AddOption(ser.GetOption(k));
  }

}

ServerLayout& ServerLayout::operator=(const ServerLayout &ser)
{  // copy constructor

  Clear();

  identifier=ser.GetId();
  identifierComment=ser.GetIdComment();
  comment=ser.GetComment();

  for (int i=0; i<ser.GetScreenSize(); i++)
  {
    AddScreen(ser.GetScreen(i), ser.GetScreenNumber(i), ser.GetScreenPosition(i), ser.GetSecondScreen(i), ser.GetScreenComment(i));
  }

  for (int j=0; j<ser.GetInputDeviceSize(); j++)
  {
    AddInputDevice(ser.GetInputDevice(j), ser.GetInputDeviceComment(j));

    for (int l=0; l<ser.GetInputDeviceOptionSize(j); l++)
    {
      AddInputDeviceOption(j, ser.GetInputDeviceOption(j,l));
    }
  }

  for (int k=0; k<ser.GetOptionSize(); k++)
  {
    AddOption(ser.GetOption(k));
  }

  return(*this);
}

void ServerLayout::AddComment(const wxString& com)
{  // set comment

  comment.Add(com);
}

void ServerLayout::SetId(const wxString& id, const wxString& com)
{  // set identifier

  identifier=id;
  identifierComment=com;
}

void ServerLayout::Clear(void)
{    // clear everything

  identifier=wxEmptyString;
  identifierComment=wxEmptyString;
  comment.Clear();
  screenEntry.Clear();
  screenComment.Clear();
  screenNumber.Clear();
  screenPosition.Clear();
  screenEntry2.Clear();
  screenSize=0;
  iDeviceEntry.Clear();
  iDeviceComment.Clear();
  if (iDeviceOptions) delete [] iDeviceOptions;
  iDeviceOptions=NULL;
  iDeviceSize=0;
  options.Clear();
}

const wxArrayString& ServerLayout::GetComment(void) const
{  // return comment

  return(comment);
}

const wxString& ServerLayout::GetId(void) const
{  // return identifier

  return(identifier);
}

const wxString& ServerLayout::GetIdComment(void) const
{  // return identifier comment

  return(identifierComment);
}

void ServerLayout::RemoveScreen(const int index)
{  // remove screen

  if (index>=screenSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::RemoveScreen"), wxICON_ERROR);
    return;
  }

  screenEntry.RemoveAt(index);
  screenComment.RemoveAt(index);
  screenNumber.RemoveAt(index);
  screenPosition.RemoveAt(index);
  screenEntry2.RemoveAt(index);
  screenSize--;
}

void ServerLayout::AddScreen(const wxString& id, const wxString& num, const wxString &pos, const wxString& second, const wxString& com)
{ // add a screen

  screenEntry.Add(id);
  screenComment.Add(com);
  screenNumber.Add(num);
  screenPosition.Add(pos);
  screenEntry2.Add(second);
  screenSize++;
}

const wxString& ServerLayout::GetScreen(const int index) const
{  // get screen id

  if (index>=screenSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::GetScreen"), wxICON_ERROR);
  }

  return(screenEntry.Item(index));
}

const wxString& ServerLayout::GetScreenComment(const int index) const
{  // get screen comment

  if (index>=screenSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::GetScreenComment"), wxICON_ERROR);
  }

  return(screenComment.Item(index));
}

const wxString& ServerLayout::GetScreenNumber(const int index) const
{ // get screen number

  if (index>=screenSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::GetScreenNumber"), wxICON_ERROR);
  }

  return(screenNumber.Item(index));

}

const wxString& ServerLayout::GetScreenPosition(const int index) const
{  // get screen position

  if (index>=screenSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::GetScreenPosition"), wxICON_ERROR);
  }

  return(screenPosition.Item(index));

}

const wxString& ServerLayout::GetSecondScreen(const int index) const
{  // get screen position

  if (index>=screenSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::GetSecondScreen"), wxICON_ERROR);
  }

  return(screenEntry2.Item(index));

}

const int ServerLayout::GetScreenSize(void) const
{  // return number of screens
  return(screenSize);
}

void ServerLayout::RemoveInputDevice(const int index)
{  // remove input device

  if (index>=iDeviceSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::RemoveInputDevice"), wxICON_ERROR);
    return;
  }

  iDeviceEntry.RemoveAt(index);
  iDeviceComment.RemoveAt(index);

  int i;  // loop counter

  if (iDeviceSize>1)
  {
    OptionList *dis=new OptionList[iDeviceSize-1];

    for (i=0; i<index; i++) dis[i]=iDeviceOptions[i];        // copy old entries
    for (i=index+1; i<iDeviceSize; i++) dis[i-1]=iDeviceOptions[i]; // and ignore index

    if (iDeviceOptions) delete [] iDeviceOptions;   // delete old list
    iDeviceOptions=dis;         // set pointer
    dis=NULL;
  }
  else
  {
    if (iDeviceOptions) delete [] iDeviceOptions;   // delete old list
    iDeviceOptions=NULL;         // set pointer
  }

  iDeviceSize--;

}

void ServerLayout::AddInputDevice(const wxString& id, const wxString& com)
{ // add an input device

  iDeviceEntry.Add(id);
  iDeviceComment.Add(com);

  OptionList *dL=new OptionList[iDeviceSize+1];

  for (int i=0; i<iDeviceSize; i++) dL[i]=iDeviceOptions[i]; // copy old entries

  // leave last entry open

  if (iDeviceOptions) delete [] iDeviceOptions;   // delete old list
  iDeviceOptions=dL;         // set pointer
  dL=NULL;

  iDeviceSize++;
}


void ServerLayout::RemoveInputDeviceOption(const int index1, const int index2)
{  // remove input device

  if (index1>=iDeviceSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::RemoveInputDeviceOption"), wxICON_ERROR);
    return;
  }

  iDeviceOptions[index1].Remove(index2);
}

void ServerLayout::AddInputDeviceOption(const int index, const Option& opt)
{ // add an input device

  if (index>=iDeviceSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::AddInputDeviceOption"), wxICON_ERROR);
    return;
  }

  iDeviceOptions[index].Add(opt);
}

void ServerLayout::SetInputDeviceOption(const int index1, const int index2, const Option& opt)
{  // set an option and value

  if (index1>=iDeviceSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::SetInputDeviceOption"), wxICON_ERROR);
    return;
  }

  iDeviceOptions[index1][index2]=opt;
}

const wxString& ServerLayout::GetInputDevice(const int index) const
{  // get input device id

 if (index>=iDeviceSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::GetInputDevice"), wxICON_ERROR);
  }

  return(iDeviceEntry.Item(index));

}

const wxString& ServerLayout::GetInputDeviceComment(const int index) const
{  // get input device comment

 if (index>=iDeviceSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::GetInputDeviceComment"), wxICON_ERROR);
  }

  return(iDeviceComment.Item(index));

}


const Option& ServerLayout::GetInputDeviceOption(const int index1, const int index2) const
{  // get input device option

 if (index1>=iDeviceSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::GetInputDevice"), wxICON_ERROR);
  }

  return(iDeviceOptions[index1][index2]);

}

Option& ServerLayout::GetInputDeviceOption(const int index1, const int index2)
{  // get input device option

 if (index1>=iDeviceSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::GetInputDeviceE"), wxICON_ERROR);
  }

  return(iDeviceOptions[index1][index2]);
}

const int ServerLayout::GetInputDeviceSize(void) const
{  // return number of input devices
  return(iDeviceSize);
}

const int ServerLayout::GetInputDeviceOptionSize(const int index) const
{  // return number of input device options

 if (index>=iDeviceSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::GetInputDeviceOptionSize"), wxICON_ERROR);
    return(0);
  }

  return(iDeviceOptions[index].GetSize());
}

void ServerLayout::RemoveOption(const int index)
{  // remove option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::RemoveOption"), wxICON_ERROR);
    return;
  }

  options.Remove(index);
}

void ServerLayout::AddOption(const Option &opt)
{  // add option

  options.Add(opt);
}

void ServerLayout::SetOption(const int index, const Option& opt)
{  // set option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::SetOption"), wxICON_ERROR);
    return;
  }

  options[index]=opt;
}

const Option& ServerLayout::GetOption(const int index) const
{  // get option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::GetOption"), wxICON_ERROR);
  }

  return(options[index]);
}

Option& ServerLayout::GetOption(const int index)
{  // get option

  if (index>=options.GetSize())
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::GetOptionE"), wxICON_ERROR);
  }

  return(options[index]);
}

const int ServerLayout::GetOptionSize(void) const
{  // return size of options

  return(options.GetSize());
}

void ServerLayout::SetScreen(const int index, const wxString& scr, const wxString& com)
{            // set screen id

  if (index>=screenSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::SetScreen"), wxICON_ERROR);
    return;
  }

  screenEntry[index]=scr;
  screenComment[index]=com;
}

void ServerLayout::SetScreenNumber(const int index, const wxString& num)
{      // set screen number

  if (index>=screenSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::SetScreenNumber"), wxICON_ERROR);
    return;
  }

  screenNumber[index]=num;

}

void ServerLayout::SetScreenPosition(const int index, const wxString& pos)
{   // set screen position

  if (index>=screenSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::SetScreenPosition"), wxICON_ERROR);
    return;
  }

  screenPosition[index]=pos;

}

void ServerLayout::SetSecondScreen(const int index, const wxString& scr)
{         // set second screen id

  if (index>=screenSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::SetSecondScreen"), wxICON_ERROR);
    return;
  }

  screenEntry2[index]=scr;

}

void ServerLayout::SetInputDevice(const int index, const wxString& dev, const wxString& com)
{  // set input device id

  if (index>=iDeviceSize)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayout::SetInputDevice"), wxICON_ERROR);
    return;
  }

  iDeviceEntry[index]=dev;
  iDeviceComment[index]=com;
}

