#ifndef SERVERLAYOUT_H
#define SERVERLAYOUT_H

// **************************************
// Header Files
// **************************************

#include "global.h"
#include "optionlist.h"

// **************************
// Class Definition
// **************************

class ServerLayout
{
public:
  ServerLayout(void);    // constructor
  ~ServerLayout(void);   // destructor
  ServerLayout(const ServerLayout &ser); // copy constructor

  ServerLayout& operator=(const ServerLayout &ser); // copy constructor

  void AddComment(const wxString& com);      // set comment
  void SetId(const wxString& id, const wxString& com=wxEmptyString);      // set identifier

  const wxArrayString& GetComment(void) const;           // return comment
  const wxString& GetId(void) const;           // return identifier
  const wxString& GetIdComment(void) const;           // return identifier comment

  void RemoveScreen(const int index);               // remove screen
  void AddScreen(const wxString& id, const wxString& num=wxEmptyString, const wxString &pos=wxEmptyString, const wxString &second=wxEmptyString, const wxString& com=wxEmptyString);  // add a screen

  void SetScreen(const int index, const wxString& scr, const wxString& com=wxEmptyString);             // set screen id
  void SetScreenNumber(const int index, const wxString& num);       // set screen number
  void SetScreenPosition(const int index, const wxString& pos);     // set screen position
  void SetSecondScreen(const int index, const wxString& scr);       // set second screen id

  const wxString& GetScreen(const int index) const;           // get screen id
  const wxString& GetScreenComment(const int index) const;           // get screen comment
  const wxString& GetScreenNumber(const int index) const;     // get screen number
  const wxString& GetScreenPosition(const int index) const;    // get screen position
  const wxString& GetSecondScreen(const int index) const;      // get second screen (for position)
  const int GetScreenSize(void) const;                       // return number of screens

  void RemoveInputDevice(const int index);               // remove input device
  void AddInputDevice(const wxString& id, const wxString& com=wxEmptyString);                       // add an input device
  void SetInputDevice(const int index, const wxString& dev, const wxString& com=wxEmptyString);   // set input device id

  const wxString& GetInputDevice(const int index) const;           // get input device id
  const wxString& GetInputDeviceComment(const int index) const;           // get input device comment
  const int GetInputDeviceSize(void) const;                       // return number of input devices

  void RemoveInputDeviceOption(const int index1, const int index2);    // remove option
  void AddInputDeviceOption(const int index, const Option &opt);              // add option
  void SetInputDeviceOption(const int index1, const int index2, const Option &opt); // set option

  Option& GetInputDeviceOption(const int index1, const int index2);                // get option
  const Option& GetInputDeviceOption(const int index1, const int index2) const;     // get option
  const int GetInputDeviceOptionSize(const int index) const;                      // return size of options

  void RemoveOption(const int index);    // remove option
  void AddOption(const Option &opt);              // add option
  void SetOption(const int index, const Option &opt); // set option

  Option& GetOption(const int index);                // get option
  const Option& GetOption(const int index) const;     // get option
  const int GetOptionSize(void) const;                      // return size of options

  void Clear(void);          // clear everything

private:
  wxString identifier;             // identifier
  wxArrayString screenEntry;      // screens
  wxArrayString screenNumber;      // screen-number of each screen
  wxArrayString screenPosition;    // screen position
  wxArrayString screenEntry2;      // screens2
  int screenSize;         // number of screens
  wxArrayString iDeviceEntry;      // input devices
  OptionList *iDeviceOptions;   // input device options
  int iDeviceSize;        // number of input devices

  wxArrayString comment;
  wxString identifierComment;
  wxArrayString screenComment;
  wxArrayString iDeviceComment;

  OptionList options;    // all options
};

#endif // SERVERLAYOUT_H
