
// **************************************
// Header Files
// **************************************

#include "serverlayoutlist.h"

// **************************
// Class Declaration
// **************************

ServerLayoutList::ServerLayoutList(void)
{  // constructor
  size=0;
  dList=NULL;
}

ServerLayoutList::~ServerLayoutList(void)
{  // destructor
  if (dList) delete [] dList;
  dList=NULL;
}

ServerLayoutList::ServerLayoutList(const ServerLayoutList &dis)
{  // copy constructor

  size=dis.GetSize();
  dList=new ServerLayout[size];

  for (int i=0; i<size;i++)
  {
    dList[i]=dis[i];
  }
}

ServerLayoutList& ServerLayoutList::operator=(const ServerLayoutList &dis)
{  // copy constructor

  Clear();

  size=dis.GetSize();
  dList=new ServerLayout[size];

  for (int i=0; i<size;i++)
  {
    dList[i]=dis[i];
  }

  return(*this);
}

void ServerLayoutList::Remove(const int index)
{  // remove a ServerLayout

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayoutList::Remove"), wxICON_ERROR);
    return;
  }

  int i;  // loop counter

  if (size>1)
  {
    ServerLayout *dis=new ServerLayout[size-1];

    for (i=0; i<index; i++) dis[i]=dList[i];        // copy old entries
    for (i=index+1; i<size; i++) dis[i-1]=dList[i]; // and ignore index

    if (dList) delete [] dList;   // delete old list
    dList=dis;         // set pointer
    dis=NULL;
  }
  else
  {
    if (dList) delete [] dList;   // delete old list
    dList=NULL;         // set pointer
  }

  size--;              // set size
}

void ServerLayoutList::Add(const ServerLayout& dis)
{  // add an option and value

  ServerLayout *dL=new ServerLayout[size+1];

  for (int i=0; i<size; i++) dL[i]=dList[i]; // copy old entries

  dL[size]=dis;   // set last entry

  if (dList) delete [] dList;   // delete old list
  dList=dL;         // set pointer
  dL=NULL;

  size++;
}

void ServerLayoutList::Clear(void)
{    // clear everything

  if (dList) delete [] dList;
  dList=NULL;

  size=0;
}

const ServerLayout& ServerLayoutList::operator[](const int index) const
{  // get single ServerLayout

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayoutList::Get"), wxICON_ERROR);
  }

  return(dList[index]);
}

ServerLayout& ServerLayoutList::operator[](const int index)
{  // get single device

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("ServerLayoutList::operator[]"), wxICON_ERROR);
    return(dList[index]);  // return empty device
  }

  return(dList[index]);
}

const int ServerLayoutList::GetSize(void) const
{  // return size

  return(size);

}
