#ifndef SETTINGS_H
#define SETTINGS_H

// **************************************
// Header Files
// **************************************

#include "global.h"

// **************************
// Class Definition
// **************************

class Settings
{

public:
  Settings(void);
  ~Settings(void);
  Settings(const Settings& set);
  Settings& operator=(const Settings& set);

  const wxString& GetProgDir(void) const;
  const wxString& GetOptionDir(void) const;
  const wxString& GetCurrentDir(void) const;
  const wxString& GetCurrentFilename(void) const;
  const wxString GetCurrentFile(void) const;
  const wxString& GetFilename(void) const;

  const bool GetLoadLastFile(void) const;
  const bool GetSaveSettings(void) const;
  const bool GetShowSplash(void) const;
  const bool GetShowBlank(void) const;
  const bool GetRememberTest(void) const;

  const long GetLanguage(void) const;

  void SetProgDir(const wxString& value);
  void SetOptionDir(const wxString& value);
  void SetCurrentDir(const wxString& value);
  void SetCurrentFilename(const wxString& value);
  void SetFilename(const wxString& value);

  void SetLoadLastFile(const bool value);
  void SetSaveSettings(const bool value);
  void SetShowSplash(const bool value);
  void SetShowBlank(const bool value);
  void SetRememberTest(const bool value);

  void SetLanguage(const long lang);

private:
  wxString setFile;   // filename for saving and loading settings
  wxString progDir;   // directory where xorg-edit is executed
  wxString optionDir; // directory where options are stored
  wxString curFile;   // current file that is open
  wxString curDir;    // current directory that is open
  bool loadLastFile;  // flag if last opened file should be loaded or /etc/X11/xorg.conf
  bool saveSettings;  // flag if settings should be saved at closing frame
  bool showSplash;    // show splash screen at start
  bool showBlank;     // show message that screen will go blank during test
  bool remTest;       // remeber for test before saving
  long language;        // language

};

#endif // SETTINGS_H

