// **************************************
// Header Files
// **************************************

#include "xorgfile.h"
#include "textframe.h"

// **************************
// Class Declaration
// **************************

XorgFile::XorgFile(void)
{
  lineNr=0;
}

XorgFile::~XorgFile()
{
}

XorgFile::XorgFile(const XorgFile& xorg)
{
  files=xorg.Files();
  modules=xorg.Modules();
  serverLayouts=xorg.ServerLayouts();
  inputDevices=xorg.InputDevices();
  devices=xorg.Devices();
  monitors=xorg.Monitors();
  screens=xorg.Screens();
  serverFlags=xorg.ServerFlags();
  extensions=xorg.Extensions();
  driEntry=xorg.Dri();
  comment=xorg.GetComment();
}

XorgFile& XorgFile::operator=(const XorgFile& xorg)
{
  files=xorg.Files();
  modules=xorg.Modules();
  serverLayouts=xorg.ServerLayouts();
  inputDevices=xorg.InputDevices();
  devices=xorg.Devices();
  monitors=xorg.Monitors();
  screens=xorg.Screens();
  serverFlags=xorg.ServerFlags();
  extensions=xorg.Extensions();
  driEntry=xorg.Dri();
  comment=xorg.GetComment();

  return(*this);
}

const FileList& XorgFile::Files(void) const
{
  return(files);
}

const ModuleList& XorgFile::Modules(void) const
{
  return(modules);
}

const ServerLayoutList& XorgFile::ServerLayouts(void) const
{
  return(serverLayouts);
}

const InputDeviceList& XorgFile::InputDevices(void) const
{
  return(inputDevices);
}

const DeviceList& XorgFile::Devices(void) const
{
  return(devices);
}

const MonitorList& XorgFile::Monitors(void) const
{
  return(monitors);
}

const ScreenList& XorgFile::Screens(void) const
{
  return(screens);
}

const ServerFlagList& XorgFile::ServerFlags(void) const
{
  return(serverFlags);
}

const ExtensionList& XorgFile::Extensions(void) const
{
  return(extensions);
}

const DriMode& XorgFile::Dri(void) const
{
  return(driEntry);
}

FileList& XorgFile::Files(void)
{
  return(files);
}

ModuleList& XorgFile::Modules(void)
{
  return(modules);
}

ServerLayoutList& XorgFile::ServerLayouts(void)
{
  return(serverLayouts);
}

InputDeviceList& XorgFile::InputDevices(void)
{
  return(inputDevices);
}

DeviceList& XorgFile::Devices(void)
{
  return(devices);
}

MonitorList& XorgFile::Monitors(void)
{
  return(monitors);
}

ScreenList& XorgFile::Screens(void)
{
  return(screens);
}

ServerFlagList& XorgFile::ServerFlags(void)
{
  return(serverFlags);
}

ExtensionList& XorgFile::Extensions(void)
{
  return(extensions);
}

DriMode& XorgFile::Dri(void)
{
  return(driEntry);
}

void XorgFile::AddComment(const wxString& com)
{
  comment.Add(com);
}

const wxArrayString& XorgFile::GetComment(void) const
{
  return(comment);
}

void XorgFile::Clear(void)
{
  lineNr=0;
  filename=wxEmptyString;

  files.Clear();
  modules.Clear();
  serverLayouts.Clear();
  inputDevices.Clear();
  devices.Clear();
  monitors.Clear();
  screens.Clear();
  serverFlags.Clear();
  extensions.Clear();
  driEntry.Clear();
  comment.Clear();
}

void XorgFile::StripSpaces(wxString& line) const
{         // convert tabs to spaces, delete spaces from left and right,

  line.Replace(_T("\t"), _T(" "));         // replace tabs with spaces
  while (line.Replace(_T("  "), _T(" "))>0);  // replace double spaces with single space
  line.Trim(true);     // strip spaces from right
  line.Trim(false);    // strip spaces from left
}

void XorgFile::StripRight(wxString& line, const int num) const
{  // delete n right characters

  line=line.Right(line.Length()-num);
  StripSpaces(line);
}

const wxString XorgFile::GetNextLine(wxTextFile& file)
{
  wxString line;

  if (lineNr==0)         // handle first line separatly
  {
    line=file.GetFirstLine();  // read first line
    lineNr++;
  }
  else
  {
    line=file.GetNextLine();    // read next line
    lineNr++;
  }

  return(line);
}

const bool XorgFile::GetArgument(wxString &arg, const wxString &line) const
{
  // get a line and extract the last argument

  wxString tempStr(line);   // copy line
  tempStr.Trim(true);       // trim line right
  tempStr.Trim(false);      // and left

  int pos=tempStr.Find('"');  // search first "

  if (pos==-1) return(true); // " not found

  // delete part before first "
  tempStr=tempStr.Right(tempStr.Length()-pos-1);
  tempStr.Trim(false);      // trim line left

  pos=tempStr.Find('"');  // search second "

  if (pos==-1) return(true); // second " not found

  arg=tempStr.Left(pos);   // get argument
  arg.Trim(true);          // trim line right
  arg.Trim(false);         // and left

  return(false);

}

const bool XorgFile::GetOption(Option &opt, const wxString &line) const
{
  // get a line and extract option and argument
  wxString tempStr(line);   // copy line
  StripSpaces(tempStr);
  tempStr.MakeLower();

  if (!tempStr.StartsWith(_T("option"))) return(true);

  tempStr=line;
  StripSpaces(tempStr);
  StripRight(tempStr, 6);   // delete "option"

  if (tempStr.Find('"')!=0) return(true); // " not found at first position
  StripRight(tempStr, 1);

  int pos=tempStr.Find('"');  // search first "
  if (pos==-1) return(true); // " not found

  opt.SetName(tempStr.Left(pos));
  StripRight(tempStr, pos+1);

  if (tempStr.Find('"')!=0)  // no argument found
  {
    opt.SetValue(wxEmptyString);
    return (false);
  }

  if (tempStr.Find('"')!=0) return(true);  // " found but not at fist position

  tempStr=tempStr.Right(tempStr.Length()-1);
  pos=tempStr.Find('"');  // search second "

  if (pos==-1) return(true); // no second " found

  opt.SetValue(tempStr.Left(pos));

  return(false);
}

const bool XorgFile::FreeComments(wxString &line, wxString& com) const
{  // delete all comments

  com=wxEmptyString;
  StripSpaces(line);      // strip spaces
  int pos=line.Find('#'); // find first comment sign
  bool first=false;   // flag if # was first char

  if (pos>=0)  // # found
  {
    if (pos==0) first=true;
    com=line.Right(line.Len()-pos-1);  // extract comment
    line=line.Left(pos);                   // extract line
    StripSpaces(line);
    StripSpaces(com);

    // it may happen that the comment has further #
    // delete the ones at beginning

    do
    {
      pos=com.Find('#');
      if (pos==0) // # found at first position
      {
        StripRight(com, 1);
      }
    } while (pos==0);
  }

  return(first);
}

const bool XorgFile::ShowError(const xorgErrorID error, const xorgSectionID section, const wxString& filename, const wxString& xLine)
{    // show error message when reading file

  wxString line(xLine);
  StripSpaces(line);     // strip spaces and comments

  wxString message;
  message << _("Error in file \"") << filename << _T("\" in ");

  switch (section)
  {
    case DEVICE:
      message << _T("Section \"Device\"");
      break;
    case MONITOR:
      message << _T("Section \"Monitor\"");
      break;
    case INPUTDEVICE:
      message << _T("Section \"Input Device\"");
      break;
    case SCREEN:
      message << _T("Section \"Screen\"");
      break;
    case SERVERLAYOUT:
      message << _T("Section \"Server Layout\"");
      break;
    case FILES:
      message << _T("Section \"Files\"");
      break;
    case SERVERFLAGS:
      message << _T("Section \"Server Flags\"");
      break;
    case MODULE:
      message << _T("Section \"Module\"");
      break;
    case DRI:
      message << _T("Section \"DRI\"");
      break;
    case EXTENSIONS:
      message << _T("Section \"Extension\"");
      break;
    case DISPLAY:
      message << _T("Subsection \"Display\"");
      break;
    case NOSEC:
      message << _("unknown/undefined part of file");
      break;
    default:  // all other sections are stored directly
      break;
  }

  switch (error)
  {
    case NOCDEVICE:
    case NOCMONITOR:
    case NOCSCREEN:
    case NOCIDEVICE:
    case NODEVICES:
    case NOMONITORS:
    case NOIDEVICES:
    case NOSCREENS:
    case NOSLAYOUTS:
    case NODEVICEID:
    case NOMONITORID:
    case NOSCREENID:
    case NOIDEVICEID:
    case NOSLAYOUTID:
    case NODEVICEDRIVER:
    case NOIDEVICEDRIVER:
      break;
    default:
      message << _(" in line number ") << lineNr << _T(": \n\"") << line << _T("\"");
      break;
  }
  message << _T(".\n\n");

  switch (error)
  {
    case NOCDEVICE:
      message << _("Device identifier does not exist in device list.");
      break;
    case NOCMONITOR:
      message << _("Monitor identifier does not exist in monitor list.");
      break;
    case NOCSCREEN:
      message << _("Screen identifier does not exist in screen list.");
      break;
    case NOCIDEVICE:
      message << _("Input device identifier does not exist in input device list.");
      break;
    case TWOSECTIONS:
      message << _("Another section has begun without ending the last one.");
      break;
    case NOSECTION:
      message << _("Section has ended without beginning.");
      break;
    case NOSUBSECTION:
      message << _("Subsection has ended without beginning.");
      break;
    case EMPTYARGUMENT:
      message << _("Argument or Option must not be empty.");
      break;
    case NODEVICEID:
      message << _("Device identifier is missing.");
      break;
    case NOMONITORID:
      message << _("Monitor identifier is missing.");
      break;
    case NOIDEVICEID:
      message << _("Input device identifier is missing.");
      break;
    case NOSCREENID:
      message << _("Screen identifier is missing.");
      break;
    case NOSLAYOUTID:
      message << _("Server layout identifier is missing.");
      break;
    case NODEVICEDRIVER:
      message << _("Device driver is missing.");
      break;
    case NOIDEVICEDRIVER:
      message << _("Input device driver is missing.");
      break;
    case NOMODUL:
      message << _("Module option given without a module.");
      break;
    case SECDRI:
      message << _("Second DRI entry found.");
      break;
    case NODEVICES:
      message << _("No devices declared. Need at least one...");
      break;
    case NOMONITORS:
      message << _("No monitors declared. Need at least one...");
      break;
    case NOIDEVICES:
      message << _("No or only one input devices declared. Need at least two...");
      break;
    case NOSCREENS:
      message << _("No screens declared. Need at least one...");
      break;
    case NOSLAYOUTS:
      message << _("No server layouts declared. Need at least one...");
      break;
    default:
      message << _("Either the file is not a valid config file or there are some options that could not be recognized.");
      break;
  }


  switch (error)
  {
    case NOCDEVICE:
    case NOCMONITOR:
    case NOCSCREEN:
    case NOCIDEVICE:
    case NODEVICES:
    case NOMONITORS:
    case NOIDEVICES:
    case NOSCREENS:
    case NOSLAYOUTS:
    case NODEVICEID:
    case NOMONITORID:
    case NOIDEVICEID:
    case NOSCREENID:
    case NOSLAYOUTID:
    case NODEVICEDRIVER:
    case NOIDEVICEDRIVER:
      message << _T("\n\n") << _("Should I ignore this and go on?") << _("\n(Warning: It is not sure what happens next!)");
      break;
    case TWOSECTIONS:
    case NOSECTION:
    case NOSUBSECTION:
    case EMPTYARGUMENT:
    case NOMODUL:
    case SECDRI:
    default:
      message << _T("\n\n") << _("Should I comment out this line and go on?") << _("\n(Warning: It is not sure what happens next!)");
      break;
  }

  if (wxMessageBox(message, _("Error: Reading file"), wxICON_QUESTION | wxYES_NO | wxYES_DEFAULT )==wxNO)
  {
    return(true);
  }
  else return(false);
}

const bool XorgFile::Load(const wxString& name)
{  // load file

  wxTextFile file(name);    // create text file handler

  if (!file.Exists())
  {
    wxString tempStr;
    tempStr << _("The file ") << _T("\"") << name << _T("\"") << _(" does not exist. Will not read anything!");
    wxMessageBox(tempStr, _("Error: File does not exist"), wxICON_ERROR | wxOK);
    return(true);
  }

  file.Open();  // open file

  wxString line;                           // extracted and converted line in file
  wxString xLine;                          // line in file
  wxString tempLine;
  wxString argument;                       // temporary string
  wxString com;                              // comment in file
  wxString screen1;                        // temporary string
  wxString screen2;                        // temporary string
  bool failure=false;                      // read failure
  bool lineComment=false;                  // flag if line is comment
  int pos=-1;                              // position of some chars
  int i, j,k ;                    // loop counter
  xorgSectionID section=NOSEC;                  // current section
  xorgErrorID error=UNKNOWN;                   // unknown error

  // elements for reading
  Device tempDevice;
  Monitor tempMonitor;
  InputDevice tempInputDevice;
  Screen tempScreen;
  Display tempDisplay;
  ServerLayout tempServerLayout;
  Option tempOption;

  // clear all old elements first
  Clear();

  filename=name;  // store filename

  do
  {
    xLine=GetNextLine(file);    // read next line
    failure=false;
    error=UNKNOWN;
    line=xLine;
    lineComment=FreeComments(line, com);  // delete all comments in line and return true or false if only a comment

    if (lineComment) // whole line was a comment
    {
      line=com;
      FreeComments(line, com); // get second comment
    }

    if (!line.IsEmpty())  // line is not empty
    {
      tempLine=line;
      line.MakeLower();   // convert to lower case letters
      // wxMessageBox(line);

      // get section in xorg.conf
      if (!lineComment && (line.StartsWith(_T("section \"device\"")) || line.StartsWith(_T("section\"device\""))))
      {
        if (section!=NOSEC)
        {
          failure=true;
          error=TWOSECTIONS;
        }
        else
        {
          section=DEVICE;
          tempDevice.Clear();   // clear device
        }
      }
      else if (!lineComment && (line.StartsWith(_T("section \"monitor\"")) || line.StartsWith(_T("section\"monitor\""))))
      {
        if (section!=NOSEC)
        {
          failure=true;
          error=TWOSECTIONS;
        }
        else
        {
          section=MONITOR;
          tempMonitor.Clear();
        }
      }
      else if (!lineComment && (line.StartsWith(_T("subsection \"display\"")) || line.StartsWith(_T("subsection\"display\""))))
      {
        if (section!=SCREEN) failure=true;
        else
        {
          section=DISPLAY;
          tempDisplay.Clear();
        }
      }
      else if (!lineComment && (line.StartsWith(_T("section \"inputdevice\"")) || line.StartsWith(_T("section\"inputdevice\""))))
      {
        if (section!=NOSEC)
        {
          failure=true;
          error=TWOSECTIONS;
        }
        else
        {
          section=INPUTDEVICE;
          tempInputDevice.Clear();
        }
      }
      else if (!lineComment && (line.StartsWith(_T("section \"screen\"")) || line.StartsWith(_T("section\"screen\""))))
      {
        if (section!=NOSEC)
        {
          failure=true;
          error=TWOSECTIONS;
        }
        else
        {
          section=SCREEN;
          tempScreen.Clear();
        }
      }
      else if (!lineComment && (line.StartsWith(_T("section \"serverlayout\"")) || line.StartsWith(_T("section\"serverlayout\""))))
      {
        if (section!=NOSEC)
        {
          failure=true;
          error=TWOSECTIONS;
        }
        else
        {
          section=SERVERLAYOUT;
          tempServerLayout.Clear();
        }
      }
      else if (!lineComment && (line.StartsWith(_T("section \"serverflags\"")) || line.StartsWith(_T("section\"serverflags\""))))
      {
        if (section!=NOSEC)
        {
          failure=true;
          error=TWOSECTIONS;
        }
        else
        {
          section=SERVERFLAGS;
        }
      }
      else if (!lineComment && (line.StartsWith(_T("section \"extensions\"")) || line.StartsWith(_T("section\"extensions\""))))
      {
        if (section!=NOSEC)
        {
          failure=true;
          error=TWOSECTIONS;
        }
        else
        {
          section=EXTENSIONS;
        }
      }
      else if (!lineComment && (line.StartsWith(_T("section \"files\"")) || line.StartsWith(_T("section\"files\""))))
      {
        if (section!=NOSEC)
        {
          failure=true;
          error=TWOSECTIONS;
        }
        else
        {
          section=FILES;
        }
      }
      else if (!lineComment && (line.StartsWith(_T("section \"module\"")) || line.StartsWith(_T("section\"module\""))))
      {
        if (section!=NOSEC)
        {
          failure=true;
          error=TWOSECTIONS;
        }
        else
        {
          section=MODULE;
        }
      }
      else if (!lineComment && (line.StartsWith(_T("section \"dri\"")) || line.StartsWith(_T("section\"dri\""))))
      {
        if (section!=NOSEC)
        {
          failure=true;
          error=TWOSECTIONS;
        }
        else
        {
          section=DRI;
        }
      }
      else if (line.StartsWith(_T("endsubsection")))
      {
        if (!lineComment && section==DISPLAY)
        {
           tempScreen.AddDisplay(tempDisplay);
           section=SCREEN;
        }
        else if (section==MODULE);  // ignore subsections
        else if (!lineComment)
        {
          error=NOSUBSECTION;
          failure=true;
        }
      }
      else if (!lineComment && line.StartsWith(_T("endsection")))
      {
        switch (section)
        {
          case INPUTDEVICE:
            inputDevices.Add(tempInputDevice);
            section=NOSEC;
            break;
          case DEVICE:
            devices.Add(tempDevice);
            section=NOSEC;
            break;
          case MONITOR:
            monitors.Add(tempMonitor);
            section=NOSEC;
            break;
          case SCREEN:
            screens.Add(tempScreen);
            section=NOSEC;
            break;
          case SERVERLAYOUT:
            serverLayouts.Add(tempServerLayout);
            section=NOSEC;
            break;
          case NOSEC:
            error=NOSECTION;
            failure=true;
            break;
          default:  // all other sections are stored directly
            section=NOSEC;
            break;
        }
      } // select sections

      if (failure)
      {
        if (ShowError(error, section, filename, xLine))
        {
          Clear();
          return(true);
        }

        lineComment=true; // create comment for the line
      }

      // handle each section separatly
      switch (section)
      {
        case FILES:
          if (line.StartsWith(_T("fontpath")))
          {
            line=tempLine; // restore line
            StripRight(line, 8);
            pos=line.Find('"');  // search first "
            if (pos!=0) failure=true; // " not found
            if (!failure)
            {
              StripRight(line, 1);
              pos=line.Find('"');  // search second "
              if (pos==-1) failure=true; // " not found
            }
            if (!failure && line.Left(pos).IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) files.AddFontPath(line.Left(pos), !lineComment, com);
            if (failure && lineComment)
            {
              line=tempLine;  // restore line
              if (!com.IsEmpty()) line+=_T(" # ")+com;
              files.AddComment(line);
              failure=false;
            }
          }
          else if (line.StartsWith(_T("modulepath")))
          {
            line=tempLine; // restore line
            StripRight(line, 10);
            pos=line.Find('"');  // search first "
            if (pos!=0) failure=true; // " not found
            if (!failure)
            {
              StripRight(line, 1);
              pos=line.Find('"');  // search second "
              if (pos==-1) failure=true; // " not found
            }
            if (!failure && line.Left(pos).IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) files.AddModulePath(line.Left(pos), !lineComment, com);
            if (failure && lineComment)
            {
              line=tempLine;  // restore line
              if (!com.IsEmpty()) line+=_T(" # ")+com;
              files.AddComment(line);
              failure=false;
            }
          }
          else if (line.StartsWith(_T("rgbpath")))
          {
            line=tempLine; // restore line
            StripRight(line, 7);
            pos=line.Find('"');  // search first "
            if (pos!=0) failure=true; // " not found
            if (!failure)
            {
              StripRight(line, 1);
              pos=line.Find('"');  // search second "
              if (pos==-1) failure=true; // " not found
            }
            if (!failure && line.Left(pos).IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) files.AddRgbPath(line.Left(pos), !lineComment, com);
            if (failure && lineComment)
            {
              line=tempLine;  // restore line
              if (!com.IsEmpty()) line+=_T(" # ")+com;
              files.AddComment(line);
              failure=false;
            }
          }
          else if (line.StartsWith(_T("section \"files\"")) || line.StartsWith(_T("section\"files\"")));
          else if (!line.IsEmpty() && !lineComment) failure=true;
          else if (lineComment)
          {
            line=tempLine;  // restore line
            if (!com.IsEmpty()) line+=_T(" # ")+com;
            files.AddComment(line);
          }
          break;
        case MODULE:
          if (line.StartsWith(_T("load")))
          {
            line=tempLine;  // restore line
            StripRight(line, 4);
            pos=line.Find('"');  // search first "
            if (pos!=0) failure=true; // " not found
            if (!failure)
            {
              StripRight(line, 1);
              pos=line.Find('"');  // search second "
              if (pos==-1) failure=true; // " not found
            }
            if (!failure && line.Left(pos).IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) modules.Add(line.Left(pos), !lineComment, com);
            if (failure && lineComment)
            {
              line=tempLine;  // restore line
              if (!com.IsEmpty()) line+=_T(" # ")+com;
              modules.AddComment(line);
              failure=false;
            }
          }
          else if (line.StartsWith(_T("subsection")))
          {
            line=tempLine;  // restore line
            StripRight(line, 10);
            pos=line.Find('"');  // search first "
            if (pos!=0) failure=true; // " not found
            if (!failure)
            {
              StripRight(line, 1);
              pos=line.Find('"');  // search second "
              if (pos==-1) failure=true; // " not found
            }
            if (!failure && line.Left(pos).IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) modules.Add(line.Left(pos), !lineComment, com);
            if (failure && lineComment)
            {
              line=tempLine;  // restore line
              if (!com.IsEmpty()) line+=_T(" # ")+com;
              modules.AddComment(line);
              failure=false;
            }
          }
          else if (line.StartsWith(_T("option")))
          {
            line=tempLine;  // restore line
            failure=GetOption(tempOption, line);
            if (!failure && modules.GetSize()==0)
            {
              error=NOMODUL;
              failure=true;
            }
            if (!failure && tempOption.GetName().IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(com);
              modules.AddOption(modules.GetSize()-1, tempOption); // set entry in class
            }
            if (failure && lineComment)
            {
              line=tempLine;  // restore line
              if (!com.IsEmpty()) line+=_T(" # ")+com;
              modules.AddComment(line);
              failure=false;
            }
          }
          else if (line.StartsWith(_T("section \"module\"")) || line.StartsWith(_T("section\"module\"")));
          else if (line.StartsWith(_T("endsubsection")));
          else if (!line.IsEmpty() && !lineComment) failure=true;
          else if (lineComment)
          {
            line=tempLine;  // restore line
            if (!com.IsEmpty()) line+=_T(" # ")+com;
            modules.AddComment(line);
          }
          break;
        case SERVERFLAGS:
          if (line.StartsWith(_T("option")))
          {
            line=tempLine;  // restore line
            failure=GetOption(tempOption, line);
            if (!failure && tempOption.GetName().IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(com);
              serverFlags.AddOption(tempOption); // set entry in class
            }
            if (failure && lineComment)
            {
              line=tempLine;  // restore line
              if (!com.IsEmpty()) line+=_T(" # ")+com;
              serverFlags.AddComment(line);
              failure=false;
            }
          }
          else if (line.StartsWith(_T("section \"serverflags\"")) || line.StartsWith(_T("section\"serverflags\"")));
          else if (!line.IsEmpty() && !lineComment) failure=true;
          else if (lineComment)
          {
            line=tempLine;  // restore line
            if (!com.IsEmpty()) line+=_T(" # ")+com;
            serverFlags.AddComment(line);
          }
          break;
        case DRI:
          if (!lineComment && line.StartsWith(_T("mode")))
          {
            line=tempLine;  // restore line
            StripRight(line, 4);
            if (!driEntry.GetMode().IsEmpty())
            {
              failure=true;  // second dri entry found
              error=SECDRI;
            }
            if (!failure && line.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure)
            {
              driEntry.SetMode(line, com);
            }
          }
          else if (line.StartsWith(_T("section \"dri\"")) || line.StartsWith(_T("section\"dri\"")));
          else if (!line.IsEmpty() && !lineComment) failure=true;
          else if (lineComment)
          {
            line=tempLine;  // restore line
            if (!com.IsEmpty()) line+=_T(" # ")+com;
            driEntry.AddComment(line);
          }
          break;
        case EXTENSIONS:
          if (line.StartsWith(_T("option")))
          {
            line=tempLine;  // restore line

            failure=GetOption(tempOption, line);
            if (!failure && tempOption.GetName().IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(com);
              extensions.AddOption(tempOption); // set entry in class
            }
            if (failure && lineComment)
            {
              line=tempLine;  // restore line
              if (!com.IsEmpty()) line+=_T(" # ")+com;
              extensions.AddComment(line);
              failure=false;
            }
          }
          else if (line.StartsWith(_T("section \"extensions\"")) || line.StartsWith(_T("section\"extensions\"")));
          else if (!line.IsEmpty() && !lineComment) failure=true;
          else if (lineComment)
          {
            line=tempLine;  // restore line
            if (!com.IsEmpty()) line+=_T(" # ")+com;
            extensions.AddComment(line);
          }
          break;
        case INPUTDEVICE:
          if (!lineComment && line.StartsWith(_T("identifier")))
          {
            line=tempLine;  // restore line
            failure=GetArgument(argument, line);
            if (!failure && argument.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempInputDevice.SetId(argument, com); // set entry in class
          }
          else if (!lineComment && line.StartsWith(_T("driver")))
          {
            line=tempLine;  // restore line
            failure=GetArgument(argument, line);
            if (!failure && argument.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempInputDevice.SetDriver(argument, com); // set entry in class
          }
          else if (line.StartsWith(_T("option")))
          {
            line=tempLine;  // restore line
            failure=GetOption(tempOption, line);
            if (!failure && tempOption.GetName().IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(com);
              tempInputDevice.AddOption(tempOption); // set entry in class
            }
            if (failure && lineComment)
            {
              line=tempLine;  // restore line
              if (!com.IsEmpty()) line+=_T(" # ")+com;
              tempInputDevice.AddComment(line);
              failure=false;
            }
          }
          else if (line.StartsWith(_T("section \"inputdevice\"")) || line.StartsWith(_T("section\"inputdevice\"")));
          else if (!line.IsEmpty() && !lineComment) failure=true;
          else if (lineComment)
          {
            line=tempLine;  // restore line
            if (!com.IsEmpty()) line+=_T(" # ")+com;
            tempInputDevice.AddComment(line);
          }
          break;
        case DEVICE:
          if (!lineComment && line.StartsWith(_T("identifier")))
          {
            line=tempLine;  // restore line
            failure=GetArgument(argument, line);
            if (!failure && argument.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempDevice.SetId(argument, com); // set entry in class
          }
          else if (!lineComment && line.StartsWith(_T("driver")))
          {
            line=tempLine;  // restore line
            failure=GetArgument(argument, line);
            if (!failure && argument.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempDevice.SetDriver(argument, com); // set entry in class
          }
          else if (!lineComment && line.StartsWith(_T("busid")))
          {
            line=tempLine;  // restore line
            failure=GetArgument(argument, line);
            if (!failure && argument.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempDevice.SetBus(argument, com); // set entry in class
          }
          else if (!lineComment && line.StartsWith(_T("videoram")))
          {
            line=tempLine;  // restore line
            StripRight(line, 8);
            if (!failure && line.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempDevice.SetVideoRam(line, com); // set entry in class
          }
          else if (!lineComment && line.StartsWith(_T("screen")))
          {
            line=tempLine;  // restore line
            StripRight(line, 6);
            if (!failure && line.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempDevice.SetScreen(line, com); // set entry in class
          }
          else if (line.StartsWith(_T("option")))
          {
            line=tempLine;  // restore line
            failure=GetOption(tempOption, line);
            if (lineComment) tempOption.SetActive(true);
            else tempOption.SetActive(false);

            if (!failure && tempOption.GetName().IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(com);
              tempDevice.AddOption(tempOption); // set entry in class
            }
            if (failure && lineComment)
            {
              line=tempLine;  // restore line
              if (!com.IsEmpty()) line+=_T(" # ")+com;
              tempDevice.AddComment(line);
              failure=false;
            }
          }
          else if (line.StartsWith(_T("vendorname")) || line.StartsWith(_T("boardname")) )
          {
            // ignore vendorname and boardname, because they are not needed
            line=tempLine;  // restore line
            if (!com.IsEmpty()) line+=_T(" # ")+com;
            tempDevice.AddComment(line);
          }
          else if (line.StartsWith(_T("section \"device\"")) || line.StartsWith(_T("section\"device\"")));
          else if (!line.IsEmpty() && !lineComment) failure=true;
          else if (lineComment)
          {
            line=tempLine;  // restore line
            if (!com.IsEmpty()) line+=_T(" # ")+com;
            tempDevice.AddComment(line);
          }
          break;
        case MONITOR:
          if (!lineComment && line.StartsWith(_T("identifier")))
          {
            line=tempLine;  // restore line
            failure=GetArgument(argument, line);
            if (!failure && argument.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempMonitor.SetId(argument, com); // set entry in class
          }
          else if (!lineComment && line.StartsWith(_T("horizsync")))
          {
            line=tempLine;  // restore line
            StripRight(line, 9);
            if (!failure && line.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempMonitor.SetHorSync(line, com); // set entry in class
          }
          else if (!lineComment && line.StartsWith(_T("vertrefresh")))
          {
            line=tempLine;  // restore line
            StripRight(line, 11);
            if (!failure && line.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempMonitor.SetVertRef(line, com); // set entry in class
          }
          else if (!lineComment && line.StartsWith(_T("gamma")))
          {
            line=tempLine;  // restore line
            StripRight(line, 5);
            if (!failure && line.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempMonitor.SetGamma(line, com); // set entry in class
          }
          else if (!lineComment && line.StartsWith(_T("displaysize")))
          {
            line=tempLine;  // restore line
            StripRight(line, 11);
            if (!failure && line.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempMonitor.SetDisplaySize(line, com); // set entry in class
          }
          else if (line.StartsWith(_T("option")))
          {
            line=tempLine;  // restore line
            failure=GetOption(tempOption, line);
            if (!failure && tempOption.GetName().IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(com);
              tempMonitor.AddOption(tempOption); // set entry in class
            }
            if (failure && lineComment)
            {
              line=tempLine;  // restore line
              if (!com.IsEmpty()) line+=_T(" # ")+com;
              tempMonitor.AddComment(line);
              failure=false;
            }
          }
          else if (line.StartsWith(_T("modeline")))
          {
            line=tempLine;  // restore line
            StripRight(line, 8);
            pos=line.Find('"');  // search first "
            if (pos!=0) failure=true; // " not found
            if (!failure)
            {
              StripRight(line, 1);
              pos=line.Find('"');  // search second "
              if (pos==-1) failure=true; // " not found
            }
            if (!failure)
            {
              argument=line.Left(pos);
              StripRight(line, pos+1);
              if (!failure && (argument.IsEmpty() || line.IsEmpty()))
              {
                failure=true;
                error=EMPTYARGUMENT;
              }
              if (!failure)
              {
                tempOption.SetActive(!lineComment);
                tempOption.SetName(argument);
                tempOption.SetValue(line);
                tempOption.SetComment(com);
                tempMonitor.AddModeline(tempOption);
              }
            }
            if (failure && lineComment)
            {
              line=tempLine;  // restore line
              if (!com.IsEmpty()) line+=_T(" # ")+com;
              tempMonitor.AddComment(line);
              failure=false;
            }
          }
          else if (line.StartsWith(_T("vendorname")) || line.StartsWith(_T("modelname")) )
          {
            // ignore vendorname and modelname, because they are not needed
            line=tempLine;  // restore line
            if (!com.IsEmpty()) line+=_T(" # ")+com;
            tempMonitor.AddComment(line);
          }
          else if (line.StartsWith(_T("section \"monitor\"")) || line.StartsWith(_T("section\"monitor\"")));
          else if (!line.IsEmpty() && !lineComment) failure=true;
          else if (lineComment)
          {
            line=tempLine;  // restore line
            if (!com.IsEmpty()) line+=_T(" # ")+com;
            tempMonitor.AddComment(line);
          }
          break;
        case SCREEN:
          if (!lineComment && line.StartsWith(_T("identifier")))
          {
            line=tempLine;  // restore line
            failure=GetArgument(argument, line);
            if (!failure && argument.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempScreen.SetId(argument, com); // set entry in class
          }
          else if (!lineComment && line.StartsWith(_T("device")))
          {
            line=tempLine;  // restore line
            failure=GetArgument(argument, line);
            if (!failure && argument.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempScreen.SetDevice(argument, com); // set entry in class
          }
          else if (!lineComment && line.StartsWith(_T("monitor")))
          {
            line=tempLine;  // restore line
            failure=GetArgument(argument, line);
            if (!failure && argument.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempScreen.SetMonitor(argument, com); // set entry in class
          }
          else if (!lineComment && line.StartsWith(_T("defaultdepth")))
          {
            line=tempLine;  // restore line
            StripRight(line, 12);
            if (!failure && line.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempScreen.SetDefaultDepth(line, com);
          }
          else if (line.StartsWith(_T("option")))
          {
            line=tempLine;  // restore line
            failure=GetOption(tempOption, line);
            if (!failure && tempOption.GetName().IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(com);
              tempScreen.AddOption(tempOption); // set entry in class
            }
            if (failure && lineComment)
            {
              line=tempLine;  // restore line
              if (!com.IsEmpty()) line+=_T(" # ")+com;
              tempScreen.AddComment(line);
              failure=false;
            }
          }
          else if (line.StartsWith(_T("section \"screen\"")) || line.StartsWith(_T("section\"screen\"")));
          else if (line.StartsWith(_T("endsubsection")));
          else if (!line.IsEmpty() && !lineComment) failure=true;
          else if (lineComment)
          {
            line=tempLine;  // restore line
            if (!com.IsEmpty()) line+=_T(" # ")+com;
            tempScreen.AddComment(line);
          }
          break;
        case DISPLAY:
          if (!lineComment && line.StartsWith(_T("depth")))
          {
            line=tempLine;  // restore line
            StripRight(line, 5);
            if (!failure && line.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempDisplay.SetDepth(line, com);
          }
          else if (!lineComment && line.StartsWith(_T("viewport")))
          {
            line=tempLine;  // restore line
            StripRight(line, 8);
            if (!failure && line.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempDisplay.SetViewPort(line, com);
          }
          else if (!lineComment && line.StartsWith(_T("virtual")))
          {
            line=tempLine;  // restore line
            StripRight(line, 7);
            if (!failure && line.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempDisplay.SetVirtual(line, com);
          }
          else if (!lineComment && line.StartsWith(_T("modes")))
          {
            line=tempLine;  // restore line
            StripRight(line, 5);

            // now there should be a number of screen modes
            pos=line.Find('"');  // search first "

            while (!failure && pos!=-1)
            {
              StripRight(line, pos+1);
              pos=line.Find('"');  // search second "
              if (pos==-1)
              {  // no second " found
                failure=true;
                break; // leave while loop
              }
              argument=line.Left(pos);
              if (!failure && argument.IsEmpty())
              {
                failure=true;
                error=EMPTYARGUMENT;
                break;
              }
              tempDisplay.AddMode(argument);
              StripRight(line, pos+1);
              pos=line.Find('"');  // search first "
            }
            tempDisplay.SetModeComment(com);
          }
          else if (line.StartsWith(_T("option")))
          {
            line=tempLine;  // restore line
            failure=GetOption(tempOption, line);
            if (!failure && tempOption.GetName().IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(com);
              tempDisplay.AddOption(tempOption); // set entry in class
            }
            if (failure && lineComment)
            {
              line=tempLine;  // restore line
              if (!com.IsEmpty()) line+=_T(" # ")+com;
              tempDisplay.AddComment(line);
              failure=false;
            }
          }
          else if (line.StartsWith(_T("subsection \"display\"")) || line.StartsWith(_T("subsection\"display\"")));
          else if (!line.IsEmpty() && !lineComment) failure=true;
          else if (lineComment)
          {
            line=tempLine;  // restore line
            if (!com.IsEmpty()) line+=_T(" # ")+com;
            tempDisplay.AddComment(line);
          }
          break;
        case SERVERLAYOUT:
          if (!lineComment && line.StartsWith(_T("identifier")))
          {
            line=tempLine;  // restore line
            failure=GetArgument(argument, line);
            if (!failure && argument.IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure) tempServerLayout.SetId(argument, com); // set entry in class
          }
          else if (!lineComment && line.StartsWith(_T("screen")))
          {
            line=tempLine;  // restore line

            // add screens
            StripRight(line, 6);

            // if the first sign is a " there is no screen number

            pos=line.Find('"');  // search first "
            if (pos==-1) failure=true; // " not found

            if (!failure)
            {
              if (pos!=0) // first the screen number
              {
                argument=line.Left(pos);
                StripSpaces(argument);
                StripRight(line, pos);
              }
              else argument=wxEmptyString;

              pos=line.Find('"');  // search first " again
              if (pos!=0) failure=true; // " not found at first position
            }

            if (!failure)  // now comes the real screen entry
            {
              StripRight(line, 1);
              pos=line.Find('"');  // search second "
              if (pos==-1) failure=true; // " not found

              screen1=line.Left(pos);
              StripSpaces(screen1);

              if (!failure && screen1.IsEmpty())
              {
                failure=true;
                error=EMPTYARGUMENT;
              }
              StripRight(line, pos+1);
            }

            if (!failure)
            {
              if (line.IsEmpty())
              {
                // no further arguments given
                tempServerLayout.AddScreen(screen1, argument);
              }
              else
              {
                pos=line.Find('"');  // search second "

                if (pos==-1)  // no other screen, absolute position
                {
                  tempServerLayout.AddScreen(screen1, argument, line, wxEmptyString, com);
                }
                else
                {  // other screen is somewhere

                  wxString opt=line.Left(pos);
                  StripSpaces(opt);
                  StripRight(line, pos+1);

                  pos=line.Find('"');  // search second "
                  if (pos==-1) failure=true; // " not found

                  if (!failure)
                  {
                    screen2=line.Left(pos);
                    StripSpaces(screen2);
                    if (!failure && screen2.IsEmpty())
                    {
                      failure=true;
                      error=EMPTYARGUMENT;
                    }
                    if (!failure) tempServerLayout.AddScreen(screen1, argument, opt, screen2, com);
                  }
                }
              }
            }
          }
          else if (!lineComment && line.StartsWith(_T("inputdevice")))
          {
            line=tempLine;  // restore line
            StripRight(line, 11);

            // first there is the name of the device
            pos=line.Find('"');  // search first "

            if (pos!=0) failure=true; // " not found

            if (!failure)
            {
              StripRight(line, 1);
              pos=line.Find('"');  // search second "
              if (pos==-1) failure=true; // " not found
            }

            if (!failure)
            {
              argument=line.Left(pos);
              if (!failure && argument.IsEmpty())
              {
                failure=true;
                error=EMPTYARGUMENT;
              }
              StripRight(line, pos+1);
              if (!failure) tempServerLayout.AddInputDevice(argument, com); // add device
            }

            if (!failure)
            {
              // now there may be a number of input device option

              pos=line.Find('"');  // search first "

              while (!failure && pos!=-1)
              {
                StripRight(line, pos+1);
                pos=line.Find('"');  // search second "
                if (pos==-1)
                {  // no second " found
                  failure=true;
                  break; // leave while loop
                }

                tempOption.SetName(line.Left(pos));
                tempOption.SetValue(wxEmptyString);
                if (!failure && tempOption.GetName().IsEmpty())
                {
                  failure=true;
                  error=EMPTYARGUMENT;
                  break; // leave while-loop
                }
                tempServerLayout.AddInputDeviceOption(tempServerLayout.GetInputDeviceSize()-1, tempOption);

                StripRight(line, pos+1);
                pos=line.Find('"');  // search first "
              } // while
            }
          }
          else if (line.StartsWith(_T("option")))
          {
            line=tempLine;  // restore line

            failure=GetOption(tempOption, line);
            if (!failure && tempOption.GetName().IsEmpty())
            {
              failure=true;
              error=EMPTYARGUMENT;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(com);
              tempServerLayout.AddOption(tempOption); // set entry in class
            }
            if (failure && lineComment)
            {
              line=tempLine;  // restore line
              if (!com.IsEmpty()) line+=_T(" # ")+com;
              tempServerLayout.AddComment(line);
              failure=false;
            }
          }
          else if (line.StartsWith(_T("section \"serverlayout\"")) || line.StartsWith(_T("section\"serverlayout\"")));
          else if (!line.IsEmpty() && !lineComment) failure=true;
          else if (lineComment)
          {
            line=tempLine;  // restore line
            if (!com.IsEmpty()) line+=_T(" # ")+com;
            tempServerLayout.AddComment(line);
          }
          break;
        default:
          if (lineComment)
          {
             if (!line.StartsWith(_T("file edited by xorg-edit")))
             {
               if (!com.IsEmpty()) comment.Add(line+_T(" # ")+com);
               else comment.Add(line);
             }
          }
          else
          {
            if (!line.StartsWith(_T("endsection"))) failure=true;
          }
          break;
      } // switch

      if (failure)
      {
        if (ShowError(error, section, filename, xLine)==true)
        {
          Clear();
          return(true);
        }
        else
        {
          line=tempLine;  // restore line
          if (!com.IsEmpty()) line+=_T(" # ")+com;

          // set comment to right section
          switch (section)
          {
            case DEVICE:
              tempDevice.AddComment(line);
              break;
            case MONITOR:
              tempMonitor.AddComment(line);
              break;
            case INPUTDEVICE:
              tempInputDevice.AddComment(line);
              break;
            case SCREEN:
              tempScreen.AddComment(line);
              break;
            case SERVERLAYOUT:
              tempServerLayout.AddComment(line);
              break;
            case FILES:
              files.AddComment(line);
              break;
            case SERVERFLAGS:
              serverFlags.AddComment(line);
              break;
            case MODULE:
              modules.AddComment(line);
              break;
            case DRI:
              driEntry.AddComment(line);
              break;
            case EXTENSIONS:
              extensions.AddComment(line);
              break;
            case DISPLAY:
              tempDisplay.AddComment(line);
              break;
            case NOSEC:
              comment.Add(line);
              break;
            default:  // all other sections are stored directly
              break;
          }
        }
      }
    } // comment
    else
    {
      if (lineComment) comment.Add(line);
    }
  } while (!file.Eof());                  // repeat until file end

  // check if device and monitor in SCREEN are found
  // check if input devices and screens in SERVERLAYOUT are found
  bool found;

  // search screen device in device entry list
  for (i=0; i<screens.GetSize(); i++)
  {
    found=false;
    for (j=0; j<devices.GetSize(); j++)
    {
      if (screens[i].GetDevice()==devices[j].GetId())
      {
        found=true;
        break; // leave for j-loop
      }
    }

    if (!found)
    {
      error=NOCDEVICE;
      if (ShowError(error, section, filename, xLine)==true)
      {
        Clear();
        return(true);
      }
    }
  }

  // search screen monitor in monitor entry list
  for (i=0; i<screens.GetSize(); i++)
  {
    found=false;
    for (j=0; j<monitors.GetSize(); j++)
    {
      if (screens[i].GetMonitor()==monitors[j].GetId())
      {
        found=true;
        break; // leave for j-loop
      }
    }

    if (!found)
    {
      error=NOCMONITOR;
      if (ShowError(error, section, filename, xLine)==true)
      {
        Clear();
        return(true);
      }
    }
  }

  // search server layout screen in screen entry list
  for (i=0; i<serverLayouts.GetSize(); i++)
  {
    for (k=0; k<serverLayouts[i].GetScreenSize(); k++)
    {
      found=false;
      for (j=0; j<screens.GetSize(); j++)
      {
        if (serverLayouts[i].GetScreen(k)==screens[j].GetId())
        {
          found=true;
          break; // leave for j-loop
        }
      }

      if (!found)
      {
        error=NOCSCREEN;
        if (ShowError(error, section, filename, xLine)==true)
        {
          Clear();
          return(true);
        }
      }

      if (serverLayouts[i].GetSecondScreen(k)!=wxEmptyString)
      {
        found=false;
        for (j=0; j<screens.GetSize(); j++)
        {
          if (serverLayouts[i].GetSecondScreen(k)==screens[j].GetId())
          {
            found=true;
            break; // leave for j-loop
          }
        }

        if (!found)
        {
          error=NOCSCREEN;
          if (ShowError(error, section, filename, xLine)==true)
          {
            Clear();
            return(true);
          }
        }
      }
    }
  }

  // search server layout input device  in input device entry list
  for (i=0; i<serverLayouts.GetSize(); i++)
  {
    for (k=0; k<serverLayouts[i].GetInputDeviceSize(); k++)
    {
      found=false;
      for (j=0; j<inputDevices.GetSize(); j++)
      {
        if (serverLayouts[i].GetInputDevice(k)==inputDevices[j].GetId())
        {
          found=true;
          break; // leave for j-loop
        }
      }

      if (!found)
      {
        error=NOCIDEVICE;
        if (ShowError(error, section, filename, xLine)==true)
        {
          Clear();
          return(true);
        }
      }
    }
  }
  return(false);
}

const bool XorgFile::Save(const wxString& name, const wxString& version)
{  // write data into file

  wxTextFile file(name);    // create text file handler
  wxString line;                // line for writing to file
  int i,j,k;           // loop counter

  // elements for writing
  Device tempDevice;
  Monitor tempMonitor;
  InputDevice tempInputDevice;
  Screen tempScreen;
  Display tempDisplay;
  ServerLayout tempServerLayout;
  Option tempOption;

  filename=name;

  // open file
  if (file.Exists())
  {
    // create copy of file
    wxString date=wxDateTime::Today().FormatDate();
    wxString time=wxDateTime::Now().FormatTime();

    // erase all chars but 0-9 from strings
    unsigned int i;

    i=0;
    while (i<date.Len())
    {
      if (!date.Mid(i,1).IsNumber()) date.Remove(i,1);
      else i++;
    }

    i=0;
    while (i<time.Len())
    {
      if (!time.Mid(i,1).IsNumber()) time.Remove(i,1);
      else i++;
    }

    if (!wxCopyFile(filename, filename+_T("_")+date+_T("_")+time))
    {
      wxString tempStr;
      tempStr << _("Could not create backup file ") << filename << _T("_") << date << _T("_") << time << _T("\"") << _(". Do you want to save anyway?");
      if (wxMessageBox(tempStr, _("Error: Saving file"), wxICON_ERROR | wxYES_NO | wxNO_DEFAULT)==wxNO) return(true);
    }

    //open file
    file.Open();                         // open file
    file.Clear();
  }
  else
  {
    // create file
    file.Create();
  }

  for (i=0; i<(int)comment.GetCount(); i++)
  {
    file.AddLine(_T("# ")+comment.Item(i));
  }
  if (comment.GetCount()==0 || !comment.Item(comment.GetCount()-1).IsEmpty()) file.AddLine(_T("#"));
  file.AddLine(_T("# File edited by ")+version+_T(" at ")+ wxDateTime::Now().FormatDate() +_T(", ")+wxDateTime::Now().FormatTime());
  file.AddLine(_T(""));

  // files section
  if (files.GetFontPathSize()>0 || files.GetModulePathSize()>0 || files.GetRgbPathSize()>0 || files.GetComment().GetCount()>0)
  {
    file.AddLine(_T("Section \"Files\""));
    for (i=0; i<(int)files.GetComment().GetCount(); i++)
    {
      file.AddLine(_T("\t# ")+files.GetComment().Item(i));
    }
    for (i=0; i<files.GetFontPathSize(); i++)
    {
      line=wxEmptyString;
      if (!files.FontPathIsActive(i)) line << _T("#");
      line << _T("\tFontPath \"") << files.GetFontPath(i)+_T("\"");
      if (!files.GetFontPathComment(i).IsEmpty()) line << _T(" # ") << files.GetFontPathComment(i);
      file.AddLine(line);
    }
    for (i=0; i<files.GetModulePathSize(); i++)
    {
      line=wxEmptyString;
      if (!files.ModulePathIsActive(i)) line << _T("#");
      line << _T("\tModulePath \"") << files.GetModulePath(i)+_T("\"");
      if (!files.GetModulePathComment(i).IsEmpty()) line << _T(" # ") << files.GetModulePathComment(i);
      file.AddLine(line);
    }
    for (i=0; i<files.GetRgbPathSize(); i++)
    {
      line=wxEmptyString;
      if (!files.RgbPathIsActive(i)) line << _T("#");
      line << _T("\tRgbPath \"") << files.GetRgbPath(i)+_T("\"");
      if (!files.GetRgbPathComment(i).IsEmpty()) line << _T(" # ") << files.GetRgbPathComment(i);
      file.AddLine(line);
    }
    file.AddLine(_T("EndSection"));
    file.AddLine(_T(""));
  }

  // server flags section
  if (serverFlags.GetOptionSize()>0 || serverFlags.GetComment().GetCount())
  {
    file.AddLine(_T("Section \"ServerFlags\""));
    for (i=0; i<(int)serverFlags.GetComment().GetCount(); i++)
    {
      file.AddLine(_T("\t# ")+serverFlags.GetComment().Item(i));
    }
    for (i=0; i<serverFlags.GetOptionSize(); i++)
    {
      tempOption=serverFlags.GetOption(i);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tOption \"")+tempOption.GetName()+_T("\"");
      if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      file.AddLine(line);
    }
    file.AddLine(_T("EndSection"));
    file.AddLine(_T(""));
  }

  // modules section
  if (modules.GetSize()>0 || modules.GetComment().GetCount()>0)
  {
    file.AddLine(_T("Section \"Module\""));
    for (i=0; i<(int)modules.GetComment().GetCount(); i++)
    {
      file.AddLine(_T("\t# ")+modules.GetComment().Item(i));
    }
    for (i=0; i<modules.GetSize(); i++)
    {
      if (modules.GetOptionSize(i)>0)
      {
        line=wxEmptyString;
        if (!modules.IsActive(i)) line << _T("#");
        line << _T("\tSubSection \"")+modules.Get(i)+_T("\"");
        if (!modules.GetModuleComment(i).IsEmpty()) line << _T(" # ") << modules.GetModuleComment(i);
        file.AddLine(line);
        for (j=0; j<modules.GetOptionSize(i); j++)
        {
          tempOption=modules.GetOption(i,j);
          line=wxEmptyString;
          if (!modules.IsActive(i) || !tempOption.IsActive()) line << _T("#");
          line << _T("\t\tOption \"")+tempOption.GetName()+_T("\"");
          if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
          if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
          file.AddLine(line);
        }
        line=wxEmptyString;
        if (!modules.IsActive(i)) line << _T("#");
        line << _T("\tEndSubSection");
        file.AddLine(line);
      }
      else
      {
        line=wxEmptyString;
        if (!modules.IsActive(i)) line << _T("#");
        line << _T("\tLoad \"")+modules.Get(i)+_T("\"");
        if (!modules.GetModuleComment(i).IsEmpty()) line << _T(" # ") << modules.GetModuleComment(i);
        file.AddLine(line);
      }
    }
    file.AddLine(_T("EndSection"));
    file.AddLine(_T(""));
  }

  // input device section
  for (i=0; i<inputDevices.GetSize(); i++)
  {
    tempInputDevice=inputDevices[i];

    // id and driver must exist
    file.AddLine(_T("Section \"InputDevice\""));

    for (j=0; j<(int)tempInputDevice.GetComment().GetCount(); j++)
    {
      file.AddLine(_T("\t# ")+tempInputDevice.GetComment().Item(j));
    }

    if (!tempInputDevice.GetIdComment().IsEmpty()) file.AddLine(_T("\tIdentifier \"")+tempInputDevice.GetId()+_T("\"")+_T(" # ")+tempInputDevice.GetIdComment());
    else file.AddLine(_T("\tIdentifier \"")+tempInputDevice.GetId()+_T("\""));
    if (!tempInputDevice.GetDriverComment().IsEmpty()) file.AddLine(_T("\tDriver \"")+tempInputDevice.GetDriver()+_T("\"")+_T(" # ")+tempInputDevice.GetDriverComment());
    else file.AddLine(_T("\tDriver \"")+tempInputDevice.GetDriver()+_T("\""));

    for (j=0; j<tempInputDevice.GetOptionSize(); j++)
    {
      tempOption=tempInputDevice.GetOption(j);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tOption \"")+tempOption.GetName()+_T("\"");
      if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      file.AddLine(line);
    }

    file.AddLine(_T("EndSection"));
    file.AddLine(_T(""));
  }


  // device section
  for (i=0; i<devices.GetSize(); i++)
  {
    tempDevice=devices[i];

    // id and driver must exist
    file.AddLine(_T("Section \"Device\""));
    for (j=0; j<(int)tempDevice.GetComment().GetCount(); j++)
    {
      file.AddLine(_T("\t# ")+tempDevice.GetComment().Item(j));
    }

    if (!tempDevice.GetIdComment().IsEmpty()) file.AddLine(_T("\tIdentifier \"")+tempDevice.GetId()+_T("\"")+_T(" # ")+tempDevice.GetIdComment());
    else file.AddLine(_T("\tIdentifier \"")+tempDevice.GetId()+_T("\""));
    if (!tempDevice.GetDriverComment().IsEmpty()) file.AddLine(_T("\tDriver \"")+tempDevice.GetDriver()+_T("\"")+_T(" # ")+tempDevice.GetDriverComment());
    else file.AddLine(_T("\tDriver \"")+tempDevice.GetDriver()+_T("\""));

    if (!tempDevice.GetBus().IsEmpty())
    {
      if (!tempDevice.GetBusComment().IsEmpty()) file.AddLine(_T("\tBusID \"")+tempDevice.GetBus()+_T("\"")+_T(" # ")+tempDevice.GetBusComment());
      else file.AddLine(_T("\tBusID \"")+tempDevice.GetBus()+_T("\""));
    }
    if (!tempDevice.GetVideoRam().IsEmpty())
    {
      if (!tempDevice.GetVideoRamComment().IsEmpty()) file.AddLine(_T("\tVideoRam \"")+tempDevice.GetVideoRam()+_T("\"")+_T(" # ")+tempDevice.GetVideoRamComment());
      else file.AddLine(_T("\tVideoRam ")+tempDevice.GetVideoRam());
    }
    if (!tempDevice.GetScreen().IsEmpty())
    {
      if (!tempDevice.GetScreenComment().IsEmpty()) file.AddLine(_T("\tScreen \"")+tempDevice.GetScreen()+_T("\"")+_T(" # ")+tempDevice.GetScreenComment());
      else file.AddLine(_T("\tScreen ")+tempDevice.GetScreen());
    }

    for (j=0; j<tempDevice.GetOptionSize(); j++)
    {
      tempOption=tempDevice.GetOption(j);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tOption \"")+tempOption.GetName()+_T("\"");
      if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      file.AddLine(line);
    }

    file.AddLine(_T("EndSection"));
    file.AddLine(_T(""));
  }

  // monitor section
  for (i=0; i<monitors.GetSize(); i++)
  {
    tempMonitor=monitors[i];

    // id must exist
    file.AddLine(_T("Section \"Monitor\""));
    for (j=0; j<(int)tempMonitor.GetComment().GetCount(); j++)
    {
      file.AddLine(_T("\t# ")+tempMonitor.GetComment().Item(j));
    }

    if (!tempMonitor.GetIdComment().IsEmpty()) file.AddLine(_T("\tIdentifier \"")+tempMonitor.GetId()+_T("\"")+_T(" # ")+tempMonitor.GetIdComment());
    else file.AddLine(_T("\tIdentifier \"")+tempMonitor.GetId()+_T("\""));

    if (!tempMonitor.GetHorSync().IsEmpty())
    {
      if (!tempMonitor.GetHorSyncComment().IsEmpty()) file.AddLine(_T("\tHorizSyn \"")+tempMonitor.GetHorSync()+_T("\"")+_T(" # ")+tempMonitor.GetHorSyncComment());
      else file.AddLine(_T("\tHorizSync ")+tempMonitor.GetHorSync());
    }
    if (!tempMonitor.GetVertRef().IsEmpty())
    {
      if (!tempMonitor.GetVertRefComment().IsEmpty()) file.AddLine(_T("\tVertRefresh \"")+tempMonitor.GetVertRef()+_T("\"")+_T(" # ")+tempMonitor.GetVertRefComment());
      else file.AddLine(_T("\tVertRefresh ")+tempMonitor.GetVertRef());
    }
    if (!tempMonitor.GetDisplaySize().IsEmpty())
    {
      if (!tempMonitor.GetDisplaySizeComment().IsEmpty()) file.AddLine(_T("\tDisplaySize \"")+tempMonitor.GetDisplaySize()+_T("\"")+_T(" # ")+tempMonitor.GetDisplaySizeComment());
      else file.AddLine(_T("\tDisplaySize ")+tempMonitor.GetDisplaySize());
    }
    if (!tempMonitor.GetGamma().IsEmpty())
    {
      if (!tempMonitor.GetGammaComment().IsEmpty()) file.AddLine(_T("\tGamma \"")+tempMonitor.GetGamma()+_T("\"")+_T(" # ")+tempMonitor.GetGammaComment());
      else file.AddLine(_T("\tGamma ")+tempMonitor.GetGamma());
    }

    for (j=0; j<tempMonitor.GetOptionSize(); j++)
    {
      tempOption=tempMonitor.GetOption(j);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tOption \"")+tempOption.GetName()+_T("\"");
      if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      file.AddLine(line);
    }

    for (j=0; j<tempMonitor.GetModelineSize(); j++)
    {
      tempOption=tempMonitor.GetModeline(j);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tModeline \"")+tempOption.GetName()+_T("\" ")+tempOption.GetValue();
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      file.AddLine(line);
    }

    file.AddLine(_T("EndSection"));
    file.AddLine(_T(""));
  }

  // screen section
  for (i=0; i<screens.GetSize(); i++)
  {
    tempScreen=screens[i];

    // id must exist
    file.AddLine(_T("Section \"Screen\""));
    for (j=0; j<(int)tempScreen.GetComment().GetCount(); j++)
    {
      file.AddLine(_T("\t# ")+tempScreen.GetComment().Item(j));
    }

    if (!tempScreen.GetIdComment().IsEmpty()) file.AddLine(_T("\tIdentifier \"")+tempScreen.GetId()+_T("\"")+_T(" # ")+tempScreen.GetIdComment());
    else file.AddLine(_T("\tIdentifier \"")+tempScreen.GetId()+_T("\""));

    if (!tempScreen.GetDevice().IsEmpty())
    {
      if (!tempScreen.GetDeviceComment().IsEmpty()) file.AddLine(_T("\tDevice \"")+tempScreen.GetDevice()+_T("\"")+_T(" # ")+tempScreen.GetDeviceComment());
      else file.AddLine(_T("\tDevice \"")+tempScreen.GetDevice()+_T("\""));
    }
    if (!tempScreen.GetMonitor().IsEmpty())
    {
      if (!tempScreen.GetMonitorComment().IsEmpty()) file.AddLine(_T("\tMonitor \"")+tempScreen.GetMonitor()+_T("\"")+_T(" # ")+tempScreen.GetMonitorComment());
      else file.AddLine(_T("\tMonitor \"")+tempScreen.GetMonitor()+_T("\""));
    }
    if (!tempScreen.GetDefaultDepth().IsEmpty())
    {
      if (!tempScreen.GetDefaultDepthComment().IsEmpty()) file.AddLine(_T("\tDefaultDepth \"")+tempScreen.GetDefaultDepth()+_T("\"")+_T(" # ")+tempScreen.GetDefaultDepthComment());
      else file.AddLine(_T("\tDefaultDepth ")+tempScreen.GetDefaultDepth());
    }

    for (j=0; j<tempScreen.GetOptionSize(); j++)
    {
      tempOption=tempScreen.GetOption(j);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tOption \"")+tempOption.GetName()+_T("\"");
      if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      file.AddLine(line);
    }

    for (j=0; j<tempScreen.GetDisplaySize(); j++)
    {
      tempDisplay=tempScreen.GetDisplay(j);

      // depth must exist
      file.AddLine(_T("\tSubSection \"Display\""));
      for (k=0; k<(int)tempDisplay.GetComment().GetCount(); k++)
      {
        file.AddLine(_T("\t\t# ")+tempDisplay.GetComment().Item(k));
      }

      if (!tempDisplay.GetDepthComment().IsEmpty()) file.AddLine(_T("\t\tDepth \"")+tempDisplay.GetDepth()+_T("\"")+_T(" # ")+tempDisplay.GetDepthComment());
      else file.AddLine(_T("\t\tDepth ")+tempDisplay.GetDepth());

      if (!tempDisplay.GetViewPort().IsEmpty())
      {
        if (!tempDisplay.GetViewPortComment().IsEmpty()) file.AddLine(_T("\t\tViewPort \"")+tempDisplay.GetViewPort()+_T("\"")+_T(" # ")+tempDisplay.GetViewPortComment());
        else file.AddLine(_T("\t\tViewPort ")+tempDisplay.GetViewPort());
      }
      if (!tempDisplay.GetVirtual().IsEmpty())
      {
        if (!tempDisplay.GetVirtualComment().IsEmpty()) file.AddLine(_T("\t\tVirtual \"")+tempDisplay.GetVirtual()+_T("\"")+_T(" # ")+tempDisplay.GetVirtualComment());
        else file.AddLine(_T("\t\tVirtual ")+tempDisplay.GetVirtual());
      }

      for (k=0; k<tempDisplay.GetOptionSize(); k++)
      {
        tempOption=tempDisplay.GetOption(k);
        line=wxEmptyString;
        if (!tempOption.IsActive()) line << _T("#");
        line << _T("\t\tOption \"")+tempOption.GetName()+_T("\"");
        if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
        if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
        file.AddLine(line);
      }

      if (tempDisplay.GetModeSize()>0)
      {
        line=_T("\t\tModes");
        for (k=0; k<tempDisplay.GetModeSize(); k++)
        {
          line+=_T(" \"")+tempDisplay.GetMode(k)+_T("\"");
        }
        if (!tempDisplay.GetModeComment().IsEmpty()) line << _T(" # ") << tempDisplay.GetModeComment();
        file.AddLine(line);
      }
      file.AddLine(_T("\tEndSubSection"));
    }

    file.AddLine(_T("EndSection"));
    file.AddLine(_T(""));
  }

  // server layout section
  for (i=0; i<serverLayouts.GetSize(); i++)
  {
    tempServerLayout=serverLayouts[i];

    // id must exist
    file.AddLine(_T("Section \"ServerLayout\""));
    for (j=0; j<(int)tempServerLayout.GetComment().GetCount(); j++)
    {
      file.AddLine(_T("\t# ")+tempServerLayout.GetComment().Item(j));
    }

    if (!tempServerLayout.GetIdComment().IsEmpty()) file.AddLine(_T("\tIdentifier \"")+tempServerLayout.GetId()+_T("\"")+_T(" # ")+tempServerLayout.GetIdComment());
    else file.AddLine(_T("\tIdentifier \"")+tempServerLayout.GetId()+_T("\""));

    for (j=0; j<tempServerLayout.GetScreenSize(); j++)
    {
      // screen name must exist
      line=_T("\tScreen");
      if (!tempServerLayout.GetScreenNumber(j).IsEmpty()) line+=_T(" ")+tempServerLayout.GetScreenNumber(j);
      line+=_T(" \"")+tempServerLayout.GetScreen(j)+_T("\"");
      if (!tempServerLayout.GetScreenPosition(j).IsEmpty()) line+=_T(" ")+tempServerLayout.GetScreenPosition(j);
      if (!tempServerLayout.GetSecondScreen(j).IsEmpty()) line+=_T(" \"")+tempServerLayout.GetSecondScreen(j)+_T("\"");
      if (!tempServerLayout.GetScreenComment(j).IsEmpty()) line+=_T(" # ")+tempServerLayout.GetScreenComment(j);
      file.AddLine(line);
    }

    for (j=0; j<tempServerLayout.GetInputDeviceSize(); j++)
    {
      // input device name name must exist
      line=_T("\tInputDevice");
      line+=_T(" \"")+tempServerLayout.GetInputDevice(j)+_T("\"");
      for (k=0; k<tempServerLayout.GetInputDeviceOptionSize(j); k++)
      {
        line+=_T(" \"")+tempServerLayout.GetInputDeviceOption(j,k).GetName()+_T("\"");
      }
      if (!tempServerLayout.GetInputDeviceComment(j).IsEmpty()) line+=_T(" # ")+tempServerLayout.GetInputDeviceComment(j);
      file.AddLine(line);
    }

    for (j=0; j<tempServerLayout.GetOptionSize(); j++)
    {
      tempOption=tempServerLayout.GetOption(j);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tOption \"")+tempOption.GetName()+_T("\"");
      if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      file.AddLine(line);
    }

    file.AddLine(_T("EndSection"));
    file.AddLine(_T(""));
  }

  // extension section
  if (extensions.GetOptionSize()>0 || extensions.GetComment().GetCount()>0)
  {
    file.AddLine(_T("Section \"Extensions\""));
    for (i=0; i<(int)extensions.GetComment().GetCount(); i++)
    {
      file.AddLine(_T("\t# ")+extensions.GetComment().Item(i));
    }

    for (i=0; i<extensions.GetOptionSize(); i++)
    {
      tempOption=extensions.GetOption(i);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tOption \"")+tempOption.GetName()+_T("\"");
      if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      file.AddLine(line);
    }
    file.AddLine(_T("EndSection"));
    file.AddLine(_T(""));
  }

  // dri section
  if (!driEntry.GetMode().IsEmpty() || driEntry.GetComment().GetCount()>0)
  {
    file.AddLine(_T("Section \"DRI\""));
    for (i=0; i<(int)driEntry.GetComment().GetCount(); i++)
    {
      file.AddLine(_T("\t# ")+driEntry.GetComment().Item(i));
    }

    if (!driEntry.GetMode().IsEmpty())
    {
      if (!driEntry.GetModeComment().IsEmpty()) file.AddLine(_T("\tMode ")+driEntry.GetMode()+_T(" # ")+driEntry.GetModeComment());
      else file.AddLine(_T("\tMode ")+driEntry.GetMode());
    }
    file.AddLine(_T("EndSection"));
  }

  // save all changes
  if (!file.Write())
  {
    wxString tempStr;
    tempStr << _("Could not write data into file ") << _T("\"") << filename << _T("\"") << _T(".");
    wxMessageBox(tempStr, _("Error: Saving file"), wxICON_ERROR);
    return(true);
  }

  return(false);

}

void XorgFile::Preview(const wxString& name, const wxString& version) const
{

  TextFrame* xText = new TextFrame(NULL, name+_(" - PREVIEW"));  // create frame

  wxString line;                // line for writing to file
  int i,j,k;               // loop counter

  // elements for writing
  Device tempDevice;
  Monitor tempMonitor;
  InputDevice tempInputDevice;
  Screen tempScreen;
  Display tempDisplay;
  ServerLayout tempServerLayout;
  Option tempOption;

  // preambel
  /*
  xText->AddLine(_T("# ")+ curDir +_T("/") + curFile + _T(" (xorg X Window System server configuration file)"));
  xText->AddLine(_T("#"));
  xText->AddLine(_T("# This file was generated by dexconf, the Debian X Configuration tool, using"));
  xText->AddLine(_T("# values from the debconf database."));
  xText->AddLine(_T("#"));
  xText->AddLine(_T("# Edit this file with caution, and see the /etc/X11/xorg.conf manual page."));
  xText->AddLine(_T("# (Type \"man /etc/X11/xorg.conf\" at the shell prompt.)"));
  xText->AddLine(_T("#"));
  xText->AddLine(_T("# This file is automatically updated on xserver-xorg package upgrades *only*"));
  xText->AddLine(_T("# if it has not been modified since the last upgrade of the xserver-xorg"));
  xText->AddLine(_T("# package."));
  xText->AddLine(_T("#"));
  xText->AddLine(_T("# If you have edited this file but would like it to be automatically updated"));
  xText->AddLine(_T("# again, run the following command:"));
  xText->AddLine(_T("# sudo dpkg-reconfigure -phigh xserver-xorg"));
  xText->AddLine(_T("#"));
  xText->AddLine(_T("# File edited by ")+version+_T(" at ")+ wxDateTime::Now().FormatDate() +_T(", ")+wxDateTime::Now().FormatTime());
  */

  for (i=0; i<(int)comment.GetCount(); i++)
  {
    xText->AddLine(_T("# ")+comment.Item(i));
  }
  if (comment.GetCount()==0 || !comment.Item(comment.GetCount()-1).IsEmpty()) xText->AddLine(_T("#"));
  xText->AddLine(_T("# File edited by ")+version+_T(" at ")+ wxDateTime::Now().FormatDate() +_T(", ")+wxDateTime::Now().FormatTime());
  xText->AddLine(_T(""));

  // files section
  if (files.GetFontPathSize()>0 || files.GetModulePathSize()>0 || files.GetRgbPathSize()>0 || files.GetComment().GetCount()>0)
  {
    xText->AddLine(_T("Section \"Files\""));
    for (i=0; i<(int)files.GetComment().GetCount(); i++)
    {
      xText->AddLine(_T("\t# ")+files.GetComment().Item(i));
    }
    for (i=0; i<files.GetFontPathSize(); i++)
    {
      line=wxEmptyString;
      if (!files.FontPathIsActive(i)) line << _T("#");
      line << _T("\tFontPath \"") << files.GetFontPath(i)+_T("\"");
      if (!files.GetFontPathComment(i).IsEmpty()) line << _T(" # ") << files.GetFontPathComment(i);
      xText->AddLine(line);
    }
    for (i=0; i<files.GetModulePathSize(); i++)
    {
      line=wxEmptyString;
      if (!files.ModulePathIsActive(i)) line << _T("#");
      line << _T("\tModulePath \"") << files.GetModulePath(i)+_T("\"");
      if (!files.GetModulePathComment(i).IsEmpty()) line << _T(" # ") << files.GetModulePathComment(i);
      xText->AddLine(line);
    }
    for (i=0; i<files.GetRgbPathSize(); i++)
    {
      line=wxEmptyString;
      if (!files.RgbPathIsActive(i)) line << _T("#");
      line << _T("\tRgbPath \"") << files.GetRgbPath(i)+_T("\"");
      if (!files.GetRgbPathComment(i).IsEmpty()) line << _T(" # ") << files.GetRgbPathComment(i);
      xText->AddLine(line);
    }
    xText->AddLine(_T("EndSection"));
    xText->AddLine(_T(""));
  }

  // server flags section
  if (serverFlags.GetOptionSize()>0 || serverFlags.GetComment().GetCount())
  {
    xText->AddLine(_T("Section \"ServerFlags\""));
    for (i=0; i<(int)serverFlags.GetComment().GetCount(); i++)
    {
      xText->AddLine(_T("\t# ")+serverFlags.GetComment().Item(i));
    }
    for (i=0; i<serverFlags.GetOptionSize(); i++)
    {
      tempOption=serverFlags.GetOption(i);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tOption \"")+tempOption.GetName()+_T("\"");
      if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      xText->AddLine(line);
    }
    xText->AddLine(_T("EndSection"));
    xText->AddLine(_T(""));
  }

  // modules section
  if (modules.GetSize()>0 || modules.GetComment().GetCount()>0)
  {
    xText->AddLine(_T("Section \"Module\""));
    for (i=0; i<(int)modules.GetComment().GetCount(); i++)
    {
      xText->AddLine(_T("\t# ")+modules.GetComment().Item(i));
    }
    for (i=0; i<modules.GetSize(); i++)
    {
      if (modules.GetOptionSize(i)>0)
      {
        line=wxEmptyString;
        if (!modules.IsActive(i)) line << _T("#");
        line << _T("\tSubSection \"")+modules.Get(i)+_T("\"");
        if (!modules.GetModuleComment(i).IsEmpty()) line << _T(" # ") << modules.GetModuleComment(i);
        xText->AddLine(line);
        for (j=0; j<modules.GetOptionSize(i); j++)
        {
          tempOption=modules.GetOption(i,j);
          line=wxEmptyString;
          if (!modules.IsActive(i) || !tempOption.IsActive()) line << _T("#");
          line << _T("\t\tOption \"")+tempOption.GetName()+_T("\"");
          if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
          if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
          xText->AddLine(line);
        }
        line=wxEmptyString;
        if (!modules.IsActive(i)) line << _T("#");
        line << _T("\tEndSubSection");
        xText->AddLine(line);
      }
      else
      {
        line=wxEmptyString;
        if (!modules.IsActive(i)) line << _T("#");
        line << _T("\tLoad \"")+modules.Get(i)+_T("\"");
        if (!modules.GetModuleComment(i).IsEmpty()) line << _T(" # ") << modules.GetModuleComment(i);
        xText->AddLine(line);
      }
    }
    xText->AddLine(_T("EndSection"));
    xText->AddLine(_T(""));
  }

  // input device section
  for (i=0; i<inputDevices.GetSize(); i++)
  {
    tempInputDevice=inputDevices[i];

    // id and driver must exist
    xText->AddLine(_T("Section \"InputDevice\""));

    for (j=0; j<(int)tempInputDevice.GetComment().GetCount(); j++)
    {
      xText->AddLine(_T("\t# ")+tempInputDevice.GetComment().Item(j));
    }

    if (!tempInputDevice.GetIdComment().IsEmpty()) xText->AddLine(_T("\tIdentifier \"")+tempInputDevice.GetId()+_T("\"")+_T(" # ")+tempInputDevice.GetIdComment());
    else xText->AddLine(_T("\tIdentifier \"")+tempInputDevice.GetId()+_T("\""));
    if (!tempInputDevice.GetDriverComment().IsEmpty()) xText->AddLine(_T("\tDriver \"")+tempInputDevice.GetDriver()+_T("\"")+_T(" # ")+tempInputDevice.GetDriverComment());
    else xText->AddLine(_T("\tDriver \"")+tempInputDevice.GetDriver()+_T("\""));

    for (j=0; j<tempInputDevice.GetOptionSize(); j++)
    {
      tempOption=tempInputDevice.GetOption(j);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tOption \"")+tempOption.GetName()+_T("\"");
      if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      xText->AddLine(line);
    }

    xText->AddLine(_T("EndSection"));
    xText->AddLine(_T(""));
  }


  // device section
  for (i=0; i<devices.GetSize(); i++)
  {
    tempDevice=devices[i];

    // id and driver must exist
    xText->AddLine(_T("Section \"Device\""));
    for (j=0; j<(int)tempDevice.GetComment().GetCount(); j++)
    {
      xText->AddLine(_T("\t# ")+tempDevice.GetComment().Item(j));
    }

    if (!tempDevice.GetIdComment().IsEmpty()) xText->AddLine(_T("\tIdentifier \"")+tempDevice.GetId()+_T("\"")+_T(" # ")+tempDevice.GetIdComment());
    else xText->AddLine(_T("\tIdentifier \"")+tempDevice.GetId()+_T("\""));
    if (!tempDevice.GetDriverComment().IsEmpty()) xText->AddLine(_T("\tDriver \"")+tempDevice.GetDriver()+_T("\"")+_T(" # ")+tempDevice.GetDriverComment());
    else xText->AddLine(_T("\tDriver \"")+tempDevice.GetDriver()+_T("\""));

    if (!tempDevice.GetBus().IsEmpty())
    {
      if (!tempDevice.GetBusComment().IsEmpty()) xText->AddLine(_T("\tBusID \"")+tempDevice.GetBus()+_T("\"")+_T(" # ")+tempDevice.GetBusComment());
      else xText->AddLine(_T("\tBusID \"")+tempDevice.GetBus()+_T("\""));
    }
    if (!tempDevice.GetVideoRam().IsEmpty())
    {
      if (!tempDevice.GetVideoRamComment().IsEmpty()) xText->AddLine(_T("\tVideoRam \"")+tempDevice.GetVideoRam()+_T("\"")+_T(" # ")+tempDevice.GetVideoRamComment());
      else xText->AddLine(_T("\tVideoRam ")+tempDevice.GetVideoRam());
    }
    if (!tempDevice.GetScreen().IsEmpty())
    {
      if (!tempDevice.GetScreenComment().IsEmpty()) xText->AddLine(_T("\tScreen \"")+tempDevice.GetScreen()+_T("\"")+_T(" # ")+tempDevice.GetScreenComment());
      else xText->AddLine(_T("\tScreen ")+tempDevice.GetScreen());
    }

    for (j=0; j<tempDevice.GetOptionSize(); j++)
    {
      tempOption=tempDevice.GetOption(j);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tOption \"")+tempOption.GetName()+_T("\"");
      if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      xText->AddLine(line);
    }

    xText->AddLine(_T("EndSection"));
    xText->AddLine(_T(""));
  }

  // monitor section
  for (i=0; i<monitors.GetSize(); i++)
  {
    tempMonitor=monitors[i];

    // id must exist
    xText->AddLine(_T("Section \"Monitor\""));
    for (j=0; j<(int)tempMonitor.GetComment().GetCount(); j++)
    {
      xText->AddLine(_T("\t# ")+tempMonitor.GetComment().Item(j));
    }

    if (!tempMonitor.GetIdComment().IsEmpty()) xText->AddLine(_T("\tIdentifier \"")+tempMonitor.GetId()+_T("\"")+_T(" # ")+tempMonitor.GetIdComment());
    else xText->AddLine(_T("\tIdentifier \"")+tempMonitor.GetId()+_T("\""));

    if (!tempMonitor.GetHorSync().IsEmpty())
    {
      if (!tempMonitor.GetHorSyncComment().IsEmpty()) xText->AddLine(_T("\tHorizSyn \"")+tempMonitor.GetHorSync()+_T("\"")+_T(" # ")+tempMonitor.GetHorSyncComment());
      else xText->AddLine(_T("\tHorizSync ")+tempMonitor.GetHorSync());
    }
    if (!tempMonitor.GetVertRef().IsEmpty())
    {
      if (!tempMonitor.GetVertRefComment().IsEmpty()) xText->AddLine(_T("\tVertRefresh \"")+tempMonitor.GetVertRef()+_T("\"")+_T(" # ")+tempMonitor.GetVertRefComment());
      else xText->AddLine(_T("\tVertRefresh ")+tempMonitor.GetVertRef());
    }
    if (!tempMonitor.GetDisplaySize().IsEmpty())
    {
      if (!tempMonitor.GetDisplaySizeComment().IsEmpty()) xText->AddLine(_T("\tDisplaySize \"")+tempMonitor.GetDisplaySize()+_T("\"")+_T(" # ")+tempMonitor.GetDisplaySizeComment());
      else xText->AddLine(_T("\tDisplaySize ")+tempMonitor.GetDisplaySize());
    }
    if (!tempMonitor.GetGamma().IsEmpty())
    {
      if (!tempMonitor.GetGammaComment().IsEmpty()) xText->AddLine(_T("\tGamma \"")+tempMonitor.GetGamma()+_T("\"")+_T(" # ")+tempMonitor.GetGammaComment());
      else xText->AddLine(_T("\tGamma ")+tempMonitor.GetGamma());
    }

    for (j=0; j<tempMonitor.GetOptionSize(); j++)
    {
      tempOption=tempMonitor.GetOption(j);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tOption \"")+tempOption.GetName()+_T("\"");
      if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      xText->AddLine(line);
    }

    for (j=0; j<tempMonitor.GetModelineSize(); j++)
    {
      tempOption=tempMonitor.GetModeline(j);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tModeline \"")+tempOption.GetName()+_T("\" ")+tempOption.GetValue();
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      xText->AddLine(line);
    }

    xText->AddLine(_T("EndSection"));
    xText->AddLine(_T(""));
  }


  // screen section
  for (i=0; i<screens.GetSize(); i++)
  {
    tempScreen=screens[i];

    // id must exist
    xText->AddLine(_T("Section \"Screen\""));
    for (j=0; j<(int)tempScreen.GetComment().GetCount(); j++)
    {
      xText->AddLine(_T("\t# ")+tempScreen.GetComment().Item(j));
    }

    if (!tempScreen.GetIdComment().IsEmpty()) xText->AddLine(_T("\tIdentifier \"")+tempScreen.GetId()+_T("\"")+_T(" # ")+tempScreen.GetIdComment());
    else xText->AddLine(_T("\tIdentifier \"")+tempScreen.GetId()+_T("\""));

    if (!tempScreen.GetDevice().IsEmpty())
    {
      if (!tempScreen.GetDeviceComment().IsEmpty()) xText->AddLine(_T("\tDevice \"")+tempScreen.GetDevice()+_T("\"")+_T(" # ")+tempScreen.GetDeviceComment());
      else xText->AddLine(_T("\tDevice \"")+tempScreen.GetDevice()+_T("\""));
    }
    if (!tempScreen.GetMonitor().IsEmpty())
    {
      if (!tempScreen.GetMonitorComment().IsEmpty()) xText->AddLine(_T("\tMonitor \"")+tempScreen.GetMonitor()+_T("\"")+_T(" # ")+tempScreen.GetMonitorComment());
      else xText->AddLine(_T("\tMonitor \"")+tempScreen.GetMonitor()+_T("\""));
    }
    if (!tempScreen.GetDefaultDepth().IsEmpty())
    {
      if (!tempScreen.GetDefaultDepthComment().IsEmpty()) xText->AddLine(_T("\tDefaultDepth \"")+tempScreen.GetDefaultDepth()+_T("\"")+_T(" # ")+tempScreen.GetDefaultDepthComment());
      else xText->AddLine(_T("\tDefaultDepth ")+tempScreen.GetDefaultDepth());
    }

    for (j=0; j<tempScreen.GetOptionSize(); j++)
    {
      tempOption=tempScreen.GetOption(j);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tOption \"")+tempOption.GetName()+_T("\"");
      if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      xText->AddLine(line);
    }

    for (j=0; j<tempScreen.GetDisplaySize(); j++)
    {
      tempDisplay=tempScreen.GetDisplay(j);

      // depth must exist
      xText->AddLine(_T("\tSubSection \"Display\""));
      for (k=0; k<(int)tempDisplay.GetComment().GetCount(); k++)
      {
        xText->AddLine(_T("\t\t# ")+tempDisplay.GetComment().Item(k));
      }

      if (!tempDisplay.GetDepthComment().IsEmpty()) xText->AddLine(_T("\t\tDepth \"")+tempDisplay.GetDepth()+_T("\"")+_T(" # ")+tempDisplay.GetDepthComment());
      else xText->AddLine(_T("\t\tDepth ")+tempDisplay.GetDepth());

      if (!tempDisplay.GetViewPort().IsEmpty())
      {
        if (!tempDisplay.GetViewPortComment().IsEmpty()) xText->AddLine(_T("\t\tViewPort \"")+tempDisplay.GetViewPort()+_T("\"")+_T(" # ")+tempDisplay.GetViewPortComment());
        else xText->AddLine(_T("\t\tViewPort ")+tempDisplay.GetViewPort());
      }
      if (!tempDisplay.GetVirtual().IsEmpty())
      {
        if (!tempDisplay.GetVirtualComment().IsEmpty()) xText->AddLine(_T("\t\tVirtual \"")+tempDisplay.GetVirtual()+_T("\"")+_T(" # ")+tempDisplay.GetVirtualComment());
        else xText->AddLine(_T("\t\tVirtual ")+tempDisplay.GetVirtual());
      }

      for (k=0; k<tempDisplay.GetOptionSize(); k++)
      {
        tempOption=tempDisplay.GetOption(k);
        line=wxEmptyString;
        if (!tempOption.IsActive()) line << _T("#");
        line << _T("\t\tOption \"")+tempOption.GetName()+_T("\"");
        if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
        if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
        xText->AddLine(line);
      }

      if (tempDisplay.GetModeSize()>0)
      {
        line=_T("\t\tModes");
        for (k=0; k<tempDisplay.GetModeSize(); k++)
        {
          line+=_T(" \"")+tempDisplay.GetMode(k)+_T("\"");
        }
        if (!tempDisplay.GetModeComment().IsEmpty()) line << _T(" # ") << tempDisplay.GetModeComment();
        xText->AddLine(line);
      }
      xText->AddLine(_T("\tEndSubSection"));
    }

    xText->AddLine(_T("EndSection"));
    xText->AddLine(_T(""));
  }

  // server layout section
  for (i=0; i<serverLayouts.GetSize(); i++)
  {
    tempServerLayout=serverLayouts[i];

    // id must exist
    xText->AddLine(_T("Section \"ServerLayout\""));
    for (j=0; j<(int)tempServerLayout.GetComment().GetCount(); j++)
    {
      xText->AddLine(_T("\t# ")+tempServerLayout.GetComment().Item(j));
    }

    if (!tempServerLayout.GetIdComment().IsEmpty()) xText->AddLine(_T("\tIdentifier \"")+tempServerLayout.GetId()+_T("\"")+_T(" # ")+tempServerLayout.GetIdComment());
    else xText->AddLine(_T("\tIdentifier \"")+tempServerLayout.GetId()+_T("\""));

    for (j=0; j<tempServerLayout.GetScreenSize(); j++)
    {
      // screen name must exist
      line=_T("\tScreen");
      if (!tempServerLayout.GetScreenNumber(j).IsEmpty()) line+=_T(" ")+tempServerLayout.GetScreenNumber(j);
      line+=_T(" \"")+tempServerLayout.GetScreen(j)+_T("\"");
      if (!tempServerLayout.GetScreenPosition(j).IsEmpty()) line+=_T(" ")+tempServerLayout.GetScreenPosition(j);
      if (!tempServerLayout.GetSecondScreen(j).IsEmpty()) line+=_T(" \"")+tempServerLayout.GetSecondScreen(j)+_T("\"");
      if (!tempServerLayout.GetScreenComment(j).IsEmpty()) line+=_T(" # ")+tempServerLayout.GetScreenComment(j);
      xText->AddLine(line);
    }

    for (j=0; j<tempServerLayout.GetInputDeviceSize(); j++)
    {
      // input device name name must exist
      line=_T("\tInputDevice");
      line+=_T(" \"")+tempServerLayout.GetInputDevice(j)+_T("\"");
      for (k=0; k<tempServerLayout.GetInputDeviceOptionSize(j); k++)
      {
        line+=_T(" \"")+tempServerLayout.GetInputDeviceOption(j,k).GetName()+_T("\"");
      }
      if (!tempServerLayout.GetInputDeviceComment(j).IsEmpty()) line+=_T(" # ")+tempServerLayout.GetInputDeviceComment(j);
      xText->AddLine(line);
    }

    for (j=0; j<tempServerLayout.GetOptionSize(); j++)
    {
      tempOption=tempServerLayout.GetOption(j);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tOption \"")+tempOption.GetName()+_T("\"");
      if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      xText->AddLine(line);
    }

    xText->AddLine(_T("EndSection"));
    xText->AddLine(_T(""));
  }

  // extension section
  if (extensions.GetOptionSize()>0 || extensions.GetComment().GetCount()>0)
  {
    xText->AddLine(_T("Section \"Extensions\""));
    for (i=0; i<(int)extensions.GetComment().GetCount(); i++)
    {
      xText->AddLine(_T("\t# ")+extensions.GetComment().Item(i));
    }

    for (i=0; i<extensions.GetOptionSize(); i++)
    {
      tempOption=extensions.GetOption(i);
      line=wxEmptyString;
      if (!tempOption.IsActive()) line << _T("#");
      line << _T("\tOption \"")+tempOption.GetName()+_T("\"");
      if (!tempOption.GetValue().IsEmpty()) line+=_T(" \"")+tempOption.GetValue()+_T("\"");
      if (!tempOption.GetComment().IsEmpty()) line << _T(" # ") << tempOption.GetComment();
      xText->AddLine(line);
    }
    xText->AddLine(_T("EndSection"));
    xText->AddLine(_T(""));
  }

  // dri section
  if (!driEntry.GetMode().IsEmpty() || driEntry.GetComment().GetCount()>0)
  {
    xText->AddLine(_T("Section \"DRI\""));
    for (i=0; i<(int)driEntry.GetComment().GetCount(); i++)
    {
      xText->AddLine(_T("\t# ")+driEntry.GetComment().Item(i));
    }

    if (!driEntry.GetMode().IsEmpty())
    {
      if (!driEntry.GetModeComment().IsEmpty()) xText->AddLine(_T("\tMode ")+driEntry.GetMode()+_T(" # ")+driEntry.GetModeComment());
      else xText->AddLine(_T("\tMode ")+driEntry.GetMode());
    }
    xText->AddLine(_T("EndSection"));
  }

  xText->ShowModal();  // show dialog modal
  delete xText;        // delete again
  xText=NULL;

}



