// **************************************
// xorg-edit
// by Dominik Wagenfuehr (2006)
// Licence: GPL
// **************************************

// **************************************
// Header Files
// **************************************

#include "guiframe.h"
#include "textframe.h"
#include "xorgapp.h"
#include "filehandler.h"
#include <wx/datetime.h>
#include <wx/utils.h>

// **************************************
// Event Tables and other Macros
// **************************************

BEGIN_EVENT_TABLE(GuiFrame, wxFrame)
  EVT_MENU(NEWXORG, GuiFrame::OnNew)
  EVT_MENU(LOADXORG, GuiFrame::OnLoad)
  EVT_MENU(SAVEXORG, GuiFrame::OnSave)
  EVT_MENU(SAVEAS, GuiFrame::OnSaveAs)
  EVT_MENU(RESTORE, GuiFrame::OnRestore)
  EVT_MENU(CLOSEXORG, GuiFrame::OnClose)
  EVT_MENU(QUITPROG, GuiFrame::OnQuit)
  EVT_MENU(SHOWXORG, GuiFrame::OnShow)
  EVT_MENU(TESTXORG, GuiFrame::OnTest)
  EVT_MENU(ABOUTPROG, GuiFrame::OnAbout)
  EVT_MENU(SAVESET, GuiFrame::OnSaveSettings)
  EVT_MENU(LANGENG, GuiFrame::SetLanguage)
  EVT_MENU(LANGGER, GuiFrame::SetLanguage)
  EVT_MENU(SAVESETEXIT, GuiFrame::SetSaveSettings)
  EVT_MENU(LOADLAST, GuiFrame::SetLoadLoastFile)
  EVT_MENU(SHOWSPLASH, GuiFrame::SetShowSplash)
  EVT_MENU(SHOWBLANK, GuiFrame::SetShowBlankTest)
  EVT_MENU(REMTEST, GuiFrame::SetRememberTest)
  EVT_MENU(SETOPTDIR, GuiFrame::SetOptionDirectory)
  EVT_NOTEBOOK_PAGE_CHANGING(CHANGETAB, GuiFrame::OnTabChange)
  EVT_TEXT(DEVID, GuiFrame::EnterDevice)
  EVT_TEXT(DEVOPT, GuiFrame::EnterDeviceOption)
  EVT_TEXT(IDEVID, GuiFrame::EnterInputDevice)
  EVT_TEXT(IDEVOPT, GuiFrame::EnterInputDeviceOption)
  EVT_TEXT(MONID, GuiFrame::EnterMonitor)
  EVT_TEXT(MONOPT, GuiFrame::EnterMonitorOption)
  EVT_TEXT(MONMODL, GuiFrame::EnterMonitorModeline)
  EVT_TEXT(SCRID, GuiFrame::EnterScreen)
  EVT_TEXT(SCROPT, GuiFrame::EnterScreenOption)
  EVT_TEXT(SCRDISP, GuiFrame::EnterScreenDiplay)
  EVT_TEXT(SCRDISPMOD, GuiFrame::EnterScreenDiplayMode)
  EVT_TEXT(SCRDISPOPT, GuiFrame::EnterScreenDiplayOption)
  EVT_TEXT(SERLID, GuiFrame::EnterServerLayout)
  EVT_TEXT(SERLOPT, GuiFrame::EnterServerLayoutOption)
  EVT_TEXT(SERLIDEVOPT, GuiFrame::EnterServerLayoutInputDeviceOption)
  EVT_TEXT(SERFOPT, GuiFrame::EnterServerFlagOption)
  EVT_TEXT(EXTOPT, GuiFrame::EnterExtension)
  EVT_TEXT(FONTPATH, GuiFrame::EnterFontPath)
  EVT_TEXT(MODPATH, GuiFrame::EnterModulePath)
  EVT_TEXT(RGBPATH, GuiFrame::EnterRgbPath)
  EVT_TEXT(MODID, GuiFrame::EnterModule)
  EVT_TEXT(MODOPT, GuiFrame::EnterModuleOption)
  EVT_TEXT(DEVDRI, GuiFrame::EnterDeviceDriver)
  EVT_TEXT(DEVBUS, GuiFrame::EnterDeviceBusID)
  EVT_TEXT(DEVSCR, GuiFrame::EnterDeviceScreen)
  EVT_TEXT(DEVRAM, GuiFrame::EnterDevicVideoRam)
  EVT_TEXT(DEVOVAL, GuiFrame::EnterDeviceOptValue)
  EVT_TEXT(MONHSY, GuiFrame::EnterMonitorHorizSync)
  EVT_TEXT(MONVREF, GuiFrame::EnterMonitorVertRefresh)
  EVT_TEXT(MONDISP, GuiFrame::EnterMonitorDisplay)
  EVT_TEXT(MONGAM, GuiFrame::EnterMonitorGamma)
  EVT_TEXT(MONOVAL, GuiFrame::EnterMonitorOptValue)
  EVT_TEXT(MONMODLOPT, GuiFrame::EnterMonitorModelineOption)
  EVT_TEXT(IDEVDRI, GuiFrame::EnterInputDeviceDriver)
  EVT_TEXT(IDEVOVAL, GuiFrame::EnterInputDeviceOptValue)
  EVT_TEXT(SCRDEP, GuiFrame::EnterScreenDepth)
  EVT_TEXT(SCROVAL, GuiFrame::EnterScreenOptValue)
  EVT_TEXT(SCRDISPVP, GuiFrame::EnterScreenDisplayVirtual)
  EVT_TEXT(SCRDISPVIR, GuiFrame::EnterScreenDisplayViewport)
  EVT_TEXT(SCRDISPOVAL, GuiFrame::EnterScreenDisplayOptValue)
  EVT_TEXT(SERLSCRNUM, GuiFrame::EnterServerLayoutScreenNumber)
  EVT_TEXT(SERLSCRPOS, GuiFrame::EnterServerLayoutScreenPosition)
  EVT_TEXT(SERLOVAL, GuiFrame::EnterServerLayoutOptValue)
  EVT_TEXT(SERFOVAL, GuiFrame::EnterServerFlagsOptValue)
  EVT_TEXT(EXTOVAL, GuiFrame::EnterExtensionOptValue)
  EVT_TEXT(MODOVAL, GuiFrame::EnterModuleOptValue)
  EVT_TEXT(DRIMOD, GuiFrame::EnterDriMode)
  EVT_CHOICE(SCRDEV, GuiFrame::SelectScreenDevice)
  EVT_CHOICE(SCRMON, GuiFrame::SelectScreenMonitor)
  EVT_CHOICE(SERLSCR1, GuiFrame::SelectServerLayoutScreen1)
  EVT_CHOICE(SERLSCR2, GuiFrame::SelectServerLayoutScreen2)
  EVT_CHOICE(SERLIDEV1, GuiFrame::SelectServerLayoutInputDevice1)
  EVT_COMBOBOX(DEVID, GuiFrame::SelectDevice)
  EVT_COMBOBOX(DEVOPT, GuiFrame::SelectDeviceOption)
  EVT_COMBOBOX(IDEVID, GuiFrame::SelectInputDevice)
  EVT_COMBOBOX(IDEVOPT, GuiFrame::SelectInputDeviceOption)
  EVT_COMBOBOX(MONID, GuiFrame::SelectMonitor)
  EVT_COMBOBOX(MONOPT, GuiFrame::SelectMonitorOption)
  EVT_COMBOBOX(MONMODL, GuiFrame::SelectMonitorModeline)
  EVT_COMBOBOX(SCRID, GuiFrame::SelectScreen)
  EVT_COMBOBOX(SCROPT, GuiFrame::SelectScreenOption)
  EVT_COMBOBOX(SCRDISP, GuiFrame::SelectScreenDiplay)
  EVT_COMBOBOX(SCRDISPMOD, GuiFrame::SelectScreenDiplayMode)
  EVT_COMBOBOX(SCRDISPOPT, GuiFrame::SelectScreenDiplayOption)
  EVT_COMBOBOX(SERLID, GuiFrame::SelectServerLayout)
  EVT_COMBOBOX(SERLOPT, GuiFrame::SelectServerLayoutOption)
  EVT_COMBOBOX(SERLSCR, GuiFrame::SelectServerLayoutScreen)
  EVT_COMBOBOX(SERLIDEV, GuiFrame::SelectServerLayoutInputDevice)
  EVT_COMBOBOX(SERLIDEVOPT, GuiFrame::SelectServerLayoutInputDeviceOption)
  EVT_COMBOBOX(SERFOPT, GuiFrame::SelectServerFlagOption)
  EVT_COMBOBOX(EXTOPT, GuiFrame::SelectExtension)
  EVT_COMBOBOX(FONTPATH, GuiFrame::SelectFontPath)
  EVT_COMBOBOX(MODPATH, GuiFrame::SelectModulePath)
  EVT_COMBOBOX(RGBPATH, GuiFrame::SelectRgbPath)
  EVT_COMBOBOX(MODID, GuiFrame::SelectModule)
  EVT_COMBOBOX(MODOPT, GuiFrame::SelectModuleOption)
  EVT_BUTTON(MONFREQ, GuiFrame::GetRefreshRate)
  EVT_BUTTON(MONDISPSIZE, GuiFrame::GetDisplaySize)
  EVT_BUTTON(MONMODLGEN, GuiFrame::GenerateModeline)
END_EVENT_TABLE()

// **************************
// Class Declaration
// **************************

// GuiFrame::GuiFrame(const wxString& title, wxLocale *lang) : wxFrame(NULL, -1, title, wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE & ~ (wxRESIZE_BORDER | wxRESIZE_BOX | wxMAXIMIZE_BOX)), app_locale(lang)
GuiFrame::GuiFrame(const wxString& title) : wxFrame(NULL, -1, title, wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE & ~ (wxRESIZE_BORDER | wxRESIZE_BOX | wxMAXIMIZE_BOX))
{
  // set constants
  optListSize=4;
  devListSize=4;

  // set title
  version=_T("xorg-edit v06.10.08");

  // set all entries to null

  fontPathCurrent=0;
  moduleCurrent=0;
  serverLayoutCurrent=0;
  inputDeviceCurrent=0;
  deviceCurrent=0;
  monitorCurrent=0;
  screenCurrent=0;
  serverFlagCurrent=0;
  extensionCurrent=0;
  deviceOptionCurrent=0;
  monitorOptionCurrent=0;
  monitorModelineCurrent=0;
  screenOptionCurrent=0;
  screenDisplayCurrent=0;
  screenDisplayModeCurrent=0;
  screenDisplayOptionCurrent=0;
  inputDeviceOptionCurrent=0;
  moduleOptionCurrent=0;
  serverLayoutOptionCurrent=0;
  serverLayoutIDevOptionCurrent=0;
  serverLayoutScreenCurrent=0;
  serverLayoutIDevCurrent=0;

  // layout for the whole frame
  wxBoxSizer *gridSizer = new wxBoxSizer(wxVERTICAL);

  layout = new wxNotebook(this, CHANGETAB);

  deviceTab = new wxPanel(layout);                       // device tab panel
  wxBoxSizer *deviceTabSizer=new wxBoxSizer(wxVERTICAL);

  monitorTab = new wxPanel(layout);                       // monitor tab panel
  wxBoxSizer *monitorTabSizer=new wxBoxSizer(wxVERTICAL);

  inputDeviceTab = new wxPanel(layout);                       // inputdevice tab panel
  wxBoxSizer *inputDeviceTabSizer=new wxBoxSizer(wxVERTICAL);

  screenTab = new wxPanel(layout);                       // screen tab panel
  wxBoxSizer *screenTabSizer=new wxBoxSizer(wxVERTICAL);

  serverTab = new wxPanel(layout);                       // server layout/server flags tab tab panel
  wxBoxSizer *serverTabSizer=new wxBoxSizer(wxVERTICAL);

  otherSectionTab = new wxPanel(layout);                       // iles/modules/extensions/dri tab panel
  wxBoxSizer *otherSectionTabSizer=new wxBoxSizer(wxVERTICAL);

  wxString tempList[devListSize];  // initial list for combo und choice elements
  wxString optList[optListSize];   // initial list for option combo und choice elements

  optList[0]=_("----------");
  optList[1]=_("Add Option");
  optList[2]=_("Delete Option");
  optList[3]=_("De/Activate Option");

  // section DEVICE elements
  deviceBox=new wxStaticBox(deviceTab, -1, _("Device"));
  wxStaticBoxSizer *deviceSizer=new wxStaticBoxSizer(deviceBox, wxVERTICAL);
  wxBoxSizer *deviceSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *deviceSizerLine2=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *deviceSizerLine3=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *deviceSizerLine4=new wxBoxSizer(wxHORIZONTAL);

  deviceDriverText=new wxStaticText(deviceTab, -1, _("Driver"));
  deviceBusIDText=new wxStaticText(deviceTab, -1, _("BusID"));
  deviceVideoRamText=new wxStaticText(deviceTab, -1, _("Video Ram"));
  deviceScreenNumberText=new wxStaticText(deviceTab, -1, _("Screen Number"));
  deviceOptionText=new wxStaticText(deviceTab, -1, _("Option"));

  tempList[0]=_("----------");
  tempList[1]=_("Add Device");
  tempList[2]=_("Delete Device");
  tempList[3]=_("Copy Device");

  deviceIdentifier=new wxComboBox(deviceTab, DEVID, _("----------"), wxDefaultPosition, wxSize(450,-1), devListSize, tempList, wxCB_DROPDOWN);

  deviceSizerLine1->Add(deviceIdentifier, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  // deviceDriver=new wxTextCtrl(deviceTab, DEVDRI, wxEmptyString, wxDefaultPosition, wxSize(140,-1));
  deviceDriver=new wxComboBox(deviceTab, DEVDRI, wxEmptyString, wxDefaultPosition, wxSize(140,-1), 0, tempList, wxCB_DROPDOWN | wxCB_SORT);
  deviceBusID=new wxTextCtrl(deviceTab, DEVBUS, wxEmptyString, wxDefaultPosition, wxSize(140,-1));

  deviceSizerLine2->Add(deviceDriverText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  deviceSizerLine2->Add(deviceDriver, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  deviceSizerLine2->Add(deviceBusIDText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT, 10);
  deviceSizerLine2->Add(deviceBusID, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  deviceVideoRam=new wxTextCtrl(deviceTab, DEVRAM, wxEmptyString, wxDefaultPosition, wxSize(140,-1));
  deviceScreen=new wxTextCtrl(deviceTab, DEVSCR, wxEmptyString, wxDefaultPosition, wxSize(100,-1));

  deviceSizerLine3->Add(deviceVideoRamText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  deviceSizerLine3->Add(deviceVideoRam, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  deviceSizerLine3->Add(deviceScreenNumberText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT, 10);
  deviceSizerLine3->Add(deviceScreen, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  deviceOption=new wxComboBox(deviceTab, DEVOPT, _("----------"), wxDefaultPosition, wxSize(285,-1), optListSize, optList, wxCB_DROPDOWN);
  // deviceOptValue=new wxTextCtrl(deviceTab, DEVOVAL, wxEmptyString, wxDefaultPosition, wxSize(110,-1));
  deviceOptValue=new wxComboBox(deviceTab, DEVOVAL, wxEmptyString, wxDefaultPosition, wxSize(110,-1), 0, tempList, wxCB_DROPDOWN | wxCB_SORT);

  deviceSizerLine4->Add(deviceOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  deviceSizerLine4->Add(deviceOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  deviceSizerLine4->Add(deviceOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  deviceSizer->Add(deviceSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  deviceSizer->Add(deviceSizerLine2, 0, wxTOP | wxBOTTOM, 3);
  deviceSizer->Add(deviceSizerLine3, 0, wxTOP | wxBOTTOM, 3);
  deviceSizer->Add(deviceSizerLine4, 0, wxTOP | wxBOTTOM, 3);

  // section MONITOR elements
  monitorBox=new wxStaticBox(monitorTab, -1, _("Monitor"));
  wxStaticBoxSizer *monitorSizer=new wxStaticBoxSizer(monitorBox, wxVERTICAL);
  wxBoxSizer *monitorSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *monitorSizerLine2=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *monitorSizerLine3=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *monitorSizerLine4=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *monitorSizerLine5=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *monitorSizerLine6=new wxBoxSizer(wxHORIZONTAL);

  monitorHorizSyncText=new wxStaticText(monitorTab, -1, _("Hor.Sync."));
  monitorVertRefreshText=new wxStaticText(monitorTab, -1, _("Vert.Refresh"));
  monitorDisplayText=new wxStaticText(monitorTab, -1, _("Display Size"));
  monitorGammaText=new wxStaticText(monitorTab, -1, _("Gamma"));
  monitorOptionText=new wxStaticText(monitorTab, -1, _("Option"));
  monitorModelineText=new wxStaticText(monitorTab, -1, _("Modeline"));

  tempList[1]=_("Add Monitor");
  tempList[2]=_("Delete Monitor");
  tempList[3]=_("Copy Monitor");

  monitorIdentifier=new wxComboBox(monitorTab, MONID, _("----------"), wxDefaultPosition, wxSize(450,-1), devListSize, tempList, wxCB_DROPDOWN);

  monitorSizerLine1->Add(monitorIdentifier, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  monitorHorizSync=new wxTextCtrl(monitorTab, MONHSY, wxEmptyString, wxDefaultPosition, wxSize(75,-1));
  monitorVertRefresh=new wxTextCtrl(monitorTab, MONVREF, wxEmptyString, wxDefaultPosition, wxSize(75,-1));
  monitorFreqButton=new wxButton(monitorTab, MONFREQ, _("Get Refresh Rate"));

  monitorSizerLine2->Add(monitorHorizSyncText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine2->Add(monitorHorizSync, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine2->Add(monitorVertRefreshText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT, 10);
  monitorSizerLine2->Add(monitorVertRefresh, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine2->Add(monitorFreqButton, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  monitorDisplay=new wxTextCtrl(monitorTab, MONDISP, wxEmptyString, wxDefaultPosition, wxSize(150,-1));
  monitorDisplaySizeButton=new wxButton(monitorTab, MONDISPSIZE, _("Get Display Size"));

  monitorSizerLine3->Add(monitorDisplayText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine3->Add(monitorDisplay, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine3->Add(monitorDisplaySizeButton, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  monitorGamma=new wxTextCtrl(monitorTab, MONGAM, wxEmptyString, wxDefaultPosition, wxSize(120,-1));

  monitorSizerLine4->Add(monitorGammaText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT, 10);
  monitorSizerLine4->Add(monitorGamma, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  tempList[1]=_("Add Modeline");
  tempList[2]=_("Delete Modeline");
  tempList[3]=_("De/Activate Modeline");

  monitorModelineName=new wxComboBox(monitorTab, MONMODL, _("----------"), wxDefaultPosition, wxSize(145,-1), devListSize, tempList, wxCB_DROPDOWN);
  monitorModelineOption=new wxTextCtrl(monitorTab, MONMODLOPT, wxEmptyString, wxDefaultPosition, wxSize(235,-1));

  monitorSizerLine5->Add(monitorModelineText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine5->Add(monitorModelineName, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine5->Add(monitorModelineOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  monitorOption=new wxComboBox(monitorTab, MONOPT, _("----------"), wxDefaultPosition, wxSize(285,-1), optListSize, optList, wxCB_DROPDOWN);
  monitorOptValue=new wxTextCtrl(monitorTab, MONOVAL, wxEmptyString, wxDefaultPosition, wxSize(110,-1));

  monitorSizerLine6->Add(monitorOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine6->Add(monitorOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine6->Add(monitorOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  monitorSizer->Add(monitorSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  monitorSizer->Add(monitorSizerLine2, 0, wxTOP | wxBOTTOM, 3);
  monitorSizer->Add(monitorSizerLine3, 0, wxTOP | wxBOTTOM, 3);
  monitorSizer->Add(monitorSizerLine4, 0, wxTOP | wxBOTTOM, 3);
  monitorSizer->Add(monitorSizerLine5, 0, wxTOP | wxBOTTOM, 3);
  monitorSizer->Add(monitorSizerLine6, 0, wxTOP | wxBOTTOM, 3);

  monitorModelineGeneratorBox=new wxStaticBox(monitorTab, -1, _("Modeline Generator"));
  wxStaticBoxSizer *monitorModelineGeneratorSizer=new wxStaticBoxSizer(monitorModelineGeneratorBox, wxVERTICAL);
  wxBoxSizer *monitorModelineGeneratorSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *monitorModelineGeneratorSizerLine2=new wxBoxSizer(wxHORIZONTAL);

  monitorModelineGeneratorXResText=new wxStaticText(monitorTab, -1, _("X"));
  monitorModelineGeneratorYResText=new wxStaticText(monitorTab, -1, _("Y"));
  monitorModelineGeneratorFreqText=new wxStaticText(monitorTab, -1, _("Refresh"));

  monitorModelineGeneratorXRes=new wxTextCtrl(monitorTab, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(90,-1));
  monitorModelineGeneratorYRes=new wxTextCtrl(monitorTab, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(90,-1));
  monitorModelineGeneratorFreq=new wxTextCtrl(monitorTab, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(70,-1));
  monitorModelineGeneratorButton=new wxButton(monitorTab, MONMODLGEN, _("Generate"));

  monitorModelineGeneratorSizerLine1->Add(monitorModelineGeneratorXResText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorModelineGeneratorSizerLine1->Add(monitorModelineGeneratorXRes, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorModelineGeneratorSizerLine1->Add(monitorModelineGeneratorYResText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorModelineGeneratorSizerLine1->Add(monitorModelineGeneratorYRes, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorModelineGeneratorSizerLine1->Add(monitorModelineGeneratorFreqText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorModelineGeneratorSizerLine1->Add(monitorModelineGeneratorFreq, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorModelineGeneratorSizerLine1->Add(monitorModelineGeneratorButton, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  monitorModelineGeneratorOutput1=new wxTextCtrl(monitorTab, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(120,-1), wxTE_READONLY);
  monitorModelineGeneratorOutput2=new wxTextCtrl(monitorTab, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(320,-1), wxTE_READONLY);

  monitorModelineGeneratorSizerLine2->Add(monitorModelineGeneratorOutput1, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorModelineGeneratorSizerLine2->Add(monitorModelineGeneratorOutput2, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  monitorModelineGeneratorSizer->Add(monitorModelineGeneratorSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  monitorModelineGeneratorSizer->Add(monitorModelineGeneratorSizerLine2, 0, wxTOP | wxBOTTOM, 3);

  // section INPUTDEVICE elements
  inputDeviceBox=new wxStaticBox(inputDeviceTab, -1, _("Input Device"));
  wxStaticBoxSizer *inputDeviceSizer=new wxStaticBoxSizer(inputDeviceBox, wxVERTICAL);
  wxBoxSizer *inputDeviceSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *inputDeviceSizerLine2=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *inputDeviceSizerLine3=new wxBoxSizer(wxHORIZONTAL);

  inputDeviceDriverText=new wxStaticText(inputDeviceTab, -1, _("Driver"));
  inputDeviceOptionText=new wxStaticText(inputDeviceTab, -1, _("Option"));

  tempList[1]=_("Add Input Device");
  tempList[2]=_("Delete Input Device");
  tempList[3]=_("Copy Input Device");

  inputDeviceIdentifier=new wxComboBox(inputDeviceTab, IDEVID, _("----------"), wxDefaultPosition, wxSize(450,-1), devListSize, tempList, wxCB_DROPDOWN);

  inputDeviceSizerLine1->Add(inputDeviceIdentifier, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  // inputDeviceDriver=new wxTextCtrl(inputDeviceTab, IDEVDRI, wxEmptyString, wxDefaultPosition, wxSize(140,-1));
  inputDeviceDriver=new wxComboBox(inputDeviceTab, IDEVDRI, wxEmptyString, wxDefaultPosition, wxSize(140,-1), 0, tempList, wxCB_DROPDOWN | wxCB_SORT);

  inputDeviceSizerLine2->Add(inputDeviceDriverText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  inputDeviceSizerLine2->Add(inputDeviceDriver, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  inputDeviceOption=new wxComboBox(inputDeviceTab, IDEVOPT, _("----------"), wxDefaultPosition, wxSize(285,-1), optListSize, optList, wxCB_DROPDOWN);
  // inputDeviceOptValue=new wxTextCtrl(inputDeviceTab, IDEVOVAL, wxEmptyString, wxDefaultPosition, wxSize(110,-1));
  inputDeviceOptValue=new wxComboBox(inputDeviceTab, IDEVOVAL, wxEmptyString, wxDefaultPosition, wxSize(110,-1), 0, tempList, wxCB_DROPDOWN | wxCB_SORT);

  inputDeviceSizerLine3->Add(inputDeviceOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  inputDeviceSizerLine3->Add(inputDeviceOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  inputDeviceSizerLine3->Add(inputDeviceOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  inputDeviceSizer->Add(inputDeviceSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  inputDeviceSizer->Add(inputDeviceSizerLine2, 0, wxTOP | wxBOTTOM, 3);
  inputDeviceSizer->Add(inputDeviceSizerLine3, 0, wxTOP | wxBOTTOM, 3);

  // section SCREEN elements
  screenBox=new wxStaticBox(screenTab, -1, _("Screen"));
  wxStaticBoxSizer *screenSizer=new wxStaticBoxSizer(screenBox, wxVERTICAL);
  wxBoxSizer *screenSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine2=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine3=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine4=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine5=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine6=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine7=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine8=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine9=new wxBoxSizer(wxHORIZONTAL);

  tempList[1]=_("Add Screen");
  tempList[2]=_("Delete Screen");
  tempList[3]=_("Copy Screen");

  screenDeviceText=new wxStaticText(screenTab, -1, _("Device"));
  screenMonitorText=new wxStaticText(screenTab, -1, _("Monitor"));
  screenDepthText=new wxStaticText(screenTab, -1, _("Default Depth"));
  screenOptionText=new wxStaticText(screenTab, -1, _("Option"));

  screenIdentifier=new wxComboBox(screenTab, SCRID, _("----------"), wxDefaultPosition, wxSize(450,-1), devListSize, tempList, wxCB_DROPDOWN);

  screenSizerLine1->Add(screenIdentifier, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenDevice=new wxChoice(screenTab, SCRDEV, wxDefaultPosition, wxSize(359,-1), 0, tempList);

  screenSizerLine2->Add(screenDeviceText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine2->Add(screenDevice, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenMonitor=new wxChoice(screenTab, SCRMON, wxDefaultPosition, wxSize(359,-1), 0, tempList);

  screenSizerLine3->Add(screenMonitorText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine3->Add(screenMonitor, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenDepth=new wxTextCtrl(screenTab, SCRDEP, wxEmptyString, wxDefaultPosition, wxSize(50,-1));

  screenSizerLine4->Add(screenDepthText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine4->Add(screenDepth, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenOption=new wxComboBox(screenTab, SCROPT, _("----------"), wxDefaultPosition, wxSize(285,-1), 4, optList, wxCB_DROPDOWN);
  screenOptValue=new wxTextCtrl(screenTab, SCROVAL, wxEmptyString, wxDefaultPosition, wxSize(110,-1));

  screenSizerLine5->Add(screenOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine5->Add(screenOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine5->Add(screenOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenDisplayDepthText=new wxStaticText(screenTab, -1, _("Display Depth"));
  screenDisplayViewportText=new wxStaticText(screenTab, -1, _("Viewport"));
  screenDisplayVirtualText=new wxStaticText(screenTab, -1, _("Virtual Size"));
  screenDisplayModesText=new wxStaticText(screenTab, -1, _("Mode"));
  screenDisplayOptionText=new wxStaticText(screenTab, -1, _("Option"));

  tempList[1]=_("Add Display");
  tempList[2]=_("Delete Display");

  screenDisplayDepth=new wxComboBox(screenTab, SCRDISP, _("----------"), wxDefaultPosition, wxSize(150,-1), devListSize-1, tempList, wxCB_DROPDOWN);

  screenSizerLine6->Add(screenDisplayDepthText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine6->Add(screenDisplayDepth, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenDisplayViewport=new wxTextCtrl(screenTab, SCRDISPVP, wxEmptyString, wxDefaultPosition, wxSize(110,-1));
  screenDisplayVirtual=new wxTextCtrl(screenTab, SCRDISPVIR, wxEmptyString, wxDefaultPosition, wxSize(110,-1));

  screenSizerLine7->Add(15,1);
  screenSizerLine7->Add(screenDisplayViewportText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine7->Add(screenDisplayViewport, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine7->Add(screenDisplayVirtualText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine7->Add(screenDisplayVirtual, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  tempList[1]=_("Add Mode");
  tempList[2]=_("Delete Mode");

  screenDisplayModes=new wxComboBox(screenTab, SCRDISPMOD, _("----------"), wxDefaultPosition, wxSize(180,-1), devListSize-1, tempList, wxCB_DROPDOWN);

  screenSizerLine8->Add(15,1);
  screenSizerLine8->Add(screenDisplayModesText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine8->Add(screenDisplayModes, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenDisplayOption=new wxComboBox(screenTab, SCRDISPOPT, _("----------"), wxDefaultPosition, wxSize(267,-1), optListSize, optList, wxCB_DROPDOWN);
  screenDisplayOptValue=new wxTextCtrl(screenTab, SCRDISPOVAL, wxEmptyString, wxDefaultPosition, wxSize(110,-1));

  screenSizerLine9->Add(15,1);
  screenSizerLine9->Add(screenDisplayOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine9->Add(screenDisplayOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine9->Add(screenDisplayOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenSizer->Add(screenSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine2, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine3, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine4, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine5, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine6, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine7, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine8, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine9, 0, wxTOP | wxBOTTOM, 3);

  // section SERVERLAYOUT elements
  serverLayoutBox=new wxStaticBox(serverTab, -1, _("Server Layout"));
  wxStaticBoxSizer *serverLayoutSizer=new wxStaticBoxSizer(serverLayoutBox, wxVERTICAL);
  wxBoxSizer *serverLayoutSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *serverLayoutSizerLine2=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *serverLayoutSizerLine3=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *serverLayoutSizerLine4=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *serverLayoutSizerLine5=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *serverLayoutSizerLine6=new wxBoxSizer(wxHORIZONTAL);

  serverLayoutScreenNumberText=new wxStaticText(serverTab, -1, _("Number"));
  serverLayoutScreenPositionText=new wxStaticText(serverTab, -1, _("Position"));
  serverLayoutOptionText=new wxStaticText(serverTab, -1, _("Option"));
  serverLayoutInputDeviceOptionText=new wxStaticText(serverTab, -1, _("Option"));

  tempList[1]=_("Add Server Layout");
  tempList[2]=_("Delete Server Layout");
  tempList[3]=_("Copy Server Layout");

  serverLayoutIdentifier=new wxComboBox(serverTab, SERLID, _("----------"), wxDefaultPosition, wxSize(450,-1), devListSize, tempList, wxCB_DROPDOWN);

  serverLayoutSizerLine1->Add(serverLayoutIdentifier, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  tempList[1]=_("Add Screen");
  tempList[2]=_("Delete Screen");

  serverLayoutScreen=new wxComboBox(serverTab, SERLSCR, _("----------"), wxDefaultPosition, wxSize(175,-1), devListSize-1, tempList, wxCB_DROPDOWN | wxCB_READONLY);
  serverLayoutScreenNumber=new wxTextCtrl(serverTab, SERLSCRNUM, wxEmptyString, wxDefaultPosition, wxSize(30,-1));
  serverLayoutScreen1=new wxChoice(serverTab, SERLSCR1, wxDefaultPosition, wxSize(170,-1), 0, tempList);

  serverLayoutSizerLine2->Add(serverLayoutScreen, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine2->Add(serverLayoutScreen1, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine2->Add(serverLayoutScreenNumberText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine2->Add(serverLayoutScreenNumber, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  serverLayoutScreenPosition=new wxTextCtrl(serverTab, SERLSCRPOS, wxEmptyString, wxDefaultPosition, wxSize(104,-1));
  serverLayoutScreen2=new wxChoice(serverTab, SERLSCR2, wxDefaultPosition, wxSize(170,-1), 0, tempList);

  serverLayoutSizerLine3->Add(15, 1);
  serverLayoutSizerLine3->Add(serverLayoutScreenPositionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine3->Add(serverLayoutScreenPosition, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine3->Add(serverLayoutScreen2, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  tempList[1]=_("Add Input Device");
  tempList[2]=_("Delete Input Device");

  serverLayoutInputDevice=new wxComboBox(serverTab, SERLIDEV, _("----------"), wxDefaultPosition, wxSize(175,-1), devListSize-1, tempList, wxCB_DROPDOWN | wxCB_READONLY);
  serverLayoutInputDevice1=new wxChoice(serverTab, SERLIDEV1, wxDefaultPosition, wxSize(170,-1), 3, tempList);

  serverLayoutSizerLine4->Add(serverLayoutInputDevice, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine4->Add(serverLayoutInputDevice1, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  serverLayoutInputDeviceOption=new wxComboBox(serverTab, SERLIDEVOPT, _("----------"), wxDefaultPosition, wxSize(285,-1), optListSize-1, optList, wxCB_DROPDOWN);
  // serverLayoutInputDeviceOptValue=new wxTextCtrl(serverTab, -1, wxEmptyString, wxDefaultPosition, wxSize(120,-1));

  serverLayoutSizerLine5->Add(15,1);
  serverLayoutSizerLine5->Add(serverLayoutInputDeviceOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine5->Add(serverLayoutInputDeviceOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  // serverLayoutSizerLine5->Add(serverLayoutInputDeviceOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  serverLayoutOption=new wxComboBox(serverTab, SERLOPT, _("----------"), wxDefaultPosition, wxSize(285,-1), optListSize-1, optList, wxCB_DROPDOWN);
  serverLayoutOptValue=new wxTextCtrl(serverTab, SERLOVAL, wxEmptyString, wxDefaultPosition, wxSize(110,-1));

  serverLayoutSizerLine6->Add(serverLayoutOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine6->Add(serverLayoutOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine6->Add(serverLayoutOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  serverLayoutSizer->Add(serverLayoutSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  serverLayoutSizer->Add(serverLayoutSizerLine6, 0, wxTOP | wxBOTTOM, 3);
  serverLayoutSizer->Add(serverLayoutSizerLine2, 0, wxTOP | wxBOTTOM, 3);
  serverLayoutSizer->Add(serverLayoutSizerLine3, 0, wxTOP | wxBOTTOM, 3);
  serverLayoutSizer->Add(serverLayoutSizerLine4, 0, wxTOP | wxBOTTOM, 3);
  serverLayoutSizer->Add(serverLayoutSizerLine5, 0, wxTOP | wxBOTTOM, 3);

  // section SERVERFLAGS elements
  serverFlagBox=new wxStaticBox(serverTab, -1, _("Server Flags"));
  wxStaticBoxSizer *serverFlagSizer=new wxStaticBoxSizer(serverFlagBox, wxVERTICAL);
  wxBoxSizer *serverFlagSizerLine1=new wxBoxSizer(wxHORIZONTAL);

  tempList[1]=_("Add Server Flag");
  tempList[2]=_("Delete Server Flag");
  tempList[3]=_("De/Activate Server Flag");

  serverFlagsOption=new wxComboBox(serverTab, SERFOPT, _("----------"), wxDefaultPosition, wxSize(334,-1), devListSize, tempList, wxCB_DROPDOWN);
  serverFlagsOptValue=new wxTextCtrl(serverTab, SERFOVAL, wxEmptyString, wxDefaultPosition, wxSize(110,-1));

  serverFlagSizerLine1->Add(serverFlagsOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverFlagSizerLine1->Add(serverFlagsOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  serverFlagSizer->Add(serverFlagSizerLine1, 0, wxTOP | wxBOTTOM, 3);

  // section EXTENSION elements
  extensionBox=new wxStaticBox(otherSectionTab, -1, _("Extensions"));
  wxStaticBoxSizer *extensionSizer=new wxStaticBoxSizer(extensionBox, wxVERTICAL);
  wxBoxSizer *extensionSizerLine1=new wxBoxSizer(wxHORIZONTAL);

  tempList[1]=_("Add Extension");
  tempList[2]=_("Delete Extension");
  tempList[3]=_("De/Activate Extension");

  extensionOption=new wxComboBox(otherSectionTab, EXTOPT, _("----------"), wxDefaultPosition, wxSize(334,-1), devListSize, tempList, wxCB_DROPDOWN);
  extensionOptValue=new wxTextCtrl(otherSectionTab, EXTOVAL, wxEmptyString, wxDefaultPosition, wxSize(110,-1));

  extensionSizerLine1->Add(extensionOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  extensionSizerLine1->Add(extensionOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  extensionSizer->Add(extensionSizerLine1, 0, wxTOP | wxBOTTOM, 3);

  // section MODULE elements
  moduleBox=new wxStaticBox(otherSectionTab, -1, _("Modules"));
  wxStaticBoxSizer *moduleSizer=new wxStaticBoxSizer(moduleBox, wxVERTICAL);
  wxBoxSizer *moduleSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *moduleSizerLine2=new wxBoxSizer(wxHORIZONTAL);

  tempList[1]=_("Add Module");
  tempList[2]=_("Delete Module");
  tempList[3]=_("De/Activate Module");

  moduleOptionText=new wxStaticText(otherSectionTab, -1, _("Option"));

  moduleIdentifier=new wxComboBox(otherSectionTab, MODID, _("----------"), wxDefaultPosition, wxSize(450,-1), devListSize, tempList, wxCB_DROPDOWN);

  moduleSizerLine1->Add(moduleIdentifier, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  moduleOption=new wxComboBox(otherSectionTab, MODOPT, _("----------"), wxDefaultPosition, wxSize(285,-1), optListSize, optList, wxCB_DROPDOWN);
  moduleOptValue=new wxTextCtrl(otherSectionTab, MODOVAL, wxEmptyString, wxDefaultPosition, wxSize(110,-1));

  moduleSizerLine2->Add(moduleOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  moduleSizerLine2->Add(moduleOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  moduleSizerLine2->Add(moduleOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  moduleSizer->Add(moduleSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  moduleSizer->Add(moduleSizerLine2, 0, wxTOP | wxBOTTOM, 3);

  // section FILES elements
  filesBox=new wxStaticBox(otherSectionTab, -1, _("Files"));
  wxStaticBoxSizer *fileSizer=new wxStaticBoxSizer(filesBox, wxVERTICAL);
  wxBoxSizer *fileSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *fileSizerLine2=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *fileSizerLine3=new wxBoxSizer(wxHORIZONTAL);

  tempList[1]=_("Add Font Path");
  tempList[2]=_("Delete Font Path");
  tempList[3]=_("De/Activate Font Path");

  fontPathText=new wxStaticText(otherSectionTab, -1, _("Font Path"));
  fontPath=new wxComboBox(otherSectionTab, FONTPATH, _("----------"), wxDefaultPosition, wxSize(375,-1), devListSize, tempList, wxCB_DROPDOWN);

  fileSizerLine1->Add(fontPathText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  fileSizerLine1->Add(fontPath, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  tempList[1]=_("Add Module Path");
  tempList[2]=_("Delete Module Path");
  tempList[3]=_("De/Activate Module Path");

  modulePathText=new wxStaticText(otherSectionTab, -1, _("Module Path"));
  modulePath=new wxComboBox(otherSectionTab, MODPATH, _("----------"), wxDefaultPosition, wxSize(367,-1), devListSize, tempList, wxCB_DROPDOWN);

  fileSizerLine2->Add(modulePathText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  fileSizerLine2->Add(modulePath, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  tempList[1]=_("Add RGB Path");
  tempList[2]=_("Delete RGB Path");
  tempList[3]=_("De/Activate RGB Path");

  rgbPathText=new wxStaticText(otherSectionTab, -1, _("RGB Path"));
  rgbPath=new wxComboBox(otherSectionTab, RGBPATH, _("----------"), wxDefaultPosition, wxSize(365,-1), devListSize, tempList, wxCB_DROPDOWN);

  fileSizerLine3->Add(rgbPathText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  fileSizerLine3->Add(rgbPath, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  fileSizer->Add(fileSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  fileSizer->Add(fileSizerLine2, 0, wxTOP | wxBOTTOM, 3);
  fileSizer->Add(fileSizerLine3, 0, wxTOP | wxBOTTOM, 3);

  // section DRI elements
  driBox=new wxStaticBox(otherSectionTab, -1, _("DRI"));
  wxStaticBoxSizer *driSizer=new wxStaticBoxSizer(driBox, wxVERTICAL);
  wxBoxSizer *driSizerLine1=new wxBoxSizer(wxHORIZONTAL);

  driMode=new wxTextCtrl(otherSectionTab, DRIMOD, wxEmptyString, wxDefaultPosition, wxSize(80,-1));

  driSizerLine1->Add(driMode, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  driSizer->Add(driSizerLine1, 0, wxTOP | wxBOTTOM, 3);

  // global sizer

  deviceTabSizer->Add(deviceSizer, 0, wxALL , 3);
  monitorTabSizer->Add(monitorSizer, 0, wxALL , 3);
  monitorTabSizer->Add(monitorModelineGeneratorSizer, 0, wxALL , 3);
  inputDeviceTabSizer->Add(inputDeviceSizer, 0, wxALL , 3);
  screenTabSizer->Add(screenSizer, 0, wxALL , 3);
  serverTabSizer->Add(serverLayoutSizer, 0, wxALL , 3);
  serverTabSizer->Add(serverFlagSizer, 0, wxALL , 3);
  otherSectionTabSizer->Add(extensionSizer, 0, wxALL , 3);
  otherSectionTabSizer->Add(moduleSizer, 0, wxALL , 3);
  otherSectionTabSizer->Add(fileSizer, 0, wxALL , 3);
  otherSectionTabSizer->Add(driSizer, 0, wxALL , 3);

  deviceTab->SetSizerAndFit(deviceTabSizer);
  monitorTab->SetSizerAndFit(monitorTabSizer);
  inputDeviceTab->SetSizerAndFit(inputDeviceTabSizer);
  screenTab->SetSizerAndFit(screenTabSizer);
  serverTab->SetSizerAndFit(serverTabSizer);
  otherSectionTab->SetSizerAndFit(otherSectionTabSizer);

  layout->AddPage(deviceTab, _("Device"));              // add device panel to the notebook
  layout->AddPage(monitorTab, _("Monitor"));              // add monitor panel to the notebook
  layout->AddPage(inputDeviceTab, _("Input Device"));         // add input device panel to the notebook
  layout->AddPage(screenTab, _("Screen"));               // add screen panel to the notebook
  layout->AddPage(serverTab, _("Server"));               // add server panel to the notebook
  layout->AddPage(otherSectionTab, _("Other Options"));              // add other options panel to the notebook

  warningText=new wxStaticText(this, -1, _("Warning: Improper use of this tool can cause damage to your\nxorg.conf and may not start your Xserver.\nYou must restart X to take the changes effect."),
                                            wxDefaultPosition, wxSize(480,-1), wxALIGN_CENTRE | wxST_NO_AUTORESIZE);

  gridSizer->Add(layout);            // add notebook layout
  gridSizer->Add(warningText, 0, wxTOP | wxBOTTOM | wxALIGN_CENTER_HORIZONTAL, 3);

  // Settings
  wxString filename;

  if (wxGetEnv(_T("HOME"), NULL))
  {  // HOME is defined
    filename=wxGetenv(_T("HOME"));
    filename+=_T("/.xorg-edit");
  }
  else
  {
    filename=wxGetCwd()+_T("/.xorg-edit");
  }
  settings.SetFilename(filename);

  settings.SetProgDir(wxGetCwd());

  if (!wxDirExists(wxGetCwd()+_T("/options")))  // option directory does not exists
  {
    settings.SetOptionDir(wxGetCwd());
  }
  else
  {
    settings.SetOptionDir(wxGetCwd()+_T("/options"));
  }

  // open xorg.conf at start if possible
  settings.SetCurrentDir(_T("/etc/X11"));
  settings.SetCurrentFilename(_T("xorg.conf"));
  settings.SetLoadLastFile(true);
  settings.SetSaveSettings(true);
  settings.SetShowSplash(true);
  settings.SetShowBlank(true);
  settings.SetRememberTest(true);
  settings.SetLanguage(wxGetApp().GetLanguage());

  // Load settings
  fileHandler.LoadSettings(settings);

  if (wxGetApp().GetLanguage()!=settings.GetLanguage())
  {
    wxGetApp().SetLanguage(settings.GetLanguage());
    // CreateGUIControls();
  }

  // Load driver lists
  LoadDrivers();

  CreateGUIControls();   // create all gui controls, like text fields, menus, buttons

  CreateStatusBar(); // create status bar
  SetSizerAndFit(gridSizer);         // sets the sizer

  if (!settings.GetLoadLastFile())
  {
    settings.SetCurrentDir(wxGetCwd());
    settings.SetCurrentFilename(wxEmptyString);
    EnableMenuItems(false);
  }
  else
  {
    if (xorg.Load(settings.GetCurrentFile()))
    {
      settings.SetCurrentDir(wxGetCwd());
      settings.SetCurrentFilename(wxEmptyString);
      EnableMenuItems(false);
    }
    else
    {
      EnableMenuItems(true);
    }
  }

  SetFrame();
  SetSaved(true);
}

GuiFrame::~GuiFrame()  // destructor
{
  // TODO: check if saved

  if (settings.GetSaveSettings())
  {
    fileHandler.SaveSettings(settings);
  }
}

void GuiFrame::EnableMenuItems(const bool enable)
{
  layout->Enable(enable);
  fileMenu->Enable(SAVEXORG, enable);
  fileMenu->Enable(SAVEAS, enable);
  fileMenu->Enable(CLOSEXORG, enable);
  fileMenu->Enable(RESTORE, enable);
  extrasMenu->Enable(SHOWXORG, enable);
  extrasMenu->Enable(TESTXORG, enable);
}

void GuiFrame::ShowWarning(void) const
{  // show warning message

  if (!settings.GetShowSplash()) return;  // do not show splash

  wxString line;

  line << _("Improper use of this tool can cause damage to your xorg.conf and may not start your Xserver. ")
       << _("You then won't have a mouse, menus or any other graphical feature to undo the changes. ")
       << _("Please see readme.txt, section \"Troubleshooting\" to get X up again _before_ this happens.\n\n")
       << _("The author is not responsible for any damage caused by using this tool!");
  wxMessageBox(line, _("WARNING"), wxICON_EXCLAMATION);

}

void GuiFrame::OnSaveSettings(wxCommandEvent &event)
{  // save settings

  fileHandler.SaveSettings(settings);
}

void GuiFrame::SetSaved(const bool save)
{  // set saved status and set title

  isSaved=save;
  if (isSaved==false)
  {
    if (!settings.GetCurrentFilename().IsEmpty()) SetTitle(settings.GetCurrentFile()+_T(" [NOT SAVED]"));
    else SetTitle(version);
  }
  else
  {
    if (!settings.GetCurrentFilename().IsEmpty()) SetTitle(settings.GetCurrentFile());
    else SetTitle(version);
  }
}

void GuiFrame::OnAbout(wxCommandEvent &event)
{  // show about dialog

  wxString info;
  info << version << _T("\n");
  info << _("written by") << _T(" Dominik Wagenfuehr\n");
  info << _("Licence") << _T(": GPL\n\n");
  info << _T("Web: http://www.cyskat.de/dee/progxorg.htm\n");
  info << _T("Wiki: http://wiki.ubuntuusers.de/xorg-edit\n\n");
  info << _T("Mail: dominik (dot) wagenfuehr (at) arcor (dot) de\n");
  wxMessageBox(info, _("About xorg-edit"));

}

void GuiFrame::OnNew(wxCommandEvent& event)
{  //create empty config file

  if (!isSaved)
  {
    if (wxMessageBox(_("File is not saved. Do you want to discard changes?"), _("Question: Discard Changes?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT) == wxNO)
    {
      return;
    }
  }

  ClearElements();
  SetFrame();
  EnableMenuItems(true);
  settings.SetCurrentDir(wxGetCwd());
  settings.SetCurrentFilename(_T("unknown"));
  SetSaved(false);
}

void GuiFrame::OnRestore(wxCommandEvent& event)
{  // load the current config file again

  if (!isSaved)
  {
    if (wxMessageBox(_("File is not saved. Do you want to discard changes?"), _("Question: Discard Changes?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT) == wxNO)
    {
      return;
    }
  }

  if (xorg.Load(settings.GetCurrentFile()))  // extract data
  {
    // EnableMenuItems(false);
  }
  else
  {
    EnableMenuItems(true);
    SetFrame();
    SetSaved(true);
  }
}

void GuiFrame::OnClose(wxCommandEvent& event)
{  // close current config

  if (!isSaved)
  {
    if (wxMessageBox(_("File is not saved. Do you want to discard changes?"), _("Question: Discard Changes?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT) == wxNO)
    {
      return;
    }
  }

  ClearElements();
  SetFrame();
  settings.SetCurrentFilename(wxEmptyString);
  SetSaved(true);

  EnableMenuItems(false);
}

void GuiFrame::OnLoad(wxCommandEvent& event)
{  // load other config file

  if (!isSaved)
  {
    if (wxMessageBox(_("File is not saved. Do you want to discard changes?"), _("Question: Discard Changes?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT) == wxNO)
    {
      return;
    }
  }

  // file dialog for opening files
  wxFileDialog openFileDialog(this, _("Choose a file"), settings.GetCurrentDir(), wxEmptyString, _T("*"), wxOPEN | wxCHANGE_DIR);

  if (openFileDialog.ShowModal() == wxID_OK)     // file has been chosen
  {
    settings.SetCurrentDir(openFileDialog.GetDirectory());
    settings.SetCurrentFilename(openFileDialog.GetFilename());
    if (xorg.Load(settings.GetCurrentFile()))  // extract data
    {
      EnableMenuItems(false);
      settings.SetCurrentFilename(wxEmptyString);
    }
    else
    {
      EnableMenuItems(true);
    }
    SetFrame();
    SetSaved(true);
  }

}

void GuiFrame::OnSave(wxCommandEvent& event)
{  // save xorg.conf

  if (settings.GetCurrentFilename().IsEmpty() || settings.GetCurrentFilename()==_T("unknown"))
  {
    OnSaveAs(event);
    return;
  }

  switch(layout->GetSelection())
  {
    case 0:  // DEVICE
      if (!CheckDeviceTab()) return;
      AcceptDevice();
      break;
    case 1:  //  MONITOR
      if (!CheckMonitorTab()) return;
      AcceptMonitor();
      break;
    case 2:  // INPUTDEVICE
      if (!CheckInputDeviceTab()) return;
      AcceptInputDevice();
      break;
    case 3:  // SCREEN
      if (!CheckScreenTab()) return;
      AcceptScreen();
      break;
    case 4:  // SERVERLAYOUT, SERVERFLAGS
      if (!CheckServerTab())  return;
      break;
    case 5:  // EXTENSIONS, MODULES, FILES, DRI
      if (!CheckOtherSectionTab()) return;
      break;
    default:
      break;
  } // switch

  if (settings.GetRememberTest())
  {
    wxString tempStr;
    tempStr << _("Do you want to test your configured file before saving? ");
    tempStr << _("In case of a misconfiguration the file won't be saved!");
    if (wxMessageBox(tempStr , _("Question: Test configuration?"), wxICON_QUESTION | wxYES_NO | wxYES_DEFAULT)==wxYES)
    {
      if (TestXorg()) return;
    }
  }

  if (!xorg.Save(settings.GetCurrentFile(), version)) SetSaved(true);

}

void GuiFrame::OnSaveAs(wxCommandEvent& event)
{  // save other config file

  switch(layout->GetSelection())
  {
    case 0:  // DEVICE
      if (!CheckDeviceTab()) return;
      AcceptDevice();
      break;
    case 1:  //  MONITOR
      if (!CheckMonitorTab()) return;
      AcceptMonitor();
      break;
    case 2:  // INPUTDEVICE
      if (!CheckInputDeviceTab()) return;
      AcceptInputDevice();
      break;
    case 3:  // SCREEN
      if (!CheckScreenTab()) return;
      AcceptScreen();
      break;
    case 4:  // SERVERLAYOUT, SERVERFLAGS
      if (!CheckServerTab())  return;
      break;
    case 5:  // EXTENSIONS, MODULES, FILES, DRI
      if (!CheckOtherSectionTab()) return;
      break;
    default:
      break;
  } // switch

  // file dialog for opening files
  wxFileDialog saveFileDialog(this, _("Choose a file"), wxEmptyString, wxEmptyString, wxEmptyString,  wxSAVE | wxOVERWRITE_PROMPT | wxCHANGE_DIR );

  if (saveFileDialog.ShowModal() == wxID_OK)     // file has been chosen
  {
    settings.SetCurrentDir(saveFileDialog.GetDirectory());
    settings.SetCurrentFilename(saveFileDialog.GetFilename());

    if (settings.GetRememberTest())
    {
      wxString tempStr;
      tempStr << _("Do you want to test your configured file before saving? ");
      tempStr << _("In case of a misconfiguration the file won't be saved!");
      if (wxMessageBox(tempStr , _("Question: Test configuration?"), wxICON_QUESTION | wxYES_NO | wxYES_DEFAULT)==wxYES)
      {
        if (TestXorg()) return;
      }
    }

    if (!xorg.Save(settings.GetCurrentFile(), version)) SetSaved(true);
  }

}

void GuiFrame::ClearElements(void)
{  // clear all elements

  xorg.Clear();

  fontPathCurrent=0;
  modulePathCurrent=0;
  rgbPathCurrent=0;

  moduleCurrent=0;
  moduleOptionCurrent=0;

  serverLayoutCurrent=0;
  serverLayoutOptionCurrent=0;
  serverLayoutIDevOptionCurrent=0;
  serverLayoutScreenCurrent=0;
  serverLayoutIDevCurrent=0;

  inputDeviceCurrent=0;
  inputDeviceOptionCurrent=0;

  deviceCurrent=0;
  deviceOptionCurrent=0;

  monitorCurrent=0;
  monitorOptionCurrent=0;
  monitorModelineCurrent=0;

  screenCurrent=0;
  screenOptionCurrent=0;
  screenDisplayCurrent=0;
  screenDisplayModeCurrent=0;
  screenDisplayOptionCurrent=0;

  serverFlagCurrent=0;

  extensionCurrent=0;
}

void GuiFrame::SetFrame(void)
{ // set all elements on screen after loading file

  UpdateDevice();
  UpdateMonitor();
  UpdateInputDevice();
  UpdateScreen();
  UpdateServerLayout();
  UpdateServerFlagOption();
  UpdateExtension();
  UpdateModule();
  UpdateFontPath();
  UpdateModulePath();
  UpdateRgbPath();
  UpdateDRI();
}

void GuiFrame::OnShow(wxCommandEvent& event)
{ // show config file

  if (!layout->IsEnabled()) return;

  switch(layout->GetSelection())
  {
    case 0:  // DEVICE
      // if (!CheckDeviceTab()) return;
      AcceptDevice();
      break;
    case 1:  //  MONITOR
      // if (!CheckMonitorTab()) return;
      AcceptMonitor();
      break;
    case 2:  // INPUTDEVICE
      // if (!CheckInputDeviceTab()) return;
      AcceptInputDevice();
      break;
    case 3:  // SCREEN
      // if (!CheckScreenTab()) return;
      AcceptScreen();
      break;
    case 4:  // SERVERLAYOUT, SERVERFLAGS
      // if (!CheckServerTab())  return;
      break;
    case 5:  // EXTENSIONS, MODULES, FILES, DRI
      // if (!CheckOtherSectionTab()) return;
      break;
    default:
      break;
  } // switch

  xorg.Preview(settings.GetCurrentFile(), version);
}

void GuiFrame::OnTest(wxCommandEvent& event)
{    // test xorg file

  TestXorg();
}

bool GuiFrame::TestXorg(void)
{    // test xorg file

  wxString tempFile=settings.GetCurrentFilename();     // store filename
  settings.SetCurrentFilename(_T("test_")+tempFile);   // set temporary filename
  if (xorg.Save(settings.GetCurrentFile(), version))
  {
    wxRemoveFile(_T("test_")+tempFile);      // delete temporary file
    settings.SetCurrentFilename(tempFile);
    return(true);  // write data into file without checking anything
  }

  if (settings.GetShowBlank())
  {
    wxMessageBox(_("I try to start a second XServer. The screen will go blank for a few seconds. Please be patient!"), _("Start XServer"), wxICON_INFORMATION);
  }

  wxArrayString output;  // output of x-test
  wxArrayString errors;  // output of x-test
  wxString command;
  int i;        // loop counter
  int display=1;         // start at xserver:1
  bool checked=false;
  TextFrame *xText=NULL;

  while(!checked)
  {
    if (display>=100)
    {  // checked 100 displays

      wxMessageBox(_("Sorry, no second XServer could be started."), _("No XServer!"), wxICON_ERROR);
      wxRemoveFile(_T("test_")+tempFile); // delete temporary file
      settings.SetCurrentFilename(tempFile);
      return(true);
    }

    checked=true;

    command.Printf(_T("X :%d"), display);
    command+=_T(" -probeonly -verbose 2 -config ");
    command+=_T("test_")+tempFile;

    if (wxExecute(command, output, errors, wxEXEC_SYNC)!=0)
    {
      // Check why the command could not be executed
      for (i=0; i<(int)errors.GetCount(); i++)
      {
        if (errors[i].Find(_T("Server is already active for display"))>=0)
        {
          display++;      // check another display
          checked=false;
          errors.Clear();
          break;  // leave for i-loop
        }

        if (errors[i].Find(_T("user not authorized to run the X server"))>=0)
        {

          command=_("Sorry, no second XServer could be started.");
          command+=_("\nIt seems that you don't have the rights to.");
          command+=_("\nDo you want to see the logfile?");

          if (wxMessageBox(command, _("No XServer!"), wxICON_ERROR | wxYES_NO | wxNO_DEFAULT)==wxYES)
          {

            xText = new TextFrame(this, _("Logfile"));  // create frame

            for (i=0; i<(int)errors.GetCount(); i++)
            {
              xText->AddLine(errors[i]);
            }

            xText->ShowModal();  // show dialog modal
            delete xText;        // delete again
            xText=NULL;
          }
          wxRemoveFile(_T("test_")+tempFile); // delete temporary file
          settings.SetCurrentFilename(tempFile);
          return(true);
        }

        if (errors[i].Find(_T("no screens found"))>=0)
        {
          command=_("You file seems to be misconfigured. Your graphical system won't start next time!");
          command+=_("\nDo you want to see the logfile?");

          if (wxMessageBox(command, _("Misconfigured File!"), wxICON_ERROR | wxYES_NO | wxNO_DEFAULT)==wxYES)
          {
            xText = new TextFrame(this, _("Logfile"));  // create frame

            for (i=0; i<(int)errors.GetCount(); i++)
            {
              xText->AddLine(errors[i]);
            }

            xText->ShowModal();  // show dialog modal
            delete xText;        // delete again
            xText=NULL;
          }
          wxRemoveFile(_T("test_")+tempFile); // delete temporary file
          settings.SetCurrentFilename(tempFile);
          return(true);
        }

      } // for i

      if (!checked)
      {
        // loop is through, another error
        command=_("Sorry, no second XServer could be started.");
        command+=_("\nDo you want to see the logfile?");

        if (wxMessageBox(command, _("No XServer!"), wxICON_ERROR | wxYES_NO | wxNO_DEFAULT)==wxYES)
        {
          xText = new TextFrame(this, _("Logfile"));  // create frame

          for (i=0; i<(int)errors.GetCount(); i++)
          {
            xText->AddLine(errors[i]);
          }

          xText->ShowModal();  // show dialog modal
          delete xText;        // delete again
          xText=NULL;
        }
        wxRemoveFile(_T("test_")+tempFile); // delete temporary file
        settings.SetCurrentFilename(tempFile);
        return(true);
      }

    }
  }

  command=_("The new configuration seems to work fine.");
  command+=_("\nDo you want to see the logfile anyway?");

  if (wxMessageBox(command, _("Everything's ok!"), wxICON_INFORMATION | wxYES_NO | wxNO_DEFAULT)==wxYES)
  {
    xText = new TextFrame(this, _("Logfile"));  // create frame

    for (i=0; i<(int)errors.GetCount(); i++)
    {
      xText->AddLine(errors[i]);
    }

    xText->ShowModal();  // show dialog modal
    delete xText;        // delete again
    xText=NULL;
  }

  // everything's fine
  wxRemoveFile(_T("test_")+tempFile); // delete temporary file
  settings.SetCurrentFilename(tempFile);

  return(false);
}

void GuiFrame::OnQuit(wxCommandEvent& event)
{  // quit program

  if (!isSaved)
  {
    if (wxMessageBox(_("File is not saved. Do you want to discard changes?"), _("Question: Discard Changes?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT) == wxNO)
    {
      return;
    }
  }

  Close();  // force closing window
}

void GuiFrame::SetSaveSettings(wxCommandEvent& event)
{    // set if settings should be saved

  settings.SetSaveSettings(event.IsChecked());
  fileHandler.SaveSettings(settings);
}

void GuiFrame::SetLoadLoastFile(wxCommandEvent& event)
{    // set if last file should be loaded

  settings.SetLoadLastFile(event.IsChecked());

}

void GuiFrame::SetShowSplash(wxCommandEvent& event)
{    // set if warning should be shown

  settings.SetShowSplash(event.IsChecked());

}

void GuiFrame::SetShowBlankTest(wxCommandEvent& event)
{    // set if warning should be shown

  settings.SetShowBlank(event.IsChecked());

}

void GuiFrame::SetRememberTest(wxCommandEvent& event)
{    // set if warning should be shown

  settings.SetRememberTest(event.IsChecked());

}

void GuiFrame::SetOptionDirectory(wxCommandEvent& event)
{    // set option directory

  // directory dialog
  wxDirDialog openDirDialog(this, _("Choose a directory"), settings.GetOptionDir());

  if (openDirDialog.ShowModal() == wxID_OK)     // directory has been chosen
  {

    if (!wxDirExists(openDirDialog.GetPath())) // option directory does not exists
    {  // fallback mode

      settings.SetOptionDir(settings.GetProgDir()+_T("/options"));

      if (!wxDirExists(settings.GetOptionDir()))  // option directory does not exists
      {
        settings.SetOptionDir(wxGetCwd());
      }
    }
    else
    {
      settings.SetOptionDir(openDirDialog.GetPath());
    }
  }

  CreateGUIControls();

}

void GuiFrame::SetLanguage(wxCommandEvent& event)
{    // set language

  if (event.GetId()==LANGENG && settings.GetLanguage()!=wxLANGUAGE_ENGLISH)
  {
    wxGetApp().SetLanguage(wxLANGUAGE_ENGLISH);
    settings.SetLanguage(wxLANGUAGE_ENGLISH);
    CreateGUIControls();
  }
  else if (event.GetId()==LANGGER && settings.GetLanguage()!=wxLANGUAGE_GERMAN)
  {
    wxGetApp().SetLanguage(wxLANGUAGE_GERMAN);
    settings.SetLanguage(wxLANGUAGE_GERMAN);
    CreateGUIControls();
  }
  else
  {
    wxGetApp().SetLanguage(wxLANGUAGE_DEFAULT);
    settings.SetLanguage(wxLANGUAGE_DEFAULT);
    CreateGUIControls();
  }
}

void GuiFrame::CreateGUIControls(void)
{  // create all gui controls, like text fields, menus, buttons

  int i;

  wxMenuBar *formerMenuBar=GetMenuBar();  // old menu bar

  fileMenu=new wxMenu;    // create file menu
  fileMenu->Append(NEWXORG, _("&New config file\tCtrl+N"), _("Create empty config file"));
  fileMenu->AppendSeparator();
  fileMenu->Append(LOADXORG, _("&Load ...\tCtrl+L"), _("Load config file"));
  fileMenu->Append(SAVEXORG, _("&Save\tCtrl+S"), _("Save config file"));
  fileMenu->Append(SAVEAS, _("Save as..."), _("Save config file under another name"));
  fileMenu->AppendSeparator();
  fileMenu->Append(RESTORE, _("Restore file"), _("Load the config file again"));
  fileMenu->AppendSeparator();
  fileMenu->Append(CLOSEXORG, _("&Close\tCtrl+Q"), _("Close current config file"));
  fileMenu->Append(QUITPROG, _("&Quit\tAlt+F4"), _("Quit the application"));

  wxMenu *langMenu = new wxMenu;  // language menu
  langMenu->AppendRadioItem(LANGENG, _("English"), _("Set language to English"));
  langMenu->AppendRadioItem(LANGGER, _("German"), _("Set language to German"));

  if (settings.GetLanguage()==wxLANGUAGE_ENGLISH)
  {
    langMenu->Check(LANGENG, true);
  }
  else if (settings.GetLanguage()==wxLANGUAGE_GERMAN)
  {
    langMenu->Check(LANGGER, true);
  }
  else
  {
    langMenu->Check(LANGENG, true);
  }

  wxMenu *settingsMenu = new wxMenu;  // settings menu
  settingsMenu->Append(SETOPTDIR, _("Set option directory: ")+settings.GetOptionDir(), _("Set directory where all driver options can be found."));
  settingsMenu->AppendSeparator();
  settingsMenu->AppendCheckItem(SAVESETEXIT, _("Save settings at exit"), _("Should the settings be saved at the end of the program?"));
  settingsMenu->AppendCheckItem(LOADLAST, _("Load last file at start"), _("Should the last edited file be opened at the begin?"));
  settingsMenu->AppendCheckItem(SHOWSPLASH, _("Show warning at start"), _("Should the warning screen be shown at every start?"));
  settingsMenu->AppendCheckItem(SHOWBLANK, _("Show blank message at test"), _("Should the message be displayed that the screen goes blank during test?"));
  settingsMenu->AppendCheckItem(REMTEST, _("Remember to test"), _("Should I remember you to test your configuration before saving?"));
  settingsMenu->AppendSeparator();
  settingsMenu->Append(SAVESET, _("Save settings now"), _("Save current settings."));

  settingsMenu->Check(SAVESETEXIT, settings.GetSaveSettings());
  settingsMenu->Check(LOADLAST, settings.GetLoadLastFile());
  settingsMenu->Check(SHOWSPLASH, settings.GetShowSplash());
  settingsMenu->Check(SHOWBLANK, settings.GetShowBlank());
  settingsMenu->Check(REMTEST, settings.GetRememberTest());

  extrasMenu=new wxMenu;    // create extras menu
  extrasMenu->Append(TESTXORG, _("&Test\tF7"), _("Test current settings"));
  extrasMenu->Append(SHOWXORG, _("&Preview...\tF5"), _("Show preview of written config file"));
  extrasMenu->AppendSeparator();
  extrasMenu->Append(-1, _("&Language"), langMenu);
  extrasMenu->Append(-1, _("&Settings"), settingsMenu);

  wxMenu *helpMenu = new wxMenu;  // help menu
  helpMenu->Append(ABOUTPROG, _("&About...\tF1"), _("Show about dialog"));

  wxMenuBar* mbar=new wxMenuBar;       // create menu bar
  mbar->Append(fileMenu, _("&File"));    // add menu item
  mbar->Append(extrasMenu, _("&Extras"));   // add menu item
  mbar->Append(helpMenu, _("&Help"));    // add menu item
  SetMenuBar(mbar);                      // set menu bar to this frame

  if (formerMenuBar) delete formerMenuBar;

  layout->SetPageText(0, _("Device"));
  layout->SetPageText(1, _("Monitor"));
  layout->SetPageText(2, _("Input Device"));
  layout->SetPageText(3, _("Screen"));
  layout->SetPageText(4, _("Server"));
  layout->SetPageText(5, _("Other Options"));

  warningText->SetLabel(_("Warning: Improper use of this tool can cause damage to your\nxorg.conf and may not start your Xserver.\nYou must restart X to take the changes effect."));

  wxString tempList[devListSize];  // initial list for combo und choice elements
  wxString optList[optListSize];   // initial list for option combo und choice elements

  optList[0]=_("----------");
  optList[1]=_("Add Option");
  optList[2]=_("Delete Option");
  optList[3]=_("De/Activate Option");

  tempList[0]=_("----------");
  tempList[1]=_("Add Device");
  tempList[2]=_("Delete Device");
  tempList[3]=_("Copy Device");

  for (i=0; i<devListSize; i++) deviceIdentifier->SetString(xorg.Devices().GetSize()+i, tempList[i]);
  if (xorg.Devices().GetSize()>0)
    for (i=0; i<optListSize; i++) deviceOption->SetString(xorg.Devices()[deviceCurrent].GetOptionSize()+i, optList[i]);
  else
    for (i=0; i<optListSize; i++) deviceOption->SetString(i, optList[i]);

  deviceBox->SetLabel( _("Device"));
  deviceDriverText->SetLabel(_("Driver"));
  deviceBusIDText->SetLabel(_("BusID"));
  deviceVideoRamText->SetLabel( _("Video Ram"));
  deviceScreenNumberText->SetLabel( _("Screen Number"));
  deviceOptionText->SetLabel(_("Option"));

  tempList[1]=_("Add Monitor");
  tempList[2]=_("Delete Monitor");
  tempList[3]=_("Copy Monitor");

  for (i=0; i<devListSize; i++) monitorIdentifier->SetString(xorg.Monitors().GetSize()+i, tempList[i]);
  if (xorg.Monitors().GetSize()>0)
    for (i=0; i<optListSize; i++) monitorOption->SetString(xorg.Monitors()[monitorCurrent].GetOptionSize()+i, optList[i]);
  else
    for (i=0; i<optListSize; i++) monitorOption->SetString(i, optList[i]);

  tempList[1]=_("Add Modeline");
  tempList[2]=_("Delete Modeline");
  tempList[3]=_("De/Activate Modeline");

  if (xorg.Monitors().GetSize()>0)
    for (i=0; i<devListSize; i++) monitorModelineName->SetString(xorg.Monitors()[monitorCurrent].GetModelineSize()+i, tempList[i]);
  else
    for (i=0; i<devListSize; i++) monitorModelineName->SetString(i, tempList[i]);

  monitorBox->SetLabel(_("Monitor"));
  monitorHorizSyncText->SetLabel( _("Hor.Sync."));
  monitorVertRefreshText->SetLabel(_("Vert.Refresh"));
  monitorDisplayText->SetLabel(_("Display Size"));
  monitorGammaText->SetLabel(_("Gamma"));
  monitorOptionText->SetLabel(_("Option"));
  monitorModelineText->SetLabel(_("Modeline"));
  monitorFreqButton->SetLabel(_("Get Refresh Rate"));
  monitorDisplaySizeButton->SetLabel(_("Get Display Size"));

  tempList[1]=_("Add Input Device");
  tempList[2]=_("Delete Input Device");
  tempList[3]=_("Copy Input Device");

  for (i=0; i<devListSize; i++) inputDeviceIdentifier->SetString(xorg.InputDevices().GetSize()+i, tempList[i]);
  if (xorg.InputDevices().GetSize()>0)
    for (i=0; i<optListSize; i++) inputDeviceOption->SetString(xorg.InputDevices()[inputDeviceCurrent].GetOptionSize()+i, optList[i]);
  else
    for (i=0; i<optListSize; i++) inputDeviceOption->SetString(i, optList[i]);

  inputDeviceBox->SetLabel(_("Input Device"));
  inputDeviceDriverText->SetLabel( _("Driver"));
  inputDeviceOptionText->SetLabel( _("Option"));

  tempList[1]=_("Add Screen");
  tempList[2]=_("Delete Screen");
  tempList[3]=_("Copy Screen");

  for (i=0; i<devListSize; i++) screenIdentifier->SetString(xorg.Screens().GetSize()+i, tempList[i]);
  if (xorg.Screens().GetSize()>0)
    for (i=0; i<optListSize; i++) screenOption->SetString(xorg.Screens()[screenCurrent].GetOptionSize()+i, optList[i]);
  else
    for (i=0; i<optListSize; i++) screenOption->SetString(i, optList[i]);

  tempList[1]=_("Add Display");
  tempList[2]=_("Delete Display");

  if (xorg.Screens().GetSize()>0)
    for (i=0; i<devListSize-1; i++) screenDisplayDepth->SetString(xorg.Screens()[screenCurrent].GetDisplaySize()+i, tempList[i]);
  else
    for (i=0; i<devListSize-1; i++) screenDisplayDepth->SetString(i, tempList[i]);

  tempList[1]=_("Add Mode");
  tempList[2]=_("Delete Mode");

  if (xorg.Screens().GetSize()>0 && xorg.Screens()[screenCurrent].GetDisplaySize()>0)
    screenDisplayViewportText->SetLabel(_("Viewport"));

  if (xorg.Screens().GetSize()>0 && xorg.Screens()[screenCurrent].GetDisplaySize()>0)
    screenDisplayVirtualText->SetLabel(_("Virtual Size"));

  if (xorg.Screens().GetSize()>0 && xorg.Screens()[screenCurrent].GetDisplaySize()>0)
    for (i=0; i<devListSize-1; i++) screenDisplayModes->SetString(xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetModeSize()+i, tempList[i]);
  else
    for (i=0; i<devListSize-1; i++) screenDisplayModes->SetString(i, tempList[i]);

  if (xorg.Screens().GetSize()>0 && xorg.Screens()[screenCurrent].GetDisplaySize()>0)
    for (i=0; i<optListSize; i++) screenDisplayOption->SetString(xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOptionSize()+i, optList[i]);
  else
    for (i=0; i<optListSize; i++) screenDisplayOption->SetString(i, optList[i]);

  screenBox->SetLabel( _("Screen"));
  screenDeviceText->SetLabel(_("Device"));
  screenMonitorText->SetLabel(_("Monitor"));
  screenDepthText->SetLabel( _("Default Depth"));
  screenOptionText->SetLabel( _("Option"));
  screenDisplayDepthText->SetLabel(_("Display Depth"));
  screenDisplayModesText->SetLabel(_("Mode"));
  screenDisplayOptionText->SetLabel(_("Option"));

  tempList[1]=_("Add Server Layout");
  tempList[2]=_("Delete Server Layout");
  tempList[3]=_("Copy Server Layout");

  for (i=0; i<devListSize; i++) serverLayoutIdentifier->SetString(xorg.ServerLayouts().GetSize()+i, tempList[i]);
  if (xorg.ServerLayouts().GetSize()>0)
    for (i=0; i<optListSize; i++) serverLayoutOption->SetString(xorg.ServerLayouts()[serverLayoutCurrent].GetOptionSize()+i, optList[i]);
  else
    for (i=0; i<optListSize; i++) serverLayoutOption->SetString(i, optList[i]);

  tempList[1]=_("Add Screen");
  tempList[2]=_("Delete Screen");

  if (xorg.ServerLayouts().GetSize()>0)
  {
    for (i=0; i<devListSize-1; i++) serverLayoutScreen->SetString(xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize()+i, tempList[i]);
    for (i=0; i<xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize(); i++)
    {
      wxString tempStr;
      tempStr << _("Screen") << _T(" ") << i;
      serverLayoutScreen->SetString(i, tempStr);
    }
    if (xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize()>0) serverLayoutScreen->SetValue(serverLayoutScreen->GetStringSelection());
  }
  else
    for (i=0; i<devListSize-1; i++) serverLayoutScreen->SetString(i, tempList[i]);

  tempList[1]=_("Add Input Device");
  tempList[2]=_("Delete Input Device");

  if (xorg.ServerLayouts().GetSize()>0)
  {
    for (i=0; i<devListSize-1; i++) serverLayoutInputDevice->SetString(xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize()+i, tempList[i]);
    for (i=0; i<xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize(); i++)
    {
      wxString tempStr;
      tempStr << _("Input Device") << _T(" ") << i;
      serverLayoutInputDevice->SetString(i, tempStr);
    }
    if (xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize()>0) serverLayoutInputDevice->SetValue(serverLayoutInputDevice->GetStringSelection());
  }
  else
    for (i=0; i<devListSize-1; i++) serverLayoutInputDevice->SetString(i, tempList[i]);

  if (xorg.ServerLayouts().GetSize()>0 && xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize()>0)
    for (i=0; i<optListSize-1; i++) serverLayoutInputDeviceOption->SetString(xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOptionSize(serverLayoutIDevCurrent)+i, optList[i]);
  else
    for (i=0; i<optListSize-1; i++) serverLayoutInputDeviceOption->SetString(i, optList[i]);

  serverLayoutBox->SetLabel(_("Server Layout"));
  serverLayoutScreenNumberText->SetLabel(_("Number"));
  serverLayoutScreenPositionText->SetLabel(_("Position"));
  serverLayoutOptionText->SetLabel(_("Option"));
  serverLayoutInputDeviceOptionText->SetLabel(_("Option"));

  tempList[1]=_("Add Server Flag");
  tempList[2]=_("Delete Server Flag");
  tempList[3]=_("De/Activate Server Flag");

  for (i=0; i<devListSize; i++) serverFlagsOption->SetString(xorg.ServerFlags().GetOptionSize()+i, tempList[i]);

  serverFlagBox->SetLabel(_("Server Flags"));

  tempList[1]=_("Add Extension");
  tempList[2]=_("Delete Extension");
  tempList[3]=_("De/Activate Extension");

  for (i=0; i<devListSize; i++) extensionOption->SetString(xorg.Extensions().GetOptionSize()+i, tempList[i]);

  extensionBox->SetLabel(_("Extensions"));

  tempList[1]=_("Add Module");
  tempList[2]=_("Delete Module");
  tempList[3]=_("De/Activate Module");

  for (i=0; i<devListSize; i++) moduleIdentifier->SetString(xorg.Modules().GetSize()+i, tempList[i]);
  if (xorg.Modules().GetSize()>0)
    for (i=0; i<optListSize; i++) moduleOption->SetString(xorg.Modules().GetOptionSize(moduleCurrent)+i, optList[i]);
  else
    for (i=0; i<optListSize; i++) moduleOption->SetString(i, optList[i]);

  moduleBox->SetLabel( _("Modules"));
  moduleOptionText->SetLabel( _("Option"));

  filesBox->SetLabel(_("Files"));

  tempList[1]=_("Add Font Path");
  tempList[2]=_("Delete Font Path");
  tempList[3]=_("De/Activate Font Path");

  for (i=0; i<devListSize; i++) fontPath->SetString(xorg.Files().GetFontPathSize()+i, tempList[i]);
  fontPathText->SetLabel(_("Font Path"));

  tempList[1]=_("Add Module Path");
  tempList[2]=_("Delete Module Path");
  tempList[3]=_("De/Activate Module Path");

  for (i=0; i<devListSize; i++) modulePath->SetString(xorg.Files().GetModulePathSize()+i, tempList[i]);
  modulePathText->SetLabel(_("Module Path"));

  tempList[1]=_("Add RGB Path");
  tempList[2]=_("Delete RGB Path");
  tempList[3]=_("De/Activate RGB Path");

  for (i=0; i<devListSize; i++) rgbPath->SetString(xorg.Files().GetRgbPathSize()+i, tempList[i]);
  rgbPathText->SetLabel(_("RGB Path"));

  driBox->SetLabel(_("DRI"));

}

///////////////////////////////////////////////////////

void GuiFrame::EnterDevice(wxCommandEvent& event)
{
  if (deviceIdentifier->GetSelection()==deviceCurrent && deviceIdentifier->GetSelection()<xorg.Devices().GetSize())
  {
    if (deviceIdentifierText!=deviceIdentifier->GetValue());
    {
      deviceIdentifierText=deviceIdentifier->GetValue();
      deviceIdentifier->SetString(deviceCurrent, deviceIdentifier->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterDeviceOption(wxCommandEvent& event)
{
  if (deviceOption->GetSelection()==deviceOptionCurrent && deviceOption->GetSelection()<xorg.Devices()[deviceCurrent].GetOptionSize())
  {
    if (xorg.Devices()[deviceCurrent].GetOption(deviceOptionCurrent).GetName()!=deviceOption->GetValue())
    {
      xorg.Devices()[deviceCurrent].GetOption(deviceOptionCurrent).SetName(deviceOption->GetValue());
      deviceOption->SetString(deviceOptionCurrent, deviceOption->GetValue());
      SetSaved(false);

      deviceOptValue->Clear();

      // set values
      wxString filename=settings.GetOptionDir()+_T("/devices/")+deviceDriver->GetValue()+_T(".xml");
      deviceDriversOption.Load(filename);

      // search select option in list
      int pos=deviceDriversOption.Find(deviceOption->GetValue());

      if (pos>=0)
      {
        if (deviceDriversOption[pos].GetValue()==_T("boolean"))
        {
          deviceOptValue->Append(_T("true"));
          deviceOptValue->Append(_T("false"));
        }
        else
        {
          // add arguments
          deviceOptValue->Append(deviceDriversOption[pos].GetArguments());
        }

        if (xorg.Devices()[deviceCurrent].GetOption(deviceOptionCurrent).GetValue().IsEmpty())
        {
          deviceOptValue->SetValue(deviceDriversOption[pos].GetDefault());
        }
        else deviceOptValue->SetValue(xorg.Devices()[deviceCurrent].GetOption(deviceOptionCurrent).GetValue());
      }
      else deviceOptValue->SetValue(xorg.Devices()[deviceCurrent].GetOption(deviceOptionCurrent).GetValue());
    }
  }
}

void GuiFrame::EnterInputDevice(wxCommandEvent& event)
{
  if (inputDeviceIdentifier->GetSelection()==inputDeviceCurrent && inputDeviceIdentifier->GetSelection()<xorg.InputDevices().GetSize())
  {
    if (inputDeviceIdentifierText!=inputDeviceIdentifier->GetValue());
    {
      inputDeviceIdentifierText=inputDeviceIdentifier->GetValue();
      inputDeviceIdentifier->SetString(inputDeviceCurrent, inputDeviceIdentifier->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterInputDeviceOption(wxCommandEvent& event)
{
  if (inputDeviceOption->GetSelection()==inputDeviceOptionCurrent && inputDeviceOption->GetSelection()<xorg.InputDevices()[inputDeviceCurrent].GetOptionSize())
  {
    if (xorg.InputDevices()[inputDeviceCurrent].GetOption(inputDeviceOptionCurrent).GetName()!=inputDeviceOption->GetValue())
    {
      xorg.InputDevices()[inputDeviceCurrent].GetOption(inputDeviceOptionCurrent).SetName(inputDeviceOption->GetValue());
      inputDeviceOption->SetString(inputDeviceOptionCurrent, inputDeviceOption->GetValue());
      SetSaved(false);

      deviceOptValue->Clear();

      // set values
      wxString filename=settings.GetOptionDir()+_T("/inputdevices/")+inputDeviceDriver->GetValue()+_T(".xml");
      inputDeviceDriversOption.Load(filename);

      // search select option in list
      int pos=inputDeviceDriversOption.Find(inputDeviceOption->GetValue());

      if (pos>=0)
      {
        if (inputDeviceDriversOption[pos].GetValue()==_T("boolean"))
        {
          inputDeviceOptValue->Append(_T("true"));
          inputDeviceOptValue->Append(_T("false"));
        }
        else
        {
          // add arguments
          inputDeviceOptValue->Append(inputDeviceDriversOption[pos].GetArguments());
        }

        if (xorg.InputDevices()[inputDeviceCurrent].GetOption(inputDeviceOptionCurrent).GetValue().IsEmpty())
        {
          inputDeviceOptValue->SetValue(inputDeviceDriversOption[pos].GetDefault());
        }
        else inputDeviceOptValue->SetValue(xorg.InputDevices()[inputDeviceCurrent].GetOption(inputDeviceOptionCurrent).GetValue());
      }
      else inputDeviceOptValue->SetValue(xorg.InputDevices()[inputDeviceCurrent].GetOption(inputDeviceOptionCurrent).GetValue());

    }
  }
}

void GuiFrame::EnterMonitor(wxCommandEvent& event)
{
  if (monitorIdentifier->GetSelection()==monitorCurrent && monitorIdentifier->GetSelection()<xorg.Monitors().GetSize())
  {
    if (monitorIdentifierText!=monitorIdentifier->GetValue());
    {
      monitorIdentifierText=monitorIdentifier->GetValue();
      monitorIdentifier->SetString(monitorCurrent, monitorIdentifier->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterMonitorOption(wxCommandEvent& event)
{
  if (monitorOption->GetSelection()==monitorOptionCurrent && monitorOption->GetSelection()<xorg.Monitors()[monitorCurrent].GetOptionSize())
  {
    if (xorg.Monitors()[monitorCurrent].GetOption(monitorOptionCurrent).GetName()!=monitorOption->GetValue())
    {
      xorg.Monitors()[monitorCurrent].GetOption(monitorOptionCurrent).SetName(monitorOption->GetValue());
      monitorOption->SetString(monitorOptionCurrent, monitorOption->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterMonitorModeline(wxCommandEvent& event)
{
  if (monitorModelineName->GetSelection()==monitorModelineCurrent && monitorModelineName->GetSelection()<xorg.Monitors()[monitorCurrent].GetModelineSize())
  {
    if (xorg.Monitors()[monitorCurrent].GetModeline(monitorModelineCurrent).GetName()!=monitorModelineName->GetValue())
    {
      xorg.Monitors()[monitorCurrent].GetModeline(monitorModelineCurrent).SetName(monitorModelineName->GetValue());
      monitorModelineName->SetString(monitorModelineCurrent, monitorModelineName->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterScreen(wxCommandEvent& event)
{
  if (screenIdentifier->GetSelection()==screenCurrent && screenIdentifier->GetSelection()<xorg.Screens().GetSize())
  {
    if (screenIdentifierText!=screenIdentifier->GetValue());
    {
      screenIdentifierText=screenIdentifier->GetValue();
      screenIdentifier->SetString(screenCurrent, screenIdentifier->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterScreenOption(wxCommandEvent& event)
{
  if (screenOption->GetSelection()==screenOptionCurrent && screenOption->GetSelection()<xorg.Screens()[screenCurrent].GetOptionSize())
  {
    if (xorg.Screens()[screenCurrent].GetOption(screenOptionCurrent).GetName()!=screenOption->GetValue())
    {
      xorg.Screens()[screenCurrent].GetOption(screenOptionCurrent).SetName(screenOption->GetValue());
      screenOption->SetString(screenOptionCurrent, screenOption->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterScreenDiplay(wxCommandEvent& event)
{
  if (screenDisplayDepth->GetSelection()==screenDisplayCurrent && screenDisplayDepth->GetSelection()<xorg.Screens()[screenCurrent].GetDisplaySize())
  {
    if (xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetDepth()!=screenDisplayDepth->GetValue())
    {
      xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).SetDepth(screenDisplayDepth->GetValue());
      screenDisplayDepth->SetString(screenDisplayCurrent, screenDisplayDepth->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterScreenDiplayMode(wxCommandEvent& event)
{
  if (screenDisplayModes->GetSelection()==screenDisplayModeCurrent && screenDisplayModes->GetSelection()<xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetModeSize())
  {
    if (xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetMode(screenDisplayModeCurrent)!=screenDisplayModes->GetValue())
    {
      xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).SetMode(screenDisplayModeCurrent, screenDisplayModes->GetValue());
      screenDisplayModes->SetString(screenDisplayModeCurrent, screenDisplayModes->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterScreenDiplayOption(wxCommandEvent& event)
{
  if (screenDisplayOption->GetSelection()==screenDisplayOptionCurrent && screenDisplayOption->GetSelection()<xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOptionSize())
  {
    if (xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOption(screenDisplayOptionCurrent).GetName()!=screenDisplayOption->GetValue())
    {
      xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOption(screenDisplayOptionCurrent).SetName(screenDisplayOption->GetValue());
      screenDisplayOption->SetString(screenDisplayOptionCurrent, screenDisplayOption->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterServerLayout(wxCommandEvent& event)
{
  if (serverLayoutIdentifier->GetSelection()==serverLayoutCurrent && serverLayoutIdentifier->GetSelection()<xorg.ServerLayouts().GetSize())
  {
    if (xorg.ServerLayouts()[serverLayoutCurrent].GetId()!=serverLayoutIdentifier->GetValue())
    {
      xorg.ServerLayouts()[serverLayoutCurrent].SetId(serverLayoutIdentifier->GetValue());
      serverLayoutIdentifier->SetString(serverLayoutCurrent, serverLayoutIdentifier->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterServerLayoutOption(wxCommandEvent& event)
{
  if (serverLayoutOption->GetSelection()==serverLayoutOptionCurrent && serverLayoutOption->GetSelection()<xorg.ServerLayouts()[serverLayoutCurrent].GetOptionSize())
  {
    if (xorg.ServerLayouts()[serverLayoutCurrent].GetOption(serverLayoutOptionCurrent).GetName()!=serverLayoutOption->GetValue())
    {
      xorg.ServerLayouts()[serverLayoutCurrent].GetOption(serverLayoutOptionCurrent).SetName(serverLayoutOption->GetValue());
      serverLayoutOption->SetString(serverLayoutOptionCurrent, serverLayoutOption->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterServerLayoutInputDeviceOption(wxCommandEvent& event)
{
  if (serverLayoutInputDeviceOption->GetSelection()==serverLayoutIDevOptionCurrent && serverLayoutInputDeviceOption->GetSelection()<xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOptionSize(serverLayoutIDevCurrent))
  {
    if (xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOption(serverLayoutIDevCurrent, serverLayoutIDevOptionCurrent).GetName()!=serverLayoutInputDeviceOption->GetValue())
    {
      // xorg.ServerLayouts()[serverLayoutCurrent].SetInputDeviceOption(serverLayoutIDevCurrent, serverLayoutIDevOptionCurrent, serverLayoutInputDeviceOption->GetValue(), serverLayoutInputDeviceOptValue->GetValue());
      xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOption(serverLayoutIDevCurrent, serverLayoutIDevOptionCurrent).SetName(serverLayoutInputDeviceOption->GetValue());
      serverLayoutInputDeviceOption->SetString(serverLayoutIDevOptionCurrent, serverLayoutInputDeviceOption->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterServerFlagOption(wxCommandEvent& event)
{
  if (serverFlagsOption->GetSelection()==serverFlagCurrent && serverFlagsOption->GetSelection()<xorg.ServerFlags().GetOptionSize())
  {
    if (xorg.ServerFlags().GetOption(serverFlagCurrent).GetName()!=serverFlagsOption->GetValue())
    {
      xorg.ServerFlags().GetOption(serverFlagCurrent).SetName(serverFlagsOption->GetValue());
      serverFlagsOption->SetString(serverFlagCurrent, serverFlagsOption->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterExtension(wxCommandEvent& event)
{
  if (extensionOption->GetSelection()==extensionCurrent && extensionOption->GetSelection()<xorg.Extensions().GetOptionSize())
  {
    if (xorg.Extensions().GetOption(extensionCurrent).GetName()!=extensionOption->GetValue())
    {
      xorg.Extensions().GetOption(extensionCurrent).SetName(extensionOption->GetValue());
      extensionOption->SetString(extensionCurrent, extensionOption->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterFontPath(wxCommandEvent& event)
{
  if (fontPath->GetSelection()==fontPathCurrent && fontPath->GetSelection()<xorg.Files().GetFontPathSize())
  {
    if (xorg.Files().GetFontPath(fontPathCurrent)!=fontPath->GetValue())
    {
      xorg.Files().SetFontPath(fontPathCurrent, fontPath->GetValue());
      fontPath->SetString(fontPathCurrent, fontPath->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterModulePath(wxCommandEvent& event)
{
  if (modulePath->GetSelection()==modulePathCurrent && modulePath->GetSelection()<xorg.Files().GetModulePathSize())
  {
    if (xorg.Files().GetModulePath(modulePathCurrent)!=modulePath->GetValue())
    {
      xorg.Files().SetModulePath(modulePathCurrent, modulePath->GetValue());
      modulePath->SetString(modulePathCurrent, modulePath->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterRgbPath(wxCommandEvent& event)
{
  if (rgbPath->GetSelection()==rgbPathCurrent && rgbPath->GetSelection()<xorg.Files().GetRgbPathSize())
  {
    if (xorg.Files().GetRgbPath(rgbPathCurrent)!=rgbPath->GetValue())
    {
      xorg.Files().SetRgbPath(rgbPathCurrent, rgbPath->GetValue());
      rgbPath->SetString(rgbPathCurrent, rgbPath->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterModule(wxCommandEvent& event)
{
  if (moduleIdentifier->GetSelection()==moduleCurrent && moduleIdentifier->GetSelection()<xorg.Modules().GetSize())
  {
    if (xorg.Modules().Get(moduleCurrent)!=moduleIdentifier->GetValue())
    {
      xorg.Modules().Set(moduleCurrent, moduleIdentifier->GetValue());
      moduleIdentifier->SetString(moduleCurrent, moduleIdentifier->GetValue());
      SetSaved(false);
    }
  }
}

void GuiFrame::EnterModuleOption(wxCommandEvent& event)
{
  if (moduleOption->GetSelection()==moduleOptionCurrent && moduleOption->GetSelection()<xorg.Modules().GetOptionSize(moduleCurrent))
  {
    if (xorg.Modules().GetOption(moduleCurrent, moduleOptionCurrent).GetName()!=moduleOption->GetValue())
    {
      xorg.Modules().GetOption(moduleCurrent, moduleOptionCurrent).SetName(moduleOption->GetValue());
      moduleOption->SetString(moduleOptionCurrent, moduleOption->GetValue());
      SetSaved(false);
    }
  }
}

// enter text fields

void GuiFrame::EnterDeviceDriver(wxCommandEvent& event)
{
  if (xorg.Devices().GetSize()==0) return;
  if (xorg.Devices()[deviceCurrent].GetDriver()!=deviceDriver->GetValue())
  {
    xorg.Devices()[deviceCurrent].SetDriver(deviceDriver->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterDeviceBusID(wxCommandEvent& event)
{
  if (xorg.Devices().GetSize()==0) return;
  if (xorg.Devices()[deviceCurrent].GetBus()!=deviceBusID->GetValue())
  {
    xorg.Devices()[deviceCurrent].SetBus(deviceBusID->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterDeviceScreen(wxCommandEvent& event)
{
  if (xorg.Devices().GetSize()==0) return;
  if (xorg.Devices()[deviceCurrent].GetScreen()!=deviceScreen->GetValue())
  {
    xorg.Devices()[deviceCurrent].SetScreen(deviceScreen->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterDevicVideoRam(wxCommandEvent& event)
{
  if (xorg.Devices().GetSize()==0) return;
  if (xorg.Devices()[deviceCurrent].GetVideoRam()!=deviceVideoRam->GetValue())
  {
    xorg.Devices()[deviceCurrent].SetVideoRam(deviceVideoRam->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterDeviceOptValue(wxCommandEvent& event)
{
  if (xorg.Devices().GetSize()==0 || xorg.Devices()[deviceCurrent].GetOptionSize()==0) return;
  if (xorg.Devices()[deviceCurrent].GetOption(deviceOptionCurrent).GetValue()!=deviceOptValue->GetValue())
  {
    xorg.Devices()[deviceCurrent].GetOption(deviceOptionCurrent).SetValue(deviceOptValue->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterMonitorHorizSync(wxCommandEvent& event)
{
  if (xorg.Monitors().GetSize()==0) return;
  if (xorg.Monitors()[monitorCurrent].GetHorSync()!=monitorHorizSync->GetValue())
  {
    xorg.Monitors()[monitorCurrent].SetHorSync(monitorHorizSync->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterMonitorVertRefresh(wxCommandEvent& event)
{
  if (xorg.Monitors().GetSize()==0) return;
  if (xorg.Monitors()[monitorCurrent].GetVertRef()!=monitorVertRefresh->GetValue())
  {
    xorg.Monitors()[monitorCurrent].SetVertRef(monitorVertRefresh->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterMonitorDisplay(wxCommandEvent& event)
{
  if (xorg.Monitors().GetSize()==0) return;
  if (xorg.Monitors()[monitorCurrent].GetDisplaySize()!=monitorDisplay->GetValue())
  {
    xorg.Monitors()[monitorCurrent].SetDisplaySize(monitorDisplay->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterMonitorGamma(wxCommandEvent& event)
{
  if (xorg.Monitors().GetSize()==0) return;
  if (xorg.Monitors()[monitorCurrent].GetGamma()!=monitorGamma->GetValue())
  {
    xorg.Monitors()[monitorCurrent].SetGamma(monitorGamma->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterMonitorOptValue(wxCommandEvent& event)
{
  if (xorg.Monitors().GetSize()==0 || xorg.Monitors()[monitorCurrent].GetOptionSize()==0) return;
  if (xorg.Monitors()[monitorCurrent].GetOption(monitorOptionCurrent).GetValue()!=monitorOptValue->GetValue())
  {
    xorg.Monitors()[monitorCurrent].GetOption(monitorOptionCurrent).SetValue(monitorOptValue->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterMonitorModelineOption(wxCommandEvent& event)
{
  if (xorg.Monitors().GetSize()==0 || xorg.Monitors()[monitorCurrent].GetModelineSize()==0) return;
  if (xorg.Monitors()[monitorCurrent].GetModeline(monitorModelineCurrent).GetValue()!=monitorModelineOption->GetValue())
  {
    xorg.Monitors()[monitorCurrent].GetModeline(monitorModelineCurrent).SetValue(monitorModelineOption->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterInputDeviceDriver(wxCommandEvent& event)
{
  if (xorg.InputDevices().GetSize()==0) return;
  if (xorg.InputDevices()[inputDeviceCurrent].GetDriver()!=inputDeviceDriver->GetValue())
  {
    xorg.InputDevices()[inputDeviceCurrent].SetDriver(inputDeviceDriver->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterInputDeviceOptValue(wxCommandEvent& event)
{
  if (xorg.InputDevices().GetSize()==0 || xorg.InputDevices()[inputDeviceCurrent].GetOptionSize()==0) return;
  if (xorg.InputDevices()[inputDeviceCurrent].GetOption(inputDeviceOptionCurrent).GetValue()!=inputDeviceOptValue->GetValue())
  {
    xorg.InputDevices()[inputDeviceCurrent].GetOption(inputDeviceOptionCurrent).SetValue(inputDeviceOptValue->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterScreenDepth(wxCommandEvent& event)
{
  if (xorg.Screens().GetSize()==0) return;
  if (xorg.Screens()[screenCurrent].GetDefaultDepth()!=screenDepth->GetValue())
  {
    xorg.Screens()[screenCurrent].SetDefaultDepth(screenDepth->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterScreenOptValue(wxCommandEvent& event)
{
  if (xorg.Screens().GetSize()==0 || xorg.Screens()[screenCurrent].GetOptionSize()==0) return;
  if (xorg.Screens()[screenCurrent].GetOption(screenOptionCurrent).GetValue()!=screenOptValue->GetValue())
  {
    xorg.Screens()[screenCurrent].GetOption(screenOptionCurrent).SetValue(screenOptValue->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterScreenDisplayViewport(wxCommandEvent& event)
{
  if (xorg.Screens().GetSize()==0 || xorg.Screens()[screenCurrent].GetDisplaySize()==0) return;
  if (xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetViewPort()!=screenDisplayViewport->GetValue())
  {
    xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).SetViewPort(screenDisplayViewport->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterScreenDisplayVirtual(wxCommandEvent& event)
{
  if (xorg.Screens().GetSize()==0 || xorg.Screens()[screenCurrent].GetDisplaySize()==0) return;
  if (xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetVirtual()!=screenDisplayVirtual->GetValue())
  {
    xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).SetVirtual(screenDisplayVirtual->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterScreenDisplayOptValue(wxCommandEvent& event)
{
  if (xorg.Screens().GetSize()==0 || xorg.Screens()[screenCurrent].GetDisplaySize()==0 || xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOptionSize()==0) return;
  if (xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOption(screenDisplayOptionCurrent).GetValue()!=screenDisplayOptValue->GetValue())
  {
    xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOption(screenDisplayOptionCurrent).SetValue(screenDisplayOptValue->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterServerLayoutScreenNumber(wxCommandEvent& event)
{
  if (xorg.ServerLayouts().GetSize()==0) return;
  if (xorg.ServerLayouts()[serverLayoutCurrent].GetScreenNumber(serverLayoutScreenCurrent)!=serverLayoutScreenNumber->GetValue())
  {
    xorg.ServerLayouts()[serverLayoutCurrent].SetScreenNumber(serverLayoutScreenCurrent, serverLayoutScreenNumber->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterServerLayoutScreenPosition(wxCommandEvent& event)
{
  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize()==0) return;
  if (xorg.ServerLayouts()[serverLayoutCurrent].GetScreenPosition(serverLayoutScreenCurrent)!=serverLayoutScreenPosition->GetValue())
  {
    xorg.ServerLayouts()[serverLayoutCurrent].SetScreenPosition(serverLayoutScreenCurrent, serverLayoutScreenPosition->GetValue());
    SetSaved(false);
  }

  wxString tempStr=serverLayoutScreenPosition->GetValue();
  tempStr.Trim(false);
  tempStr.Trim(true);
  tempStr.MakeLower();

  if (tempStr==_T("rightof") || tempStr==_T("leftof") || tempStr==_T("above") || tempStr==_T("below"))
  {
    serverLayoutScreen2->Enable(true);
  }
  else
  {
    serverLayoutScreen2->Enable(false);
    xorg.ServerLayouts()[serverLayoutCurrent].SetSecondScreen(serverLayoutScreenCurrent, wxEmptyString);
  }
}

void GuiFrame::EnterServerLayoutOptValue(wxCommandEvent& event)
{
  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts()[serverLayoutCurrent].GetOptionSize()==0) return;
  if (xorg.ServerLayouts()[serverLayoutCurrent].GetOption(serverLayoutOptionCurrent).GetValue()!=serverLayoutOptValue->GetValue())
  {
    xorg.ServerLayouts()[serverLayoutCurrent].GetOption(serverLayoutOptionCurrent).SetValue(serverLayoutOptValue->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterServerFlagsOptValue(wxCommandEvent& event)
{
  if (xorg.ServerFlags().GetOptionSize()==0) return;
  if (xorg.ServerFlags().GetOption(serverFlagCurrent).GetValue()!=serverFlagsOptValue->GetValue())
  {
    xorg.ServerFlags().GetOption(serverFlagCurrent).SetValue(serverFlagsOptValue->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterExtensionOptValue(wxCommandEvent& event)
{
  if (xorg.Extensions().GetOptionSize()==0) return;
  if (xorg.Extensions().GetOption(extensionCurrent).GetValue()!=extensionOptValue->GetValue())
  {
    xorg.Extensions().GetOption(extensionCurrent).SetValue(extensionOptValue->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterModuleOptValue(wxCommandEvent& event)
{
  if (xorg.Modules().GetSize()==0 || xorg.Modules().GetOptionSize(moduleCurrent)==0) return;
  if (xorg.Modules().GetOption(moduleCurrent, moduleOptionCurrent).GetValue()!=moduleOptValue->GetValue())
  {
    xorg.Modules().GetOption(moduleCurrent, moduleOptionCurrent).SetValue(moduleOptValue->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::EnterDriMode(wxCommandEvent& event)
{
  if (xorg.Dri().GetMode()!=driMode->GetValue())
  {
    xorg.Dri().SetMode(driMode->GetValue());
    SetSaved(false);
  }
}

void GuiFrame::SelectScreenDevice(wxCommandEvent& event)
{
  if (xorg.Screens()[screenCurrent].GetDevice()!=screenDevice->GetStringSelection())
  {
    xorg.Screens()[screenCurrent].SetDevice(screenDevice->GetStringSelection());
    SetSaved(false);
  }
}

void GuiFrame::SelectScreenMonitor(wxCommandEvent& event)
{
  if (xorg.Screens()[screenCurrent].GetMonitor()!=screenMonitor->GetStringSelection())
  {
    xorg.Screens()[screenCurrent].SetMonitor(screenMonitor->GetStringSelection());
    SetSaved(false);
  }
}

void GuiFrame::SelectServerLayoutScreen1(wxCommandEvent& event)
{
  if (xorg.ServerLayouts()[serverLayoutCurrent].GetScreen(serverLayoutScreenCurrent)!=serverLayoutScreen1->GetStringSelection())
  {
    xorg.ServerLayouts()[serverLayoutCurrent].SetScreen(serverLayoutScreenCurrent, serverLayoutScreen1->GetStringSelection());
    SetSaved(false);
  }
}

void GuiFrame::SelectServerLayoutScreen2(wxCommandEvent& event)
{
  if (serverLayoutScreen2->IsEnabled())
  {
    if (xorg.ServerLayouts()[serverLayoutCurrent].GetSecondScreen(serverLayoutScreenCurrent)!=serverLayoutScreen2->GetStringSelection())
    {
      xorg.ServerLayouts()[serverLayoutCurrent].SetSecondScreen(serverLayoutScreenCurrent, serverLayoutScreen2->GetStringSelection());
      SetSaved(false);
    }
  }
  else
  {
    if (xorg.ServerLayouts()[serverLayoutCurrent].GetSecondScreen(serverLayoutScreenCurrent)!=wxEmptyString)
    {
      xorg.ServerLayouts()[serverLayoutCurrent].SetSecondScreen(serverLayoutScreenCurrent, wxEmptyString);
      SetSaved(false);
    }
  }
}

void GuiFrame::SelectServerLayoutInputDevice1(wxCommandEvent& event)
{
  if (xorg.ServerLayouts()[serverLayoutCurrent].GetInputDevice(serverLayoutIDevCurrent)!=serverLayoutInputDevice1->GetStringSelection())
  {
    xorg.ServerLayouts()[serverLayoutCurrent].SetInputDevice(serverLayoutIDevCurrent, serverLayoutInputDevice1->GetStringSelection());
    SetSaved(false);
  }
}

void GuiFrame::SelectDevice(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Devices().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Devices().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddDevice();
    else if (selection==2) DelDevice();
    else if (selection==3) CopyDevice();
  }
  else if (CheckDevice())
  {
    AcceptDevice();
    deviceCurrent=selection;
  }

  UpdateDevice();
}

void GuiFrame::SelectDeviceOption(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Devices()[deviceCurrent].GetOptionSize())  // special command (add, delete)
  {
    selection-=xorg.Devices()[deviceCurrent].GetOptionSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddDeviceOption();
    else if (selection==2) DelDeviceOption();
    else if (selection==3) DeActDeviceOption();
  }
  else if (CheckDeviceOption()) deviceOptionCurrent=selection;

  UpdateDeviceOption();
}

void GuiFrame::SelectInputDevice(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.InputDevices().GetSize())  // special command (add, delete)
  {
    selection-=xorg.InputDevices().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddInputDevice();
    else if (selection==2) DelInputDevice();
    else if (selection==3) CopyInputDevice();
  }
  else if (CheckInputDevice()) inputDeviceCurrent=selection;

  UpdateInputDevice();
}

void GuiFrame::SelectInputDeviceOption(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.InputDevices()[inputDeviceCurrent].GetOptionSize())  // special command (add, delete)
  {
    selection-=xorg.InputDevices()[inputDeviceCurrent].GetOptionSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddInputDeviceOption();
    else if (selection==2) DelInputDeviceOption();
    else if (selection==3) DeActInputDeviceOption();
  }
  else if (CheckInputDeviceOption()) inputDeviceOptionCurrent=selection;

  UpdateInputDeviceOption();
}

void GuiFrame::SelectMonitor(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Monitors().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Monitors().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddMonitor();
    else if (selection==2) DelMonitor();
    else if (selection==3) CopyMonitor();
  }
  else if (CheckMonitor())
  {
    AcceptMonitor();
    monitorCurrent=selection;
  }

  UpdateMonitor();
}

void GuiFrame::SelectMonitorOption(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Monitors()[monitorCurrent].GetOptionSize())  // special command (add, delete)
  {
    selection-=xorg.Monitors()[monitorCurrent].GetOptionSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddMonitorOption();
    else if (selection==2) DelMonitorOption();
    else if (selection==3) DeActMonitorOption();
  }
  else if (CheckMonitorOption()) monitorOptionCurrent=selection;

  UpdateMonitorOption();
}

void GuiFrame::SelectMonitorModeline(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Monitors()[monitorCurrent].GetModelineSize())  // special command (add, delete)
  {
    selection-=xorg.Monitors()[monitorCurrent].GetModelineSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddMonitorModeline();
    else if (selection==2) DelMonitorModeline();
    else if (selection==3) DeActMonitorModeline();
  }
  else if (CheckMonitorModeline()) monitorModelineCurrent=selection;

  UpdateMonitorModeline();
}

void GuiFrame::SelectScreen(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Screens().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Screens().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddScreen();
    else if (selection==2) DelScreen();
    else if (selection==3) CopyScreen();
  }
  else if (CheckScreen())
  {
    AcceptScreen();
    screenCurrent=selection;
  }

  UpdateScreen();
}

void GuiFrame::SelectScreenOption(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Screens()[screenCurrent].GetOptionSize())  // special command (add, delete)
  {
    selection-=xorg.Screens()[screenCurrent].GetOptionSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddScreenOption();
    else if (selection==2) DelScreenOption();
    else if (selection==3) DeActScreenOption();
  }
  else if (CheckScreenOption()) screenOptionCurrent=selection;

  UpdateScreenOption();
}

void GuiFrame::SelectScreenDiplay(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Screens()[screenCurrent].GetDisplaySize())  // special command (add, delete)
  {
    selection-=xorg.Screens()[screenCurrent].GetDisplaySize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddScreenDisplay();
    else if (selection==2) DelScreenDisplay();
  }
  else if (CheckScreenDisplay()) screenDisplayCurrent=selection;

  UpdateScreenDisplay();
}

void GuiFrame::SelectScreenDiplayMode(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetModeSize())  // special command (add, delete)
  {
    selection-=xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetModeSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddScreenDisplayMode();
    else if (selection==2) DelScreenDisplayMode();
  }
  else if (CheckScreenDisplayMode()) screenDisplayModeCurrent=selection;

  UpdateScreenDisplayMode();
}

void GuiFrame::SelectScreenDiplayOption(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOptionSize())  // special command (add, delete)
  {
    selection-=xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOptionSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddScreenDisplayOption();
    else if (selection==2) DelScreenDisplayOption();
    else if (selection==3) DeActScreenDisplayOption();
  }
  else if (CheckScreenDisplayOption()) screenDisplayOptionCurrent=selection;

  UpdateScreenDisplayOption();
}

void GuiFrame::SelectServerLayout(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.ServerLayouts().GetSize())  // special command (add, delete)
  {
    selection-=xorg.ServerLayouts().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddServerLayout();
    else if (selection==2) DelServerLayout();
    else if (selection==3) CopyServerLayout();
  }
  else if (CheckServerLayout()) serverLayoutCurrent=selection;

  UpdateServerLayout();
}

void GuiFrame::SelectServerLayoutScreen(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize())  // special command (add, delete)
  {
    selection-=xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddServerLayoutScreen();
    else if (selection==2) DelServerLayoutScreen();
  }
  else if (CheckServerLayoutScreen()) serverLayoutScreenCurrent=selection;

  UpdateServerLayoutScreen();
}

void GuiFrame::SelectServerLayoutInputDevice(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize())  // special command (add, delete)
  {
    selection-=xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddServerLayoutInputDevice();
    else if (selection==2) DelServerLayoutInputDevice();
  }
  else if (CheckServerLayoutInputDevice()) serverLayoutIDevCurrent=selection;

  UpdateServerLayoutInputDevice();
}

void GuiFrame::SelectServerLayoutOption(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.ServerLayouts()[serverLayoutCurrent].GetOptionSize())  // special command (add, delete)
  {
    selection-=xorg.ServerLayouts()[serverLayoutCurrent].GetOptionSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddServerLayoutOption();
    else if (selection==2) DelServerLayoutOption();
    else if (selection==3) DeActServerLayoutOption();
  }
  else if (CheckServerLayoutOption()) serverLayoutOptionCurrent=selection;

  UpdateServerLayoutOption();
}

void GuiFrame::SelectServerLayoutInputDeviceOption(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOptionSize(serverLayoutIDevCurrent))  // special command (add, delete)
  {
    selection-=xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOptionSize(serverLayoutIDevCurrent);   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddServerLayoutInputDeviceOption();
    else if (selection==2) DelServerLayoutInputDeviceOption();
  }
  else if (CheckServerLayoutInputDeviceOption()) serverLayoutIDevOptionCurrent=selection;

  UpdateServerLayoutInputDeviceOption();
}

void GuiFrame::SelectServerFlagOption(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.ServerFlags().GetOptionSize())  // special command (add, delete)
  {
    selection-=xorg.ServerFlags().GetOptionSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddServerFlagOption();
    else if (selection==2) DelServerFlagOption();
    else if (selection==3) DeActServerFlagOption();
  }
  else if (CheckServerFlagOption()) serverFlagCurrent=selection;

  UpdateServerFlagOption();
}

void GuiFrame::SelectExtension(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Extensions().GetOptionSize())  // special command (add, delete)
  {
    selection-=xorg.Extensions().GetOptionSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddExtension();
    else if (selection==2) DelExtension();
    else if (selection==3) DeActExtension();
  }
  else if (CheckExtension()) extensionCurrent=selection;

  UpdateExtension();
}

void GuiFrame::SelectFontPath(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Files().GetFontPathSize())  // special command (add, delete)
  {
    selection-=xorg.Files().GetFontPathSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddFontPath();
    else if (selection==2) DelFontPath();
    else if (selection==3) DeActFontPath();
  }
  else if (CheckFontPath()) fontPathCurrent=selection;

  UpdateFontPath();
}

void GuiFrame::SelectModulePath(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Files().GetModulePathSize())  // special command (add, delete)
  {
    selection-=xorg.Files().GetModulePathSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddModulePath();
    else if (selection==2) DelModulePath();
    else if (selection==3) DeActModulePath();
  }
  else if (CheckModulePath()) modulePathCurrent=selection;

  UpdateModulePath();
}

void GuiFrame::SelectRgbPath(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Files().GetRgbPathSize())  // special command (add, delete)
  {
    selection-=xorg.Files().GetRgbPathSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddRgbPath();
    else if (selection==2) DelRgbPath();
    else if (selection==3) DeActRgbPath();
  }
  else if (CheckRgbPath()) rgbPathCurrent=selection;

  UpdateRgbPath();
}

void GuiFrame::SelectModule(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Modules().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Modules().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddModule();
    else if (selection==2) DelModule();
    else if (selection==3) DeActModule();
  }
  else if (CheckModule()) moduleCurrent=selection;

  UpdateModule();
}

void GuiFrame::SelectModuleOption(wxCommandEvent& event)
{
  int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Modules().GetOptionSize(moduleCurrent))  // special command (add, delete)
  {
    selection-=xorg.Modules().GetOptionSize(moduleCurrent);   // subtract, so that we only have cases 0,1,2

    if (selection==1) AddModuleOption();
    else if (selection==2) DelModuleOption();
    else if (selection==3) DeActModuleOption();
  }
  else if (CheckModuleOption()) moduleOptionCurrent=selection;

  UpdateModuleOption();
}

void GuiFrame::GetRefreshRate(wxCommandEvent& event)
{  // get refresh rate with ddcprobe

  wxArrayString output;  // output of ddcprobe
  int i;        // loop counter

  if (wxExecute(_T("ddcprobe"), output, wxEXEC_SYNC)!=0)
  {
    // ddcprobe could not be executed
    wxMessageBox(_("Sorry, could not determine refresh rate."), _("No result!"), wxICON_ERROR);
    return;
  }

  // search for "monitorrange"
  for (i=0; i<(int)output.GetCount(); i++)
  {
    if (output[i].StartsWith(_T("monitorrange:")))
    {
      wxString range=output[i];
      StripRight(range, 13);

      int pos=range.Find(',');
      if (pos==-1)
      {
        // no comma found
        wxMessageBox(_("Sorry, could not determine refresh rate."), _("No result!"), wxICON_ERROR);
        return;
      }

      wxString hor=range.Left(pos);
      StripSpaces(hor);
      wxString vert=range.Right(range.Len()-pos-1);
      StripSpaces(vert);

      monitorHorizSync->SetValue(hor);
      monitorVertRefresh->SetValue(vert);

      EnterMonitorHorizSync(event);
      EnterMonitorVertRefresh(event);

      break;
    }
  }

  if (i==(int)output.GetCount())
  {
    // monitorrange could not be found
    wxMessageBox(_("Sorry, could not determine refresh rate."), _("No result!"), wxICON_ERROR);
    return;
  }
}

void GuiFrame::GetDisplaySize(wxCommandEvent& event)
{  // calculate display size with xdpyinfo

  wxArrayString output;  // output of xdpyinfo
  int i;        // loop counter

  if (wxExecute(_T("xdpyinfo"), output, wxEXEC_SYNC)!=0)
  {
    // xdpyinfo could not be executed
    wxMessageBox(_("Sorry, could not determine display size."), _("No result!"), wxICON_ERROR);
    return;
  }

  // search for resolution and screen size
  for (i=0; i<(int)output.GetCount(); i++)
  {
    if (output[i].StartsWith(_T("  dimensions:")))
    {
      wxString line=output[i];
      StripRight(line, 13);

      int pos=line.Find(_T("pixels"));
      if (pos==-1)
      {
        // no pixel entry found
        wxMessageBox(_("Sorry, could not determine display size."), _("No result!"), wxICON_ERROR);
        return;
      }

      wxString resolution=line.Left(pos);
      StripSpaces(resolution);

      StripRight(line, pos+6);  // remove pixels

      if (line.GetChar(0)!='(' || line.Last()!=')')
      {
        // no (...) found
        wxMessageBox(_("Sorry, could not determine display size."), _("No result!"), wxICON_ERROR);
      }

      line=line.Mid(1, line.Len()-2);
      StripSpaces(line);

      pos=line.Find(_T("millimeters"));
      if (pos==-1)
      {
        // no pixel entry found
        wxMessageBox(_("Sorry, could not determine display size."), _("No result!"), wxICON_ERROR);
        return;
      }

      wxString size=line.Left(pos);
      StripSpaces(size);

      pos=resolution.Find('x');
      if (pos==-1)
      {
        // no pixel entry found
        wxMessageBox(_("Sorry, could not determine display size."), _("No result!"), wxICON_ERROR);
        return;
      }

      wxString resHor=resolution.Left(pos);
      wxString resVer=resolution.Right(resolution.Len()-pos-1);

      pos=size.Find('x');
      if (pos==-1)
      {
        // no pixel entry found
        wxMessageBox(_("Sorry, could not determine display size."), _("No result!"), wxICON_ERROR);
        return;
      }

      wxString sizeHor=size.Left(pos);
      wxString sizeVer=size.Right(size.Len()-pos-1);

      // Note: resolution is unimportant here... will be used later for DPI maybe

      monitorDisplay->SetValue(sizeHor+_(" ")+sizeVer);
      EnterMonitorDisplay(event);

      break;
    }
  }

  if (i==(int)output.GetCount())
  {
    // resolution could not be found
    wxMessageBox(_("Sorry, could not determine display size."), _("No result!"), wxICON_ERROR);
    return;
  }
}

void GuiFrame::GenerateModeline(wxCommandEvent& event)
{  // generate modeline with gtf

  wxArrayString output;  // output of gtf
  int i;        // loop counter

  // check fields
  if (monitorModelineGeneratorXRes->GetValue().IsEmpty() || monitorModelineGeneratorYRes->GetValue().IsEmpty() || monitorModelineGeneratorFreq->GetValue().IsEmpty())
  {
    wxMessageBox(_("Please fill out all fields!"), _("Error generating modeline"), wxICON_ERROR);
    return;
  }


  if (wxExecute(_T("gtf ")+monitorModelineGeneratorXRes->GetValue()+_T(" ")+monitorModelineGeneratorYRes->GetValue()+_T(" ")+monitorModelineGeneratorFreq->GetValue(), output, wxEXEC_SYNC)!=0)
  {
    // gtf could not be executed
    wxMessageBox(_("Sorry, could not generate modeline."), _("No result!"), wxICON_ERROR);
    return;
  }

  // search for resolution and screen size
  for (i=0; i<(int)output.GetCount(); i++)
  {
    if (output[i].StartsWith(_T("  Modeline")))
    {
      wxString line=output[i];
      StripRight(line, 10);  // delete "Modeline"

      int pos=line.Find('"');
      if (pos!=0)
      {
        // no " found
        wxMessageBox(_("Sorry, could not generate modeline."), _("No result!"), wxICON_ERROR);
        return;
      }
      StripRight(line, 1); // delete "

      pos=line.Find('"');
      if (pos==-1)
      {
        // no second " found
        wxMessageBox(_("Sorry, could not generate modeline."), _("No result!"), wxICON_ERROR);
        return;
      }

      wxString mode=line.Left(pos);
      StripSpaces(mode);

      StripRight(line, pos+1);

      monitorModelineGeneratorOutput1->SetValue(mode);
      monitorModelineGeneratorOutput2->SetValue(line);

      break;
    }
  }

  if (i==(int)output.GetCount())
  {
    // resolution could not be found
    wxMessageBox(_("Sorry, could not generate modeline."), _("No result!"), wxICON_ERROR);
    return;
  }
}

void GuiFrame::UpdateDevice(void)
{
  while (deviceIdentifier->GetCount()>devListSize) deviceIdentifier->Delete(0);

  deviceIdentifier->Enable(true);

  if (xorg.Devices().GetSize()==0)
  {
    deviceCurrent=0;
    deviceOptionCurrent=0;
    deviceDriver->Enable(false);
    deviceBusID->Enable(false);
    deviceScreen->Enable(false);
    deviceVideoRam->Enable(false);
    deviceIdentifier->SetEditable(false);
    deviceDriver->SetValue(wxEmptyString);
    deviceBusID->SetValue(wxEmptyString);
    deviceScreen->SetValue(wxEmptyString);
    deviceVideoRam->SetValue(wxEmptyString);
    deviceOptValue->SetValue(wxEmptyString);
    deviceIdentifierText=wxEmptyString;
  }
  else
  {
    deviceOptionCurrent=0;
    deviceDriver->Enable(true);
    deviceBusID->Enable(true);
    deviceScreen->Enable(true);
    deviceVideoRam->Enable(true);
    deviceIdentifier->SetEditable(true);
    deviceDriver->SetValue(xorg.Devices()[deviceCurrent].GetDriver());
    deviceBusID->SetValue(xorg.Devices()[deviceCurrent].GetBus());
    deviceScreen->SetValue(xorg.Devices()[deviceCurrent].GetScreen());
    deviceVideoRam->SetValue(xorg.Devices()[deviceCurrent].GetVideoRam());

    for (int i=0; i<xorg.Devices().GetSize(); i++)
    {
      deviceIdentifier->Insert(xorg.Devices()[i].GetId(), i);
    }
    deviceIdentifierText=xorg.Devices()[deviceCurrent].GetId();
  }
  deviceIdentifier->SetSelection(deviceCurrent);

  UpdateDeviceOption();
}

void GuiFrame::UpdateDeviceOption(void)
{
  while (deviceOption->GetCount()>optListSize) deviceOption->Delete(0);

  if (xorg.Devices().GetSize()==0) deviceOption->Enable(false);
  else deviceOption->Enable(true);

  if (xorg.Devices().GetSize()==0 || xorg.Devices()[deviceCurrent].GetOptionSize()==0)
  {
    deviceOptionCurrent=0;
    deviceOptValue->Enable(false);
    deviceOptValue->SetValue(wxEmptyString);
    deviceOption->SetEditable(false);
    deviceOption->SetSelection(deviceOptionCurrent);
  }
  else
  {
    if (xorg.Devices()[deviceCurrent].GetOption(deviceOptionCurrent).IsActive())
    {
      deviceOptValue->Enable(true);
      deviceOption->SetEditable(true);
    }
    else
    {
      deviceOptValue->Enable(false);
      deviceOption->SetEditable(false);
    }

    for (int i=0; i<xorg.Devices()[deviceCurrent].GetOptionSize(); i++)
    {
      if (xorg.Devices()[deviceCurrent].GetOption(i).IsActive())
        deviceOption->Insert(xorg.Devices()[deviceCurrent].GetOption(i).GetName(), i);
      else
        deviceOption->Insert(xorg.Devices()[deviceCurrent].GetOption(i).GetName()+_(" [inactive]"), i);
    }
    deviceOption->SetSelection(deviceOptionCurrent);

    deviceOptValue->Clear();

    // set values
    wxString filename=settings.GetOptionDir()+_T("/devices/")+deviceDriver->GetValue()+_T(".xml");

    deviceDriversOption.Load(filename);

    // search select option in list
    int pos=deviceDriversOption.Find(xorg.Devices()[deviceCurrent].GetOption(deviceOptionCurrent).GetName());

    if (pos>=0)
    {
      if (deviceDriversOption[pos].GetValue()==_T("boolean"))
      {
        deviceOptValue->Append(_T("true"));
        deviceOptValue->Append(_T("false"));
      }
      else
      {
        // add arguments
        deviceOptValue->Append(deviceDriversOption[pos].GetArguments());
      }
    }
    deviceOptValue->SetValue(xorg.Devices()[deviceCurrent].GetOption(deviceOptionCurrent).GetValue());
  }
}

void GuiFrame::UpdateInputDevice(void)
{
  while (inputDeviceIdentifier->GetCount()>devListSize) inputDeviceIdentifier->Delete(0);

  inputDeviceIdentifier->Enable(true);

  if (xorg.InputDevices().GetSize()==0)
  {
    inputDeviceCurrent=0;
    inputDeviceOptionCurrent=0;
    inputDeviceDriver->Enable(false);
    inputDeviceIdentifier->SetEditable(false);
    inputDeviceDriver->SetValue(wxEmptyString);
    inputDeviceIdentifierText=wxEmptyString;
  }
  else
  {
    inputDeviceOptionCurrent=0;
    inputDeviceDriver->Enable(true);
    inputDeviceIdentifier->SetEditable(true);
    inputDeviceDriver->SetValue(xorg.InputDevices()[inputDeviceCurrent].GetDriver());

    for (int i=0; i<xorg.InputDevices().GetSize(); i++)
    {
      inputDeviceIdentifier->Insert(xorg.InputDevices()[i].GetId(), i);
    }
    inputDeviceIdentifierText=xorg.InputDevices()[inputDeviceCurrent].GetId();
  }
  inputDeviceIdentifier->SetSelection(inputDeviceCurrent);

  UpdateInputDeviceOption();

}

void GuiFrame::UpdateInputDeviceOption(void)
{
  while (inputDeviceOption->GetCount()>optListSize) inputDeviceOption->Delete(0);

  if (xorg.InputDevices().GetSize()==0) inputDeviceOption->Enable(false);
  else inputDeviceOption->Enable(true);

  if (xorg.InputDevices().GetSize()==0 || xorg.InputDevices()[inputDeviceCurrent].GetOptionSize()==0)
  {
    inputDeviceOptionCurrent=0;
    inputDeviceOptValue->Enable(false);
    inputDeviceOptValue->SetValue(wxEmptyString);
    inputDeviceOption->SetEditable(false);
    inputDeviceOption->SetSelection(inputDeviceOptionCurrent);
  }
  else
  {
    if (xorg.InputDevices()[inputDeviceCurrent].GetOption(inputDeviceOptionCurrent).IsActive())
    {
      inputDeviceOptValue->Enable(true);
      inputDeviceOption->SetEditable(true);
    }
    else
    {
      inputDeviceOptValue->Enable(false);
      inputDeviceOption->SetEditable(false);
    }

    for (int i=0; i<xorg.InputDevices()[inputDeviceCurrent].GetOptionSize(); i++)
    {
      if (xorg.InputDevices()[inputDeviceCurrent].GetOption(i).IsActive())
        inputDeviceOption->Insert(xorg.InputDevices()[inputDeviceCurrent].GetOption(i).GetName(), i);
      else
        inputDeviceOption->Insert(xorg.InputDevices()[inputDeviceCurrent].GetOption(i).GetName()+_(" [inactive]"), i);
    }

    inputDeviceOption->SetSelection(inputDeviceOptionCurrent);

    inputDeviceOptValue->Clear();

    // set values
    wxString filename=settings.GetOptionDir()+_T("/inputdevices/")+deviceDriver->GetValue()+_T(".xml");

    inputDeviceDriversOption.Load(filename);

    // search select option in list
    int pos=inputDeviceDriversOption.Find(xorg.InputDevices()[inputDeviceCurrent].GetOption(inputDeviceOptionCurrent).GetName());

    if (pos>=0)
    {
      if (inputDeviceDriversOption[pos].GetValue()==_T("boolean"))
      {
        inputDeviceOptValue->Append(_T("true"));
        inputDeviceOptValue->Append(_T("false"));
      }
      else
      {
        // add arguments
        inputDeviceOptValue->Append(inputDeviceDriversOption[pos].GetArguments());
      }
    }
    inputDeviceOptValue->SetValue(xorg.InputDevices()[inputDeviceCurrent].GetOption(inputDeviceOptionCurrent).GetValue());
  }
}

void GuiFrame::UpdateMonitor(void)
{
  while (monitorIdentifier->GetCount()>devListSize) monitorIdentifier->Delete(0);

  monitorIdentifier->Enable(true);

  if (xorg.Monitors().GetSize()==0)
  {
    monitorCurrent=0;
    monitorOptionCurrent=0;
    monitorModelineCurrent=0;
    monitorHorizSync->Enable(false);
    monitorVertRefresh->Enable(false);
    monitorDisplay->Enable(false);
    monitorGamma->Enable(false);
    monitorIdentifier->SetEditable(false);
    monitorFreqButton->Enable(false);
    monitorDisplaySizeButton->Enable(false);
    monitorHorizSync->SetValue(wxEmptyString);
    monitorVertRefresh->SetValue(wxEmptyString);
    monitorDisplay->SetValue(wxEmptyString);
    monitorGamma->SetValue(wxEmptyString);
    monitorIdentifierText=wxEmptyString;
  }
  else
  {
    monitorOptionCurrent=0;
    monitorModelineCurrent=0;
    monitorHorizSync->Enable(true);
    monitorVertRefresh->Enable(true);
    monitorDisplay->Enable(true);
    monitorGamma->Enable(true);
    monitorIdentifier->SetEditable(true);
    monitorFreqButton->Enable(true);
    monitorDisplaySizeButton->Enable(true);
    monitorHorizSync->SetValue(xorg.Monitors()[monitorCurrent].GetHorSync());
    monitorVertRefresh->SetValue(xorg.Monitors()[monitorCurrent].GetVertRef());
    monitorDisplay->SetValue(xorg.Monitors()[monitorCurrent].GetDisplaySize());
    monitorGamma->SetValue(xorg.Monitors()[monitorCurrent].GetGamma());

    for (int i=0; i<xorg.Monitors().GetSize(); i++)
    {
      monitorIdentifier->Insert(xorg.Monitors()[i].GetId(), i);
    }
    monitorIdentifierText=xorg.Monitors()[monitorCurrent].GetId();
  }
  monitorIdentifier->SetSelection(monitorCurrent);

  UpdateMonitorOption();
  UpdateMonitorModeline();

}

void GuiFrame::UpdateMonitorOption(void)
{
  while (monitorOption->GetCount()>optListSize) monitorOption->Delete(0);

  if (xorg.Monitors().GetSize()==0) monitorOption->Enable(false);
  else monitorOption->Enable(true);

  if (xorg.Monitors().GetSize()==0 || xorg.Monitors()[monitorCurrent].GetOptionSize()==0)
  {
    monitorOptionCurrent=0;
    monitorOptValue->Enable(false);
    monitorOptValue->SetValue(wxEmptyString);
    monitorOption->SetEditable(false);
  }
  else
  {
    monitorOptValue->SetValue(xorg.Monitors()[monitorCurrent].GetOption(monitorOptionCurrent).GetValue());

    if (xorg.Monitors()[monitorCurrent].GetOption(monitorOptionCurrent).IsActive())
    {
      monitorOptValue->Enable(true);
      monitorOption->SetEditable(true);
    }
    else
    {
      monitorOptValue->Enable(false);
      monitorOption->SetEditable(false);
    }

    for (int i=0; i<xorg.Monitors()[monitorCurrent].GetOptionSize(); i++)
    {
      if (xorg.Monitors()[monitorCurrent].GetOption(i).IsActive())
        monitorOption->Insert(xorg.Monitors()[monitorCurrent].GetOption(i).GetName(), i);
      else
        monitorOption->Insert(xorg.Monitors()[monitorCurrent].GetOption(i).GetName()+_(" [inactive]"), i);
    }
  }
  monitorOption->SetSelection(monitorOptionCurrent);
}

void GuiFrame::UpdateMonitorModeline(void)
{
  while (monitorModelineName->GetCount()>devListSize) monitorModelineName->Delete(0);

  if (xorg.Monitors().GetSize()==0) monitorModelineName->Enable(false);
  else monitorModelineName->Enable(true);

  if (xorg.Monitors().GetSize()==0 || xorg.Monitors()[monitorCurrent].GetModelineSize()==0)
  {
    monitorModelineCurrent=0;
    monitorModelineOption->Enable(false);
    monitorModelineOption->SetValue(wxEmptyString);
    monitorModelineName->SetEditable(false);
  }
  else
  {
    monitorModelineOption->SetValue(xorg.Monitors()[monitorCurrent].GetModeline(monitorModelineCurrent).GetValue());

    if (xorg.Monitors()[monitorCurrent].GetModeline(monitorModelineCurrent).IsActive())
    {
      monitorModelineOption->Enable(true);
      monitorModelineName->SetEditable(true);
    }
    else
    {
      monitorModelineOption->Enable(false);
      monitorModelineName->SetEditable(false);
    }

    for (int i=0; i<xorg.Monitors()[monitorCurrent].GetModelineSize(); i++)
    {
      if (xorg.Monitors()[monitorCurrent].GetModeline(i).IsActive())
        monitorModelineName->Insert(xorg.Monitors()[monitorCurrent].GetModeline(i).GetName(), i);
      else
        monitorModelineName->Insert(xorg.Monitors()[monitorCurrent].GetModeline(i).GetName()+_(" [inactive]"), i);
    }
  }
  monitorModelineName->SetSelection(monitorModelineCurrent);
}

void GuiFrame::UpdateScreen(void)
{
  while (screenIdentifier->GetCount()>devListSize) screenIdentifier->Delete(0);
  screenDevice->Clear();
  screenMonitor->Clear();

  screenIdentifier->Enable(true);

  int i;

  if (xorg.Screens().GetSize()==0)
  {
    screenCurrent=0;
    screenOptionCurrent=0;
    screenDisplayCurrent=0;
    screenDisplayModeCurrent=0;
    screenDisplayOptionCurrent=0;
    screenDepth->Enable(false);
    screenDevice->Enable(false);
    screenMonitor->Enable(false);
    screenIdentifier->SetEditable(false);
    screenDepth->SetValue(wxEmptyString);
    screenIdentifierText=wxEmptyString;
  }
  else
  {
    screenOptionCurrent=0;
    screenDisplayCurrent=0;
    screenDisplayModeCurrent=0;
    screenDisplayOptionCurrent=0;
    screenDepth->Enable(true);
    screenDevice->Enable(true);
    screenMonitor->Enable(true);
    screenIdentifier->SetEditable(true);
    screenDepth->SetValue(xorg.Screens()[screenCurrent].GetDefaultDepth());

    for (i=0; i<xorg.Devices().GetSize(); i++)
    {
      screenDevice->Append(xorg.Devices()[i].GetId());
    }
    screenDevice->SetStringSelection(xorg.Screens()[screenCurrent].GetDevice());

    for (i=0; i<xorg.Monitors().GetSize(); i++)
    {
      screenMonitor->Append(xorg.Monitors()[i].GetId());
    }
    screenMonitor->SetStringSelection(xorg.Screens()[screenCurrent].GetMonitor());

    for (i=0; i<xorg.Screens().GetSize(); i++)
    {
      screenIdentifier->Insert(xorg.Screens()[i].GetId(), i);
    }
    screenIdentifierText=xorg.Screens()[screenCurrent].GetId();

  }
  screenIdentifier->SetSelection(screenCurrent);

  UpdateScreenOption();
  UpdateScreenDisplay();
}

void GuiFrame::UpdateScreenOption(void)
{
  while (screenOption->GetCount()>optListSize) screenOption->Delete(0);

  if (xorg.Screens().GetSize()==0) screenOption->Enable(false);
  else screenOption->Enable(true);

  if (xorg.Screens().GetSize()==0 || xorg.Screens()[screenCurrent].GetOptionSize()==0)
  {
    screenOptionCurrent=0;
    screenOptValue->Enable(false);
    screenOptValue->SetValue(wxEmptyString);
    screenOption->SetEditable(false);
  }
  else
  {
    screenOptValue->SetValue(xorg.Screens()[screenCurrent].GetOption(screenOptionCurrent).GetValue());

    if (xorg.Screens()[screenCurrent].GetOption(screenOptionCurrent).IsActive())
    {
      screenOptValue->Enable(true);
      screenOption->SetEditable(true);
    }
    else
    {
      screenOptValue->Enable(false);
      screenOption->SetEditable(false);
    }

    for (int i=0; i<xorg.Screens()[screenCurrent].GetOptionSize(); i++)
    {
      if (xorg.Screens()[screenCurrent].GetOption(i).IsActive())
        screenOption->Insert(xorg.Screens()[screenCurrent].GetOption(i).GetName(), i);
      else
        screenOption->Insert(xorg.Screens()[screenCurrent].GetOption(i).GetName()+_(" [inactive]"), i);
    }
  }
  screenOption->SetSelection(screenOptionCurrent);
}

void GuiFrame::UpdateScreenDisplay(void)
{
  while (screenDisplayDepth->GetCount()>devListSize-1) screenDisplayDepth->Delete(0);

  if (xorg.Screens().GetSize()==0) screenDisplayDepth->Enable(false);
  else screenDisplayDepth->Enable(true);

  if (xorg.Screens().GetSize()==0 || xorg.Screens()[screenCurrent].GetDisplaySize()==0)
  {
    screenDisplayCurrent=0;
    screenDisplayModeCurrent=0;
    screenDisplayOptionCurrent=0;
    screenDisplayViewport->Enable(false);
    screenDisplayViewport->SetValue(wxEmptyString);
    screenDisplayVirtual->Enable(false);
    screenDisplayVirtual->SetValue(wxEmptyString);
    screenDisplayDepth->SetEditable(false);
  }
  else
  {
    screenDisplayViewport->Enable(true);
    screenDisplayViewport->SetValue(xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetViewPort());
    screenDisplayVirtual->Enable(true);
    screenDisplayVirtual->SetValue(xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetVirtual());
    screenDisplayDepth->SetEditable(true);

    screenDisplayModeCurrent=0;
    screenDisplayOptionCurrent=0;
    for (int i=0; i<xorg.Screens()[screenCurrent].GetDisplaySize(); i++)
    {
      screenDisplayDepth->Insert(xorg.Screens()[screenCurrent].GetDisplay(i).GetDepth(), i);
    }
  }
  screenDisplayDepth->SetSelection(screenDisplayCurrent);

  UpdateScreenDisplayOption();
  UpdateScreenDisplayMode();
}

void GuiFrame::UpdateScreenDisplayMode(void)
{
  while (screenDisplayModes->GetCount()>devListSize-1) screenDisplayModes->Delete(0);

  if (xorg.Screens().GetSize()==0 || xorg.Screens()[screenCurrent].GetDisplaySize()==0) screenDisplayModes->Enable(false);
  else screenDisplayModes->Enable(true);

  if (xorg.Screens().GetSize()==0 || xorg.Screens()[screenCurrent].GetDisplaySize()==0 || xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetModeSize()==0)
  {
    screenDisplayModeCurrent=0;
    screenDisplayModes->SetEditable(false);
  }
  else
  {
    screenDisplayModes->SetEditable(true);
    for (int i=0; i<xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetModeSize(); i++)
    {
      screenDisplayModes->Insert(xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetMode(i), i);
    }
  }
  screenDisplayModes->SetSelection(screenDisplayModeCurrent);

}

void GuiFrame::UpdateScreenDisplayOption(void)
{
  while (screenDisplayOption->GetCount()>optListSize) screenDisplayOption->Delete(0);

  if (xorg.Screens().GetSize()==0 || xorg.Screens()[screenCurrent].GetDisplaySize()==0) screenDisplayOption->Enable(false);
  else screenDisplayOption->Enable(true);

  if (xorg.Screens().GetSize()==0 || xorg.Screens()[screenCurrent].GetDisplaySize()==0 || xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOptionSize()==0)
  {
    screenDisplayOptionCurrent=0;
    screenDisplayOptValue->Enable(false);
    screenDisplayOptValue->SetValue(wxEmptyString);
    screenDisplayOption->SetEditable(false);
  }
  else
  {
    screenDisplayOptValue->SetValue(xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOption(screenDisplayOptionCurrent).GetValue());

    if (xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOption(screenDisplayOptionCurrent).IsActive())
    {
      screenDisplayOptValue->Enable(true);
      screenDisplayOption->SetEditable(true);
    }
    else
    {
      screenDisplayOptValue->Enable(false);
      screenDisplayOption->SetEditable(false);
    }

    for (int i=0; i<xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOptionSize(); i++)
    {
      if (xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOption(i).IsActive())
        screenDisplayOption->Insert(xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOption(i).GetName(), i);
      else
        screenDisplayOption->Insert(xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOption(i).GetName()+_(" [inactive]"), i);
    }
  }
  screenDisplayOption->SetSelection(screenDisplayOptionCurrent);
}

void GuiFrame::UpdateServerLayout(void)
{
  while (serverLayoutIdentifier->GetCount()>devListSize) serverLayoutIdentifier->Delete(0);

  serverLayoutIdentifier->Enable(true);

  if (xorg.ServerLayouts().GetSize()==0)
  {
    serverLayoutCurrent=0;
    serverLayoutOptionCurrent=0;
    serverLayoutScreenCurrent=0;
    serverLayoutIDevCurrent=0;
    serverLayoutIDevOptionCurrent=0;
    serverLayoutIdentifier->SetEditable(false);
  }
  else
  {
    serverLayoutOptionCurrent=0;
    serverLayoutScreenCurrent=0;
    serverLayoutIDevCurrent=0;
    serverLayoutIDevOptionCurrent=0;
    serverLayoutIdentifier->SetEditable(true);
    for (int i=0; i<xorg.ServerLayouts().GetSize(); i++)
    {
      serverLayoutIdentifier->Insert(xorg.ServerLayouts()[i].GetId(), i);
    }
  }
  serverLayoutIdentifier->SetSelection(serverLayoutCurrent);

  UpdateServerLayoutScreen();
  UpdateServerLayoutInputDevice();
  UpdateServerLayoutOption();

}

void GuiFrame::UpdateServerLayoutOption(void)
{
  while (serverLayoutOption->GetCount()>optListSize) serverLayoutOption->Delete(0);

  if (xorg.ServerLayouts().GetSize()==0) serverLayoutOption->Enable(false);
  else serverLayoutOption->Enable(true);

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts()[serverLayoutCurrent].GetOptionSize()==0)
  {
    serverLayoutOptionCurrent=0;
    serverLayoutOptValue->Enable(false);
    serverLayoutOptValue->SetValue(wxEmptyString);
    serverLayoutOption->SetEditable(false);
  }
  else
  {
    serverLayoutOptValue->SetValue(xorg.ServerLayouts()[serverLayoutCurrent].GetOption(serverLayoutOptionCurrent).GetValue());

    if (xorg.ServerLayouts()[serverLayoutCurrent].GetOption(serverLayoutOptionCurrent).IsActive())
    {
      serverLayoutOptValue->Enable(true);
      serverLayoutOption->SetEditable(true);
    }
    else
    {
      serverLayoutOptValue->Enable(false);
      serverLayoutOption->SetEditable(false);
    }

    for (int i=0; i<xorg.ServerLayouts()[serverLayoutCurrent].GetOptionSize(); i++)
    {
      if (xorg.ServerLayouts()[serverLayoutCurrent].GetOption(i).IsActive())
        serverLayoutOption->Insert(xorg.ServerLayouts()[serverLayoutCurrent].GetOption(i).GetName(), i);
      else
        serverLayoutOption->Insert(xorg.ServerLayouts()[serverLayoutCurrent].GetOption(i).GetName()+_(" [inactive]"), i);
    }
  }
  serverLayoutOption->SetSelection(serverLayoutOptionCurrent);
}

void GuiFrame::UpdateServerLayoutScreen(void)
{
  while (serverLayoutScreen->GetCount()>devListSize-1) serverLayoutScreen->Delete(0);

  if (xorg.ServerLayouts().GetSize()==0) serverLayoutScreen->Enable(false);
  else serverLayoutScreen->Enable(true);

  serverLayoutScreen1->Clear();
  serverLayoutScreen2->Clear();

  int i;

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize()==0)
  {
    serverLayoutScreenCurrent=0;
    serverLayoutScreen1->Enable(false);
    serverLayoutScreen2->Enable(false);
    serverLayoutScreenNumber->Enable(false);
    serverLayoutScreenPosition->Enable(false);
    serverLayoutScreenNumber->SetValue(wxEmptyString);
    serverLayoutScreenPosition->SetValue(wxEmptyString);
  }
  else
  {
    serverLayoutScreenNumber->Enable(true);
    serverLayoutScreenPosition->Enable(true);
    serverLayoutScreen1->Enable(true);
    serverLayoutScreen2->Enable(true);

    for (i=0; i<xorg.Screens().GetSize(); i++)
    {
      serverLayoutScreen1->Append(xorg.Screens()[i].GetId());
    }
    serverLayoutScreen1->SetStringSelection(xorg.ServerLayouts()[serverLayoutCurrent].GetScreen(serverLayoutScreenCurrent));

    serverLayoutScreenNumber->SetValue(xorg.ServerLayouts()[serverLayoutCurrent].GetScreenNumber(serverLayoutScreenCurrent));
    serverLayoutScreenPosition->SetValue(xorg.ServerLayouts()[serverLayoutCurrent].GetScreenPosition(serverLayoutScreenCurrent));

    for (i=0; i<xorg.Screens().GetSize(); i++)
    {
      serverLayoutScreen2->Append(xorg.Screens()[i].GetId());
    }

    wxString tempStr=serverLayoutScreenPosition->GetValue();
    tempStr.Trim(false);
    tempStr.Trim(true);
    tempStr.MakeLower();

    if (tempStr==_T("rightof") || tempStr==_T("leftof") || tempStr==_T("above") || tempStr==_T("below"))
    {
      serverLayoutScreen2->Enable(true);
      serverLayoutScreen2->SetStringSelection(xorg.ServerLayouts()[serverLayoutCurrent].GetSecondScreen(serverLayoutScreenCurrent));
    }
    else
    {
      serverLayoutScreen2->Enable(false);
    }

    for (i=0; i<xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize(); i++)
    {
      wxString tempStr;
      tempStr << _("Screen") << _T(" ") << i;
      serverLayoutScreen->Insert(tempStr, i);
    }
  }
  serverLayoutScreen->SetSelection(serverLayoutScreenCurrent);
}

void GuiFrame::UpdateServerLayoutInputDevice(void)
{
  while (serverLayoutInputDevice->GetCount()>devListSize-1) serverLayoutInputDevice->Delete(0);

  if (xorg.ServerLayouts().GetSize()==0) serverLayoutInputDevice->Enable(false);
  else serverLayoutInputDevice->Enable(true);

  serverLayoutInputDevice1->Clear();

  int i;

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize()==0)
  {
    serverLayoutIDevCurrent=0;
    serverLayoutIDevOptionCurrent=0;
    serverLayoutInputDevice1->Enable(false);
  }
  else
  {
    serverLayoutIDevOptionCurrent=0;
    serverLayoutInputDevice1->Enable(true);

    for (i=0; i<xorg.InputDevices().GetSize(); i++)
    {
      serverLayoutInputDevice1->Append(xorg.InputDevices()[i].GetId());
    }
    serverLayoutInputDevice1->SetStringSelection(xorg.ServerLayouts()[serverLayoutCurrent].GetInputDevice(serverLayoutIDevCurrent));

    for (i=0; i<xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize(); i++)
    {
      wxString tempStr;
      tempStr << _("Input Device") << _T(" ") << i;
      serverLayoutInputDevice->Insert(tempStr, i);
    }
  }
  serverLayoutInputDevice->SetSelection(serverLayoutIDevCurrent);

  UpdateServerLayoutInputDeviceOption();
}

void GuiFrame::UpdateServerLayoutInputDeviceOption(void)
{
  while (serverLayoutInputDeviceOption->GetCount()>optListSize-1) serverLayoutInputDeviceOption->Delete(0);

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize()==0) serverLayoutInputDeviceOption->Enable(false);
  else serverLayoutInputDeviceOption->Enable(true);

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize()==0 || xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOptionSize(serverLayoutIDevCurrent)==0)
  {
    serverLayoutIDevOptionCurrent=0;
    // serverLayoutInputDeviceOptValue->Enable(false);
    // serverLayoutInputDeviceOptValue->SetValue(wxEmptyString);
    serverLayoutInputDeviceOption->SetEditable(false);
  }
  else
  {
    serverLayoutInputDeviceOption->SetEditable(true);
    // serverLayoutInputDeviceOptValue->Enable(true);
    // serverLayoutInputDeviceOptValue->SetValue(wxEmptyString);

    for (int i=0; i<xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOptionSize(serverLayoutIDevCurrent); i++)
    {
      serverLayoutInputDeviceOption->Insert(xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOption(serverLayoutIDevCurrent, i).GetName(), i);
    }
  }
  serverLayoutInputDeviceOption->SetSelection(serverLayoutIDevOptionCurrent);
}

void GuiFrame::UpdateServerFlagOption(void)
{
  while (serverFlagsOption->GetCount()>devListSize) serverFlagsOption->Delete(0);

  serverFlagsOption->Enable(true);

  if (xorg.ServerFlags().GetOptionSize()==0)
  {
    serverFlagCurrent=0;
    serverFlagsOptValue->Enable(false);
    serverFlagsOptValue->SetValue(wxEmptyString);
    serverFlagsOption->SetEditable(false);
  }
  else
  {
    serverFlagsOptValue->SetValue(xorg.ServerFlags().GetOption(serverFlagCurrent).GetValue());

    if (xorg.ServerFlags().GetOption(serverFlagCurrent).IsActive())
    {
      serverFlagsOptValue->Enable(true);
      serverFlagsOption->SetEditable(true);
    }
    else
    {
      serverFlagsOptValue->Enable(false);
      serverFlagsOption->SetEditable(false);
    }

    for (int i=0; i<xorg.ServerFlags().GetOptionSize(); i++)
    {
      if (xorg.ServerFlags().GetOption(i).IsActive())
        serverFlagsOption->Insert(xorg.ServerFlags().GetOption(i).GetName(), i);
      else
        serverFlagsOption->Insert(xorg.ServerFlags().GetOption(i).GetName()+_(" [inactive]"), i);
    }
  }
  serverFlagsOption->SetSelection(serverFlagCurrent);
}

void GuiFrame::UpdateExtension(void)
{
  while (extensionOption->GetCount()>devListSize) extensionOption->Delete(0);

  extensionOption->Enable(true);

  if (xorg.Extensions().GetOptionSize()==0)
  {
    extensionCurrent=0;
    extensionOptValue->Enable(false);
    extensionOptValue->SetValue(wxEmptyString);
    extensionOption->SetEditable(false);
  }
  else
  {
    extensionOptValue->SetValue(xorg.Extensions().GetOption(extensionCurrent).GetValue());

    if (xorg.Extensions().GetOption(extensionCurrent).IsActive())
    {
      extensionOptValue->Enable(true);
      extensionOption->SetEditable(true);
    }
    else
    {
      extensionOptValue->Enable(false);
      extensionOption->SetEditable(false);
    }

    for (int i=0; i<xorg.Extensions().GetOptionSize(); i++)
    {
      if (xorg.Extensions().GetOption(i).IsActive())
        extensionOption->Insert(xorg.Extensions().GetOption(i).GetName(), i);
      else
        extensionOption->Insert(xorg.Extensions().GetOption(i).GetName()+_(" [inactive]"), i);
    }
  }
  extensionOption->SetSelection(extensionCurrent);
  if (xorg.Extensions().GetOptionSize()==0 || !xorg.Extensions().GetOption(extensionCurrent).IsActive()) extensionOptValue->Enable(false);
}

void GuiFrame::UpdateFontPath(void)
{
  while (fontPath->GetCount()>devListSize) fontPath->Delete(0);

  fontPath->Enable(true);

  if (xorg.Files().GetFontPathSize()==0)
  {
    fontPathCurrent=0;
    fontPath->SetEditable(false);
  }
  else
  {
    if (xorg.Files().FontPathIsActive(fontPathCurrent))
    {
      fontPath->SetEditable(true);
    }
    else
    {
      fontPath->SetEditable(false);
    }

    for (int i=0; i<xorg.Files().GetFontPathSize(); i++)
    {
      if (xorg.Files().FontPathIsActive(i))
        fontPath->Insert(xorg.Files().GetFontPath(i), i);
      else
        fontPath->Insert(xorg.Files().GetFontPath(i)+_(" [inactive]"), i);
    }
  }
  fontPath->SetSelection(fontPathCurrent);
}

void GuiFrame::UpdateModulePath(void)
{
  while (modulePath->GetCount()>devListSize) modulePath->Delete(0);

  modulePath->Enable(true);

  if (xorg.Files().GetModulePathSize()==0)
  {
    modulePathCurrent=0;
    modulePath->SetEditable(false);
  }
  else
  {
    if (xorg.Files().ModulePathIsActive(modulePathCurrent))
    {
      modulePath->SetEditable(true);
    }
    else
    {
      modulePath->SetEditable(false);
    }

    for (int i=0; i<xorg.Files().GetModulePathSize(); i++)
    {
      if (xorg.Files().ModulePathIsActive(i))
        modulePath->Insert(xorg.Files().GetModulePath(i), i);
      else
        modulePath->Insert(xorg.Files().GetModulePath(i)+_(" [inactive]"), i);
    }
  }
  modulePath->SetSelection(modulePathCurrent);
}

void GuiFrame::UpdateRgbPath(void)
{
  while (rgbPath->GetCount()>devListSize) rgbPath->Delete(0);

  rgbPath->Enable(true);

  if (xorg.Files().GetRgbPathSize()==0)
  {
    rgbPathCurrent=0;
    rgbPath->SetEditable(false);
  }
  else
  {
    if (xorg.Files().RgbPathIsActive(rgbPathCurrent))
    {
      rgbPath->SetEditable(true);
    }
    else
    {
      rgbPath->SetEditable(false);
    }

    for (int i=0; i<xorg.Files().GetRgbPathSize(); i++)
    {
      if (xorg.Files().RgbPathIsActive(i))
        rgbPath->Insert(xorg.Files().GetRgbPath(i), i);
      else
        rgbPath->Insert(xorg.Files().GetRgbPath(i)+_(" [inactive]"), i);
    }
  }
  rgbPath->SetSelection(rgbPathCurrent);
}

void GuiFrame::UpdateModule(void)
{
  while (moduleIdentifier->GetCount()>devListSize) moduleIdentifier->Delete(0);

  moduleIdentifier->Enable(true);

  if (xorg.Modules().GetSize()==0)
  {
    moduleCurrent=0;
    moduleOptionCurrent=0;
    moduleIdentifier->SetEditable(false);
  }
  else
  {
    moduleOptionCurrent=0;

    if (xorg.Modules().IsActive(moduleCurrent))
    {
      moduleIdentifier->SetEditable(true);
    }
    else
    {
      moduleIdentifier->SetEditable(false);
    }

    for (int i=0; i<xorg.Modules().GetSize(); i++)
    {
      if (xorg.Modules().IsActive(i))
        moduleIdentifier->Insert(xorg.Modules().Get(i), i);
      else
        moduleIdentifier->Insert(xorg.Modules().Get(i)+_(" [inactive]"), i);
    }
  }
  moduleIdentifier->SetSelection(moduleCurrent);

  UpdateModuleOption();
}

void GuiFrame::UpdateModuleOption(void)
{
  while (moduleOption->GetCount()>optListSize) moduleOption->Delete(0);

  if (xorg.Modules().GetSize()==0 || !xorg.Modules().IsActive(moduleCurrent)) moduleOption->Enable(false);
  else moduleOption->Enable(true);

  if (xorg.Modules().GetSize()==0 || xorg.Modules().GetOptionSize(moduleCurrent)==0)
  {
    moduleOptionCurrent=0;
    moduleOptValue->Enable(false);
    moduleOptValue->SetValue(wxEmptyString);
    moduleOption->SetEditable(false);
  }
  else
  {
    moduleOptValue->SetValue(xorg.Modules().GetOption(moduleCurrent, moduleOptionCurrent).GetValue());

    if (xorg.Modules().IsActive(moduleCurrent) && xorg.Modules().GetOption(moduleCurrent, moduleOptionCurrent).IsActive())
    {
      moduleOptValue->Enable(true);
      moduleOption->SetEditable(true);
    }
    else
    {
      moduleOptValue->Enable(false);
      moduleOption->SetEditable(false);
    }

    for (int i=0; i<xorg.Modules().GetOptionSize(moduleCurrent); i++)
    {
      if (xorg.Modules().GetOption(moduleCurrent, i).IsActive())
        moduleOption->Insert(xorg.Modules().GetOption(moduleCurrent, i).GetName(), i);
      else
        moduleOption->Insert(xorg.Modules().GetOption(moduleCurrent, i).GetName()+_(" [inactive]"), i);
    }
  }
  moduleOption->SetSelection(moduleOptionCurrent);
}

void GuiFrame::UpdateDRI(void)
{
  driMode->SetValue(xorg.Dri().GetMode());
}

void GuiFrame::AddDevice(void)
{
  if (!CheckDevice()) return;

  AcceptDevice();

  wxString tempStr;
  Device tempDevice;

  tempStr.Printf(_T("Default Device %d"), xorg.Devices().GetSize()+1);
  tempDevice.SetId(tempStr);

  xorg.Devices().Add(tempDevice);
  deviceCurrent=xorg.Devices().GetSize()-1;
  UpdateScreen();
  SetSaved(false);
}

void GuiFrame::DelDevice(void)
{
  if (xorg.Devices().GetSize()==0) return;

  if (wxMessageBox(_("Do you really want to delete this device?"), _("Question: Delete Device?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  bool found=false;   // entry found
  int i;         // loop counter

  // check if there are some screen that use this device
  for (i=0; i<xorg.Screens().GetSize(); i++)
  {
    if (xorg.Screens()[i].GetDevice()==xorg.Devices()[deviceCurrent].GetId())
    {
      found=true;
      break;  // leave for i-loop
    }
  }

  if (found)
  {
    wxString tempStr;
    tempStr << _("This device is still used by screen \"") << xorg.Screens()[i].GetId() << _T("\".");
    tempStr << _("\n\nShould I delete this references too? ");
    tempStr << _("Otherwise you must manually resolve the dependence before deleting this device.");

    if (wxMessageBox(tempStr, _("Error: Device used"), wxICON_ERROR | wxYES_NO | wxNO_DEFAULT)==wxNO) return;
    else
    {
      for (i=0; i<xorg.Screens().GetSize(); i++)
      {
        if (xorg.Screens()[i].GetDevice()==xorg.Devices()[deviceCurrent].GetId())
        {
          xorg.Screens()[i].SetDevice(wxEmptyString);
        }
      }
    }
  }

  xorg.Devices().Remove(deviceCurrent);
  if (deviceCurrent==xorg.Devices().GetSize() && deviceCurrent!=0) deviceCurrent--;
  UpdateScreen();
  SetSaved(false);
}

void GuiFrame::CopyDevice(void)
{
  if (xorg.Devices().GetSize()==0) return;

  if (!CheckDevice()) return;

  AcceptDevice();

  Device tempDevice(xorg.Devices()[deviceCurrent]);
  tempDevice.SetId(xorg.Devices()[deviceCurrent].GetId()+_(" (Copy)"));
  xorg.Devices().Add(tempDevice);

  deviceCurrent=xorg.Devices().GetSize()-1;
  UpdateScreen();
  SetSaved(false);
}

void GuiFrame::AddDeviceOption(void)
{
  if (!CheckDeviceOption()) return;

  xorg.Devices()[deviceCurrent].AddOption(Option());
  deviceOptionCurrent=xorg.Devices()[deviceCurrent].GetOptionSize()-1;
  SetSaved(false);
}

void GuiFrame::DelDeviceOption(void)
{
  if (xorg.Devices()[deviceCurrent].GetOptionSize()==0) return;

  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.Devices()[deviceCurrent].RemoveOption(deviceOptionCurrent);
  if (deviceOptionCurrent==xorg.Devices()[deviceCurrent].GetOptionSize() && deviceOptionCurrent!=0) deviceOptionCurrent--;
  SetSaved(false);
}

void GuiFrame::DeActDeviceOption(void)
{
  if (xorg.Devices()[deviceCurrent].GetOptionSize()==0) return;

  xorg.Devices()[deviceCurrent].GetOption(deviceOptionCurrent).SetActive(!xorg.Devices()[deviceCurrent].GetOption(deviceOptionCurrent).IsActive());
  SetSaved(false);
}

void GuiFrame::AddInputDevice(void)
{
  if (!CheckInputDevice()) return;
  AcceptInputDevice();

  wxString tempStr;
  InputDevice tempDevice;

  tempStr.Printf(_T("Default Input Device %d"), xorg.InputDevices().GetSize()+1);
  tempDevice.SetId(tempStr);

  xorg.InputDevices().Add(tempDevice);
  inputDeviceCurrent=xorg.InputDevices().GetSize()-1;
  UpdateServerLayout();
  SetSaved(false);
}

void GuiFrame::DelInputDevice(void)
{
  if (xorg.InputDevices().GetSize()==0) return;

  if (wxMessageBox(_("Do you really want to delete this input device?"), _("Question: Delete Input Device?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  bool found=false;   // entry found
  int i,j;       // loop counter

  // check if there are some server layout that use this input device
  for (i=0; i<xorg.ServerLayouts().GetSize(); i++)
  {
    for (j=0; j<xorg.ServerLayouts()[i].GetInputDeviceSize(); j++)
    {
      if (xorg.ServerLayouts()[i].GetInputDevice(j)==xorg.InputDevices()[inputDeviceCurrent].GetId())
      {
        found=true;
        break; // leave for j-loop
      }
    }
    if (found) break; // leave for i-loop
  }

  if (found)
  {
    wxString tempStr;
    tempStr << _("This input device is still used by server layout \"") << xorg.ServerLayouts()[i].GetId() << _T("\".");
    tempStr << _("\n\nShould I delete this references too? ");
    tempStr << _("Otherwise you must manually resolve the dependence before deleting this input device.");

    if (wxMessageBox(tempStr, _("Error: Input Device used"), wxICON_ERROR | wxYES_NO | wxNO_DEFAULT)==wxNO) return;
    else
    {
      for (i=0; i<xorg.ServerLayouts().GetSize(); i++)
      {
        for (j=0; j<xorg.ServerLayouts()[i].GetInputDeviceSize(); j++)
        {
          if (xorg.ServerLayouts()[i].GetInputDevice(j)==xorg.InputDevices()[inputDeviceCurrent].GetId())
          {
            xorg.ServerLayouts()[i].RemoveInputDevice(j);
            if (i==serverLayoutCurrent && j==serverLayoutIDevCurrent && j==xorg.ServerLayouts()[i].GetInputDeviceSize() && j!=0) serverLayoutIDevCurrent--;
            j--;
          }
        }
      }
    }
  }

  xorg.InputDevices().Remove(inputDeviceCurrent);
  if (inputDeviceCurrent==xorg.InputDevices().GetSize() && inputDeviceCurrent!=0) inputDeviceCurrent--;
  UpdateServerLayout();
  SetSaved(false);
}

void GuiFrame::CopyInputDevice(void)
{
  if (xorg.InputDevices().GetSize()==0) return;

  if (!CheckInputDevice()) return;
  AcceptInputDevice();

  InputDevice tempDevice(xorg.InputDevices()[inputDeviceCurrent]);
  tempDevice.SetId(xorg.InputDevices()[inputDeviceCurrent].GetId()+_(" (Copy)"));

  xorg.InputDevices().Add(tempDevice);
  inputDeviceCurrent=xorg.InputDevices().GetSize()-1;
  UpdateServerLayout();
  SetSaved(false);
}

void GuiFrame::AddInputDeviceOption(void)
{
  if (!CheckInputDeviceOption()) return;
  xorg.InputDevices()[inputDeviceCurrent].AddOption(Option());
  inputDeviceOptionCurrent=xorg.InputDevices()[inputDeviceCurrent].GetOptionSize()-1;
  SetSaved(false);
}

void GuiFrame::DelInputDeviceOption(void)
{
  if (xorg.InputDevices()[inputDeviceCurrent].GetOptionSize()==0) return;

  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.InputDevices()[inputDeviceCurrent].RemoveOption(inputDeviceOptionCurrent);
  if (inputDeviceOptionCurrent==xorg.InputDevices()[inputDeviceCurrent].GetOptionSize() && inputDeviceOptionCurrent!=0) inputDeviceOptionCurrent--;
  SetSaved(false);
}

void GuiFrame::DeActInputDeviceOption(void)
{
  if (xorg.InputDevices()[inputDeviceCurrent].GetOptionSize()==0) return;

  xorg.InputDevices()[inputDeviceCurrent].GetOption(inputDeviceOptionCurrent).SetActive(!xorg.InputDevices()[inputDeviceCurrent].GetOption(inputDeviceOptionCurrent).IsActive());
  SetSaved(false);
}

void GuiFrame::AddMonitor(void)
{
  if (!CheckMonitor()) return;
  AcceptMonitor();

  wxString tempStr;
  Monitor tempDevice;

  tempStr.Printf(_T("Default Monitor %d"), xorg.Monitors().GetSize()+1);
  tempDevice.SetId(tempStr);

  xorg.Monitors().Add(tempDevice);
  monitorCurrent=xorg.Monitors().GetSize()-1;
  UpdateScreen();
  SetSaved(false);
}

void GuiFrame::DelMonitor(void)
{
  if (xorg.Monitors().GetSize()==0) return;

  if (wxMessageBox(_("Do you really want to delete this monitor?"), _("Question: Delete Monitor?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  bool found=false;    // entry found
  int i;      // loop counter

  // check if there are some screen that use this monitor
  for (i=0; i<xorg.Screens().GetSize(); i++)
  {
    if (xorg.Screens()[i].GetMonitor()==xorg.Monitors()[monitorCurrent].GetId())
    {
      found=true;
      break; // leave for i-loop
    }
  }

  if (found)
  {
    wxString tempStr;
    tempStr << _("This monitor is still used by screen \"") << xorg.Screens()[i].GetId() << _T("\".");
    tempStr << _("\n\nShould I delete this references too? ");
    tempStr << _("Otherwise you must manually resolve the dependence before deleting this monitor.");

    if (wxMessageBox(tempStr, _("Error: Monitor used"), wxICON_ERROR | wxYES_NO | wxNO_DEFAULT)==wxNO) return;
    else
    {
      for (i=0; i<xorg.Screens().GetSize(); i++)
      {
        if (xorg.Screens()[i].GetMonitor()==xorg.Monitors()[monitorCurrent].GetId())
        {
          xorg.Screens()[i].SetMonitor(wxEmptyString);
        }
      }
    }
  }

  xorg.Monitors().Remove(monitorCurrent);
  if (monitorCurrent==xorg.Monitors().GetSize() && monitorCurrent!=0) monitorCurrent--;
  UpdateScreen();
  SetSaved(false);
}

void GuiFrame::CopyMonitor(void)
{
  if (xorg.Monitors().GetSize()==0) return;

  if (!CheckMonitor()) return;
  AcceptMonitor();

  Monitor tempDevice(xorg.Monitors()[monitorCurrent]);
  tempDevice.SetId(xorg.Monitors()[monitorCurrent].GetId()+_(" (Copy)"));

  xorg.Monitors().Add(tempDevice);
  monitorCurrent=xorg.Monitors().GetSize()-1;
  UpdateScreen();
  SetSaved(false);
}

void GuiFrame::AddMonitorOption(void)
{
  if (!CheckMonitorOption()) return;
  xorg.Monitors()[monitorCurrent].AddOption(Option());
  monitorOptionCurrent=xorg.Monitors()[monitorCurrent].GetOptionSize()-1;
  SetSaved(false);
}

void GuiFrame::DelMonitorOption(void)
{
  if (xorg.Monitors()[monitorCurrent].GetOptionSize()==0) return;

  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.Monitors()[monitorCurrent].RemoveOption(monitorOptionCurrent);
  if (monitorOptionCurrent==xorg.Monitors()[monitorCurrent].GetOptionSize() && monitorOptionCurrent!=0) monitorOptionCurrent--;
  SetSaved(false);
}

void GuiFrame::DeActMonitorOption(void)
{
  if (xorg.Monitors()[monitorCurrent].GetOptionSize()==0) return;

  xorg.Monitors()[monitorCurrent].GetOption(monitorOptionCurrent).SetActive(!xorg.Monitors()[monitorCurrent].GetOption(monitorOptionCurrent).IsActive());
  SetSaved(false);
}

void GuiFrame::AddMonitorModeline(void)
{
  if (!CheckMonitorModeline()) return;
  xorg.Monitors()[monitorCurrent].AddModeline(Option());
  monitorModelineCurrent=xorg.Monitors()[monitorCurrent].GetModelineSize()-1;
  SetSaved(false);
}

void GuiFrame::DelMonitorModeline(void)
{
  if (xorg.Monitors()[monitorCurrent].GetModelineSize()==0) return;

  if (wxMessageBox(_("Do you really want to delete this modeline?"), _("Question: Delete Modeline?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.Monitors()[monitorCurrent].RemoveModeline(monitorModelineCurrent);
  if (monitorModelineCurrent==xorg.Monitors()[monitorCurrent].GetModelineSize() && monitorModelineCurrent!=0) monitorModelineCurrent--;
  SetSaved(false);
}

void GuiFrame::DeActMonitorModeline(void)
{
  if (xorg.Monitors()[monitorCurrent].GetModelineSize()==0) return;

  xorg.Monitors()[monitorCurrent].GetModeline(monitorModelineCurrent).SetActive(!xorg.Monitors()[monitorCurrent].GetModeline(monitorModelineCurrent).IsActive());
  SetSaved(false);
}

void GuiFrame::AddScreen(void)
{
  if (!CheckScreen()) return;
  AcceptScreen();

  if (xorg.Devices().GetSize()==0)
  {
    wxMessageBox(_("No devices available to add!"), _("Error in Screen"), wxICON_ERROR);
    return;
  }

  if (xorg.Monitors().GetSize()==0)
  {
    wxMessageBox(_("No monitors available to add!"), _("Error in Screen"), wxICON_ERROR);
    return;
  }

  wxString tempStr;
  Screen tempDevice;

  tempStr.Printf(_T("Default Screen %d"), xorg.Screens().GetSize()+1);
  tempDevice.SetId(tempStr);
  tempDevice.SetDevice(xorg.Devices()[0].GetId());   // get first device
  tempDevice.SetMonitor(xorg.Monitors()[0].GetId());   // get first monitor

  xorg.Screens().Add(tempDevice);
  screenCurrent=xorg.Screens().GetSize()-1;
  UpdateServerLayout();
  SetSaved(false);
}

void GuiFrame::DelScreen(void)
{
  if (xorg.Screens().GetSize()==0) return;

  if (wxMessageBox(_("Do you really want to delete this screen?"), _("Question: Delete Screen?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  bool found=false;   // entry found
  int i,j;       // loop counter

  // check if there are some server layout that use this screen
  for (i=0; i<xorg.ServerLayouts().GetSize(); i++)
  {
    for (j=0; j<xorg.ServerLayouts()[i].GetScreenSize(); j++)
    {
      if (xorg.ServerLayouts()[i].GetScreen(j)==xorg.Screens()[screenCurrent].GetId() || xorg.ServerLayouts()[i].GetSecondScreen(j)==xorg.Screens()[screenCurrent].GetId())
      {
        found=true;
        break; // leave for j-loop
      }
    }
    if (found) break; // leave for i-loop
  }

  if (found)
  {
    wxString tempStr;
    tempStr << _("This screen is still used by server layout \"") << xorg.ServerLayouts()[i].GetId() << _T("\".");
    tempStr << _("\n\nShould I delete this references too? ");
    tempStr << _("Otherwise you must manually resolve the dependence before deleting this screen.");

    if (wxMessageBox(tempStr, _("Error: Screen used"), wxICON_ERROR | wxYES_NO | wxNO_DEFAULT)==wxNO) return;
    else
    {
      for (i=0; i<xorg.ServerLayouts().GetSize(); i++)
      {
        for (j=0; j<xorg.ServerLayouts()[i].GetScreenSize(); j++)
        {
          if (xorg.ServerLayouts()[i].GetScreen(j)==xorg.Screens()[screenCurrent].GetId() || xorg.ServerLayouts()[i].GetSecondScreen(j)==xorg.Screens()[screenCurrent].GetId())
          {
            xorg.ServerLayouts()[i].RemoveScreen(j);
            if (i==serverLayoutCurrent && j==serverLayoutScreenCurrent && j==xorg.ServerLayouts()[i].GetScreenSize() && j!=0) serverLayoutScreenCurrent--;
            j--;
          }
        }
      }
    }
  }

  xorg.Screens().Remove(screenCurrent);
  if (screenCurrent==xorg.Screens().GetSize() && screenCurrent!=0) screenCurrent--;
  UpdateServerLayout();
  SetSaved(false);
}

void GuiFrame::CopyScreen(void)
{
  if (xorg.Screens().GetSize()==0) return;

  if (!CheckScreen()) return;
  AcceptScreen();

  Screen tempDevice(xorg.Screens()[screenCurrent]);
  tempDevice.SetId(xorg.Screens()[screenCurrent].GetId()+_(" (Copy)"));
  xorg.Screens().Add(tempDevice);
  screenCurrent=xorg.Screens().GetSize()-1;
  UpdateServerLayout();
  SetSaved(false);
}


void GuiFrame::AddScreenOption(void)
{
  if (!CheckScreenOption()) return;
  xorg.Screens()[screenCurrent].AddOption(Option());
  screenOptionCurrent=xorg.Screens()[screenCurrent].GetOptionSize()-1;
  SetSaved(false);
}

void GuiFrame::DelScreenOption(void)
{
  if (xorg.Screens()[screenCurrent].GetOptionSize()==0) return;
  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.Screens()[screenCurrent].RemoveOption(screenOptionCurrent);
  if (screenOptionCurrent==xorg.Screens()[screenCurrent].GetOptionSize() && screenOptionCurrent!=0) screenOptionCurrent--;
  SetSaved(false);
}

void GuiFrame::DeActScreenOption(void)
{
  if (xorg.Screens()[screenCurrent].GetOptionSize()==0) return;

  xorg.Screens()[screenCurrent].GetOption(screenOptionCurrent).SetActive(!xorg.Screens()[screenCurrent].GetOption(screenOptionCurrent).IsActive());
  SetSaved(false);
}

void GuiFrame::AddScreenDisplay(void)
{
  if (!CheckScreenDisplay()) return;

  wxString tempStr;
  Display tempDevice;

  xorg.Screens()[screenCurrent].AddDisplay(tempDevice);
  screenDisplayCurrent=xorg.Screens()[screenCurrent].GetDisplaySize()-1;
  SetSaved(false);
}

void GuiFrame::DelScreenDisplay(void)
{
  if (xorg.Screens()[screenCurrent].GetDisplaySize()==0) return;
  if (wxMessageBox(_("Do you really want to delete this display?"), _("Question: Delete Display?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.Screens()[screenCurrent].RemoveDisplay(screenDisplayCurrent);
  if (screenDisplayCurrent==xorg.Screens()[screenCurrent].GetDisplaySize() && screenDisplayCurrent!=0) screenDisplayCurrent--;
  SetSaved(false);
}

void GuiFrame::AddScreenDisplayMode(void)
{
  if (!CheckScreenDisplayMode());
  xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).AddMode(wxEmptyString);
  screenDisplayModeCurrent=xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetModeSize()-1;
  SetSaved(false);
}

void GuiFrame::DelScreenDisplayMode(void)
{
  if (xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetModeSize()==0) return;
  if (wxMessageBox(_("Do you really want to delete this mode?"), _("Question: Delete Mode?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).RemoveMode(screenDisplayModeCurrent);
  if (screenDisplayModeCurrent==xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetModeSize() && screenDisplayModeCurrent!=0) screenDisplayModeCurrent--;
  SetSaved(false);
}

void GuiFrame::AddScreenDisplayOption(void)
{
  if (!CheckScreenDisplayOption()) return;
  xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).AddOption(Option());
  screenDisplayOptionCurrent=xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOptionSize()-1;
  SetSaved(false);
}

void GuiFrame::DelScreenDisplayOption(void)
{
  if (xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOptionSize()==0) return;
  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).RemoveOption(screenDisplayOptionCurrent);
  if (screenDisplayOptionCurrent==xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOptionSize() && screenDisplayOptionCurrent!=0) screenDisplayOptionCurrent--;
  SetSaved(false);
}

void GuiFrame::DeActScreenDisplayOption(void)
{
  if (xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOptionSize()==0) return;

  xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOption(screenDisplayOptionCurrent).SetActive(!xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOption(screenDisplayOptionCurrent).IsActive());
  SetSaved(false);
}

void GuiFrame::AddServerLayout(void)
{
  if (!CheckServerLayout()) return;

  wxString tempStr;
  ServerLayout tempDevice;

  tempStr.Printf(_T("Default Layout %d"), xorg.ServerLayouts().GetSize()+1);
  tempDevice.SetId(tempStr);

  xorg.ServerLayouts().Add(tempDevice);
  serverLayoutCurrent=xorg.ServerLayouts().GetSize()-1;
  SetSaved(false);
}

void GuiFrame::DelServerLayout(void)
{
  if (xorg.ServerLayouts().GetSize()==0) return;
  if (wxMessageBox(_("Do you really want to delete this server layout?"), _("Question: Delete Server Layout?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.ServerLayouts().Remove(serverLayoutCurrent);
  if (serverLayoutCurrent==xorg.ServerLayouts().GetSize() && serverLayoutCurrent!=0) serverLayoutCurrent--;
  SetSaved(false);
}

void GuiFrame::CopyServerLayout(void)
{
  if (xorg.ServerLayouts().GetSize()==0) return;

  if (!CheckServerLayout()) return;

  wxString tempStr;
  ServerLayout tempDevice(xorg.ServerLayouts()[serverLayoutCurrent]);
  tempDevice.SetId(xorg.ServerLayouts()[serverLayoutCurrent].GetId()+_(" (Copy)"));
  xorg.ServerLayouts().Add(tempDevice);
  serverLayoutCurrent=xorg.ServerLayouts().GetSize()-1;
  SetSaved(false);
}


void GuiFrame::AddServerLayoutScreen(void)
{
  if (!CheckServerLayoutScreen()) return;

  if (xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize()>=xorg.Screens().GetSize())
  {
    wxMessageBox(_("No (more) screens available to add!"), _("Error in Server Layout"), wxICON_ERROR);
    return;
  }

  // search correct screen to add
  wxString tempStr;
  int i, j;

  for (i=0; i<xorg.Screens().GetSize(); i++)
  {
    tempStr=xorg.Screens()[i].GetId();
    for (j=0; j<xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize(); j++)
    {
      if (tempStr==xorg.ServerLayouts()[serverLayoutCurrent].GetScreen(j)) break;
    }
    if (j==xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize())
    {
      break;
    }
  }
  xorg.ServerLayouts()[serverLayoutCurrent].AddScreen(tempStr);
  serverLayoutScreenCurrent=xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize()-1;
  SetSaved(false);
}

void GuiFrame::DelServerLayoutScreen(void)
{
  if (xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize()==0) return;
  if (wxMessageBox(_("Do you really want to delete this screen?"), _("Question: Delete Screen?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.ServerLayouts()[serverLayoutCurrent].RemoveScreen(serverLayoutScreenCurrent);
  if (serverLayoutScreenCurrent==xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize() && serverLayoutScreenCurrent!=0) serverLayoutScreenCurrent--;
  SetSaved(false);
}

void GuiFrame::AddServerLayoutInputDevice(void)
{
  if (!CheckServerLayoutInputDevice()) return;

  if (xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize()>=xorg.InputDevices().GetSize())
  {
    wxMessageBox(_("No (more) input devices available to add!"), _("Error in Server Layout"), wxICON_ERROR);
    return;
  }

  // search correct screen to add
  wxString tempStr;
  int i, j;

  for (i=0; i<xorg.InputDevices().GetSize(); i++)
  {
    tempStr=xorg.InputDevices()[i].GetId();
    for (j=0; j<xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize(); j++)
    {
      if (tempStr==xorg.ServerLayouts()[serverLayoutCurrent].GetInputDevice(j)) break;
    }
    if (j==xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize())
    {
      break;
    }
  }

  xorg.ServerLayouts()[serverLayoutCurrent].AddInputDevice(tempStr);
  serverLayoutIDevCurrent=xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize()-1;
  SetSaved(false);
}

void GuiFrame::DelServerLayoutInputDevice(void)
{
  if (xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize()==0) return;
  if (wxMessageBox(_("Do you really want to delete this input device?"), _("Question: Delete Input Device?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.ServerLayouts()[serverLayoutCurrent].RemoveInputDevice(serverLayoutIDevCurrent);
  if (serverLayoutIDevCurrent==xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize() && serverLayoutIDevCurrent!=0) serverLayoutIDevCurrent--;
  SetSaved(false);
}

void GuiFrame::AddServerLayoutOption(void)
{
  if (!CheckServerLayoutOption()) return;

  xorg.ServerLayouts()[serverLayoutCurrent].AddOption(Option());
  serverLayoutOptionCurrent=xorg.ServerLayouts()[serverLayoutCurrent].GetOptionSize()-1;
  SetSaved(false);
}

void GuiFrame::DelServerLayoutOption(void)
{
  if (xorg.ServerLayouts()[serverLayoutCurrent].GetOptionSize()==0) return;
  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.ServerLayouts()[serverLayoutCurrent].RemoveOption(serverLayoutOptionCurrent);
  if (serverLayoutOptionCurrent==xorg.ServerLayouts()[serverLayoutCurrent].GetOptionSize() && serverLayoutOptionCurrent!=0) serverLayoutOptionCurrent--;
  SetSaved(false);
}

void GuiFrame::DeActServerLayoutOption(void)
{
  if (xorg.ServerLayouts()[serverLayoutCurrent].GetOptionSize()==0) return;

  xorg.ServerLayouts()[serverLayoutCurrent].GetOption(serverLayoutOptionCurrent).SetActive(!xorg.ServerLayouts()[serverLayoutCurrent].GetOption(serverLayoutOptionCurrent).IsActive());
  SetSaved(false);
}

void GuiFrame::AddServerLayoutInputDeviceOption(void)
{
  if (!CheckServerLayoutInputDeviceOption()) return;

  xorg.ServerLayouts()[serverLayoutCurrent].AddInputDeviceOption(serverLayoutIDevCurrent, Option());
  serverLayoutIDevOptionCurrent=xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOptionSize(serverLayoutIDevCurrent)-1;
  SetSaved(false);
}

void GuiFrame::DelServerLayoutInputDeviceOption(void)
{
  if (xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOptionSize(inputDeviceCurrent)==0) return;
  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.ServerLayouts()[serverLayoutCurrent].RemoveInputDeviceOption(serverLayoutIDevCurrent, serverLayoutIDevOptionCurrent);
  if (serverLayoutIDevOptionCurrent==xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOptionSize(serverLayoutIDevCurrent) && serverLayoutIDevOptionCurrent!=0) serverLayoutIDevOptionCurrent--;
  SetSaved(false);
}

void GuiFrame::AddServerFlagOption(void)
{
  if (!CheckServerFlagOption()) return;

  xorg.ServerFlags().AddOption(Option());
  serverFlagCurrent=xorg.ServerFlags().GetOptionSize()-1;
  SetSaved(false);
}

void GuiFrame::DelServerFlagOption(void)
{
  if (xorg.ServerFlags().GetOptionSize()==0) return;
  if (wxMessageBox(_("Do you really want to delete this server flag?"), _("Question: Delete Server Flag?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.ServerFlags().RemoveOption(serverFlagCurrent);
  if (serverFlagCurrent==xorg.ServerFlags().GetOptionSize() && serverFlagCurrent!=0) serverFlagCurrent--;
  SetSaved(false);
}

void GuiFrame::DeActServerFlagOption(void)
{
  if (xorg.ServerFlags().GetOptionSize()==0) return;

  xorg.ServerFlags().GetOption(serverFlagCurrent).SetActive(!xorg.ServerFlags().GetOption(serverFlagCurrent).IsActive());
  SetSaved(false);
}

void GuiFrame::AddExtension(void)
{
  if (!CheckExtension()) return;

  xorg.Extensions().AddOption(Option());
  extensionCurrent=xorg.Extensions().GetOptionSize()-1;
  SetSaved(false);
}

void GuiFrame::DelExtension(void)
{
  if (xorg.Extensions().GetOptionSize()==0) return;
  if (wxMessageBox(_("Do you really want to delete this extension?"), _("Question: Delete Extension?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.Extensions().RemoveOption(extensionCurrent);
  if (extensionCurrent==xorg.Extensions().GetOptionSize() && extensionCurrent!=0) extensionCurrent--;
  SetSaved(false);
}

void GuiFrame::DeActExtension(void)
{
  if (xorg.Extensions().GetOptionSize()==0) return;

  xorg.Extensions().GetOption(extensionCurrent).SetActive(!xorg.Extensions().GetOption(extensionCurrent).IsActive());
  SetSaved(false);
}

void GuiFrame::AddFontPath(void)
{
  if (!CheckFontPath()) return;

  xorg.Files().AddFontPath(wxEmptyString, true);
  fontPathCurrent=xorg.Files().GetFontPathSize()-1;
  SetSaved(false);
}

void GuiFrame::DelFontPath(void)
{
  if (xorg.Files().GetFontPathSize()==0) return;
  if (wxMessageBox(_("Do you really want to delete this font path?"), _("Question: Delete Font Path?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.Files().RemoveFontPath(fontPathCurrent);
  if (fontPathCurrent==xorg.Files().GetFontPathSize() && fontPathCurrent!=0) fontPathCurrent--;
  SetSaved(false);
}

void GuiFrame::DeActFontPath(void)
{
  if (xorg.Files().GetFontPathSize()==0) return;
  xorg.Files().SetFontPathActive(fontPathCurrent, !xorg.Files().FontPathIsActive(fontPathCurrent));
  SetSaved(false);
}

void GuiFrame::AddModulePath(void)
{
  if (!CheckModulePath()) return;

  xorg.Files().AddModulePath(wxEmptyString, true);
  modulePathCurrent=xorg.Files().GetModulePathSize()-1;
  SetSaved(false);
}

void GuiFrame::DelModulePath(void)
{
  if (xorg.Files().GetModulePathSize()==0) return;
  if (wxMessageBox(_("Do you really want to delete this module path?"), _("Question: Delete Module Path?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.Files().RemoveModulePath(modulePathCurrent);
  if (modulePathCurrent==xorg.Files().GetModulePathSize() && modulePathCurrent!=0) modulePathCurrent--;
  SetSaved(false);
}

void GuiFrame::DeActModulePath(void)
{
  if (xorg.Files().GetModulePathSize()==0) return;
  xorg.Files().SetModulePathActive(modulePathCurrent, !xorg.Files().ModulePathIsActive(modulePathCurrent));
  SetSaved(false);
}

void GuiFrame::AddRgbPath(void)
{
  if (!CheckRgbPath()) return;

  xorg.Files().AddRgbPath(wxEmptyString, true);
  rgbPathCurrent=xorg.Files().GetRgbPathSize()-1;
  SetSaved(false);
}

void GuiFrame::DelRgbPath(void)
{
  if (xorg.Files().GetRgbPathSize()==0) return;
  if (wxMessageBox(_("Do you really want to delete this RGB path?"), _("Question: Delete RGB Path?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.Files().RemoveRgbPath(rgbPathCurrent);
  if (rgbPathCurrent==xorg.Files().GetRgbPathSize() && rgbPathCurrent!=0) rgbPathCurrent--;
  SetSaved(false);
}

void GuiFrame::DeActRgbPath(void)
{
  if (xorg.Files().GetRgbPathSize()==0) return;
  xorg.Files().SetRgbPathActive(rgbPathCurrent, !xorg.Files().RgbPathIsActive(rgbPathCurrent));
  SetSaved(false);
}

void GuiFrame::AddModule(void)
{
  if (!CheckModule()) return;

  xorg.Modules().Add(wxEmptyString, true);
  moduleCurrent=xorg.Modules().GetSize()-1;
  SetSaved(false);
}

void GuiFrame::DelModule(void)
{
  if (xorg.Modules().GetSize()==0) return;
  if (wxMessageBox(_("Do you really want to delete this module?"), _("Question: Delete Module?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.Modules().Remove(moduleCurrent);
  if (moduleCurrent==xorg.Modules().GetSize() && moduleCurrent!=0) moduleCurrent--;
  SetSaved(false);
}

void GuiFrame::DeActModule(void)
{
  if (xorg.Modules().GetSize()==0) return;

  xorg.Modules().SetActive(moduleCurrent, !xorg.Modules().IsActive(moduleCurrent));
  SetSaved(false);
}

void GuiFrame::AddModuleOption(void)
{
  if (!CheckModuleOption()) return;

  xorg.Modules().AddOption(moduleCurrent, Option());
  moduleOptionCurrent=xorg.Modules().GetOptionSize(moduleCurrent)-1;
  SetSaved(false);
}

void GuiFrame::DelModuleOption(void)
{
  if (xorg.Modules().GetOptionSize(moduleCurrent)==0) return;
  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return;

  xorg.Modules().RemoveOption(moduleCurrent, moduleOptionCurrent);
  if (moduleOptionCurrent==xorg.Modules().GetOptionSize(moduleCurrent) && moduleOptionCurrent!=0) moduleOptionCurrent--;
  SetSaved(false);
}

void GuiFrame::DeActModuleOption(void)
{
  if (xorg.Modules().GetOptionSize(moduleCurrent)==0) return;

  xorg.Modules().GetOption(moduleCurrent, moduleOptionCurrent).SetActive(!xorg.Modules().GetOption(moduleCurrent, moduleOptionCurrent).IsActive());
  SetSaved(false);
}


// changes

void GuiFrame::OnTabChange(wxNotebookEvent& event)
{    // some tab has changed, sav changes if possible

  if (event.GetSelection()!=event.GetOldSelection())  // tab has really changed
  {
    switch(event.GetOldSelection())
    {
      case 0:  // DEVICE
        if (!CheckDeviceTab()) event.Veto();
        else AcceptDevice();
        break;
      case 1:  // MONITOR
        if (!CheckMonitorTab()) event.Veto();
        else AcceptMonitor();
        break;
      case 2:  // INPUTDEVICE
        if (!CheckInputDeviceTab()) event.Veto();
        else AcceptInputDevice();
        break;
      case 3:  // SCREEN
        if (!CheckScreenTab()) event.Veto();
        else AcceptScreen();
        break;
      case 4:  // SERVERLAYOUT, SERVERFLAGS
        if (!CheckServerTab()) event.Veto();
        break;
      case 5:  // EXTENSIONS, MODULES, FILES, DRI
        if (!CheckOtherSectionTab()) event.Veto();
        break;
      default:
        break;
    } // switch
  }

}

bool GuiFrame::CheckDeviceTab(void)
{
  if (!CheckDevice()) return(false);
  return(true);
}

bool GuiFrame::CheckDevice(void)
{
  if (xorg.Devices().GetSize()==0) return(true);

  long tempL=0;
  wxString tempStr;

  // check if empty
  if (deviceIdentifierText.IsEmpty())
  {
    wxMessageBox(_("Device identifier must not be empty!"), _("Error in Device"), wxICON_ERROR);
    deviceIdentifier->SetSelection(deviceCurrent);
    // deviceIdentifier->SetFocus();
    return(false);
  }

  // check if unique
  for (int i=0; i<xorg.Devices().GetSize(); i++)
  {
    if (i!=deviceCurrent && xorg.Devices()[i].GetId()==deviceIdentifierText)
    {
      wxMessageBox(_("Device identifier already exists!"), _("Error in Device"), wxICON_ERROR);
      deviceIdentifier->SetSelection(deviceCurrent);
      // deviceIdentifier->SetFocus();
      return(false);
    }
  }

  // check if empty
  if (deviceDriver->GetValue().IsEmpty())
  {
    wxMessageBox(_("Device driver must not be empty!"), _("Error in Device"), wxICON_ERROR);
    deviceIdentifier->SetSelection(deviceCurrent);
    // deviceDriver->SetFocus();
    return(false);
  }

  if (!deviceScreen->GetValue().IsEmpty())
  {
    tempStr=deviceScreen->GetValue();
    tempStr.Trim(false);
    tempStr.Trim(true);
    tempStr.ToLong(&tempL);

    if (!tempStr.IsNumber() || tempL<0)
    {
      wxMessageBox(_("Device screen must be a non-negative integer!"), _("Error in Device"), wxICON_ERROR);
      deviceIdentifier->SetSelection(deviceCurrent);
      deviceScreen->SetFocus();
      return(false);
    }
    tempStr.Printf(_T("%d"), tempL);
    deviceScreen->SetValue(tempStr);
  }

  if (!deviceVideoRam->GetValue().IsEmpty())
  {
    tempStr=deviceVideoRam->GetValue();
    tempStr.Trim(false);
    tempStr.Trim(true);
    tempStr.ToLong(&tempL);

    if (!tempStr.IsNumber() || tempL<0)
    {
      wxMessageBox(_("Device video ram must be a non-negative integer!"), _("Error in Device"), wxICON_ERROR);
      deviceIdentifier->SetSelection(deviceCurrent);
      deviceVideoRam->SetFocus();
      return(false);
    }
    tempStr.Printf(_T("%d"), tempL);
    deviceVideoRam->SetValue(tempStr);
  }

  if (!CheckDeviceOption()) return(false);
  return(true);
}

bool GuiFrame::CheckDeviceOption(void)
{
  if (xorg.Devices()[deviceCurrent].GetOptionSize()==0) return(true);

  if (xorg.Devices()[deviceCurrent].GetOption(deviceOptionCurrent).GetName().IsEmpty())
  {
    wxMessageBox(_("Device option must not be empty!"), _("Error in Device"), wxICON_ERROR);
    deviceIdentifier->SetSelection(deviceCurrent);
    deviceOption->SetSelection(deviceOptionCurrent);
    deviceOption->SetFocus();
    return(false);
  }

  for (int i=0; i<xorg.Devices()[deviceCurrent].GetOptionSize(); i++)
  {
    if (i!=deviceOptionCurrent && xorg.Devices()[deviceCurrent].GetOption(i).GetName()==xorg.Devices()[deviceCurrent].GetOption(deviceOptionCurrent).GetName())
    {
      wxMessageBox(_("Device option already exists!"), _("Error in Device"), wxICON_ERROR);
      deviceIdentifier->SetSelection(deviceCurrent);
      deviceOption->SetSelection(deviceOptionCurrent);
      deviceOption->SetFocus();
      return(false);
    }
  }

  // check options
  wxString filename=settings.GetOptionDir()+_T("/devices/")+deviceDriver->GetValue()+_T(".xml");
  deviceDriversOption.Load(filename);

  // search selected option in list
  int pos=deviceDriversOption.Find(xorg.Devices()[deviceCurrent].GetOption(deviceOptionCurrent).GetName());

  if (pos==-1) return(true); // no error

  if (deviceDriversOption[pos].GetValue()==_T("boolean"))
  {
    if (deviceOptValue->GetValue()!=_T("1") && deviceOptValue->GetValue()!=_T("on") && deviceOptValue->GetValue()!=_T("true") && deviceOptValue->GetValue()!=_T("yes")
     && deviceOptValue->GetValue()!=_T("0") && deviceOptValue->GetValue()!=_T("off") && deviceOptValue->GetValue()!=_T("false") && deviceOptValue->GetValue()!=_T("no"))
    {
      wxMessageBox(_("Device option must be boolean!"), _("Error in Device"), wxICON_ERROR);
      deviceIdentifier->SetSelection(deviceCurrent);
      deviceOption->SetSelection(deviceOptionCurrent);
      deviceOptValue->SetFocus();
      return(false);
    }
  }
  else
  {
    if (deviceDriversOption[pos].GetValue()==_T("integer"))
    {
      bool isInt=true;

      if (deviceOptValue->GetValue().StartsWith(_T("0x")))
      {  // possible hexadecimal number

        for (int i=2; i<(int)deviceOptValue->GetValue().Len(); i++)
        {
          int c=deviceOptValue->GetValue()[i];

          if (c!='0' && c!='1' && c!='2' && c!='3' && c!='4' && c!='5' && c!='6'
           && c!='7' && c!='8' && c!='9' && c!='0' && c!='A' && c!='B' && c!='C'
           && c!='D' && c!='E' && c!='F' && c!='a' && c!='b' && c!='c' && c!='d'
           && c!='e' && c!='f')
          {
            isInt=false;
            break;
          }
        }
      }
      else if (!deviceOptValue->GetValue().IsNumber())
      {
        isInt=false;
      }

      if (!isInt)
      {
        wxMessageBox(_("Device option must be integer!"), _("Error in Device"), wxICON_ERROR);
        deviceIdentifier->SetSelection(deviceCurrent);
        deviceOption->SetSelection(deviceOptionCurrent);
        deviceOptValue->SetFocus();
        return(false);
      }
    }

    if (deviceDriversOption[pos].IsStrict())
    {
      if (deviceDriversOption[pos].GetArguments().Index(deviceOptValue->GetValue())==wxNOT_FOUND)
      {
        wxMessageBox(_("Device option must be in argument list!"), _("Error in Device"), wxICON_ERROR);
        deviceIdentifier->SetSelection(deviceCurrent);
        deviceOption->SetSelection(deviceOptionCurrent);
        deviceOptValue->SetFocus();
        return(false);
      }
    }
  }

  return(true);
}

bool GuiFrame::CheckMonitorTab(void) const
{
  if (!CheckMonitor()) return(false);
  return(true);
}

bool GuiFrame::CheckMonitor(void) const
{
  if (xorg.Monitors().GetSize()==0) return(true);

  // check if empty
  if (monitorIdentifierText.IsEmpty())
  {
    wxMessageBox(_("Monitor identifier must not be empty!"), _("Error in Monitor"), wxICON_ERROR);
    monitorIdentifier->SetSelection(monitorCurrent);
    monitorIdentifier->SetFocus();
    return(false);
  }

  // check if unique
  for (int i=0; i<xorg.Monitors().GetSize(); i++)
  {
    if (i!=monitorCurrent && xorg.Monitors()[i].GetId()==monitorIdentifierText)
    {
      wxMessageBox(_("Monitor identifier already exists!"), _("Error in Monitor"), wxICON_ERROR);
      monitorIdentifier->SetSelection(monitorCurrent);
      monitorIdentifier->SetFocus();
      return(false);
    }
  }

  if (!CheckMonitorOption()) return(false);
  if (!CheckMonitorModeline()) return(false);

  return(true);
}

bool GuiFrame::CheckMonitorOption(void) const
{
  if (xorg.Monitors()[monitorCurrent].GetOptionSize()==0) return(true);

  if (xorg.Monitors()[monitorCurrent].GetOption(monitorOptionCurrent).GetName().IsEmpty())
  {
    wxMessageBox(_("Monitor option must not be empty!"), _("Error in Monitor"), wxICON_ERROR);
    monitorIdentifier->SetSelection(monitorCurrent);
    monitorOption->SetSelection(monitorOptionCurrent);
    monitorOption->SetFocus();
    return(false);
  }

  for (int i=0; i<xorg.Monitors()[monitorCurrent].GetOptionSize(); i++)
  {
    if (i!=monitorOptionCurrent && xorg.Monitors()[monitorCurrent].GetOption(i).GetName()==xorg.Monitors()[monitorCurrent].GetOption(monitorOptionCurrent).GetName())
    {
      wxMessageBox(_("Monitor option already exists!"), _("Error in Monitor"), wxICON_ERROR);
      monitorIdentifier->SetSelection(monitorCurrent);
      monitorOption->SetSelection(monitorOptionCurrent);
      monitorOption->SetFocus();
      return(false);
    }
  }

  return(true);
}

bool GuiFrame::CheckMonitorModeline(void) const
{
  if (xorg.Monitors()[monitorCurrent].GetModelineSize()==0) return(true);

  wxString tempStr;
  int number=0;
  int pos=0;

  if (xorg.Monitors()[monitorCurrent].GetModeline(monitorModelineCurrent).GetName().IsEmpty())
  {
    wxMessageBox(_("Monitor modeline name must not be empty!"), _("Error in Monitor"), wxICON_ERROR);
    monitorIdentifier->SetSelection(monitorCurrent);
    monitorModelineName->SetSelection(monitorModelineCurrent);
    monitorModelineName->SetFocus();
    return(false);
  }

  for (int i=0; i<xorg.Monitors()[monitorCurrent].GetOptionSize(); i++)
  {
    if (i!=monitorModelineCurrent && xorg.Monitors()[monitorCurrent].GetModeline(i).GetName()==xorg.Monitors()[monitorCurrent].GetModeline(monitorModelineCurrent).GetName())
    {
      wxMessageBox(_("Monitor modeline already exists!"), _("Error in Monitor"), wxICON_ERROR);
      monitorIdentifier->SetSelection(monitorCurrent);
      monitorModelineName->SetSelection(monitorModelineCurrent);
      monitorModelineName->SetFocus();
      return(false);
    }
  }

  tempStr=monitorModelineOption->GetValue();
  tempStr.Trim(true);
  tempStr.Trim(false);

  // count entries of modeline, must be at least nine + options
  do
  {
    pos=tempStr.Find(' ');
    if (pos!=-1)
    {
      number++;
      tempStr=tempStr.Right(tempStr.Length()-pos-1);  // truncate
      tempStr.Trim(false);    // strip spaces from left
    }
  } while (pos!=-1);

  if (number<8)
  {
    wxMessageBox(_("Monitor modeline must have 9 entries at least!"), _("Error in Monitor"), wxICON_ERROR);
    monitorIdentifier->SetSelection(monitorCurrent);
    monitorModelineName->SetSelection(monitorModelineCurrent);
    monitorModelineOption->SetFocus();
    return(false);
  }
  return(true);
}

bool GuiFrame::CheckInputDeviceTab(void)
{
  if (!CheckInputDevice()) return(false);
  return(true);
}

bool GuiFrame::CheckInputDevice(void)
{
  if (xorg.InputDevices().GetSize()==0) return(true);

  // check if empty
  if (inputDeviceIdentifierText.IsEmpty())
  {
    wxMessageBox(_("Device identifier must not be empty!"), _("Error in Input Device"), wxICON_ERROR);
    inputDeviceIdentifier->SetSelection(inputDeviceCurrent);
    inputDeviceIdentifier->SetFocus();
    return(false);
  }

  // check if unique
  for (int i=0; i<xorg.InputDevices().GetSize(); i++)
  {
    if (i!=inputDeviceCurrent && xorg.InputDevices()[i].GetId()==inputDeviceIdentifierText)
    {
      wxMessageBox(_("Device identifier already exists!"), _("Error in Input Device"), wxICON_ERROR);
      inputDeviceIdentifier->SetSelection(inputDeviceCurrent);
      inputDeviceIdentifier->SetFocus();
      return(false);
    }
  }

  // check if empty
  if (inputDeviceDriver->GetValue().IsEmpty())
  {
    wxMessageBox(_("Input device driver must not be empty!"), _("Error in Input Device"), wxICON_ERROR);
    inputDeviceIdentifier->SetSelection(inputDeviceCurrent);
    inputDeviceDriver->SetFocus();
    return(false);
  }

  if (!CheckInputDeviceOption()) return(false);
  return(true);
}

bool GuiFrame::CheckInputDeviceOption(void)
{
  if (xorg.InputDevices()[inputDeviceCurrent].GetOptionSize()==0) return(true);

  if (xorg.InputDevices()[inputDeviceCurrent].GetOption(inputDeviceOptionCurrent).GetName().IsEmpty())
  {
    wxMessageBox(_("Input device option must not be empty!"), _("Error in Input Device"), wxICON_ERROR);
    inputDeviceIdentifier->SetSelection(inputDeviceCurrent);
    inputDeviceOption->SetSelection(inputDeviceOptionCurrent);
    inputDeviceOption->SetFocus();
    return(false);
  }

  for (int i=0; i<xorg.InputDevices()[inputDeviceCurrent].GetOptionSize(); i++)
  {
    if (i!=inputDeviceOptionCurrent && xorg.InputDevices()[inputDeviceCurrent].GetOption(i).GetName()==xorg.InputDevices()[inputDeviceCurrent].GetOption(inputDeviceOptionCurrent).GetName())
    {
      wxMessageBox(_("Input device option already exists!"), _("Error in Input Device"), wxICON_ERROR);
      inputDeviceIdentifier->SetSelection(inputDeviceCurrent);
      inputDeviceOption->SetSelection(inputDeviceOptionCurrent);
      inputDeviceOption->SetFocus();
      return(false);
    }
  }

  // check options
  wxString filename=settings.GetOptionDir()+_T("/inputdevices/")+inputDeviceDriver->GetValue()+_T(".xml");
  inputDeviceDriversOption.Load(filename);

  // search selected option in list
  int pos=inputDeviceDriversOption.Find(xorg.InputDevices()[inputDeviceCurrent].GetOption(inputDeviceOptionCurrent).GetName());

  if (pos==-1) return(true); // no error

  if (inputDeviceDriversOption[pos].GetValue()==_T("boolean"))
  {
    if (inputDeviceOptValue->GetValue()!=_T("1") && inputDeviceOptValue->GetValue()!=_T("on") && inputDeviceOptValue->GetValue()!=_T("true") && inputDeviceOptValue->GetValue()!=_T("yes")
     && inputDeviceOptValue->GetValue()!=_T("0") && inputDeviceOptValue->GetValue()!=_T("off") && inputDeviceOptValue->GetValue()!=_T("false") && inputDeviceOptValue->GetValue()!=_T("no"))
    {
      wxMessageBox(_("Input device option must be boolean!"), _("Error in Input Device"), wxICON_ERROR);
      inputDeviceIdentifier->SetSelection(inputDeviceCurrent);
      inputDeviceOption->SetSelection(inputDeviceOptionCurrent);
      inputDeviceOptValue->SetFocus();
      return(false);
    }
  }
  else
  {
    if (inputDeviceDriversOption[pos].GetValue()==_T("integer"))
    {
      bool isInt=true;

      if (inputDeviceOptValue->GetValue().StartsWith(_T("0x")))
      {  // possible hexadecimal number

        for (int i=2; i<(int)deviceOptValue->GetValue().Len(); i++)
        {
          int c=inputDeviceOptValue->GetValue()[i];

          if (c!='0' && c!='1' && c!='2' && c!='3' && c!='4' && c!='5' && c!='6'
           && c!='7' && c!='8' && c!='9' && c!='0' && c!='A' && c!='B' && c!='C'
           && c!='D' && c!='E' && c!='F' && c!='a' && c!='b' && c!='c' && c!='d'
           && c!='e' && c!='f')
          {
            isInt=false;
            break;
          }
        }
      }
      else if (!inputDeviceOptValue->GetValue().IsNumber())
      {
        isInt=false;
      }

      if (!isInt)
      {
        wxMessageBox(_("Input device option must be integer!"), _("Error in Input Device"), wxICON_ERROR);
        inputDeviceIdentifier->SetSelection(inputDeviceCurrent);
        inputDeviceOption->SetSelection(inputDeviceOptionCurrent);
        inputDeviceOptValue->SetFocus();
        return(false);
      }
    }

    if (inputDeviceDriversOption[pos].IsStrict())
    {
      if (inputDeviceDriversOption[pos].GetArguments().Index(inputDeviceOptValue->GetValue())==wxNOT_FOUND)
      {
        wxMessageBox(_("Input device option must be in argument list!"), _("Error in Input Device"), wxICON_ERROR);
        inputDeviceIdentifier->SetSelection(inputDeviceCurrent);
        inputDeviceOption->SetSelection(inputDeviceOptionCurrent);
        inputDeviceOptValue->SetFocus();
        return(false);
      }
    }
  }

  return(true);
}

bool GuiFrame::CheckScreenTab(void) const
{
  if (!CheckScreen()) return(false);
  return(true);
}

bool GuiFrame::CheckScreen(void) const
{
  if (xorg.Screens().GetSize()==0) return(true);

  long tempL=0;
  wxString tempStr;

  // check if empty
  if (screenIdentifierText.IsEmpty())
  {
    wxMessageBox(_("Screen identifier must not be empty!"), _("Error in Screen"), wxICON_ERROR);
    screenIdentifier->SetSelection(screenCurrent);
    screenIdentifier->SetFocus();
    return(false);
  }

  // check if unique
  for (int i=0; i<xorg.Screens().GetSize(); i++)
  {
    if (i!=screenCurrent && xorg.Screens()[i].GetId()==screenIdentifierText)
    {
      wxMessageBox(_("Screen identifier already exists!"), _("Error in Screen"), wxICON_ERROR);
      screenIdentifier->SetSelection(screenCurrent);
      screenIdentifier->SetFocus();
      return(false);
    }
  }

  tempStr=screenDepth->GetValue();
  tempStr.Trim(false);
  tempStr.Trim(true);
  tempStr.ToLong(&tempL);

  if (tempStr.IsEmpty())
  {
    wxMessageBox(_("Screen depth must not be empty!"), _("Error in Screen"), wxICON_ERROR);
    screenIdentifier->SetSelection(screenCurrent);
    screenDepth->SetFocus();
    return(false);
  }

  if (!tempStr.IsNumber() || tempL<0)
  {
    wxMessageBox(_("Screen depth must be a non-negative integer!"), _("Error in Screen"), wxICON_ERROR);
    screenIdentifier->SetSelection(screenCurrent);
    screenDepth->SetFocus();
    return(false);
  }
  tempStr.Printf(_T("%d"), tempL);
  screenDepth->SetValue(tempStr);

  if (!CheckScreenOption()) return(false);
  if (!CheckScreenDisplay()) return(false);
  return(true);
}

bool GuiFrame::CheckScreenOption(void) const
{
  if (xorg.Screens()[screenCurrent].GetOptionSize()==0) return(true);

  if (xorg.Screens()[screenCurrent].GetOption(screenOptionCurrent).GetName().IsEmpty())
  {
    wxMessageBox(_("Screen option must not be empty!"), _("Error in Screen"), wxICON_ERROR);
    screenIdentifier->SetSelection(screenCurrent);
    screenOption->SetSelection(screenOptionCurrent);
    screenOption->SetFocus();
    return(false);
  }
  for (int i=0; i<xorg.Screens()[screenCurrent].GetOptionSize(); i++)
  {
    if (i!=screenOptionCurrent && xorg.Screens()[screenCurrent].GetOption(i).GetName()==xorg.Screens()[screenCurrent].GetOption(screenOptionCurrent).GetName())
    {
      wxMessageBox(_("Screen option already exists!"), _("Error in Screen"), wxICON_ERROR);
      screenIdentifier->SetSelection(screenCurrent);
      screenOption->SetSelection(screenOptionCurrent);
      screenOption->SetFocus();
      return(false);
    }
  }
  return(true);
}

bool GuiFrame::CheckScreenDisplay(void) const
{
  if (xorg.Screens()[screenCurrent].GetDisplaySize()==0) return(true);
  if (!CheckScreenDisplayOption()) return(false);
  if (!CheckScreenDisplayMode()) return(false);

  long tempL=0;
  wxString tempStr;

  tempStr=xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetDepth();
  tempStr.Trim(false);
  tempStr.Trim(true);
  tempStr.ToLong(&tempL);

  if (tempStr.IsEmpty())
  {
    wxMessageBox(_("Screen display depth must not be empty!"), _("Error in Screen"), wxICON_ERROR);
    screenIdentifier->SetSelection(screenCurrent);
    screenDisplayDepth->SetSelection(screenDisplayCurrent);
    screenDisplayDepth->SetFocus();
    return(false);
  }

  if (!tempStr.IsNumber() || tempL<0)
  {
    wxMessageBox(_("Screen display depth must be a non-negative integer!"), _("Error in Screen"), wxICON_ERROR);
    screenIdentifier->SetSelection(screenCurrent);
    screenDisplayDepth->SetSelection(screenDisplayCurrent);
    screenDisplayDepth->SetFocus();
    return(false);
  }

  for (int i=0; i<xorg.Screens()[screenCurrent].GetDisplaySize(); i++)
  {
    if (i!=screenDisplayCurrent && xorg.Screens()[screenCurrent].GetDisplay(i).GetDepth()==xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetDepth())
    {
      wxMessageBox(_("Screen display already exists!"), _("Error in Screen"), wxICON_ERROR);
      screenIdentifier->SetSelection(screenCurrent);
      screenDisplayDepth->SetSelection(screenDisplayCurrent);
      screenDisplayDepth->SetFocus();
      return(false);
    }
  }

  return(true);
}

bool GuiFrame::CheckScreenDisplayOption(void) const
{
  if (xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOptionSize()==0) return(true);

  if (xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOption(screenDisplayOptionCurrent).GetName().IsEmpty())
  {
    wxMessageBox(_("Screen display option must not be empty!"), _("Error in Screen"), wxICON_ERROR);
    screenIdentifier->SetSelection(screenCurrent);
    screenDisplayDepth->SetSelection(screenDisplayCurrent);
    screenDisplayOption->SetSelection(screenDisplayOptionCurrent);
    screenDisplayOption->SetFocus();
    return(false);
  }
  for (int i=0; i<xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOptionSize(); i++)
  {
    if (i!=screenDisplayOptionCurrent && xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOption(i).GetName()==xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetOption(screenDisplayOptionCurrent).GetName())
    {
      wxMessageBox(_("Screen display option already exists!"), _("Error in Screen"), wxICON_ERROR);
      screenIdentifier->SetSelection(screenCurrent);
      screenDisplayDepth->SetSelection(screenDisplayCurrent);
      screenDisplayOption->SetSelection(screenDisplayOptionCurrent);
      screenDisplayOption->SetFocus();
      return(false);
    }
  }
  return(true);
}

bool GuiFrame::CheckScreenDisplayMode(void) const
{
  if (xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetModeSize()==0) return(true);

  if (xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetMode(screenDisplayModeCurrent).IsEmpty())
  {
    wxMessageBox(_("Screen display mode must not be empty!"), _("Error in Screen"), wxICON_ERROR);
    screenIdentifier->SetSelection(screenCurrent);
    screenDisplayDepth->SetSelection(screenDisplayCurrent);
    screenDisplayModes->SetSelection(screenDisplayModeCurrent);
    screenDisplayModes->SetFocus();
    return(false);
  }
  for (int i=0; i<xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetModeSize(); i++)
  {
    if (i!=screenDisplayModeCurrent && xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetMode(i)==xorg.Screens()[screenCurrent].GetDisplay(screenDisplayCurrent).GetMode(screenDisplayModeCurrent))
    {
      wxMessageBox(_("Screen display mode already exists!"), _("Error in Screen"), wxICON_ERROR);
      screenIdentifier->SetSelection(screenCurrent);
      screenDisplayDepth->SetSelection(screenDisplayCurrent);
      screenDisplayModes->SetSelection(screenDisplayModeCurrent);
      screenDisplayModes->SetFocus();
      return(false);
    }
  }

  return(true);
}

bool GuiFrame::CheckServerTab(void) const
{
  if (!CheckServerLayout()) return(false);
  if (!CheckServerFlagOption()) return(false);
  return(true);
}

bool GuiFrame::CheckServerLayout(void) const
{
  if (xorg.ServerLayouts().GetSize()==0) return(true);

  // check if empty
  if (xorg.ServerLayouts()[serverLayoutCurrent].GetId().IsEmpty())
  {
    wxMessageBox(_("Server layout identifier must not be empty!"), _("Error in Server Layout"), wxICON_ERROR);
    serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
    serverLayoutIdentifier->SetFocus();
    return(false);
  }

  // check if unique
  for (int i=0; i<xorg.ServerLayouts().GetSize(); i++)
  {
    if (i!=serverLayoutCurrent && xorg.ServerLayouts()[i].GetId()==xorg.ServerLayouts()[serverLayoutCurrent].GetId())
    {
      wxMessageBox(_("Server layout identifier already exists!"), _("Error in Server Layout"), wxICON_ERROR);
      serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
      serverLayoutIdentifier->SetFocus();
      return(false);
    }
  }

  if(!CheckServerLayoutOption()) return(false);
  if(!CheckServerLayoutScreen()) return(false);
  if(!CheckServerLayoutInputDevice()) return(false);
  return(true);
}

bool GuiFrame::CheckServerLayoutOption(void) const
{
  if (xorg.ServerLayouts()[serverLayoutCurrent].GetOptionSize()==0) return(true);

  if (xorg.ServerLayouts()[serverLayoutCurrent].GetOption(serverLayoutOptionCurrent).GetName().IsEmpty())
  {
    wxMessageBox(_("Server layout option must not be empty!"), _("Error in Server Layout"), wxICON_ERROR);
    serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
    serverLayoutOption->SetSelection(serverLayoutOptionCurrent);
    serverLayoutOption->SetFocus();
    return(false);
  }
  for (int i=0; i<xorg.ServerLayouts()[serverLayoutCurrent].GetOptionSize(); i++)
  {
    if (i!=serverLayoutOptionCurrent && xorg.ServerLayouts()[serverLayoutCurrent].GetOption(i).GetName()==xorg.ServerLayouts()[serverLayoutCurrent].GetOption(serverLayoutOptionCurrent).GetName())
    {
      wxMessageBox(_("Server layout option already exists!"), _("Error in Server Layout"), wxICON_ERROR);
      serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
      serverLayoutOption->SetSelection(serverLayoutOptionCurrent);
      serverLayoutOption->SetFocus();
      return(false);
    }
  }
  return(true);
}

bool GuiFrame::CheckServerLayoutScreen(void) const
{
  if (xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize()==0) return(true);

  long tempL=0;
  wxString tempStr, valueX, valueY;
  int pos=0;

  // check if unique
  for (int i=0; i<xorg.ServerLayouts()[serverLayoutCurrent].GetScreenSize(); i++)
  {
    if (i!=serverLayoutScreenCurrent && xorg.ServerLayouts()[serverLayoutCurrent].GetScreen(i)==serverLayoutScreen1->GetStringSelection())
    {
      wxMessageBox(_("Screen already used in this server layout. Please select another one!"), _("Error in Server Layout"), wxICON_ERROR);
      serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
      serverLayoutScreen->SetSelection(serverLayoutScreenCurrent);
      serverLayoutScreen1->SetFocus();
      return(false);
    }
  }

  // check if double
  if (serverLayoutScreen2->IsEnabled() && serverLayoutScreen1->GetStringSelection()==serverLayoutScreen2->GetStringSelection())
  {
    wxMessageBox(_("Screen 1 and 2 are the same. Please select another one!"), _("Error in Server Layout"), wxICON_ERROR);
    serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
    serverLayoutScreen->SetSelection(serverLayoutScreenCurrent);
    serverLayoutScreen2->SetFocus();
    return(false);
  }

  tempStr=serverLayoutScreenNumber->GetValue();
  tempStr.Trim(false);
  tempStr.Trim(true);
  tempStr.ToLong(&tempL);

  if (!tempStr.IsEmpty())
  {
    if (!tempStr.IsNumber() || tempL<0)
    {
      wxMessageBox(_("Server layout screen number must be a non-negative integer!"), _("Error in Server Layout"), wxICON_ERROR);
      serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
      serverLayoutScreen->SetSelection(serverLayoutScreenCurrent);
      serverLayoutScreenNumber->SetFocus();
      return(false);
    }
    tempStr.Printf(_T("%d"), tempL);
    serverLayoutScreenNumber->SetValue(tempStr);
  }

  tempStr=serverLayoutScreenPosition->GetValue();
  tempStr.Trim(false);
  tempStr.Trim(true);
  tempStr.MakeLower();

  if (tempStr.StartsWith(_T("absolute")))
  {
    pos=tempStr.Find(' ');
    if (pos==-1)
    {
      wxMessageBox(_("Server layout screen position is wrong!"), _("Error in Server Layout"), wxICON_ERROR);
      serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
      serverLayoutScreen->SetSelection(serverLayoutScreenCurrent);
      serverLayoutScreenPosition->SetFocus();
      return(false);
    }
    tempStr=tempStr.Right(tempStr.Len()-pos-1);
    tempStr.Trim(false);

    pos=tempStr.Find(' ');
    if (pos==-1)
    {
      wxMessageBox(_("Server layout screen position is wrong!"), _("Error in Server Layout"), wxICON_ERROR);
      serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
      serverLayoutScreen->SetSelection(serverLayoutScreenCurrent);
      serverLayoutScreenPosition->SetFocus();
      return(false);
    }

    valueX=tempStr.Left(pos);
    tempStr=tempStr.Right(tempStr.Len()-pos-1);
    tempStr.Trim(false);

    pos=tempStr.Find(' ');
    if (pos!=-1)
    {
      wxMessageBox(_("Server layout screen position is wrong!"), _("Error in Server Layout"), wxICON_ERROR);
      serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
      serverLayoutScreen->SetSelection(serverLayoutScreenCurrent);
      serverLayoutScreenPosition->SetFocus();
      return(false);
    }
    valueY=tempStr;

    if (!valueX.IsNumber() || !valueY.IsNumber())
    {
      wxMessageBox(_("Absolute values of server layout screen position must be non-negative!"), _("Error in Server Layout"), wxICON_ERROR);
      serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
      serverLayoutScreen->SetSelection(serverLayoutScreenCurrent);
      serverLayoutScreenPosition->SetFocus();
      return(false);
    }

  }
  else if (tempStr==_T("rightof") || tempStr==_T("leftof") || tempStr==_T("above") || tempStr==_T("below"))
  {
    // everythings fine
  }
  else if (tempStr==_T("relative"))
  {
    wxMessageBox(_("I'm sorry, but the \"relative\" option for screen position is not supported yet!"), _("Error in Server Layout"), wxICON_ERROR);
    serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
    serverLayoutScreen->SetSelection(serverLayoutScreenCurrent);
    serverLayoutScreenPosition->SetFocus();
    return(false);
  }
  else if (!tempStr.IsEmpty())
  {
    pos=tempStr.Find(' ');
    if (pos==-1)
    {
      wxMessageBox(_("Server layout screen position is wrong!"), _("Error in Server Layout"), wxICON_ERROR);
      serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
      serverLayoutScreen->SetSelection(serverLayoutScreenCurrent);
      serverLayoutScreenPosition->SetFocus();
      return(false);
    }

    valueX=tempStr.Left(pos);
    tempStr=tempStr.Right(tempStr.Len()-pos-1);
    tempStr.Trim(false);

    pos=tempStr.Find(' ');
    if (pos!=-1)
    {
      wxMessageBox(_("Server layout screen position is wrong!"), _("Error in Server Layout"), wxICON_ERROR);
      serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
      serverLayoutScreen->SetSelection(serverLayoutScreenCurrent);
      serverLayoutScreenPosition->SetFocus();
      return(false);
    }
    valueY=tempStr;

    if (!valueX.IsNumber() || !valueY.IsNumber())
    {
      wxMessageBox(_("Absolute values of server layout screen position must be non-negative!"), _("Error in Server Layout"), wxICON_ERROR);
      serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
      serverLayoutScreen->SetSelection(serverLayoutScreenCurrent);
      serverLayoutScreenPosition->SetFocus();
      return(false);
    }
  }

  return(true);
}

bool GuiFrame::CheckServerLayoutInputDevice(void) const
{
  if (xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize()==0) return(true);

  // check if unique
  for (int i=0; i<xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceSize(); i++)
  {
    if (i!=serverLayoutIDevCurrent && xorg.ServerLayouts()[serverLayoutCurrent].GetInputDevice(i)==serverLayoutInputDevice1->GetStringSelection())
    {
      wxMessageBox(_("Input device already used in this server layout. Please select another one!"), _("Error in Server Layout"), wxICON_ERROR);
      serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
      serverLayoutInputDevice->SetSelection(serverLayoutIDevCurrent);
      serverLayoutInputDevice1->SetFocus();
      return(false);
    }
  }

  if(!CheckServerLayoutInputDeviceOption()) return(false);

  return(true);
}

bool GuiFrame::CheckServerLayoutInputDeviceOption(void) const
{
  if (xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOptionSize(serverLayoutIDevCurrent)==0) return(true);

  if (xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOption(serverLayoutIDevCurrent, serverLayoutIDevOptionCurrent).GetName().IsEmpty())
  {
    wxMessageBox(_("Server layout input device option must not be empty!"), _("Error in Server Layout"), wxICON_ERROR);
    serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
    serverLayoutInputDevice->SetSelection(serverLayoutIDevCurrent);
    serverLayoutInputDeviceOption->SetSelection(serverLayoutIDevOptionCurrent);
    serverLayoutInputDeviceOption->SetFocus();
    return(false);
  }
  for (int i=0; i<xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOptionSize(serverLayoutIDevCurrent); i++)
  {
    if (i!=serverLayoutIDevOptionCurrent && xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOption(serverLayoutIDevCurrent, i).GetName()==xorg.ServerLayouts()[serverLayoutCurrent].GetInputDeviceOption(serverLayoutIDevCurrent, serverLayoutIDevOptionCurrent).GetName())
    {
      wxMessageBox(_("Server layout input device option already exists!"), _("Error in Server Layout"), wxICON_ERROR);
      serverLayoutIdentifier->SetSelection(serverLayoutCurrent);
      serverLayoutInputDevice->SetSelection(serverLayoutIDevCurrent);
      serverLayoutInputDeviceOption->SetSelection(serverLayoutIDevOptionCurrent);
      serverLayoutInputDeviceOption->SetFocus();
      return(false);
    }
  }
  return(true);
}

bool GuiFrame::CheckServerFlagOption(void) const
{
  if (xorg.ServerFlags().GetOptionSize()==0) return(true);

  if (xorg.ServerFlags().GetOption(serverFlagCurrent).GetName().IsEmpty())
  {
    wxMessageBox(_("Server flag must not be empty!"), _("Error in Server Flag"), wxICON_ERROR);
    serverFlagsOption->SetSelection(serverFlagCurrent);
    serverFlagsOption->SetFocus();
    return(false);
  }
  for (int i=0; i<xorg.ServerFlags().GetOptionSize(); i++)
  {
    if (i!=serverFlagCurrent && xorg.ServerFlags().GetOption(i).GetName()==xorg.ServerFlags().GetOption(serverFlagCurrent).GetName())
    {
      wxMessageBox(_("Server flag already exists!"), _("Error in Server Flag"), wxICON_ERROR);
      serverFlagsOption->SetSelection(serverFlagCurrent);
      serverFlagsOption->SetFocus();
      return(false);
    }
  }
  return(true);
}

bool GuiFrame::CheckOtherSectionTab(void) const
{
  if(!CheckExtension()) return(false);
  if(!CheckModule()) return(false);
  if(!CheckFiles()) return(false);
  if(!CheckDRI()) return(false);
  return(true);
}

bool GuiFrame::CheckExtension(void) const
{
  if (xorg.Extensions().GetOptionSize()==0) return(true);

  if (xorg.Extensions().GetOption(extensionCurrent).GetName().IsEmpty())
  {
    wxMessageBox(_("Extension must not be empty!"), _("Error in Extension"), wxICON_ERROR);
    extensionOption->SetSelection(extensionCurrent);
    extensionOption->SetFocus();
    return(false);
  }
  for (int i=0; i<xorg.Extensions().GetOptionSize(); i++)
  {
    if (i!=extensionCurrent && xorg.Extensions().GetOption(i).GetName()==xorg.Extensions().GetOption(extensionCurrent).GetName())
    {
      wxMessageBox(_("Extension already exists!"), _("Error in Extension"), wxICON_ERROR);
      extensionOption->SetSelection(extensionCurrent);
      extensionOption->SetFocus();
      return(false);
    }
  }
  return(true);
}

bool GuiFrame::CheckModule(void) const
{
  if (xorg.Modules().GetSize()==0) return(true);

  if (xorg.Modules().Get(moduleCurrent).IsEmpty())
  {
    wxMessageBox(_("Module name must not be empty!"), _("Error in Modules"), wxICON_ERROR);
    moduleIdentifier->SetSelection(moduleCurrent);
    moduleIdentifier->SetFocus();
    return(false);
  }
  for (int i=0; i<xorg.Modules().GetSize(); i++)
  {
    if (i!=moduleCurrent && xorg.Modules().Get(i)==xorg.Modules().Get(moduleCurrent))
    {
      wxMessageBox(_("Module name already exists!"), _("Error in Modules"), wxICON_ERROR);
      moduleIdentifier->SetSelection(moduleCurrent);
      moduleIdentifier->SetFocus();
      return(false);
    }
  }

  if(!CheckModuleOption()) return(false);
  return(true);
}

bool GuiFrame::CheckModuleOption(void) const
{
  if (xorg.Modules().GetOptionSize(moduleCurrent)==0) return(true);

  if (xorg.Modules().GetOption(moduleCurrent, moduleOptionCurrent).GetName().IsEmpty())
  {
    wxMessageBox(_("Module option must not be empty!"), _("Error in Modules"), wxICON_ERROR);
    moduleIdentifier->SetSelection(moduleCurrent);
    moduleOption->SetSelection(moduleOptionCurrent);
    moduleOption->SetFocus();
    return(false);
  }
  for (int i=0; i<xorg.Modules().GetOptionSize(moduleCurrent); i++)
  {
    if (i!=moduleOptionCurrent && xorg.Modules().GetOption(moduleCurrent, i).GetName()==xorg.Modules().GetOption(moduleCurrent, moduleOptionCurrent).GetName())
    {
      wxMessageBox(_("Module option already exists!"), _("Error in Modules"), wxICON_ERROR);
      moduleIdentifier->SetSelection(moduleCurrent);
      moduleOption->SetSelection(moduleOptionCurrent);
      moduleOption->SetFocus();
      return(false);
    }
  }
  return(true);
}

bool GuiFrame::CheckFiles(void) const
{
  if(!CheckFontPath()) return(false);
  if(!CheckModulePath()) return(false);
  if(!CheckRgbPath()) return(false);
  return(true);
}

bool GuiFrame::CheckFontPath(void) const
{
  if (xorg.Files().GetFontPathSize()==0) return(true);

  if (xorg.Files().GetFontPath(fontPathCurrent).IsEmpty())
  {
    wxMessageBox(_("Font path must not be empty!"), _("Error in Files"), wxICON_ERROR);
    fontPath->SetSelection(fontPathCurrent);
    fontPath->SetFocus();
    return(false);
  }
  for (int i=0; i<xorg.Files().GetFontPathSize(); i++)
  {
    if (i!=fontPathCurrent && xorg.Files().GetFontPath(i)==xorg.Files().GetFontPath(fontPathCurrent))
    {
      wxMessageBox(_("Font path already exists!"), _("Error in Files"), wxICON_ERROR);
      fontPath->SetSelection(fontPathCurrent);
      fontPath->SetFocus();
      return(false);
    }
  }
  return(true);
}

bool GuiFrame::CheckModulePath(void) const
{
  if (xorg.Files().GetModulePathSize()==0) return(true);

  if (xorg.Files().GetModulePath(modulePathCurrent).IsEmpty())
  {
    wxMessageBox(_("Module path must not be empty!"), _("Error in Files"), wxICON_ERROR);
    modulePath->SetSelection(modulePathCurrent);
    modulePath->SetFocus();
    return(false);
  }
  for (int i=0; i<xorg.Files().GetModulePathSize(); i++)
  {
    if (i!=modulePathCurrent && xorg.Files().GetModulePath(i)==xorg.Files().GetModulePath(modulePathCurrent))
    {
      wxMessageBox(_("Module path already exists!"), _("Error in Files"), wxICON_ERROR);
      modulePath->SetSelection(modulePathCurrent);
      modulePath->SetFocus();
      return(false);
    }
  }
  return(true);
}

bool GuiFrame::CheckRgbPath(void) const
{
  if (xorg.Files().GetRgbPathSize()==0) return(true);

  if (xorg.Files().GetRgbPath(rgbPathCurrent).IsEmpty())
  {
    wxMessageBox(_("RGB path must not be empty!"), _("Error in Files"), wxICON_ERROR);
    rgbPath->SetSelection(rgbPathCurrent);
    rgbPath->SetFocus();
    return(false);
  }
  for (int i=0; i<xorg.Files().GetRgbPathSize(); i++)
  {
    if (i!=rgbPathCurrent && xorg.Files().GetRgbPath(i)==xorg.Files().GetRgbPath(rgbPathCurrent))
    {
      wxMessageBox(_("RGB path already exists!"), _("Error in Files"), wxICON_ERROR);
      rgbPath->SetSelection(rgbPathCurrent);
      rgbPath->SetFocus();
      return(false);
    }
  }
  return(true);
}

bool GuiFrame::CheckDRI(void) const
{
  return(true); // nothing to check
}

// accept functions

void GuiFrame::AcceptDevice(void)
{
  if (xorg.Devices().GetSize()==0) return;

  if (xorg.Devices()[deviceCurrent].GetId()!=deviceIdentifierText)
  {
    // check if there are some screen that use this device and change the name there
    for (int i=0; i<xorg.Screens().GetSize(); i++)
    {
      if (xorg.Screens()[i].GetDevice()==xorg.Devices()[deviceCurrent].GetId())
      {
        xorg.Screens()[i].SetDevice(deviceIdentifierText);
      }
    }
    xorg.Devices()[deviceCurrent].SetId(deviceIdentifierText);

    // reinit screen
    UpdateScreen();
  }
}

void GuiFrame::AcceptMonitor(void)
{
  if (xorg.Monitors().GetSize()==0) return;

  if (xorg.Monitors()[monitorCurrent].GetId()!=monitorIdentifierText)
  {
    // check if there are some screen that use this monitor and change the name there
    for (int i=0; i<xorg.Screens().GetSize(); i++)
    {
      if (xorg.Screens()[i].GetMonitor()==xorg.Monitors()[monitorCurrent].GetId())
      {
        xorg.Screens()[i].SetMonitor(monitorIdentifierText);
      }
    }
    xorg.Monitors()[monitorCurrent].SetId(monitorIdentifierText);

    // reinit screen
    UpdateScreen();
  }
}

void GuiFrame::AcceptInputDevice(void)
{
  if (xorg.InputDevices().GetSize()==0) return;

  if (xorg.InputDevices()[inputDeviceCurrent].GetId()!=inputDeviceIdentifierText)
  {
    // check if there are some server layout that use this input device and change name there
    for (int i=0; i<xorg.ServerLayouts().GetSize(); i++)
    {
      for (int j=0; j<xorg.ServerLayouts()[i].GetInputDeviceSize(); j++)
      {
        if (xorg.ServerLayouts()[i].GetInputDevice(j)==xorg.InputDevices()[inputDeviceCurrent].GetId())
        {
          xorg.ServerLayouts()[i].SetInputDevice(j, inputDeviceIdentifierText);
        }
      }
    }
    xorg.InputDevices()[inputDeviceCurrent].SetId(inputDeviceIdentifierText);

    // reinit server layout
    UpdateServerLayout();
  }
}

void GuiFrame::AcceptScreen(void)
{
  if (xorg.Screens().GetSize()==0) return;

  if (xorg.Screens()[screenCurrent].GetId()!=screenIdentifierText)
  {
    // check if there are some server layout that use this screen and change name there
    for (int i=0; i<xorg.ServerLayouts().GetSize(); i++)
    {
      for (int j=0; j<xorg.ServerLayouts()[i].GetScreenSize(); j++)
      {
        if (xorg.ServerLayouts()[i].GetScreen(j)==xorg.Screens()[screenCurrent].GetId())
        {
          xorg.ServerLayouts()[i].SetScreen(j, screenIdentifierText);
        }
        if (xorg.ServerLayouts()[i].GetSecondScreen(j)==xorg.Screens()[screenCurrent].GetId())
        {
          xorg.ServerLayouts()[i].SetSecondScreen(j, screenIdentifierText);
        }
      }
    }
    xorg.Screens()[screenCurrent].SetId(screenIdentifierText);

    // reinit server layout
    UpdateServerLayout();
  }
}

void GuiFrame::LoadDrivers(void)
{  // load driver lists

  wxString filename;

  filename=settings.GetOptionDir()+_T("/devices/list.xml");
  fileHandler.LoadDrivers(deviceDriverList, filename);
  deviceDriver->Clear();
  deviceDriver->Append(deviceDriverList);

  filename=settings.GetOptionDir()+_T("/inputdevices/list.xml");
  fileHandler.LoadDrivers(inputDeviceDriverList, filename);
  inputDeviceDriver->Clear();
  inputDeviceDriver->Append(inputDeviceDriverList);

}

void GuiFrame::StripSpaces(wxString& line) const
{         // convert tabs to spaces, delete spaces from left and right,

  line.Replace(_T("\t"), _T(" "));         // replace tabs with spaces
  while (line.Replace(_T("  "), _T(" "))>0);  // replace double spaces with single space
  line.Trim(true);     // strip spaces from right
  line.Trim(false);    // strip spaces from left
}

void GuiFrame::StripRight(wxString& line, const int num) const
{  // delete n right characters

  line=line.Right(line.Length()-num);
  StripSpaces(line);
}

